<?php
namespace Teknisa\CADEST\Modules\Produto\Controllers;

use Teknisa\CADEST\Modules\Produto\Services\Produto as ProdutoService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;

class ProdutoArvore extends \Teknisa\CADEST\Controllers\TeknisaCrud {

    protected $dataSourceName = "produto_para_arvore";
    protected $dataSourceNameToSave = "produto";
    protected $produtoService;

    public function __construct($crudService, $parameterBag, ProdutoService $produtoService, $environmment) {
        parent::__construct($crudService, $parameterBag);
        $this->produtoService = $produtoService;
        $this->operador = $environmment->getCdOperador();
        $this->nrorg = $environmment->getNrOrgTrab();
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions     = $filterCriteria->getConditions();
        $container      = $request->getParameters()['origin']['containerName'];

        //uncomment lines when Zeedhi Framework manage to get faster
        if(count($conditions) == 0) {
            $filterCriteria->addCondition("CDPRODNIVANTE", "IS_NULL", null);
        }
        
        $operador = false;
        foreach($conditions as $filter){
            if($filter['columnName'] == 'P_CDOPERADOR')
                $operador = true;
        }
        
        if(!$operador){
            if($container == 'ger01902_arvoreproduto_controller')
                $filterCriteria->addCondition("P_CDOPERADOR", "=", 'T');
            else
                $filterCriteria->addCondition("P_CDOPERADOR", "=", $this->operador);
        }
        
        if($container != 'ger01902_arvoreproduto_controller')
            $filterCriteria->addCondition("PERMVISUPROD", "<>", 'B');
            
        parent::find($request, $response);
        
        $dataSets = $response->getDataSets();
        if (count($dataSets) > 0) {
            $rows = $dataSets[0]->getRows();

            $rows = array_map(function($row) {
                $row['_hasChild'] = $row['HASCHILD'] === 'true' ? true : false;
                unset($row['HASCHILD']);
                return $row;
            }, $rows);

            $response->setDataSets(array(new DataSet('produto_para_arvore', $rows)));
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {  
            $row['CDARVPROD'] = $this->produtoService->removeMask($row['CDARVPROD']);
            
            $row['CDPRODNIVANTE'] = isset($row['CDARVPRODNIVANTE']) ? $row['CDARVPRODNIVANTE'] : $row['CDPRODNIVANTE'];
            $this->produtoService->arvprodContainsProdnivante($row['CDARVPROD'], $row['CDPRODNIVANTE']);
            
            $row['CDNVPRODUTO'] = $this->produtoService->getCdnvprodutoByCdarvprod($row['CDARVPROD']);
            if ($row['__is_new'] === true) {
                $this->produtoService->existArvprod($row['CDARVPROD']);
                $row['CDPRODUTO'] = $this->produtoService->getNewCdproduto($row['CDARVPROD']);
                $row['__columns_to_return'] = array('CDPRODUTO');
            }
            $this->produtoService->correctLevelByArvprod($row['CDARVPROD'], $row['CDNVPRODUTO']);
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveMask(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = 'paramgeral';
        $this->dataSourceNameToSave = 'paramgeral';
        parent::save($request, $response);
        
         $this->dataSourceName = 'paramimpprod';
        $this->dataSourceNameToSave = 'paramimpprod';
        
        $newFilterCriteria = new FilterCriteria($this->dataSourceName, array());
        $filter = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $responseParam = new DTO\Response;
        parent::find($filter, $responseParam);
        if($responseParam->getDataSets()[0]){
            $dataSet = $responseParam->getDataSets()[0]->getRows();
        }else{
            $dataSet = $responseParam->getDataSets();
        }
        $dataSet = new DataSet("paramimpprod", $dataSet);
        $deleteRequest = new DTO\Request\DataSet($dataSet, $request->getMethod(), '/paramimpprod/delete', $request->getUserId());
            
        parent::delete($deleteRequest, $responseParam);
        $this->produtoService->addNewParamimpprod();
    }

    public function suggestions(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $sugestedarvprod = $this->produtoService->getSugestedCdarvprod($row['PARENTCDPRODUTO'], $row['PARENTNVPRODUTO']);
        $return = array(
            'CDDARVPROD'=>$sugestedarvprod
        );
        
        $dataSet = new DataSet('sugestedArvprod', array($return));
        $response->setDataSets(array($dataSet));
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            $this->produtoService->canBeDeletedTree($row['CDARVPROD'], $row['CDPRODUTO']);
            $row['NRORG'] = $this->nrorg;
            parent::delete($request, $response);
        } catch(\Exception $e) { 
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}
