function QueryController(templateManager, QueryService, metaDataFactory, ScreenService, $compile){

    var self = this;
    var QUERY_SELECT = "SELECT";
    var QUERY_PROCEDURE = "PROCEDURE";
    var QUERY_TEMPLATE = "WITH A AS (%q) SELECT * FROM (SELECT ROWNUM AS ZH_ROWNUM, A.* FROM  A)  WHERE ZH_ROWNUM >= %s AND ZH_ROWNUM <= %e";
    var ENDL = "\n\n";
    var pageSize = 300;
    var pageIndex = 0;
    var hasNextPage = false;
    var loadingMore = false;
    var backupRow;


    function insertRowNumberInQuery(query, pageStart){
        query = QUERY_TEMPLATE.replace("%q", query);
        query = query.replace("%s", pageStart);
        query = query.replace("%e", pageStart + pageSize - 1);
        return query;
    }

    /**
     * Clear the breaklines
     *
     * @param {query} query to clear
     */
    function cleanQuery(query){
        query = query.replace(/(\r\n|\n|\r)/gm," ");
        return query;
    }

    function addRowNumber(query){
        var queryWidget = templateManager.container.getWidget('query');

        /*if(query.includes(';')){
            query = cleanQuery(query);
            var querys = query.split(';');
            querys.pop();
            querys.forEach(function(query, index, querys){
                if(query.toUpperCase().includes(QUERY_SELECT)){
                    query = insertRowNumberInQuery(query, (pageIndex * pageSize) + 1);
                }
            });
            query = querys;
        } else */if(query.toUpperCase().includes(QUERY_SELECT) && !query.toUpperCase().includes(QUERY_PROCEDURE)){
            query = insertRowNumberInQuery(query, (pageIndex * pageSize) + 1);
        }
        return query;
    }

    this.showResult = function(result, loadingMore){
        if(result instanceof Array){
            var resultWidget = templateManager.container.getWidget("results");
            if(!loadingMore){
                self.clearResults(resultWidget);
                for (var object in result[0]){
                    self.createField(resultWidget, object);
                }
                resultWidget.dataSource.data = result;
                hasNextPage = result.length == pageSize ? true : false;
            } else {
                hasNextPage = result.length == pageSize ? true : false;
                resultWidget.dataSource.data = resultWidget.dataSource.data.concat(result);
            }
        }

        if(!loadingMore){
            ScreenService.openSwipe(templateManager.container.getWidget("results"));
        }
    };

    this.loadMoreResults = function(){
        if(hasNextPage){
            pageIndex++;
            this.queryExecute(true);
        }
    };

    this.openParameterModal = function(parameters){
        var modal = templateManager.container.getWidget("parameterModal");
        var replace = true;

        if(parameters.length === modal.dataSource.data.length){
            replace = false;
            modal.dataSource.data.forEach(function(data,index){
                if(parameters[index] !== data.parameter){
                    replace = true;
                }
            });
        }

        if(replace){
            modal.dataSource.data = [];
            parameters.forEach(function(parameter){
                modal.dataSource.data.push(
                    {
                        parameter: parameter,
                        value: "",
                        ISNULL: false
                    }
                );
            });
        }

        ScreenService.openPopup(modal);
    };

    this.openConsoleSwipe = function(){
        ScreenService.closePopup();
        ScreenService.openSwipe(templateManager.container.getWidget("console"));
        templateManager.container.getWidget("console").activate();
    };

    this.closeResultsSwipe = function(){
        pageIndex = 0;
        ScreenService.closeSwipe().then(function(){
            templateManager.container.getWidget("query").activate();
        });
    };

    this.minifyQuery = function(){
        var queryWidget = templateManager.container.getWidget('query');
        queryWidget.getField("query").value(cleanQuery(queryWidget.getField("query").value()));
        ScreenService.closePopup();
    };

    this.hasParameters = function(query,returnParameters){
        var parameters = [];
        var isParameter = false;
        var parameter = '';
        var isString = false;

        if(query instanceof Array){
            query.forEach(function(q){
                if(self.hasParameters(q,true)){
                    parameters = parameters.concat(self.hasParameters(q,true));
                }
            });
        } else {
            for(var j = 0;j<query.length;j++){
                if((query.charAt(j) == '\'' || query.charAt(j) == '\"')){
                    isString = !isString;
                } else if(!isString){
                    if(query.charAt(j) == ' ' || query.charAt(j) == ',' || query.charAt(j) == '(' || query.charAt(j) == ')'){
                        if(isParameter){
                            parameters.push(parameter);
                        }
                        parameter = '';
                        isParameter = false;
                    } else if(isParameter){
                        parameter += query.charAt(j);
                    } else if(query.charAt(j) == ':'){
                        isParameter = true;
                    }
                }
            }
            if(isParameter){
                parameters.push(parameter);
            }
        }

        return (parameters.length && returnParameters) ? parameters : parameters.length;
    };

    function buildValueSQL(field){
        return field.ISNULL ? 'NULL' : ('\'' + field.value + '\'');
    }

    this.setParameters = function(args){
        if(backupRow.query instanceof Array){
                args.dataSource.data.forEach(function(field){
                    for(var i =0;i<backupRow.query.length;i++){
                        backupRow.query[i] = backupRow.query[i].replace(":" + field.parameter, buildValueSQL(field));
                    }
                });
        } else {
            args.dataSource.data.forEach(function(field){
                backupRow.query = backupRow.query.replace(":" + field.parameter, buildValueSQL(field));
            });
        }

        self.processQuery(backupRow);
        backupRow = undefined;
        ScreenService.closePopup();
    };

    this.logMessage = function(message){
        var log = templateManager.container.getWidget('console').getField('log');
        if(!log.value()){
            log.value(message + ENDL);
        } else log.value(log.value() + message + ENDL);
    };

    this.cleanConsole = function(){
        var log = templateManager.container.getWidget('console').getField('log');
        log.value('');
    };

    this.processQuery = function(row ,loadingMore){
        var resultWidget = templateManager.container.getWidget("results");

        try {
            QueryService.RunQuery(row).then(function(response){
                var successMessage;
                if(!response.dataset.result[0]){
                    self.logMessage(successMessage);
                    self.showResult(0);
                } else if (!response.dataset.result[0].type){
                    successMessage = getSuccessMessage(response);
                    handleNonTypedResult(response.dataset.result, loadingMore, successMessage );
                } else {
                    successMessage = getSuccessMessage(response);
                    getResultMessage(response, successMessage);
                }

            }, function(errorResponse){
                handleError(errorResponse);
            });

        } catch (err){
            ScreenService.alertNotification('Unable to execute query');
        }

        resultWidget.isVisible = true;
        resultWidget.formTemplate = "widget/grid.html";
    };

    function getSuccessMessage(response){
        var successMessage;
        var length = response.dataset.result[0].rows ? response.dataset.result[0].rows.length : response.dataset.result.length;
        successMessage = length > pageSize ? "Showing " + length + " paginated results" : "Query executed successfully, " + length + " results obtained";

        return successMessage;
    }

    function handleNonTypedResult(result, loadingMore, successMessage = "Success!"){
        if(result[0].Error){
            self.logMessage(result[0].Error);
            self.showResult(result, loadingMore);
        } else if(result[0].Mensagem){
            self.logMessage(result[0].Mensagem);
            self.showResult(result, loadingMore);
        } else {
            self.showResult(result, loadingMore);
            self.logMessage(successMessage);
        }
    }

    function getResultMessage(response, successMessage){
        for(var object in response.dataset.result){
            switch (response.dataset.result[object].type){
                case 'error':
                    self.logMessage(response.dataset.result[object].error[0].Error);
                    self.openConsoleSwipe();
                    break;
                case 'dataset':
                    self.logMessage(successMessage);
                    self.showResult(response.dataset.result[object].rows);
                    break;
                case 'message':
                    self.logMessage(response.dataset.result[object].message[0].Mensagem);
                    self.openConsoleSwipe();
                    break;
            }
        }
    }

    function handleError(errorResponse){
        var resultWidget = templateManager.container.getWidget("results");
        var response = {
            "Error": errorResponse.data.error
        };
        self.logMessage(errorResponse.data.error);
        self.openConsoleSwipe();
        self.createField(resultWidget, "Error");
        resultWidget.dataSource.data.push(response);
    }

    this.queryExecute = function(loadingMore = false){
        var queryWidget = templateManager.container.getWidget('query');
        var resultWidget = templateManager.container.getWidget("results");
        if(!loadingMore)this.clearResults(resultWidget);
        if(verifyNeededFields()){

            var query = queryWidget.currentRow.query;

            query = addRowNumber(query);

            var row = Util.clone(queryWidget.currentRow);

            /*if(self.hasParameters(query,false) && !loadingMore){
                row.query = query;
                backupRow = row;
                self.openParameterModal(self.hasParameters(query,true));
            } else {*/
                row.query = query;
                self.processQuery(row, loadingMore);
            //}
        }
    };

    function verifyNeededFields(){
        var queryWidget = templateManager.container.getWidget('query');
        var resultWidget = templateManager.container.getWidget("results");
        if (!queryWidget.currentRow.query){
            ScreenService.alertNotification("Please inform the query");
            resultWidget.isVisible = true;
            return false;
        } else {
            return true;
        }
    }

    /**
     * Creates fields to show the query results
     *
     * @param {widget} Widget where the fields will be created
     * @param {name}   Name to the field to be created
     */
    this.createField = function(widget, name){
        var field = {
            "name": ScreenService.i18n(name),
            "label": name,
            "showOnForm": true,
            "showOnView": true,
            "showOnList": true,
            "readOnly": true,
            "template": "field/text-edit.html"
        };

        metaDataFactory.factoryField(field, widget, true);
    };

    /**
     * Clear the fields created to the last query executed
     *
     * @param {widget} widget to clear
     */
    this.clearResults = function(widget){
        widget.fields = [];
        widget.dataSource.data = [];
        templateManager.container.getWidget("query").activate();
    };

    this.containsLimiter= function(query){
        return ((query.includes("FETCH FIRST")) && (query.includes("ROWS ONLY")));
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('QueryController', QueryController);
});