function ger01700_unidade_controller(engine, Endereco_Controller, eventAggregator, General, ger01700_unidade_service, restEngine, ScreenService, MessageService, templateManager, ZHPromise, LibLoginController, localStorageEngine) {
	
	var self = this;

	/* **************************************************
	 * Procedimento que bloqueia teclas específicas do
	 * teclado de acordo com o fieldName recebido.
	 * Versão personalizada do método "justInteger" do
	 * General.
	 *
	 * @param string fieldName
	 */
	self.blockInvalidKeys = function (fieldName) {
		var element = $("#" + fieldName);
		element.keypress(function (e) {
			if (ger01700_unidade_service.validateKeys(e.which, fieldName))
				e.preventDefault();
		});
	};

	/* **************************************************
	 * Procedimento que ordena e espera a atualização do
	 * produto, informando o usuário do resultado da
	 * execução.
	 *
	 * @param Widget widget
	 */
	self.buildAtualizaProduto = function (widget) {
		ger01700_unidade_service.atualizaProduto(widget.currentRow.CDFILIAL).then(function (response) {
			if (response.dataset.success)
				ScreenService.showMessage(response.dataset.success.MESSAGE);
			ScreenService.closePopup();
		}, self.promiseErrorProcedure);
	};

	/* **************************************************
	 * Função que requisita e armazena na widget a
	 * validação de endereço caso seja necessário ou caso
	 * forçada por parâmetro.
	 *
	 * @param Widget widget
	 * @param boolean force
	 *
	 * @return Promise
	 */
	self.buildEndefiliValidation = function (widget, force) {
		var deferred = ZHPromise.defer();
		var unidadeWidget = widget.getParent();

		if (!widget.validation || (widget.validation.CDFILIAL != unidadeWidget.currentRow.CDFILIAL) || force) {
			widget.validation = {};

			var CDFILIAL = (widget.getParent().currentRow.__is_new) ? 'IGNORE' : widget.currentRow.CDFILIAL;

			ger01700_unidade_service.validateEndefili(CDFILIAL).then(function (response) {
				if (!widget.getParent().currentRow.__is_new) {
					widget.validation.allTypes = response.dataset.validation.ALLTYPES;
					widget.validation.usedTypes = response.dataset.validation.USEDTYPES;
					widget.validation.freeTypes = response.dataset.validation.FREETYPES;
					widget.validation.mayInsert = response.dataset.validation.MAYINSERT;
				}

				widget.validation.requireEndFields = response.dataset.validation.REQENDFIELDS;

				deferred.resolve(widget.validation);
			}, self.promiseErrorProcedure);
		} else
			deferred.resolve(widget.validation);

		return deferred;
	};

	/* **************************************************
	 * Função que monta um filtro à partir do filterName
	 * para o field recebido. Retorna true caso o filtro
	 * tenha sido criado ou alterado.
	 *
	 * @param Field field
	 * @param string filterName
	 *
	 * @return boolean
	 */
	self.buildFilter = function (field, filterName) {
		var changed = 404;
		var widget = field.widget;

		if (!field.dataSourceFilter)
			field.dataSourceFilter = [];

		field.dataSourceFilter.forEach(function (filter) {
			if (filter.name == filterName) {
				if (filter.value != widget.currentRow[filterName]) {
					filter.value = widget.currentRow[filterName];
					changed = true;
				} else
					changed = false;
			}
		});

		if (changed === 404)
			field.dataSourceFilter.push({
				name: filterName,
				operator: "=",
				value: widget.currentRow[filterName]
			});

		return changed !== false;
	};

	/* **************************************************
	 * Procedimento que requisita as validações de
	 * Paramgeral.
	 *
	 * @param Widget widget
	 *
	 * @return Promise
	 */
	self.buildParametrizacaoValidation = function (widget) {
		var deferred = ZHPromise.defer();

		if (!widget.validation || (Object.keys(widget.validation).length === 0 && widget.validation.constructor === Object)) {
			ger01700_unidade_service.validateParametrizacao().then(function (response) {
				if (!widget.validation)
					widget.validation = {};

				widget.validation.showParamTab = response.dataset.validation.SHOWPARAMTAB;

				deferred.resolve(widget.validation);
			}, self.promiseErrorProcedure);
		} else
			deferred.resolve(widget.validation);

		return deferred;
	};

	/* **************************************************
	 * Procedimento que coloca na widget atual os códigos
	 * necessários de sua widget parent. Pode receber uma
	 * string com o nome da propriedade desejada ou um
	 * array de propriedades. Permite o envio do nome da
	 * parentWidget para eventuais casos extra ordinários
	 *
	 * @param Widget currentWidget
	 * @param String|Array primaryKeys
	 * @param String parentName
	 */
	self.buildParentPrimaryKeys = function (currentWidget, primaryKeys, parentName) {
		var parentWidget = (parentName) ? currentWidget.container.getWidget(parentName) : currentWidget.getParent();

		if (Util.isString(primaryKeys))
			primaryKeys = [primaryKeys];

		primaryKeys.forEach(function (PK) {
			currentWidget.currentRow[PK] = parentWidget.currentRow[PK];
		});
	};

	/* **************************************************
	 * Procedimento que inicializa as widgets filhas da
	 * widget recebida, colocando o valueColumn em suas
	 * currentRows e em seus filtros.
	 *
	 * @param Widget widget
	 * @param string valueColumn
	 */
	self.buildSubWidgetFilters = function (widget, valueColumn) {
		widget.widgets.forEach(function (subWidget) {
			var found = 404;

			subWidget.dataSourceFilter.forEach(function (filter) {
				if (filter.name == valueColumn) {
					filter.value = widget.currentRow[valueColumn];
					found = true;
				}
			});

			if (found !== true)
				subWidget.dataSourceFilter.push({
					name: valueColumn,
					operator: "=",
					value: widget.currentRow[valueColumn]
				});

			subWidget.currentRow[valueColumn] = widget.currentRow[valueColumn];
		});
	};

	/* **************************************************
	 * Procedimento que gerencia o cancelamento do
	 * formulário da widget Unidade.
	 *
	 * @param Widget widget
	 */
	self.cancelFormUnidade = function (widget) {
		if (widget.currentRow.__is_new)
			Endereco_Controller.cancelInsertion(widget.name);
		else
			General.widgetCancelForm(widget);
	};

	/* **************************************************
	 * Procedimento que checa os campos opcionais do
	 * endereço da filial, validando-os de acordo com os
	 * parâmetros do banco de dados.
	 *
	 * @param Widget widget
	 */
	self.checkAddressFields = function (widget) {
		var validation = widget.validation;
		var addressFieldNames = ["NMBAIRRO",
                                 "DSENDEFILISEP",
                                 "NRENDEFILISEP",
                                 "DSLOGRADOURO",
                                 "NRCEPFILI"];

		addressFieldNames.forEach(function (fieldName) {
			var field = widget.getField(fieldName);

			if (validation.requireEndFields ^ (field.validations && field.validations.required)) {
				if (field.dependencies && field.dependencies.parentFields)
					Object.keys(field.dependencies.parentFields).forEach(function (dependency) {
						if (!(validation.requireEndFields ^ field.dependencies.parentFields[dependency].require))
							field.dependencies.parentFields[dependency].require = !validation.requireEndFields;
					});

				if (!field.validations)
					field.validations = {};

				self.switchFieldRequirement(field);
			}
		});
	};

	/* **************************************************
	 * Função que retorna true se o número de um endereço
	 * for válido, ou se não existir número.
	 *
	 * @param Widget widget
	 *
	 * @return boolean
	 */
	self.checkAddressNumber = function (widget) {
		return (!widget.currentRow.NRENDEFILISEP) ? true : /^[0-9]+[a-zA-Z]?$/.test(widget.currentRow.NRENDEFILISEP);
	};

	/* **************************************************
	 * Função que retorna true se um tipo de endereço
	 * puder ser inserido.
	 *
	 * @param Widget widget
	 *
	 * @return boolean
	 */
	self.checkAddressType = function (widget) {
		var validTypes = widget.validation.freeTypes;
		var currentType = widget.currentRow.IDTPENDEFILI;

		return validTypes.indexOf(currentType) != -1;
	};

	/* **************************************************
	 * Procedimento que checa o input de CEP, validando o
	 * dado inserido e retornando os dados do endereço da
	 * API dos Correios, tratando erros e efetuando
	 * validações de acordo com o necessário.
	 *
	 * @param Field field
	 */
	self.checkCEP = function (field) {
		var widget = field.widget;
		var CEP = field.value();

		if (CEP && CEP.length == 10) {
			ger01700_unidade_service.validateCEP(CEP).then(function (response) {
				var address = response.dataset.ENDERECO;
				if(widget.currentRow.__is_new == true){
					widget.currentRow.CDPAIS = (address.CDPAIS) ? address.CDPAIS : '';
					widget.currentRow.NMPAIS = (address.NMPAIS) ? address.NMPAIS : '';
					widget.currentRow.SGESTADO = (address.SGESTADO) ? address.SGESTADO : '';
					widget.currentRow.NMESTADO = (address.NMESTADO) ? address.NMESTADO : '';
					widget.currentRow.CDMUNICIPIO = (address.CDMUNICIPIO) ? address.CDMUNICIPIO : '';
					widget.currentRow.NMMUNICIPIO = (address.NMMUNICIPIO) ? address.NMMUNICIPIO : '';
					widget.currentRow.CDBAIRRO = (address.CDBAIRRO) ? address.CDBAIRRO : '';
					widget.currentRow.NMBAIRRO = (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.NMBAIRFILI = (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.CDLOGRADOURO = (address.CDLOGRADOURO) ? address.CDLOGRADOURO : '';
					widget.currentRow.DSLOGRADOURO = (address.DSLOGRADOURO) ? address.DSLOGRADOURO : '';
					widget.currentRow.DSENDEFILISEP = (address.DSENDERECO) ? address.DSENDERECO : '';
				}else{
					widget.currentRow.CDPAIS = widget.currentRow.CDPAIS? widget.currentRow.CDPAIS : (address.CDPAIS) ? address.CDPAIS : '';
					widget.currentRow.NMPAIS = widget.currentRow.NMPAIS? widget.currentRow.NMPAIS : (address.NMPAIS) ? address.NMPAIS : '';
					widget.currentRow.SGESTADO = widget.currentRow.SGESTADO? widget.currentRow.SGESTADO : (address.SGESTADO) ? address.SGESTADO : '';
					widget.currentRow.NMESTADO = widget.currentRow.NMESTADO? widget.currentRow.NMESTADO : (address.NMESTADO) ? address.NMESTADO : '';
					widget.currentRow.CDMUNICIPIO = widget.currentRow.CDMUNICIPIO? widget.currentRow.CDMUNICIPIO : (address.CDMUNICIPIO) ? address.CDMUNICIPIO : '';
					widget.currentRow.NMMUNICIPIO = widget.currentRow.NMMUNICIPIO? widget.currentRow.NMMUNICIPIO : (address.NMMUNICIPIO) ? address.NMMUNICIPIO : '';
					widget.currentRow.CDBAIRRO = widget.currentRow.CDBAIRRO? widget.currentRow.CDBAIRRO : (address.CDBAIRRO) ? address.CDBAIRRO : '';
					widget.currentRow.NMBAIRRO = widget.currentRow.NMBAIRRO? widget.currentRow.NMBAIRRO : (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.NMBAIRFILI = widget.currentRow.NMBAIRFILI? widget.currentRow.NMBAIRFILI : (address.NMBAIRRO) ? address.NMBAIRRO : '';
					widget.currentRow.CDLOGRADOURO = widget.currentRow.CDLOGRADOURO? widget.currentRow.CDLOGRADOURO : (address.CDLOGRADOURO) ? address.CDLOGRADOURO : '';
					widget.currentRow.DSLOGRADOURO = widget.currentRow.DSLOGRADOURO? widget.currentRow.DSLOGRADOURO : (address.DSLOGRADOURO) ? address.DSLOGRADOURO : '';
					widget.currentRow.DSENDEFILISEP = widget.currentRow.DSENDEFILISEP? widget.currentRow.DSENDEFILISEP : (address.DSENDERECO) ? address.DSENDERECO : '';
				}

				self.constructFieldFilters(widget.getField('NMESTADO'));
				self.constructFieldFilters(widget.getField('NMMUNICIPIO'));
				self.constructFieldFilters(widget.getField('NMBAIRRO'));

				self.checkFieldDependencies(widget.getField('NMPAIS'));
			}, function (error) {
				var errorMessage = 'Timeout';

				widget.currentRow.NRCEPFILI = '';

				if (error.data)
					errorMessage = error.data.error;
				
				// ScreenService.showMessage("Erro na requisição do CEP '" + CEP + "': " + errorMessage, "warning");
				
				/* MSG NOVA */
				var variaveis = {
                    CEP: CEP,
                    errorMessage: errorMessage
                };
				
				MessageService.showMessageByCode("CEPINVALIDO", variaveis, {"type":"warning"});
				/* FIM MSG NOVA */
			});
		}
	};  

	/* **************************************************
	 * Procedimento que checa se o contrato da unidade
	 * foi rescindido e obriga ou não o preenchimento do
	 * field da data de rescisão.
	 *
	 * @param Widget widget
	 */
	self.checkContratoRescindido = function (widget) {
		var dateField = widget.getField("DTRESCCONTRATO");

		if (widget.currentRow.IDCNTRRESC == 'S') {
			dateField.readOnly = false;
			dateField.validations = {
				required: true
			};
		} else {
			dateField.readOnly = true;
			dateField.validations = {};
			self.nullifyFieldValues(dateField);
		}
	};

	/* **************************************************
	 * Procedimento que configura datas mínimas/máximas
	 * entre fields de acordo com a configuração das
	 * propriedades "successor" e "precursor".
	 *
	 * A propriedade "successor" indica o nome do field
	 * cuja data deve suceder a do field atual.
	 *
	 * A propriedade "precursor" indica o nome do field
	 * cuja data deve preceder a do field atual.
	 *
	 * @param Field currentField
	 */
	self.checkDateInterval = function (currentField) {
		var widget = currentField.widget;
		if (currentField.successor) {
			var laterDateField = widget.getField(currentField.successor);

			laterDateField.minDate = widget.currentRow[currentField.name];
		} else if (currentField.precursor) {
			var earlierDateField = widget.getField(currentField.precursor);

			earlierDateField.maxDate = widget.currentRow[currentField.name];
		}
	};

	/* **************************************************
	 * Função que valida um email de acordo com os termos
	 * do RFC2822.
	 *
	 * Found at
	 *    en.wikibooks.org/wiki/JavaScript/Best_practices
	 *
	 * @param String email
	 *
	 * @return boolean
	 */
	self.checkEmail = function (email) {
		//These comments use the following terms from RFC2822:
			//local-part, domain, domain-literal and dot-atom.
			//Does the address contain a local-part followed an @ followed by a domain?
			//Note the use of lastIndexOf to find the last @ in the address
			//since a valid email address may have a quoted @ in the local-part.
			//Does the domain name have at least two parts, i.e. at least one dot,
			//after the @? If not, is it a domain-literal?

		//This will accept some invalid email addresses
		//BUT it doesn't reject valid ones.

		var atSym = email.lastIndexOf("@");

		//no local-part
		if (atSym < 1)
			return false;

		//no domain
		if (atSym == email.length - 1)
			return false;

		//there may only be 64 octets in the local-part
		if (atSym > 64)
			return false;

		//there may only be 255 octets in the domain
		if (email.length - atSym > 255)
			return false;

		//is the domain plausible?
		var lastDot = email.lastIndexOf(".");

		//check if it is a dot-atom such as example.com
		if (lastDot > atSym + 1 && lastDot < email.length - 1)
			return true;

		//check if could be a domain-literal.
		if (email.charAt(atSym + 1) == '[' && email.charAt(email.length - 1) == ']')
			return true;

		return false;
	};

	////////////////////////////////////////////////////////
	//                 FIELD DEPENDENCIES                 //
	// A cadeia de funções e procedimentos que começa a   //
	// partir do método checkFieldDependencies supõe que  //
	// os fields que o chamam terão o objeto dependencies //
	//                                                    //
	// Tal objeto pode possuir as propriedades:           //
	//     parentFields, um objeto cujo cada propriedade  //
	//         indica uma relação de dependência com um   //
	//         field daquele nome. Pode apresentar as     //
	//         propriedades:                              //
	//             block                                  //
	//                 indica que o field deve ter seu    //
	//                 acesso bloqueado caso o parent não //
	//                 esteja preenchido corretamente     //
	//             filter                                 //
	//                 indica que o field deve receber um //
	//                 filtro criado a partir daquele     //
	//                 parent. Só funciona em fields do   //
	//                 tipo select                        //
	//             require                                //
	//                 indica que o field tornar-se-á     //
	//                 obrigatório caso o parent esteja   //
	//                 preenchido, e opcional caso        //
	//                 contrário                          //
	//     childrenFields, um array que indica os names   //
	//         dos fields que dependem diretamente desse  //
	//         field                                      //
	//     isAvailable, que é definida pela cadeia de     //
	//         funções e procedimentos, indica se o field //
	//         deveria estar acessível ou não             //
	//     isRequired, que é definida pela cadeia de      //
	//         funções e procedimentos, indica se o field //
	//         deveria estar obrigatório ou não           //
	//                                                    //
	////////////////////////////////////////////////////////

	/* **************************************************
	 * Procedimento que realiza a checagem do objeto
	 * dependencies. A passagem do parâmetro flags não é
	 * necesária, pois sua utilidade é interna à cadeia
	 * de métodos.
	 *
	 * Retorna um Array com todos os fields verificados
	 * pela cadeia de métodos.
	 *
	 * @param Field field
	 * @param Array flags
	 *
	 * @return Array
	 */
	self.checkFieldDependencies = function (field, flags) {
		if (!flags) flags = [];

		if (!flags.find(function (flag) {
				return flag === field.name;
			}))
			flags.push(field.name);

		if (field.dependencies) {
			if (self.checkFieldParentage(field))
				field.reload();

			flags = self.checkFieldFiliation(field, flags);
		}

		field.widget.isValid();
		templateManager.updateTemplate();

		return flags;
	};

	/* **************************************************
	 * Procedimento que chama o método de checagem do
	 * objeto dependencies para cada um dos fields
	 * dependentes do field atual. A passagem do
	 * parâmetro flags não é necesária, pois sua
	 * utilidade é interna à cadeia de métodos.
	 *
	 * Retorna um Array com todos os fields verificados
	 * pela cadeia de métodos.
	 *
	 * @param Field field
	 * @param Array flags
	 *
	 * @return Array
	 */
	self.checkFieldFiliation = function (field, flags) {
		if (field.dependencies.childrenFields && field.dependencies.childrenFields.length > 0) {
			var widget = field.widget;

			field.dependencies.childrenFields.forEach(function (child) {
				if (!flags.find(function (flag) {
						return flag === child;
					})) {

					var childField = widget.getField(child);

					flags = self.checkFieldDependencies(childField, flags);
				}
			});
		}

		return flags;
	};

	/* **************************************************
	 * Função que checa a disponibilidade do field, e, se
	 * necessário, limpa os dados, cria os filtros,
	 * requer o field ou bloqueia-o.
	 * Retorna true caso o filtro do field tenha sido
	 * alterado.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkFieldParentage = function (field) {
		if (field.dependencies.parentFields) {
			if (!field.validations)
				field.validations = {};

			self.checkFieldValidity(field);

			field.readOnly = !field.dependencies.isAvailable;

			if (field.dependencies.isRequired ^ field.validations.required)
				self.switchFieldRequirement(field);

			if (!field.dependencies.isAvailable) {
				self.nullifyFieldValues(field);
				return false;
			} else if (self.constructFieldFilters(field)) {
				self.nullifyFieldValues(field);
				return true;
			}
		}

		return false;
	};

	/* **************************************************
	 * Procedimento que checa os parents do field e
	 * configura o flag de disponibilidade de acordo.
	 *
	 * @param Field field
	 */
	self.checkFieldValidity = function (field) {
		var widget = field.widget;
		var forceRequire = (field.validations.required === true);

		field.dependencies.isAvailable = true;
		field.dependencies.isRequired = false;

		Object.keys(field.dependencies.parentFields).forEach(function (parent) {
			var parentField = widget.getField(parent);

			var valueColumn = self.getFieldValueColumn(parentField);

			if (field.dependencies.parentFields[parent].block && !widget.currentRow[valueColumn])
				field.dependencies.isAvailable = false;

			if (field.dependencies.parentFields[parent].require) {
				forceRequire = false;

				if (widget.currentRow[valueColumn])
					field.dependencies.isRequired = true;
			}
		});

		if (forceRequire)
			field.dependencies.isRequired = true;
	};

	/* **************************************************
	 * Procedimento que checa a função operacional da
	 * unidade e obriga ou não o preenchimento do field
	 * de fornecedor.
	 *
	 * @param Widget widget
	 */
	self.checkFuncaoOperacional = function (widget) {
		var fornField = widget.getField("NMRAZSOCFORN");

		if (widget.currentRow.IDFUNCAOFILI == 'O') {
			fornField.validations = {
				required: true
			};
		} else {
			fornField.validations = {};
			self.nullifyFieldValues(fornField);
		}
	};

	/* **************************************************
	 * Procedimento que altera a visibilidade da action
	 * de inserção de acordo com a disponibilidade de
	 * tipos de endereço.
	 *
	 * @param Widget widget
	 */
	self.checkInsertAction = function (widget) {
		var insertAction = widget.actions.find(function (action) {
			return action.name == 'ADD';
		});

		insertAction.isVisible = widget.validation.mayInsert;
	};

	/* **************************************************
	 * Função que checa a validade de um field que pode
	 * conter múltiplos emails, separados apenas pelo
	 * caractere ";". Possíveis whitespaces que estiverem
	 * separando emails serão removidos.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkMultipleEmails = function (field) {
		var validation = {
			isValid: null,
			which: []
		};

		if (field.value()) {
			var emails = field.value();
			validation.isValid = true;

			var length = emails.length;
			while (emails[--(length)] === ';')
				emails = emails.slice(0, -1);

			emails = emails.split(';');

			for (var i = 0; i < emails.length; i++)
				emails[i] = emails[i].trim();

			emails.forEach(function (email) {
				if (!self.checkEmail(email)) {
					validation.isValid = false;
					validation.which.push(email);
				}
			});

			field.value(emails.join(";"));
		} else
			validation.isValid = !(field.validations && field.validations.required);

		return validation;
	};

	/* **************************************************
	 * Procedimento que checa o input de inscrição,
	 * validando-o e colocando o tipo de inscrição na
	 * currentRow de acordo.
	 *
	 * Nota: Esse método pressupõe que o input de
	 * inscrição possui as propriedades valueField e
	 * descriptionField, apesar de ser do template
	 * text-edit. Ele também supõe que o valueField
	 * e o descriptionField estarão na currentRow.
	 *
	 * @param Field field
	 */
	this.checkNumeroDeInscricao = function (field) {
		var validatedInscricao = ger01700_unidade_service.validateNumeroDeInscricao(field.widget.currentRow[field.descriptionField]);
		var widget = field.widget;

		if (validatedInscricao.valid) {
			switch (validatedInscricao.tipo.cod) {
			case 'O': //Código genérico
				validatedInscricao.rawValue = General.fill(validatedInscricao.parameter, 5);
				break;
			case 'I': //Inscrição genérica
				validatedInscricao.rawValue = General.fill(validatedInscricao.parameter, 20);
				break;
			default: //CPF && CNPJ
				validatedInscricao.rawValue = validatedInscricao.parameter;
			}

			widget.currentRow.IDTPIJURFILI = validatedInscricao.tipo.cod;
			widget.currentRow[field.valueField] = validatedInscricao.rawValue;
			widget.currentRow[field.descriptionField] = validatedInscricao.value;
		} else {
			if (validatedInscricao.error.msg)
				ScreenService.showMessage(validatedInscricao.error.msg, "warning");

			widget.currentRow.IDTPIJURFILI = null;
			widget.currentRow[field.valueField] = null;
			widget.currentRow[field.descriptionField] = null;
		}
	};

	/* **************************************************
	 * Função que checa se o valor percentual é válido na
	 * escala definida, ou de 0 à 100 caso nenhuma escala
	 * for recebida.
	 *
	 * @param integer percentage
	 * @param integer minValue
	 * @param integer maxValue
	 *
	 * @return boolean
	 */
	self.checkPercentage = function (percentage, minValue, maxValue) {
		minValue = minValue | 0;
		maxValue = maxValue | 100;

		return (percentage >= minValue && percentage <= maxValue);
	};

	/* **************************************************
	 * Procedimento que requisita a validação da tab de
	 * parametrização e troca a visibilidade da widget
	 * de acordo com a configuração da base.
	 *
	 * @param Widget widget
	 */
	self.checkTabParametrizacao = function (widget) {
		self.buildParametrizacaoValidation(widget).then(function (validation) {
			var paramWidget = widget.container.getWidget('parametrizacao');

			paramWidget.isVisible = paramWidget.showAsTab = validation.showParamTab;
		});
	};

	/* **************************************************
	 * Função que checa se o tipo de endereço selecionado
	 * em um field está dentre os endereços já
	 * previamente cadastrados para aquela filial.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.checkTipoDeEndereco = function (field) {
		var widget = field.widget;

		var validTypes = widget.validation.usedTypes;
		var currentType = field.value();
		
		if(currentType == 'I') return true;
		else return validTypes.indexOf(currentType) != -1;
		
	};

	/* **************************************************
	 * Procedimento que checa a validez de uma nova PK de
	 * Unidade.
	 *
	 * @param Widget widget
	 */
	self.checkUnidadeCode = function (widget) {
		ger01700_unidade_service.validateUnidadePK(widget.currentRow.CDFILIAL).then(function (response) {
			if (!response.dataset.validation.VALID) {
				
				// ScreenService.showMessage("Já existe uma filial com o código '" + response.dataset.validation.CODE + "'!");
				
				/* MSG NOVA */
				var variaveis = {
                    CODFILIAL: response.dataset.validation.CODE
                };
				
				MessageService.showMessageByCode("FILIALEXISTENTE", variaveis, {"type":"error"});
				/* FIM MSG NOVA */

				widget.currentRow.CDFILIAL = '';

				self.checkFieldDependencies(widget.getField('CDFILIAL'));
			}
		}.bind(self));
	};

	/* **************************************************
	 * Procedimento que realiza todas as validações
	 * necessárias para a widget Endefili.
	 *
	 * @param Widget widget
	 */
	self.checkWidgetEndefili = function (widget, onEnter) {
		self.buildParentPrimaryKeys(widget, "CDFILIAL");

		self.buildEndefiliValidation(widget).then(function () {
			self.checkAddressFields(widget);

			if (!widget.currentRow.__is_new)
				self.checkInsertAction(widget);
		});

		if (!onEnter) {
			self.checkCEP(widget.getField('NRCEPFILI'));

			self.checkFieldDependencies(widget.getField('NMPAIS'));

			self.buildParentPrimaryKeys(widget, 'CDFILIAL');
		}
	};

	/* **************************************************
	 * Procedimento que realiza todas as validações
	 * necessárias para a widget Unidade.
	 *
	 * @param Widget widget
	 */
	self.checkWidgetUnidade = function (widget) {
		self.checkContratoRescindido(widget);

		self.checkFieldDependencies(widget.getField("NMRAZSOCEMPR"));
		self.checkFieldDependencies(widget.getField("CDFILIAL"));

		self.checkFuncaoOperacional(widget);
		
		if (!widget.currentRow.__is_new) {
			self.checkDateInterval(widget.getField("DTINCONTRFIL"));
			self.checkDateInterval(widget.getField("DTFICONTRFIL"));

			widget.currentRow.MSKINSJURFILI = widget.currentRow.NRINSJURFILI;

			if (widget.currentRow.NRINSJURFILI !== undefined)
				this.checkNumeroDeInscricao(widget.getField("MSKINSJURFILI"));

			self.buildSubWidgetFilters(widget, "CDFILIAL");

			self.checkTabParametrizacao(widget);
		}
	};
	
	/* **************************************************
	 * Função que checa os parents do field e cria os
	 * filtros de acordo. Retorna true caso algum filtro
	 * tenha sido criado ou mudado, e falso caso
	 * contrário ou se o field não for do tipo select.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.constructFieldFilters = function (field) {
		if (field.template.indexOf("select") != -1) {
			var changed = false;
			var widget = field.widget;

			Object.keys(field.dependencies.parentFields).forEach(function (parent) {
				if (field.dependencies.parentFields[parent].filter) {
					var parentField = widget.getField(parent);
					var valueColumn = self.getFieldValueColumn(parentField);

					changed = self.buildFilter(field, valueColumn) || changed;
				}
			});

			return changed;
		}
		return false;
	};

	/* **************************************************
	 * Procedimento que valida e executa a exclusão de
	 * um registro do tipo Endereço.
	 *
	 * @param Widget widget
	 */
	self.deleteWidgetEndefili = function (widget) {
		const currentType = widget.currentRow.IDTPENDEFILI;
		const widgetDados = templateManager.container.getWidget("dados");
		
		if(self.validateAddressTypeBetweenWidgets(widgetDados,currentType)){
			General.deleteCrud(widget).then(function () {
				return self.buildEndefiliValidation(widget, true);
			}).then(function () {
				self.checkInsertAction(widget);
				self.checkAddressFields(widget);
			});
		}
	};
	
	self.validateAddressTypeBetweenWidgets = function(widgetDados,currentType){
		let boolCheck = true;
		
		widgetDados.reload().then(_=>{
			if(currentType == 'P'){
				ScreenService.showMessage("O endereço principal não pode ser excluído.");
				boolCheck =false;
			}else if(widgetDados.currentRow.IDENDEMISFIL === currentType){
				ScreenService.showMessage("Exclusão não permitida! Endereço vinculado aos Dados Fiscais (Endereço para Emissão)");
				boolCheck =false;
			} else if(widgetDados.currentRow.IDENDCFOPFIL === currentType){
				ScreenService.showMessage("Exclusão não permitida! Endereço vinculado aos Dados Fiscais (Endereço para C.F.O.P.)");
				boolCheck =false;
			}
		});
		return boolCheck;
	};

	/* **************************************************
	 * Procedimento que valida a widget Endefili antes de
	 * habilitar sua edição.
	 *
	 * @param Widget widget
	 */
	self.editWidgetEndefili = function (widget) {
		self.checkWidgetEndefili(widget);

		General.widgetEdit(widget);
	};

	/* **************************************************
	 * Procedimento que valida a widget Unidade antes de
	 * habilitar sua edição.
	 *
	 * @param Widget widget
	 */
	self.editWidgetUnidade = function (widget) {
		self.checkWidgetUnidade(widget);

		General.widgetEdit(widget);
	};

	/* **************************************************
	 * Função que recebe um field e retorna o nome da
	 * coluna que contém o value referente ao mesmo.
	 *
	 * @param Field field
	 *
	 * @return string
	 */
	self.getFieldValueColumn = function (field) {
		var valueColumn = field.name;

		if (field.outData && field.valueField) {
			Object.keys(field.outData).forEach(function (key) {
				if (field.outData[key] == field.valueField)
					valueColumn = field.valueField;
				else if (key == field.valueField)
					valueColumn = field.outData[key];
			});
		}

		return valueColumn;
	};

	/* **************************************************
	 * Procedimento que limpa todos os valores do outData
	 *
	 * Checa se a widget está em modo view e apenas limpa
	 * os dados caso "clearView" estiver setado como true
	 *
	 * @param Field field
	 * @param boolean clearView
	 */
	self.nullifyFieldValues = function (field, clearView) {
		var widget = field.widget;

		if (widget.mode == 'edit' || clearView) {
			if (field.outData) {
				Object.keys(field.outData).forEach(function (key) {
					widget.currentRow[field.outData[key]] = null;
				});
			}

			widget.currentRow[field.name] = null;
		}
	};

	/* **************************************************
	 * Procedimento genérico para erros de promise.
	 *
	 * @param Error error
	 */
	self.promiseErrorProcedure = function (error) {
		var errorMessage = 'Timeout';

		if (error.data)
			errorMessage = error.data.error;

		ScreenService.showMessage(errorMessage);
	};

	/* **************************************************
	 * Função que salva um novo registro de Unidade com
	 * seu respectivo registro de Endereço Principal.
	 *
	 * @param Widget widget
	 *
	 * @return Promise
	 */
	self.saveNewUnidade = function (widget) {
		widget.currentRow.CDOPERADOR = getLocalVar('CDOPERADOR');
		widget.currentRow.NRORGFILIAL = getLocalVar('NRORG');
		
		var addressWidget = templateManager.container.getWidget(widget.addressWidgetToForm);
		self.normalizaFuncaoFili(widget);

		Object.keys(addressWidget.currentRow).forEach(function (key) {
			if(!widget.currentRow[key])
				widget.currentRow[key] = addressWidget.currentRow[key];
		});

		if (addressWidget.isValid() && !self.checkAddressNumber(addressWidget)) {
			var deferred = ZHPromise.defer();

			deferred.reject("Número inválido.");
			ScreenService.showMessage("Número inválido.");

			return deferred;
		} else
			return General.widgetSave(widget); //.then(relacionaOperadorLogado(widget));
	};
	
	self.newEndefili = function(widget){
		let adressType = widget.getField('IDTPENDEFILI');
		adressType.dataSource.data = adressType.dataSource.data.filter(row => {
			return !widget.dataSource.data.some(wRow => {
				return wRow.IDTPENDEFILI == row.IDTPENDEFILI;
			});
		});
		General.widgetNewRow(widget);
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Unidade, caso o formulário esteja válido e os
	 * tipos de endereço selecionados estiverem de acordo
	 * com os existentes para aquela filial. Por fim,
	 * atualiza os valores da widget principal de acordo.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetDadosFiscais = function (widget) {
		const emissionAddress= widget.getField("IDENDEMISFIL");
		const CFOPAddress= widget.getField("IDENDCFOPFIL");
		
		self.normalizaFuncaoFili(widget);
		
		if (!self.checkTipoDeEndereco(emissionAddress))
				ScreenService.showMessage("Endereço para Emissão Inválido!");
		else if (!self.checkTipoDeEndereco(CFOPAddress))
				ScreenService.showMessage("Endereço para C.F.O.P. Inválido!");
		else if (!self.checkPercentage((widget.getField("VRPECREPREST").value() | 0)))
			ScreenService.showMessage("Percentual de crédito deve ser entre 0 e 100.");
		else
			General.widgetSave(widget).then(function () {
				self.updateParentRow(widget);
			});
		
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, caso o formulário esteja válido e o tipo
	 * de endereço não estiver registrado para aquela
	 * filial. Por fim, requisita novamente as validações
	 * e verifica a action de inserção.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetEndefili = function (widget) {
		if (widget.isValid()) {
			self.normalizaFuncaoFili(widget);
			if (!self.checkAddressNumber(widget))
				ScreenService.showMessage("Número inválido.");
			else {
				if (!widget.currentRow.__is_new || self.checkAddressType(widget)) {
					General.widgetSave(widget).then(function () {
						widget.reload([]).then(function(){
							widget.reload().then(function(){
								return self.buildEndefiliValidation(widget, true);
							});
						});
					}).then(function () {
						self.checkInsertAction(widget);
						self.checkAddressFields(widget);
					});
				} else
					ScreenService.showMessage("Tipo de endereço já cadastrado.");
			}
		}
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, caso o formulário esteja válido e os
	 * emails inseridos estiverem válidos. Por fim, força
	 * a atualização da widget de endereço.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetNFE = function (widget) {
		var email = self.checkMultipleEmails(widget.getField("DSEMAILEMXMLNFE"));
		self.normalizaFuncaoFili(widget);
		
		
		if (!email.isValid){
			// ScreenService.showMessage("Email(s) [\"" + email.which.join("\", \"") + "\"] Inválido(s)!");
			var EMAILS = email.which.join("\", \"");
			
			/* MSG NOVA */
			var variaveis = {
                EMAILS: EMAILS
            };
			
			MessageService.showMessageByCode("EMAILINVALIDO", variaveis, {"type":"error"});
			/* FIM MSG NOVA */
			
		}else{
			General.widgetSave(widget).then(function () {
				this.atualizaAbaNfe(widget);
			}.bind(this));
			
		}
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo
	 * Endereço, validando o email de login de SMTP. Por
	 * fim, força a atualização da widget de endereço.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetSMTP = function (widget) {
		var emailValue = widget.getField("DSLOGINSMTP").value() ? widget.getField("DSLOGINSMTP").value() : '';
		self.normalizaFuncaoFili(widget);
		
		if (!self.checkEmail(emailValue))
			ScreenService.showMessage("Email de Login Inválido!");
		else
			General.widgetSave(widget).then(function () {
				widget.container.getWidget("endereco").reload();
			});
	};

	/* **************************************************
	 * Procedimento que salva um registro do tipo Unidade
	 * de maneiras diferentes, de acordo com o estado de
	 * inserção ou alteração.
	 *
	 * @param Widget widget
	 */
	self.saveWidgetUnidade = function (widget) {
		self.normalizaFuncaoFili(widget);
		
		if(General.isValidFiliaisValuesCP(widget.currentRow.CDFILIAL)){
			if (widget.currentRow.__is_new) {
				
				Endereco_Controller.save(widget.name, self.saveNewUnidade, [widget]);
			}
			else 
				General.widgetSave(widget);
		}else{
			ScreenService.showMessage('Unidade não habilitada para utilização! Favor entrar em contato com o administrador do sistema para que seja cadastrada a unidade no Portal do Cliente (CP).');
		}
	};
	
	var relacionaOperadorLogado = function (widget){
		let olddataSourceName = widget.dataSourceName;
		let currentRow = widget.currentRow;
		let operador = getLocalVar('CDOPERADOR');
		currentRow.CDOPERADOR = operador;
		currentRow.__is_new = true;
		widget.dataSourceName = 'cadEst#/crud/filioper';
		General.saveCrud(widget).then(() =>{
			widget.dataSourceName = olddataSourceName;
		});
		
	};
	
	self.normalizaFuncaoFili = function(widget){
		if(widget.currentRow.IDFUNCAOFILI == "U")
		 	widget.currentRow.IDFUNCAOFILI = null;
	};
	
	/* **************************************************
	 * Procedimento que checa se o número de telefone
	 * tem entre 8 a 9 caractéres, e caso não tenha, 
	 * apaga o campo.
	 *
	 * @param Field field
	 */
	self.checkTelefone = function(field) {
		var value = String(field.value());
		if(value.length<8) {
			field.clearValue();
		}
	};

	/* **************************************************
	 * Procedimento que coloca na currentRow o primeiro
	 * dado do dataSource da widget. Pensado para ser
	 * utilizado em widgets do tipo form.
	 *
	 * @param Widget widget
	 */
	self.setForms = function (widget) {
		widget.currentRow = widget.dataSource.data[0];
	};

	/* **************************************************
	 * Função que troca a condição "required" de um field
	 * e retorna um boolean que indica se o field passou
	 * ou não a sê-lo.
	 *
	 * @param Field field
	 *
	 * @return boolean
	 */
	self.switchFieldRequirement = function (field) {
		if (field.validations) {
			if (field.validations.required) {
				delete field.validations.required;
				return false;
			} else {
				field.validations.required = true;
				return true;
			}
		}
		return false;
	};

	/* **************************************************
	 * Procedimento que copia os dados divergentes da
	 * widget atual para a widget parent.
	 *
	 * @param Widget widget
	 */
	self.updateParentRow = function (widget) {
		var parentWidget = widget.getParent();

		Object.keys(parentWidget.currentRow).forEach(function (value) {
			if (widget.currentRow.hasOwnProperty(value) && parentWidget.currentRow[value] != widget.currentRow[value])
				parentWidget.currentRow[value] = widget.currentRow[value];
		});
	};

	//--------------------------------[GER01742]--------------------------------

	var callbackError = function(err) {
		var message = '';
		if (err.data && err.data.error) {
			message = err.data.error;
		} else {
			message = JSON.stringify(err);
		}
		ScreenService.showMessage(message);
	};

	self.confirmUnitChange = function (unidade) {
		var mensagem = "Deseja confirmar a troca de unidade?";
		ScreenService.confirmMessage(mensagem, "question", changeUnit.bind(unidade));
	};

    const changeUnit = function() {
		ger01700_unidade_service.alterarEstrutura(this).then(updateLocalSession.bind(this), callbackError);
	};

	self.paramfilial = function() {
		var nrorg = arguments[0] || localStorageEngine.getLocalVar("NRORG") || 'NRORG_LOGADO';
		var cdfilial = arguments[1] || localStorageEngine.getLocalVar("CDFILIAL") || 'FILIAL_LOGADA';
		return restEngine.post('cadEst#/crud/paramfilial', {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "NRORG",
					"operator": "=",
					"value": nrorg
				},
				{
					"name": "CDFILIAL",
					"operator": "=",
					"value": cdfilial
				}
			]
		});

	};

	self.retornaDataProcessamento = function(cdfilial, nrorg) {
		return self.paramfilial(nrorg, cdfilial).then(function(callback) {
			var result = callback.dataset.paramfilial;
			var dtprocessa = result.length > 0 ?
								result[0].DTPROCESSA : null;

			if(dtprocessa)
				return moment(dtprocessa,'DD/MM/YYYY HH:mm:ss').format('L');
			else
				return moment().format('L');
		});
	};

	var buildUserAndHeader = function (name, descriptions, headerInfo) {
		var openWindowOperador = function () {
			ScreenService.openWindow("ger00002_operador");
		};
		ScreenService.buildUserData(name, descriptions, openWindowOperador);
		ScreenService.setHeaderInfo(headerInfo);
	};

	var setLocalVars = function (unidade) {
		setLocalVar("CDFILIAL", unidade.CDFILIAL);
		setLocalVar("NMFILIAL", unidade.NMFILIAL);
		setLocalVar("DTPROCESSA", unidade.DTPROCESSA);
	};
	
	var updateBackendSession = function (unidade) {
		var user = localStorage.getItem('LOGIN_USER');
		var acl = localStorageEngine.getLocalVar('accessProfiles');
		var otherSessionValues = {};
		otherSessionValues.CDFILIAL = localStorageEngine.getLocalVar("CDFILIAL");
		otherSessionValues.NMFILIAL = localStorageEngine.getLocalVar('NMFILIAL');
		otherSessionValues.DTPROCESSA = localStorageEngine.getLocalVar('DTPROCESSA');
		LibLoginController.updateSession(user, acl, otherSessionValues).then(result => {
			engine.openWindow(templateManager.project.mainWindow);
			eventAggregator.publish("onUnitChange", this);
			return 'OK!';
		});
	};

	var updateLocalSession = function() {
		self.retornaDataProcessamento(self.CDFILIAL, self.NRORG).then(function (dtprocessa) {
			var nmoperador = getLocalVar("NMOPERADOR");
			var unitDescription = self.CDFILIAL + ' - ' + self.NMFILIAL;
			var headerInfo = dtprocessa + " | " + unitDescription;
			buildUserAndHeader(nmoperador, [dtprocessa, unitDescription], headerInfo);
			self.DTPROCESSA = dtprocessa;
			setLocalVars(this);
			updateBackendSession(this);
		}.bind(this), callbackError);
	};
	
	this.gerarCliente = function(widget) {
		General.callRestEngine("cadEst#/filialCliente", "Row", widget.currentRow).then(() => {
			ScreenService.successNotification("Cliente gerado com sucesso");	
			ScreenService.closePopup();
		},
		(error) => {
			ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
		}); 	
	};
	
	this.atualizaAbaNfe = function(widget) {
		General.callRestEngine("cadEst#/atualizaAbaNfe", "Row", widget.currentRow).then(() => {
			// ScreenService.successNotification("Salvo com sucesso");
			widget.container.getWidget("nfe").reload();
		},
		(error) => {
			ScreenService.showMessage(error.data.error.replace("Exception: ", ""));
		}); 	
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('ger01700_unidade_controller', ger01700_unidade_controller);
});
