function ger01700_unidade_service(Endereco, General, ger01700_unidade_repository, restEngine) {
    var self = this;

    self.alterarEstrutura = function ({NMFILIAL, CDFILIAL, NRORG}) {
        const filial = {NMFILIAL, CDFILIAL, NRORG};

        return General.callRestEngine('cadEst#/alterarEstrutura', 'Row', filial);
    };

    /* **************************************************
     * Função que prepara a atualização dos produtos de
     * uma unidade e retorna o resultado da operação.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.atualizaProduto = function(cdfilial) {
        var cdnvproduto = General.getLevelByMask('GER_CDPICTPROD','1').length;

        return ger01700_unidade_repository.requestAtualizaProduto(cdfilial, cdnvproduto);
    };

    /* **************************************************
     * Função que retorna um request de validação de CEP
     * efetuado para a API dos Correios.
     *
     * @param String CEP
     *
     * @return Promise
     */
    self.validateCEP = function(CEP) {
        return Endereco.buscarCep(CEP);
    };

    /* **************************************************
     * Função que requisita a validação de endereço.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.validateEndefili = function(cdfilial) {
        return ger01700_unidade_repository.requestEndefiliValidation(cdfilial);
    };

    /* **************************************************
     * Função que retorna a validez de uma key para um
     * field específico, ou para números inteiros.
     *
     * @param int key
     * @param string fieldName
     *
     * @return boolean
     */
    self.validateKeys = function(key, fieldName) {
        var integerNumbers = [48, 49, 50, 51,  52,  53,  54,  55,  56,  57];
        var validKeys = {};

        validKeys.integer = integerNumbers;
        validKeys.MSKINSJURFILI = integerNumbers.concat([45, 46, 47]); //'-', '.', '/'

        if(fieldName && validKeys.hasOwnProperty(fieldName))
            return validKeys[fieldName].indexOf(key) < 0;
        else
            return validKeys.integer.indexOf(key) < 0;
    };

    /* **************************************************
     * Função que trata o objeto de inscrição, montando
     * as mensagens de erro caso necessário.
     *
     * @param int key
     * @param string fieldName
     *
     * @return object(inscricao)
     */
    self.validateNumeroDeInscricao = function(inputValue) {
        var inscricao = General.inscricaoFormat(inputValue);
        
         /* **************************************************
          * parseInt() e toString() apenas para retirar os zeros a esquera que a função  
          * "General.inscricaoFormat(inputValue)" coloca 
          */

        if(!inscricao.valid) {
            inscricao.error = {};
            switch(inscricao.tipo.cod) {
                case 'J':
                    inscricao.error.msg = "CNPJ \"" + inscricao.value + "\" inválido!";
                    break;
                case 'F':
                    inscricao.error.msg = "CPF \"" + inscricao.value + "\" inválido!";
                    break;
                case null:
                    break;
                case 'I':
                /* falls through */
                default:
                    inscricao.error.msg = inscricao.tipo.desc + " \"" + inscricao.value + "\" inválida!";
            }
        }

        return inscricao;
    };

    /* **************************************************
     * Função que requisita as validações de Paramgeral.
     *
     * @return Promise
     */
    self.validateParametrizacao = function() {
        return ger01700_unidade_repository.requestParametrizacaoValidation();
    };

    /* **************************************************
     * Função que requisita as validações da PK de uma
     * unidade.
     *
     * @param string cdfilial
     *
     * @return Promise
     */
    self.validateUnidadePK = function(cdfilial) {
        return ger01700_unidade_repository.requestUnidadeCodeValidation(cdfilial);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ger01700_unidade_service', ger01700_unidade_service);
});
