function Endereco_Controller(General, ScreenService, templateManager, ZHPromise) {
    var self = this;

    //////////////////////////////////////////////////////////
    //                                                      //
    // ESSE CONTROLLER ESPERA AS SEGUINTES CARACTERÍSTICAS: //
    //                                                      //
    // na widget principal:                                 //
    //     addressWidget, que indica o nome da widget       //
    //     do grid de endereço;                             //
    //                                                      //
    //     addressWidgetToForm, que indica o nome da        //
    //     widget do form de endereço.                      //
    //                                                      //
    // na widget de formulário de endereço:                 //
    //     addressTypeColumn, que indica o nome do field    //
    //     do tipo de endereço.                             //
    //                                                      //
    //     event WidgetOnEnter com o método                 //
    //                     addressWidgetOnEnter em seu code //
    //                                                      //
    //////////////////////////////////////////////////////////

    /* **************************************************
     * Procedimento qeu verifica a flag de visitação da
     * widget de formulário de endereço.
     *
     * @param Widget widget
     */
    self.addressWidgetOnEnter = function(widget) {
        if(widget.visited == 404)
            setTimeout(function() {widget.isValid();}, 5000); //blame Wagão for setTimeout
                                                              //R.I.P. Wagão ??? - 2016

        widget.visited = true;
    };

    /* **************************************************
     * Procedimento que cancela o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.cancelInsertion = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);

        self.clearRow(mainWidget);

        self.exit(mainWidgetName, validationFunction, validationParams);
    };

    /* **************************************************
     * Procedimento que remove a currentRow da widget
     * principal para o caso de cancelamento da inserção.
     *
     * @param Widget mainWidget
     */
    self.clearRow = function(mainWidget) {
        var indexOfCurrentRow = mainWidget.dataSource.data.indexOf(mainWidget.currentRow);

        if(indexOfCurrentRow !== -1) mainWidget.dataSource.data.pop();

        mainWidget.setCurrentRow({});
    };

    /* **************************************************
     * Procedimento que encerra o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.exit = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), true);

        self.prepareView(mainWidget);
    };

    /* **************************************************
     * Procedimento que abre o swipe de inserção da
     * widget principal junto com a sua widget de
     * formulário de endereço.
     *
     * Recebe o nome da widget principal que será aberta,
     * e permite o uso de uma função de validação
     * opcional a ser executada após a abertura do swipe,
     * que pode ou não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.newRow = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        formAddressWidget.visited = false;

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), false);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        self.prepareForm(mainWidget, formAddressWidget);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);
    };

    /* **************************************************
     * Procedimento que configura as widgets da inserção
     * de um registro com endereço para dito processo.
     *
     * @param Widget mainWidget
     * @param Widget formAddressWidget
     */
    self.prepareForm = function(mainWidget, formAddressWidget) {
        mainWidget.newRow();

        formAddressWidget.setCurrentRow({});
        formAddressWidget.edit();

        formAddressWidget.currentRow.__is_new = true;
        formAddressWidget.currentRow[formAddressWidget.addressTypeColumn] = 'P';

        formAddressWidget.getField(formAddressWidget.addressTypeColumn).readOnly = true;
    };

    /* **************************************************
     * Procedimento que configura a widget principal para
     * sair do modo de inserção.
     *
     * @param Widget mainWidget
     */
    self.prepareView = function(mainWidget) {
        mainWidget.list();

        ScreenService.closeSwipe();

        mainWidget.activate();
    };

    /* **************************************************
     * Procedimento que checa a validez das duas widgets
     * de inserção e efetua o salvamento quando possível.
     *
     * Também avisa o usuário sobre qual(is) widget(s)
     * estão inválidas, e direciona-o caso necessário.
     *
     * Recebe o nome da widget principal que será salva,
     * além da função que será executada para efetuar o
     * salvamento. Permite o uso de funções de sucesso e
     * erro opcionais a serem executadas após o
     * salvamento.
     *
     * IMPORTANTE: A função "saveParams" DEVE retornar
     * promise!
     *
     * @param string mainWidgetName
     * @param Function saveFunction
     * @param Array saveParams
     * @param Function successFunction
     * @param Function errorFunction
     */
    self.save = function(mainWidgetName, saveFunction, saveParams, successFunction, errorFunction) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        var mainIsValid = mainWidget.isValid();
        
        this.checkAndRegisterNeighborhood(formAddressWidget.getField('NMBAIRRO'), formAddressWidget.visited === true).then(resultado => {
            if(formAddressWidget.visited === true){
                if(resultado.dataset.bairro.length > 0)
                    formAddressWidget.getField('NMBAIRRO').dataSource.data = resultado.dataset.bairro;
                if(resultado.dataset.bairro.length == 1){
                    let bairro = resultado.dataset.bairro[0];
                    formAddressWidget.currentRow.CDBAIRRO = bairro.CDBAIRRO;
                    formAddressWidget.currentRow.NMBAIRRO = bairro.NMBAIRRO;
                }
            } 
            //checks address widget validity
                /* note: Widget's "isValid" function returns true if the widget wasn't visited,
                 * even if it's not actually valid. So, its necessary to check the real address
                 * validity, in this case, using a flag. */
            var addressIsValid = (formAddressWidget.isValid() && (formAddressWidget.visited === true));
    
            switch(true) {
                case mainIsValid && addressIsValid:
                    if(saveFunction instanceof Function)
                        saveFunction.apply(this, saveParams).then(function(callback) {
                            self.exit(mainWidgetName);
    
                            if(successFunction instanceof Function)
                                successFunction.apply(this, callback);
                        }, errorFunction);
                    break;
    
                case !mainIsValid && !addressIsValid:
                    if(formAddressWidget.visited === true)
                        ScreenService.showMessage("Preencha todos os campos obrigatórios das duas telas!");
                    else
                        formAddressWidget.visited = 404;
                    break;
    
                case !mainIsValid && !mainWidget.isActive():
                    ScreenService.showMessage("Preencha os campos obrigatórios!");
    
                    mainWidget.currentWidget = mainWidget;
    
                    mainWidget.activate();
                    break;
    
                case !addressIsValid && !formAddressWidget.isActive():
                    formAddressWidget.visited = 404;
    
                    ScreenService.showMessage("Preencha os campos obrigatórios!");
    
                    formAddressWidget.activate();
                    break;
            }
        });
        
    };

    /* **************************************************
     * Procedimento que troca entre a widget de grid de
     * endereço e widget de formulário de endereço.
     *
     * Para que o procedimento funcione corretamente, a
     * widget de grid deve ter sua visibilidade marcada
     * inicialmente como true.
     *
     * @param Widget mainAddressWidget
     * @param Widget formAddressWidget
     */
    self.switchAddressVisibility = function(mainAddressWidget, formAddressWidget) {
        var previousState = mainAddressWidget.isVisible;

        mainAddressWidget.enabled = mainAddressWidget.showAsTab = mainAddressWidget.isVisible = !previousState;

        formAddressWidget.enabled = formAddressWidget.isVisible = formAddressWidget.showAsTab = previousState;
    };
    
    this.checkAndRegisterNeighborhood = function(neighborhoodField){
        if(!neighborhoodField.value() && neighborhoodField.dataSource.data.length == 0){
            let neighborhoodValue = neighborhoodField.dataSourceFilter.filter(data => data.operator == 'LIKE_ALL' && data.value != '%%');
            if(neighborhoodValue.length > 0){
                neighborhoodValue = neighborhoodValue[0].value.replace(/%/g, '');
                let neighborhoodSequentialFilter = neighborhoodField.dataSourceFilter;
                neighborhoodSequentialFilter.forEach(filter =>{
                    if(filter.name == 'NMMUNICIPIO')
                        filter.value = neighborhoodValue;
                });
                
                return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', neighborhoodSequentialFilter);
            }else{
                return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', []);
            }
        }else
            return General.callRestEngine('cadEst#/validateNeighborhood', 'FilterData', []);
    };
    
    Number.prototype.pad = function(size) {
      var s = String(this);
      while (s.length < (size || 2)) {s = "0" + s;}
      return s;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Endereco_Controller', Endereco_Controller);
});