// FILE: bower_components/zh-interval-component/dist/zh-interval-component.js

// FILE: src/interval/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-interval-component';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/interval/registers/IntervalRegister.js
function IntervalRegister(DecodeValueFieldService, validateEngine, ScreenService) {
    var self = this;

    /**
     * DecodeValueField Register
     **/
    var DEFAULT_DISPLAY_FORMAT = 'YYYY-MM-DD';
    var DEFAULT_DATE_FORMAT = 'YYYY-MM-DD';

    self.decodeValueField = function(field, fieldRow) {
        var displayValue = "";
        var dateFormat = field.dateFormat || DEFAULT_DATE_FORMAT;
        var formattedDate = field.displayFormat || DEFAULT_DISPLAY_FORMAT;
        if(fieldRow){
            if (fieldRow.end || fieldRow.start || fieldRow.period) {
                var beginDate = fieldRow.start ? moment(fieldRow.start, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                var endDate = fieldRow.end ? moment(fieldRow.end, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                if (!fieldRow.period) {
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate;
                    return displayValue;
                } else {
                    var periodLabel = fieldRow.period.label;
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate + ' (' + periodLabel + ') ';
                    return displayValue;
                }

            }
        }

        return ScreenService.i18n("From ") + formattedDate.replace(/[a-z]/gi, '_') + ScreenService.i18n(" to ") + formattedDate.replace(/[a-z]/gi, '_');
    };

    DecodeValueFieldService.register("interval.html", self.decodeValueField);

    /**
     * ValidateEngine Register
     **/
    function required(value, validationAttrs) {
        return {
            "valid": requiredStart(value, validationAttrs).valid && requiredEnd(value, validationAttrs).valid,
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Required field")
        };
    }

    function requiredStart(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'start');   
    }

    function requiredEnd(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'end');
    }

    function buildRequiredMessage(value, validationAttrs, property) {
        return {
            "valid": value && Util.isObject(value) && !Util.isEmptyOrBlank(value[property]),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n('Required field')
        };
    }

    var registeredValidations = validateEngine.getMappedValidations();
    var validationsToRegister = {
        "requiredInterval": required,
        "requiredIntervalStart": requiredStart,
        "requiredIntervalEnd": requiredEnd
    };

    for (var validationName in validationsToRegister) {
        if (!registeredValidations.hasOwnProperty(validationName)) {
            validateEngine.registerValidation(validationName, validationsToRegister[validationName]);
        }
    }
}

Configuration(IntervalRegister);

// FILE: src/interval/controllers/IntervalController.js
function IntervalController($scope, ZhUtil, ScreenService, metaDataFactory, $rootScope) {
	var self = this;
	self.field = $scope.field;
	self.widget = $scope.widget;

	/**
	 * Constant values
	 **/
	var DEFAULT_DISPLAY_FORMAT = 'YYYY-MM-DD';
	var DEFAULT_DATE_FORMAT = 'YYYY-MM-DD';
	var DEFAULT_START_LABEL = 'Begin on';
	var DEFAULT_END_LABEL = 'End on';
	var INCLUSIVE_SUFFIX = "inclusive";

	/**
	 * Initial values
	 **/
	var sourceElement = self.field || self.widget;
	$scope.dateFormat = sourceElement.dateFormat || DEFAULT_DATE_FORMAT;
	$scope.displayFormat = sourceElement.displayFormat || DEFAULT_DISPLAY_FORMAT;
	$scope.startLabel = ScreenService.i18n(sourceElement.startLabel || DEFAULT_START_LABEL);
	$scope.endLabel = ScreenService.i18n(sourceElement.endLabel || DEFAULT_END_LABEL);

	self.formatDateForShowOnField = function(property) {
		var valueOnCurrentRow = self.field.widget.currentRow[self.field.name]; //Generate mask type like __/__/____
		var formattedDate = moment().format($scope.displayFormat).replace(/[0-9]/g, '_');
		if (valueOnCurrentRow && (property === 'start' || property === 'end') && valueOnCurrentRow[property]) {
			formattedDate = moment(valueOnCurrentRow[property], $scope.dateFormat).format($scope.displayFormat);
		}
		return formattedDate;
	};

	function buildIntervals() {
		var fieldIntervals = self.field.intervals;
		var finalIntervals = [];

		//Set default Intervals
		if (fieldIntervals === undefined) {
			fieldIntervals = [{
				label: "Today",
				period: "today"
			}, {
				label: "Yesterday",
				period: "yesterday"
			}, {
				label: "Tomorrow",
				period: "tomorrow"
			}, {
				label: "This Week",
				period: "this-week"
			}, {
				label: "Last Week",
				period: "last-week"
			}, {
				label: "Next Week",
				period: "next-week"
			}, {
				label: "This Quarter",
				period: "this-quarter"
			}, {
				label: "Last Quarter",
				period: "last-quarter"
			}, {
				label: "Next Quarter",
				period: "next-quarter"
			}, {
				label: "This Month",
				period: "this-month"
			}, {
				label: "Last Month",
				period: "last-month"
			}, {
				label: "Next Month",
				period: "next-month"
			}, {
				label: "This Year",
				period: "this-year"
			}, {
				label: "Last Year",
				period: "last-year"
			}, {
				label: "Next Year",
				period: "next-year"
			}];
		}

		// Iterates in field periods to build each determined row
		if (Util.isArray(fieldIntervals)) {
			fieldIntervals.forEach(function(interval) {
				try {
					/**
					 *   @Note:
					 * - The customized flag is used to tell that such period was created by developer.
					 * - The personalized flag is used to tell that such period will be selected by the user because it's not in the period list.
					 */

					var row;

					// Evaluates possible custom intervals
					if (isCustomPeriod(interval.period)) {
						row = ZhUtil.evaluate(interval.period);
						row.customized = true;
					}
					else {
						row = buildObjectWithPeriodDetails(interval.period);
					}

					finalIntervals.push({
						"_period_label": interval.label,
						"_period": row,
						"_period_name": interval.period
					});
				}
				catch (e) {}
			});
		}

		return finalIntervals;
	}

	function isCustomPeriod(periodName) {
		return periodName[0] === '@';
	}

	function buildObjectWithPeriodDetails(word) {
		var decoded = decodeWordExpression(word);
		var buildIntervalMethod = (decoded.suffix === INCLUSIVE_SUFFIX) ? buildIntervalInclusive : buildInterval;
		return buildIntervalMethod(decoded.prefix, decoded.amount, decoded.unit);
	}

	function decodeWordExpression(word) {
		var exceptionWords = {
			'today': 'this-day',
			'yesterday': 'last-day',
			'tomorrow': 'next-day'
		};

		// Transform exception words due to function needs
		if (word in exceptionWords) {
			word = exceptionWords[word];
		}

		var prefix, amount, unit, suffix;
		var splitedString = word.split("-");

		if (splitedString.length === 2) {
			prefix = splitedString[0];
			unit = splitedString[1];
		}
		else if (splitedString.length > 2) {
			prefix = splitedString[0];
			amount = splitedString[1];
			unit = splitedString[2];
			suffix = splitedString[3];
		}
		else {
			throw "Used period has an incorrect format.";
		}

		return {
			prefix: prefix,
			amount: amount || 1,
			unit: unit,
			suffix: suffix
		};
	}

	function buildInterval(prefix, amount, unit) {
		var start = moment().startOf(unit),
			end = moment();
		switch (prefix) {
			case 'last':
				end.subtract(1, unit);
				start.subtract(amount, unit);
				break;
			case 'next':
				start.add(1, unit);
				end.add(amount, unit);
				break;
		}
		end.endOf(unit);
		return factoryInterval(start, end);
	}

	function buildIntervalInclusive(prefix, amount, unit) {
		var start = moment().startOf(unit),
			end = moment();
		switch (prefix) {
			case 'last':
				start.subtract(amount - 1, unit);
				break;
			case 'next':
				end.add(amount - 1, unit);
				break;
		}
		end.endOf(unit);
		return factoryInterval(start, end);
	}

	function factoryInterval(start, end) {
		return {
			"start": start.format($scope.dateFormat),
			"end": end.format($scope.dateFormat),
			"customized": false
		};
	}

	function clearInternalValues() {
		if(!self.field.isReadOnly()){
			var intervalField = self.field;
			var parentWidget = intervalField.widget;
			delete parentWidget.currentRow[intervalField.name];
			intervalField.validate(intervalField.value());
			if (self.field.change) {
				self.field.change();
			}
		}
	}

	function createIntervalWidget() {
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "args.owner.widget.parent.closeIntervalPopup(args.owner)",
				"id": 0
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"name": "ActionEvent",
				"code": "args.owner.widget.parent.closeIntervalPopup(args.owner); args.owner.widget.parent.applyIntervalToWidgetRow();",
				"id": 1
			}]
		}];

		return metaDataFactory.widgetFactory({
			"id": "0",
			"name": "Interval",
			"label": self.field.label,
			"template": "zh-interval-component#widget/interval.html",
			"container": self.field.widget.container,
			"isVisible": true,
			"popupNoBlock": false,
			"startLabel": self.field.startLabel,
			"endLabel": self.field.endLabel,
			"dateFormat": $scope.dateFormat,
			"displayFormat": self.field.displayFormat || $scope.dateFormat,
			"widgets": [{
				"id": "0.1",
				"name": "Calendar",
				"label": self.field.label,
				"template": "widget/calendar.html",
				"container": self.field.widget.container,
				"parent": "",
				"dateFormat": $scope.dateFormat,
				"displayFormat": self.field.displayFormat || $scope.dateFormat,
				"isVisible": true,
				"floatingControl": false,
				"dataSource": {
					"data": self.field.dataSource
				},
				"showDescriptionOrder": false,
				"events": [{
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.parent.selectIntervalDate(args)"
				}, {
					"name": "WidgetOnEnter",
					"code": "args.owner.parent.selectDatesBasedOnCurrentRow()"
				}],
				"actions": actions
			}, {
				"id": "0.2",
				"name": "List",
				"label": self.field.label,
				"template": "widget/list.html",
				"container": self.field.widget.container,
				"parent": "",
				"isVisible": true,
				"showDescriptionOrder": false,
				"floatingControl": false,
				"fields": [{
					"label": "Period",
					"name": "_period_label",
					"translate": true,
					"showOnList": true
				}],
				"dataSource": {
					"data": buildIntervals()
				},
				"events": [{
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.parent.selectIntervalPeriod(args.owner)"
				}],
				"actions": actions
			}],
			"events": []
		});
	}

	this.openIntervalWidget = function() {
		if(!self.field.isReadOnly()){
			self.field.intervalWidget = createIntervalWidget();
			applyIntervalWidgetInitialValues();
			resetIntervalValuesOnCurrentRow();
			createIntervalWidgetMethods();
			ScreenService.openPopup(self.field.intervalWidget).then(function() {
				if(self.field.intervalWidget === $rootScope._currentPopup){
					$rootScope._currentPopup = self.field.intervalWidget.widgets[0];
				}
			});
		}
	};

	function applyIntervalWidgetInitialValues() {
		var intervalWidget = self.field.intervalWidget;
		intervalWidget.isIntervalEmpty = true;
		intervalWidget.beingSelected = 'start';
		intervalWidget.setBeingSelected = setBeingSelected;
		intervalWidget.carouselWidget = 'calendar';

		intervalWidget.intervalData = {};
		intervalWidget.intervalData.start = getInitialstartDate();
		intervalWidget.intervalData.end = getInitialEndDate();
		intervalWidget.widgets[0].currentDate = intervalWidget.intervalData.start;
		intervalWidget.formattedData = {};
		intervalWidget.formattedData.start = moment(getInitialstartDate(), $scope.dateFormat).format($scope.displayFormat);
		intervalWidget.formattedData.end = moment(getInitialEndDate(), $scope.dateFormat).format($scope.displayFormat);
	}

	function getInitialstartDate() {
		return getInitialDate('start');
	}

	function getInitialEndDate() {
		return getInitialDate('end');
	}

	function getInitialDate(property) {
		var field = self.field;
		var parentWidget = field.widget;
		var currentRowProperty = parentWidget.currentRow[field.name];
		return currentRowProperty ? currentRowProperty[property] : moment().format($scope.dateFormat);
	}

	function resetIntervalValuesOnCurrentRow() {
		var intervalField = self.field;
		var parentWidget = intervalField.widget;
		if (!parentWidget.currentRow[intervalField.name]) {
			parentWidget.currentRow[intervalField.name] = {
				start: null,
				end: null,
				period: null
			};
		}
	}

	function setBeingSelected(value) {
		self.field.intervalWidget.beingSelected = value;
	}

	function createIntervalWidgetMethods() {
		self.field.intervalWidget.applyIntervalToWidgetRow = function() {
			var intervalField = self.field;
			var parentWidget = intervalField.widget;
			var intervalData = intervalField.intervalWidget.intervalData;

			parentWidget.currentRow[intervalField.name] = {
				start: intervalData.start,
				end: intervalData.end,
				period: intervalData.period
			};
			intervalField.validate(intervalField.value());
			if (intervalField.change) {
				intervalField.change();
			}
		};

		self.field.intervalWidget.selectDatesBasedOnCurrentRow = function() {
			var field = self.field;
			var currentRow = field.widget.currentRow;
			var intervalWidget = field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			var formattedData = intervalWidget.formattedData;
			var startDate = currentRow[field.name] ? currentRow[field.name].start : null;
			var endDate = currentRow[field.name] ? currentRow[field.name].end : null;

			if (Util.isValidDate(startDate, $scope.dateFormat) && Util.isValidDate(endDate, $scope.dateFormat)) {
				intervalData.start = startDate;
				intervalData.end = endDate;
				formattedData.start = moment(startDate, $scope.dateFormat).format($scope.displayFormat);
				formattedData.end = moment(endDate, $scope.dateFormat).format($scope.displayFormat);
				selectIntervalOnCalendar();
			}
			else {
				intervalData.start = moment().format($scope.dateFormat);
				intervalData.end = moment().format($scope.dateFormat);
				formattedData.start = moment().format($scope.displayFormat);
				formattedData.end = moment().format($scope.displayFormat);
				selectIntervalOnCalendar();
			}
		};

		self.field.intervalWidget.selectIntervalDate = function(args) {
			var intervalWidget = self.field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			var formattedData = intervalWidget.formattedData;
			var selectedDate = args.row;
			var momentSelectedDate = moment(args.row, $scope.dateFormat);
			var momentstartDate = moment(intervalData.start, $scope.dateFormat);
			var momentEndDate = moment(intervalData.end, $scope.dateFormat);

			if (intervalWidget.beingSelected !== 'start' &&
				intervalWidget.beingSelected !== 'end'
			) {
				intervalWidget.beingSelected = 'start';
			}

			intervalData[intervalWidget.beingSelected] = selectedDate;
			formattedData[intervalWidget.beingSelected] = moment(selectedDate, $scope.dateFormat).format($scope.displayFormat);

			var shouldToggleBeingSelected = true;

			if (intervalWidget.beingSelected === 'start' &&
				momentSelectedDate.isAfter(momentEndDate)
			) {
				intervalData.end = selectedDate;
			}
			else if (intervalWidget.beingSelected === 'end' &&
				momentSelectedDate.isBefore(momentstartDate)
			) {
				shouldToggleBeingSelected = false;
				intervalData.start = selectedDate;
			}

			if (shouldToggleBeingSelected) {
				toggleBeingSelected();
			}

			clearOldInterval();
			selectIntervalOnCalendar();
			intervalWidget.isIntervalEmpty = false;
		};

		self.field.intervalWidget.closeIntervalPopup = function(owner) {
			ScreenService.closePopup();
			var intervalField = self.field;
			var toBeActivatedWidget = this.lastActiveWidget;
			var intervalPeriod = toBeActivatedWidget.currentRow[intervalField.name];

			if (intervalPeriod === undefined) {
				getLastActivatedWidget();
			}
			else if (intervalPeriod.start === null && intervalPeriod.end === null && intervalPeriod.period === null) {
				delete toBeActivatedWidget.currentRow[intervalField.name];
			}

			getLastActivatedWidget();
		};

		self.field.intervalWidget.selectIntervalPeriod = function(listWidget) {
			var intervalWidget = self.field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			intervalData.start = listWidget.currentRow._period.start;
			intervalData.end = listWidget.currentRow._period.end;
			intervalData.period = {
				'name': listWidget.currentRow._period_name,
				'label': listWidget.currentRow._period_label,
				'customized': listWidget.currentRow._period.customized
			};

			intervalWidget.formattedData.start = moment(intervalData.start, $scope.dateFormat).format($scope.displayFormat);
			intervalWidget.formattedData.end = moment(intervalData.end, $scope.dateFormat).format($scope.displayFormat);

			clearOldInterval();
			selectIntervalOnCalendar();

			ScreenService.closePopup();
			self.field.widget.activate();
			intervalWidget.applyIntervalToWidgetRow();
		};

		self.field.intervalWidget.isCalendarCurrentWidget = function() {
			return self.field.intervalWidget.carouselWidget === 'calendar';
		};

		self.field.intervalWidget.isPeriodSelectorCurrentWidget = function() {
			return self.field.intervalWidget.carouselWidget === 'periodSelector';
		};

		self.field.intervalWidget.toggleCurrentWidget = function() {
			var intervalWidget = self.field.intervalWidget;
			if (intervalWidget.isPeriodSelectorCurrentWidget()) {
				intervalWidget.carouselWidget = 'calendar';
				intervalWidget.beingSelected = 'start';
			}
			else {
				intervalWidget.carouselWidget = 'periodSelector';
				intervalWidget.beingSelected = 'none';
			}

			getLastActivatedWidget();
			intervalWidget.nextWidget();
		};
	}


	function getLastActivatedWidget() {
		var lastActive = self.field.intervalWidget.lastActiveWidget;
		var toBeActivatedWidget = self.field.intervalWidget.lastActiveWidget;
		toBeActivatedWidget.activate();
		toBeActivatedWidget.lastActiveWidget = lastActive;
	}

	function toggleBeingSelected() {
		var intervalWidget = self.field.intervalWidget;
		intervalWidget.beingSelected = intervalWidget.beingSelected === 'start' ? 'end' : 'start';
	}

	function clearOldInterval() {
		var calendarWidget = self.field.intervalWidget.widgets[0];
		calendarWidget.selectedDates = [];
	}

	function selectIntervalOnCalendar() {
		var intervalWidget = self.field.intervalWidget || self.widget;
		var calendarWidget = intervalWidget.widgets[0];

		var dateFormat = self.field.dateFormat;
		var startDate = intervalWidget.intervalData.start;
		var endDate = intervalWidget.intervalData.end;

		var momentstartDate = moment(startDate, dateFormat);
		var momentEndDate = moment(endDate, dateFormat);
		var momentCurrentDate = momentstartDate;

		while (momentCurrentDate.isBefore(momentEndDate) || momentCurrentDate.isSame(momentEndDate)) {
			calendarWidget.setDateAsSelected(momentCurrentDate.format(dateFormat), dateFormat);
			momentCurrentDate.add(1, 'day');
		}

	}

	this.getPeriodLabel = function() {
		var label = '';
		var property = self.field.widget.currentRow[self.field.name];
		if (property && property.period && property.period.label) {
			label = '(' + property.period.label + ')';
		}
		return label;
	};

	if (self.field) {
		var field = self.field;
		var parentWidget = field.widget;

		field.value(self.field.value() || self.field.defaultValue);
		field.isRowSet = function() {
			var currentRow = this.widget.currentRow;
			return currentRow[this.name] && currentRow[this.name].start && currentRow[this.name].end;
		};
		field.clearValue = clearInternalValues;

		field.getDatesByPeriodName = function(periodName, periodLabel) {
			periodLabel = periodLabel || '';
			var date = isCustomPeriod(periodName) ? ZhUtil.evaluate(periodName) : buildObjectWithPeriodDetails(periodName);

			return {
				"start": date.start,
				"end": date.end,
				"period": {
					"name": periodName,
					"label": periodLabel || '',
					"customized": date.customized
				}
			};
		};

		if (parentWidget.currentRow[field.name] &&
			parentWidget.currentRow[field.name].period &&
			parentWidget.currentRow[field.name].period.name
		) {
			var date = field.getDatesByPeriodName(parentWidget.currentRow[field.name].period.name);
			parentWidget.currentRow[field.name].start = date.start;
			parentWidget.currentRow[field.name].end = date.end;
		}
	}

	/**
	 * Validations
	 **/
	$scope.isRequired = function(field) {
		return field.validations && (field.validations.requiredInterval || field.validations.requiredIntervalStart || field.validations.requiredIntervalEnd);
	};
}

// FILE: bower_components/zh-grid-configuration/dist/zh-grid-configuration.js

// FILE: bower_components/sortablejs/Sortable.js
/**!
 * Sortable
 * @author	RubaXa   <trash@rubaxa.org>
 * @author	owenm    <owen23355@gmail.com>
 * @license MIT
 */

(function sortableModule(factory) {
	"use strict";

	if (typeof define === "function" && define.amd) {
		define(factory);
	}
	else if (typeof module != "undefined" && typeof module.exports != "undefined") {
		module.exports = factory();
	}
	else {
		/* jshint sub:true */
		window["Sortable"] = factory();
	}
})(function sortableFactory() {
	"use strict";

	if (typeof window === "undefined" || !window.document) {
		return function sortableError() {
			throw new Error("Sortable.js requires a window with a document");
		};
	}

	var dragEl,
		parentEl,
		ghostEl,
		cloneEl,
		rootEl,
		nextEl,
		lastDownEl,

		scrollEl,
		scrollParentEl,
		scrollCustomFn,

		oldIndex,
		newIndex,

		activeGroup,
		putSortable,

		autoScrolls = [],
		scrolling = false,

		awaitingDragStarted = false,
		ignoreNextClick = false,
		sortables = [],

		pointerElemChangedInterval,
		lastPointerElemX,
		lastPointerElemY,

		tapEvt,
		touchEvt,

		moved,


		lastTarget,
		lastDirection,
		pastFirstInvertThresh = false,
		isCircumstantialInvert = false,
		lastMode, // 'swap' or 'insert'

		targetMoveDistance,


		forRepaintDummy,
		realDragElRect, // dragEl rect after current animation

		/** @const */
		R_SPACE = /\s+/g,

		expando = 'Sortable' + (new Date).getTime(),

		win = window,
		document = win.document,
		parseInt = win.parseInt,
		setTimeout = win.setTimeout,

		$ = win.jQuery || win.Zepto,
		Polymer = win.Polymer,

		captureMode = {
			capture: false,
			passive: false
		},

		IE11OrLess = !!navigator.userAgent.match(/(?:Trident.*rv[ :]?11\.|msie|iemobile)/i),
		Edge = !!navigator.userAgent.match(/Edge/i),
		// FireFox = !!navigator.userAgent.match(/firefox/i),

		CSSFloatProperty = Edge || IE11OrLess ? 'cssFloat' : 'float',

		// This will not pass for IE9, because IE9 DnD only works on anchors
		supportDraggable = ('draggable' in document.createElement('div')),

		supportCssPointerEvents = (function() {
			// false when <= IE11
			if (IE11OrLess) {
				return false;
			}
			var el = document.createElement('x');
			el.style.cssText = 'pointer-events:auto';
			return el.style.pointerEvents === 'auto';
		})(),

		_silent = false,
		_alignedSilent = false,

		abs = Math.abs,
		min = Math.min,

		savedInputChecked = [],

		_detectDirection = function(el, options) {
			var elCSS = _css(el),
				elWidth = parseInt(elCSS.width),
				child1 = _getChild(el, 0, options),
				child2 = _getChild(el, 1, options),
				firstChildCSS = child1 && _css(child1),
				secondChildCSS = child2 && _css(child2),
				firstChildWidth = firstChildCSS && parseInt(firstChildCSS.marginLeft) + parseInt(firstChildCSS.marginRight) + _getRect(child1).width,
				secondChildWidth = secondChildCSS && parseInt(secondChildCSS.marginLeft) + parseInt(secondChildCSS.marginRight) + _getRect(child2).width;
			if (elCSS.display === 'flex') {
				return elCSS.flexDirection === 'column' || elCSS.flexDirection === 'column-reverse'
				? 'vertical' : 'horizontal';
			}
			return (child1 &&
				(
					firstChildCSS.display === 'block' ||
					firstChildCSS.display === 'flex' ||
					firstChildCSS.display === 'table' ||
					firstChildCSS.display === 'grid' ||
					firstChildWidth >= elWidth &&
					elCSS[CSSFloatProperty] === 'none' ||
					child2 &&
					elCSS[CSSFloatProperty] === 'none' &&
					firstChildWidth + secondChildWidth > elWidth
				) ?
				'vertical' : 'horizontal'
			);
		},

		/**
		 * Detects first nearest empty sortable to X and Y position using emptyInsertThreshold.
		 * @param  {Number} x      X position
		 * @param  {Number} y      Y position
		 * @return {HTMLElement}   Element of the first found nearest Sortable
		 */
		_detectNearestEmptySortable = function(x, y) {
			for (var i = 0; i < sortables.length; i++) {
				if (sortables[i].children.length) continue;

				var rect = _getRect(sortables[i]),
					threshold = sortables[i][expando].options.emptyInsertThreshold,
					insideHorizontally = x >= (rect.left - threshold) && x <= (rect.right + threshold),
					insideVertically = y >= (rect.top - threshold) && y <= (rect.bottom + threshold);

				if (insideHorizontally && insideVertically) {
					return sortables[i];
				}
			}
		},

		_isClientInRowColumn = function(x, y, el, axis, options) {
			var targetRect = _getRect(el),
				targetS1Opp = axis === 'vertical' ? targetRect.left : targetRect.top,
				targetS2Opp = axis === 'vertical' ? targetRect.right : targetRect.bottom,
				mouseOnOppAxis = axis === 'vertical' ? x : y;

			return targetS1Opp < mouseOnOppAxis && mouseOnOppAxis < targetS2Opp;
		},

		_isElInRowColumn = function(el1, el2, axis) {
			var el1Rect = el1 === dragEl && realDragElRect || _getRect(el1),
				el2Rect = el2 === dragEl && realDragElRect || _getRect(el2),
				el1S1Opp = axis === 'vertical' ? el1Rect.left : el1Rect.top,
				el1S2Opp = axis === 'vertical' ? el1Rect.right : el1Rect.bottom,
				el1OppLength = axis === 'vertical' ? el1Rect.width : el1Rect.height,
				el2S1Opp = axis === 'vertical' ? el2Rect.left : el2Rect.top,
				el2S2Opp = axis === 'vertical' ? el2Rect.right : el2Rect.bottom,
				el2OppLength = axis === 'vertical' ? el2Rect.width : el2Rect.height;

			return (
				el1S1Opp === el2S1Opp ||
				el1S2Opp === el2S2Opp ||
				(el1S1Opp + el1OppLength / 2) === (el2S1Opp + el2OppLength / 2)
			);
		},

		_getParentAutoScrollElement = function(el, includeSelf) {
			// skip to window
			if (!el || !el.getBoundingClientRect) return win;

			var elem = el;
			var gotSelf = false;
			do {
				// we don't need to get elem css if it isn't even overflowing in the first place (performance)
				if (elem.clientWidth < elem.scrollWidth || elem.clientHeight < elem.scrollHeight) {
					var elemCSS = _css(elem);
					if (
						elem.clientWidth < elem.scrollWidth && (elemCSS.overflowX == 'auto' || elemCSS.overflowX == 'scroll') ||
						elem.clientHeight < elem.scrollHeight && (elemCSS.overflowY == 'auto' || elemCSS.overflowY == 'scroll')
					) {
						if (!elem || !elem.getBoundingClientRect || elem === document.body) return win;

						if (gotSelf || includeSelf) return elem;
						gotSelf = true;
					}
				}
			/* jshint boss:true */
			} while (elem = elem.parentNode);

			return win;
		},

		_autoScroll = _throttle(function (/**Event*/evt, /**Object*/options, /**HTMLElement*/rootEl, /**Boolean*/isFallback) {
			// Bug: https://bugzilla.mozilla.org/show_bug.cgi?id=505521
			if (options.scroll) {
				var _this = rootEl ? rootEl[expando] : window,
					sens = options.scrollSensitivity,
					speed = options.scrollSpeed,

					x = evt.clientX,
					y = evt.clientY,

					winWidth = window.innerWidth,
					winHeight = window.innerHeight,

					scrollThisInstance = false;

				// Detect scrollEl
				if (scrollParentEl !== rootEl) {
					_clearAutoScrolls();

					scrollEl = options.scroll;
					scrollCustomFn = options.scrollFn;

					if (scrollEl === true) {
						scrollEl = _getParentAutoScrollElement(rootEl, true);
						scrollParentEl = scrollEl;
					}
				}


				var layersOut = 0;
				var currentParent = scrollEl;
				do {
					var	el = currentParent,
						rect = _getRect(el),

						top = rect.top,
						bottom = rect.bottom,
						left = rect.left,
						right = rect.right,

						width = rect.width,
						height = rect.height,

						scrollWidth,
						scrollHeight,

						css,

						vx,
						vy,

						canScrollX,
						canScrollY,

						scrollPosX,
						scrollPosY;


					if (el !== win) {
						scrollWidth = el.scrollWidth;
						scrollHeight = el.scrollHeight;

						css = _css(el);

						canScrollX = width < scrollWidth && (css.overflowX === 'auto' || css.overflowX === 'scroll');
						canScrollY = height < scrollHeight && (css.overflowY === 'auto' || css.overflowY === 'scroll');

						scrollPosX = el.scrollLeft;
						scrollPosY = el.scrollTop;
					} else {
						scrollWidth = document.documentElement.scrollWidth;
						scrollHeight = document.documentElement.scrollHeight;

						css = _css(document.documentElement);

						canScrollX = width < scrollWidth && (css.overflowX === 'auto' || css.overflowX === 'scroll' || css.overflowX === 'visible');
						canScrollY = height < scrollHeight && (css.overflowY === 'auto' || css.overflowY === 'scroll' || css.overflowY === 'visible');

						scrollPosX = document.documentElement.scrollLeft;
						scrollPosY = document.documentElement.scrollTop;
					}

					vx = canScrollX && (abs(right - x) <= sens && (scrollPosX + width) < scrollWidth) - (abs(left - x) <= sens && !!scrollPosX);

					vy = canScrollY && (abs(bottom - y) <= sens && (scrollPosY + height) < scrollHeight) - (abs(top - y) <= sens && !!scrollPosY);


					if (!autoScrolls[layersOut]) {
						for (var i = 0; i <= layersOut; i++) {
							if (!autoScrolls[i]) {
								autoScrolls[i] = {};
							}
						}
					}

					if (autoScrolls[layersOut].vx != vx || autoScrolls[layersOut].vy != vy || autoScrolls[layersOut].el !== el) {
						autoScrolls[layersOut].el = el;
						autoScrolls[layersOut].vx = vx;
						autoScrolls[layersOut].vy = vy;

						clearInterval(autoScrolls[layersOut].pid);

						if (el && (vx != 0 || vy != 0)) {
							scrollThisInstance = true;
							/* jshint loopfunc:true */
							autoScrolls[layersOut].pid = setInterval((function () {
								// emulate drag over during autoscroll (fallback), emulating native DnD behaviour
								if (isFallback && this.layer === 0) {
									Sortable.active._emulateDragOver(true);
								}
								var scrollOffsetY = autoScrolls[this.layer].vy ? autoScrolls[this.layer].vy * speed : 0;
								var scrollOffsetX = autoScrolls[this.layer].vx ? autoScrolls[this.layer].vx * speed : 0;

								if ('function' === typeof(scrollCustomFn)) {
									if (scrollCustomFn.call(_this, scrollOffsetX, scrollOffsetY, evt, touchEvt, autoScrolls[this.layer].el) !== 'continue') {
										return;
									}
								}
								if (autoScrolls[this.layer].el === win) {
									win.scrollTo(win.pageXOffset + scrollOffsetX, win.pageYOffset + scrollOffsetY);
								} else {
									autoScrolls[this.layer].el.scrollTop += scrollOffsetY;
									autoScrolls[this.layer].el.scrollLeft += scrollOffsetX;
								}
							}).bind({layer: layersOut}), 24);
						}
					}
					layersOut++;
				} while (options.bubbleScroll && currentParent !== win && (currentParent = _getParentAutoScrollElement(currentParent, false)));
				scrolling = scrollThisInstance; // in case another function catches scrolling as false in between when it is not
			}
		}, 30),

		_clearAutoScrolls = function () {
			autoScrolls.forEach(function(autoScroll) {
				clearInterval(autoScroll.pid);
			});
			autoScrolls = [];
		},

		_prepareGroup = function (options) {
			function toFn(value, pull) {
				return function(to, from, dragEl, evt) {
					var ret;
					var sameGroup = to.options.group.name &&
									from.options.group.name &&
									to.options.group.name === from.options.group.name;

					if (value == null && (pull || sameGroup)) {
						// Default pull value
						// Default pull and put value if same group
						return true;
					} else if (value == null || value === false) {
						return false;
					} else if (pull && value === 'clone') {
						return value;
					} else if (typeof value === 'function') {
						return toFn(value(to, from, dragEl, evt));
					} else {
						var otherGroup = (pull ? to : from).options.group.name;

						return (value === true ||
						(typeof value === 'string' && value === otherGroup) ||
						(value.join && value.indexOf(otherGroup) > -1));
					}
				};
			}

			var group = {};
			var originalGroup = options.group;

			if (!originalGroup || typeof originalGroup != 'object') {
				originalGroup = {name: originalGroup};
			}

			group.name = originalGroup.name;
			group.checkPull = toFn(originalGroup.pull, true);
			group.checkPut = toFn(originalGroup.put);
			group.revertClone = originalGroup.revertClone;

			options.group = group;
		},

		_checkAlignment = function(evt) {
			if (!dragEl || !dragEl.parentNode) return;
			dragEl.parentNode[expando] && dragEl.parentNode[expando]._computeIsAligned(evt);
		},

		_isTrueParentSortable = function(el, target) {
			var trueParent = target;
			while (!trueParent[expando]) {
				trueParent = trueParent.parentNode;
			}

			return el === trueParent;
		},

		_artificalBubble = function(sortable, originalEvt, method) {
			// Artificial IE bubbling
			var nextParent = sortable.parentNode;
			while (nextParent && !nextParent[expando]) {
				nextParent = nextParent.parentNode;
			}

			if (nextParent) {
				nextParent[expando][method](_extend(originalEvt, {
					artificialBubble: true
				}));
			}
		},

		_hideGhostForTarget = function() {
			if (!supportCssPointerEvents && ghostEl) {
				_css(ghostEl, 'display', 'none');
			}
		},

		_unhideGhostForTarget = function() {
			if (!supportCssPointerEvents && ghostEl) {
				_css(ghostEl, 'display', '');
			}
		};


	// #1184 fix - Prevent click event on fallback if dragged but item not changed position
	document.addEventListener('click', function(evt) {
		if (ignoreNextClick) {
			evt.preventDefault();
			evt.stopPropagation && evt.stopPropagation();
			evt.stopImmediatePropagation && evt.stopImmediatePropagation();
			ignoreNextClick = false;
			return false;
		}
	}, true);

	var nearestEmptyInsertDetectEvent = function(evt) {
		if (dragEl) {
			var nearest = _detectNearestEmptySortable(evt.clientX, evt.clientY);

			if (nearest) {
				nearest[expando]._onDragOver({
					clientX: evt.clientX,
					clientY: evt.clientY,
					target: nearest,
					rootEl: nearest
				});
			}
		}
	};
	// We do not want this to be triggered if completed (bubbling canceled), so only define it here
	document.addEventListener('dragover', nearestEmptyInsertDetectEvent);
	document.addEventListener('mousemove', nearestEmptyInsertDetectEvent);

	/**
	 * @class  Sortable
	 * @param  {HTMLElement}  el
	 * @param  {Object}       [options]
	 */
	function Sortable(el, options) {
		if (!(el && el.nodeType && el.nodeType === 1)) {
			throw 'Sortable: `el` must be HTMLElement, not ' + {}.toString.call(el);
		}

		this.el = el; // root element
		this.options = options = _extend({}, options);


		// Export instance
		el[expando] = this;

		// Default options
		var defaults = {
			group: null,
			sort: true,
			disabled: false,
			store: null,
			handle: null,
			scroll: true,
			scrollSensitivity: 30,
			scrollSpeed: 10,
			bubbleScroll: true,
			draggable: /[uo]l/i.test(el.nodeName) ? 'li' : '>*',
			swapThreshold: 1, // percentage; 0 <= x <= 1
			invertSwap: false, // invert always
			invertedSwapThreshold: null, // will be set to same as swapThreshold if default
			removeCloneOnHide: true,
			direction: function() {
				return _detectDirection(el, this.options);
			},
			ghostClass: 'sortable-ghost',
			chosenClass: 'sortable-chosen',
			dragClass: 'sortable-drag',
			ignore: 'a, img',
			filter: null,
			preventOnFilter: true,
			animation: 0,
			easing: null,
			setData: function (dataTransfer, dragEl) {
				dataTransfer.setData('Text', dragEl.textContent);
			},
			dropBubble: false,
			dragoverBubble: false,
			dataIdAttr: 'data-id',
			delay: 0,
			touchStartThreshold: parseInt(window.devicePixelRatio, 10) || 1,
			forceFallback: false,
			fallbackClass: 'sortable-fallback',
			fallbackOnBody: false,
			fallbackTolerance: 0,
			fallbackOffset: {x: 0, y: 0},
			supportPointer: Sortable.supportPointer !== false && (
				('PointerEvent' in window) ||
				window.navigator && ('msPointerEnabled' in window.navigator) // microsoft
			),
			emptyInsertThreshold: 5
		};


		// Set default options
		for (var name in defaults) {
			!(name in options) && (options[name] = defaults[name]);
		}

		_prepareGroup(options);

		// Bind all private methods
		for (var fn in this) {
			if (fn.charAt(0) === '_' && typeof this[fn] === 'function') {
				this[fn] = this[fn].bind(this);
			}
		}

		// Setup drag mode
		this.nativeDraggable = options.forceFallback ? false : supportDraggable;

		// Bind events
		if (options.supportPointer) {
			_on(el, 'pointerdown', this._onTapStart);
		} else {
			_on(el, 'mousedown', this._onTapStart);
			_on(el, 'touchstart', this._onTapStart);
		}

		if (this.nativeDraggable) {
			_on(el, 'dragover', this);
			_on(el, 'dragenter', this);
		}

		sortables.push(this.el);

		// Restore sorting
		options.store && options.store.get && this.sort(options.store.get(this) || []);
	}

	Sortable.prototype = /** @lends Sortable.prototype */ {
		constructor: Sortable,

		_computeIsAligned: function(evt) {
			var target;

			if (ghostEl && !supportCssPointerEvents) {
				_hideGhostForTarget();
				target = document.elementFromPoint(evt.clientX, evt.clientY);
				_unhideGhostForTarget();
			} else {
				target = evt.target;
			}

			target = _closest(target, this.options.draggable, this.el, false);
			if (_alignedSilent) return;
			if (!dragEl || dragEl.parentNode !== this.el) return;

			var children = this.el.children;
			for (var i = 0; i < children.length; i++) {
				// Don't change for target in case it is changed to aligned before onDragOver is fired
				if (_closest(children[i], this.options.draggable, this.el, false) && children[i] !== target) {
					children[i].sortableMouseAligned = _isClientInRowColumn(evt.clientX, evt.clientY, children[i], this._getDirection(evt, null), this.options);
				}
			}
			// Used for nulling last target when not in element, nothing to do with checking if aligned
			if (!_closest(target, this.options.draggable, this.el, true)) {
				lastTarget = null;
			}

			_alignedSilent = true;
			setTimeout(function() {
				_alignedSilent = false;
			}, 30);

		},

		_getDirection: function(evt, target) {
			return (typeof this.options.direction === 'function') ? this.options.direction.call(this, evt, target, dragEl) : this.options.direction;
		},

		_onTapStart: function (/** Event|TouchEvent */evt) {
			if (!evt.cancelable) return;

			var _this = this,
				el = this.el,
				options = this.options,
				preventOnFilter = options.preventOnFilter,
				type = evt.type,
				touch = evt.touches && evt.touches[0],
				target = (touch || evt).target,
				originalTarget = evt.target.shadowRoot && ((evt.path && evt.path[0]) || (evt.composedPath && evt.composedPath()[0])) || target,
				filter = options.filter,
				startIndex;

			_saveInputCheckedState(el);


			// IE: Calls events in capture mode if event element is nested. This ensures only correct element's _onTapStart goes through.
			// This process is also done in _onDragOver
			if (IE11OrLess && !evt.artificialBubble && !_isTrueParentSortable(el, target)) {
				return;
			}

			// Don't trigger start event when an element is been dragged, otherwise the evt.oldindex always wrong when set option.group.
			if (dragEl) {
				return;
			}

			if (/mousedown|pointerdown/.test(type) && evt.button !== 0 || options.disabled) {
				return; // only left button and enabled
			}

			// cancel dnd if original target is content editable
			if (originalTarget.isContentEditable) {
				return;
			}

			target = _closest(target, options.draggable, el, false);

			if (!target) {
				if (IE11OrLess) {
					_artificalBubble(el, evt, '_onTapStart');
				}
				return;
			}

			if (lastDownEl === target) {
				// Ignoring duplicate `down`
				return;
			}

			// Get the index of the dragged element within its parent
			startIndex = _index(target, options.draggable);

			// Check filter
			if (typeof filter === 'function') {
				if (filter.call(this, evt, target, this)) {
					_dispatchEvent(_this, originalTarget, 'filter', target, el, el, startIndex);
					preventOnFilter && evt.cancelable && evt.preventDefault();
					return; // cancel dnd
				}
			}
			else if (filter) {
				filter = filter.split(',').some(function (criteria) {
					criteria = _closest(originalTarget, criteria.trim(), el, false);

					if (criteria) {
						_dispatchEvent(_this, criteria, 'filter', target, el, el, startIndex);
						return true;
					}
				});

				if (filter) {
					preventOnFilter && evt.cancelable && evt.preventDefault();
					return; // cancel dnd
				}
			}

			if (options.handle && !_closest(originalTarget, options.handle, el, false)) {
				return;
			}

			// Prepare `dragstart`
			this._prepareDragStart(evt, touch, target, startIndex);
		},


		_handleAutoScroll: function(evt, fallback) {
			if (!dragEl || !this.options.scroll) return;
			var x = evt.clientX,
				y = evt.clientY,

				elem = document.elementFromPoint(x, y),
				_this = this;

			// IE does not seem to have native autoscroll,
			// Edge's autoscroll seems too conditional,
			// Firefox and Chrome are good
			if (fallback || Edge || IE11OrLess) {
				_autoScroll(evt, _this.options, elem, fallback);

				// Listener for pointer element change
				var ogElemScroller = _getParentAutoScrollElement(elem, true);
				if (
					scrolling &&
					(
						!pointerElemChangedInterval ||
						x !== lastPointerElemX ||
						y !== lastPointerElemY
					)
				) {

					pointerElemChangedInterval && clearInterval(pointerElemChangedInterval);
					// Detect for pointer elem change, emulating native DnD behaviour
					pointerElemChangedInterval = setInterval(function() {
						if (!dragEl) return;
						// could also check if scroll direction on newElem changes due to parent autoscrolling
						var newElem = _getParentAutoScrollElement(document.elementFromPoint(x, y), true);
						if (newElem !== ogElemScroller) {
							ogElemScroller = newElem;
							_clearAutoScrolls();
							_autoScroll(evt, _this.options, ogElemScroller, fallback);
						}
					}, 10);
					lastPointerElemX = x;
					lastPointerElemY = y;
				}

			} else {
				// if DnD is enabled (and browser has good autoscrolling), first autoscroll will already scroll, so get parent autoscroll of first autoscroll
				if (!_this.options.bubbleScroll || _getParentAutoScrollElement(elem, true) === window) {
					_clearAutoScrolls();
					return;
				}
				_autoScroll(evt, _this.options, _getParentAutoScrollElement(elem, false), false);
			}
		},

		_prepareDragStart: function (/** Event */evt, /** Touch */touch, /** HTMLElement */target, /** Number */startIndex) {
			var _this = this,
				el = _this.el,
				options = _this.options,
				ownerDocument = el.ownerDocument,
				dragStartFn;

			if (target && !dragEl && (target.parentNode === el)) {
				rootEl = el;
				dragEl = target;
				parentEl = dragEl.parentNode;
				nextEl = dragEl.nextSibling;
				lastDownEl = target;
				activeGroup = options.group;
				oldIndex = startIndex;

				tapEvt = {
					target: dragEl,
					clientX: (touch || evt).clientX,
					clientY: (touch || evt).clientY
				};

				this._lastX = (touch || evt).clientX;
				this._lastY = (touch || evt).clientY;

				dragEl.style['will-change'] = 'all';
				// undo animation if needed
				dragEl.style.transition = '';
				dragEl.style.transform = '';

				dragStartFn = function () {
					// Delayed drag has been triggered
					// we can re-enable the events: touchmove/mousemove
					_this._disableDelayedDrag();

					// Make the element draggable
					dragEl.draggable = _this.nativeDraggable;

					// Bind the events: dragstart/dragend
					_this._triggerDragStart(evt, touch);

					// Drag start event
					_dispatchEvent(_this, rootEl, 'choose', dragEl, rootEl, rootEl, oldIndex);

					// Chosen item
					_toggleClass(dragEl, options.chosenClass, true);
				};

				// Disable "draggable"
				options.ignore.split(',').forEach(function (criteria) {
					_find(dragEl, criteria.trim(), _disableDraggable);
				});

				if (options.supportPointer) {
					_on(ownerDocument, 'pointerup', _this._onDrop);
					_on(ownerDocument, 'pointercancel', _this._onDrop);
				} else {
					_on(ownerDocument, 'mouseup', _this._onDrop);
					_on(ownerDocument, 'touchend', _this._onDrop);
					_on(ownerDocument, 'touchcancel', _this._onDrop);
				}

				if (options.delay) {
					// If the user moves the pointer or let go the click or touch
					// before the delay has been reached:
					// disable the delayed drag
					_on(ownerDocument, 'mouseup', _this._disableDelayedDrag);
					_on(ownerDocument, 'touchend', _this._disableDelayedDrag);
					_on(ownerDocument, 'touchcancel', _this._disableDelayedDrag);
					_on(ownerDocument, 'mousemove', _this._delayedDragTouchMoveHandler);
					_on(ownerDocument, 'touchmove', _this._delayedDragTouchMoveHandler);
					options.supportPointer && _on(ownerDocument, 'pointermove', _this._delayedDragTouchMoveHandler);

					_this._dragStartTimer = setTimeout(dragStartFn, options.delay);
				} else {
					dragStartFn();
				}
			}
		},

		_delayedDragTouchMoveHandler: function (/** TouchEvent|PointerEvent **/e) {
			var touch = e.touches ? e.touches[0] : e;
			if (min(abs(touch.clientX - this._lastX), abs(touch.clientY - this._lastY))
					>= this.options.touchStartThreshold
			) {
				this._disableDelayedDrag();
			}
		},

		_disableDelayedDrag: function () {
			var ownerDocument = this.el.ownerDocument;

			clearTimeout(this._dragStartTimer);
			_off(ownerDocument, 'mouseup', this._disableDelayedDrag);
			_off(ownerDocument, 'touchend', this._disableDelayedDrag);
			_off(ownerDocument, 'touchcancel', this._disableDelayedDrag);
			_off(ownerDocument, 'mousemove', this._delayedDragTouchMoveHandler);
			_off(ownerDocument, 'touchmove', this._delayedDragTouchMoveHandler);
			_off(ownerDocument, 'pointermove', this._delayedDragTouchMoveHandler);
		},

		_triggerDragStart: function (/** Event */evt, /** Touch */touch) {
			touch = touch || (evt.pointerType == 'touch' ? evt : null);

			if (!this.nativeDraggable || touch) {
				if (this.options.supportPointer) {
					touch && _on(document, 'touchmove', _preventScroll); // must be touchmove to prevent scroll
					_on(document, 'pointermove', this._onTouchMove);
				} else if (touch) {
					_on(document, 'touchmove', _preventScroll);
					_on(document, 'touchmove', this._onTouchMove);
				} else {
					_on(document, 'mousemove', this._onTouchMove);
				}
			} else {
				_on(dragEl, 'dragend', this);
				_on(rootEl, 'dragstart', this._onDragStart);
			}

			try {
				if (document.selection) {
					// Timeout neccessary for IE9
					_nextTick(function () {
						document.selection.empty();
					});
				} else {
					window.getSelection().removeAllRanges();
				}
			} catch (err) {
			}
		},

		_dragStarted: function (fallback) {
			awaitingDragStarted = false;
			if (rootEl && dragEl) {
				if (this.nativeDraggable) {
					_on(document, 'dragover', this._handleAutoScroll);
					_on(document, 'dragover', _checkAlignment);
				}
				var options = this.options;

				// Apply effect
				!fallback && _toggleClass(dragEl, options.dragClass, false);
				_toggleClass(dragEl, options.ghostClass, true);

				// In case dragging an animated element
				_css(dragEl, 'transform', '');

				Sortable.active = this;

				fallback && this._appendGhost();

				// Drag start event
				_dispatchEvent(this, rootEl, 'start', dragEl, rootEl, rootEl, oldIndex);
			} else {
				this._nulling();
			}
		},

		_emulateDragOver: function (bypassLastTouchCheck) {
			if (touchEvt) {
				if (this._lastX === touchEvt.clientX && this._lastY === touchEvt.clientY && !bypassLastTouchCheck) {
					return;
				}
				this._lastX = touchEvt.clientX;
				this._lastY = touchEvt.clientY;

				_hideGhostForTarget();

				var target = document.elementFromPoint(touchEvt.clientX, touchEvt.clientY);
				var parent = target;

				while (target && target.shadowRoot) {
					target = target.shadowRoot.elementFromPoint(touchEvt.clientX, touchEvt.clientY);
					parent = target;
				}

				if (parent) {
					do {
						if (parent[expando]) {
							var inserted;

							inserted = parent[expando]._onDragOver({
								clientX: touchEvt.clientX,
								clientY: touchEvt.clientY,
								target: target,
								rootEl: parent
							});

							if (inserted && !this.options.dragoverBubble) {
								break;
							}
						}

						target = parent; // store last element
					}
					/* jshint boss:true */
					while (parent = parent.parentNode);
				}
				dragEl.parentNode[expando]._computeIsAligned(touchEvt);

				_unhideGhostForTarget();
			}
		},


		_onTouchMove: function (/**TouchEvent*/evt) {
			if (tapEvt) {
				if (!evt.cancelable) return;
				var	options = this.options,
					fallbackTolerance = options.fallbackTolerance,
					fallbackOffset = options.fallbackOffset,
					touch = evt.touches ? evt.touches[0] : evt,
					matrix = ghostEl && _matrix(ghostEl),
					scaleX = ghostEl && matrix && matrix.a,
					scaleY = ghostEl && matrix && matrix.d,
					dx = ((touch.clientX - tapEvt.clientX) + fallbackOffset.x) / (scaleX ? scaleX : 1),
					dy = ((touch.clientY - tapEvt.clientY) + fallbackOffset.y) / (scaleY ? scaleY : 1),
					translate3d = evt.touches ? 'translate3d(' + dx + 'px,' + dy + 'px,0)' : 'translate(' + dx + 'px,' + dy + 'px)';


				// only set the status to dragging, when we are actually dragging
				if (!Sortable.active && !awaitingDragStarted) {
					if (fallbackTolerance &&
						min(abs(touch.clientX - this._lastX), abs(touch.clientY - this._lastY)) < fallbackTolerance
					) {
						return;
					}
					this._onDragStart(evt, true);
				}

				this._handleAutoScroll(touch, true);


				moved = true;
				touchEvt = touch;


				_css(ghostEl, 'webkitTransform', translate3d);
				_css(ghostEl, 'mozTransform', translate3d);
				_css(ghostEl, 'msTransform', translate3d);
				_css(ghostEl, 'transform', translate3d);

				evt.cancelable && evt.preventDefault();
			}
		},

		_appendGhost: function () {
			if (!ghostEl) {
				var rect = _getRect(dragEl, this.options.fallbackOnBody ? document.body : rootEl, true),
					css = _css(dragEl),
					options = this.options;

				ghostEl = dragEl.cloneNode(true);

				_toggleClass(ghostEl, options.ghostClass, false);
				_toggleClass(ghostEl, options.fallbackClass, true);
				_toggleClass(ghostEl, options.dragClass, true);

				_css(ghostEl, 'box-sizing', 'border-box');
				_css(ghostEl, 'margin', 0);
				_css(ghostEl, 'top', rect.top);
				_css(ghostEl, 'left', rect.left);
				_css(ghostEl, 'width', rect.width);
				_css(ghostEl, 'height', rect.height);
				_css(ghostEl, 'opacity', '0.8');
				_css(ghostEl, 'position', 'fixed');
				_css(ghostEl, 'zIndex', '100000');
				_css(ghostEl, 'pointerEvents', 'none');

				options.fallbackOnBody && document.body.appendChild(ghostEl) || rootEl.appendChild(ghostEl);
			}
		},

		_onDragStart: function (/**Event*/evt, /**boolean*/fallback) {
			var _this = this;
			var dataTransfer = evt.dataTransfer;
			var options = _this.options;

			// Setup clone
			cloneEl = _clone(dragEl);

			cloneEl.draggable = false;
			cloneEl.style['will-change'] = '';

			this._hideClone();

			_toggleClass(cloneEl, _this.options.chosenClass, false);


			// #1143: IFrame support workaround
			_this._cloneId = _nextTick(function () {
				if (!_this.options.removeCloneOnHide) {
					rootEl.insertBefore(cloneEl, dragEl);
				}
				_dispatchEvent(_this, rootEl, 'clone', dragEl);
			});


			!fallback && _toggleClass(dragEl, options.dragClass, true);

			// Set proper drop events
			if (fallback) {
				ignoreNextClick = true;
				_this._loopId = setInterval(_this._emulateDragOver, 50);
			} else {
				// Undo what was set in _prepareDragStart before drag started
				_off(document, 'mouseup', _this._onDrop);
				_off(document, 'touchend', _this._onDrop);
				_off(document, 'touchcancel', _this._onDrop);
				_off(document, 'pointercancel', _this._onDrop);

				if (dataTransfer) {
					dataTransfer.effectAllowed = 'move';
					options.setData && options.setData.call(_this, dataTransfer, dragEl);
				}

				_on(document, 'drop', _this);

				// #1276 fix:
				_css(dragEl, 'transform', 'translateZ(0)');
			}

			awaitingDragStarted = true;

			_this._dragStartId = _nextTick(_this._dragStarted.bind(_this, fallback));
			_on(document, 'selectstart', _this);
		},

		// Returns true - if no further action is needed (either inserted or another condition)
		_onDragOver: function (/**Event*/evt) {
			var el = this.el,
				target = evt.target,
				dragRect,
				targetRect,
				revert,
				options = this.options,
				group = options.group,
				activeSortable = Sortable.active,
				isOwner = (activeGroup === group),
				canSort = options.sort,
				_this = this;

			if (_silent) return;

			// IE event order fix
			if (IE11OrLess && !evt.rootEl && !evt.artificialBubble && !_isTrueParentSortable(el, target)) {
				return;
			}

			// Return invocation when no further action is needed in another sortable
			function completed() {
				if (activeSortable) {
					// Set ghost class to new sortable's ghost class
					_toggleClass(dragEl, putSortable ? putSortable.options.ghostClass : activeSortable.options.ghostClass, false);
					_toggleClass(dragEl, options.ghostClass, true);
				}

				if (putSortable !== _this && _this !== Sortable.active) {
					putSortable = _this;
				} else if (_this === Sortable.active) {
					putSortable = null;
				}


				// Null lastTarget if it is not inside a previously swapped element
				if ((target === dragEl && !dragEl.animated) || (target === el && !target.animated)) {
					lastTarget = null;
				}
				// no bubbling and not fallback
				if (!options.dragoverBubble && !evt.rootEl && target !== document) {
					_this._handleAutoScroll(evt);
					dragEl.parentNode[expando]._computeIsAligned(evt);
				}

				!options.dragoverBubble && evt.stopPropagation && evt.stopPropagation();

				return true;
			}

			// Call when dragEl has been inserted
			function changed() {
				_dispatchEvent(_this, rootEl, 'change', target, el, rootEl, oldIndex, _index(dragEl, options.draggable), evt);
			}


			if (evt.preventDefault !== void 0) {
				evt.cancelable && evt.preventDefault();
			}


			moved = true;

			target = _closest(target, options.draggable, el, true);

			// target is dragEl or target is animated
			if (!!_closest(evt.target, null, dragEl, true) || target.animated) {
				return completed();
			}

			if (target !== dragEl) {
				ignoreNextClick = false;
			}

			if (activeSortable && !options.disabled &&
				(isOwner
					? canSort || (revert = !rootEl.contains(dragEl)) // Reverting item into the original list
					: (
						putSortable === this ||
						(
							(this.lastPutMode = activeGroup.checkPull(this, activeSortable, dragEl, evt)) &&
							group.checkPut(this, activeSortable, dragEl, evt)
						)
					)
				)
			) {
				var axis = this._getDirection(evt, target);

				dragRect = _getRect(dragEl);

				if (revert) {
					this._hideClone();
					parentEl = rootEl; // actualization

					if (nextEl) {
						rootEl.insertBefore(dragEl, nextEl);
					} else {
						rootEl.appendChild(dragEl);
					}

					return completed();
				}

				if ((el.children.length === 0) || (el.children[0] === ghostEl) ||
					_ghostIsLast(evt, axis, el) && !dragEl.animated
				) {
					//assign target only if condition is true
					if (el.children.length !== 0 && el.children[0] !== ghostEl && el === evt.target) {
						target = _lastChild(el);
					}

					if (target) {
						targetRect = _getRect(target);
					}

					if (isOwner) {
						activeSortable._hideClone();
					} else {
						activeSortable._showClone(this);
					}

					if (_onMove(rootEl, el, dragEl, dragRect, target, targetRect, evt, !!target) !== false) {
						el.appendChild(dragEl);
						parentEl = el; // actualization
						realDragElRect = null;

						changed();
						this._animate(dragRect, dragEl);
						target && this._animate(targetRect, target);
						return completed();
					}
				}
				else if (target && target !== dragEl && (target.parentNode[expando] !== void 0) && target !== el) {
					var direction = 0,
						targetBeforeFirstSwap,
						aligned = target.sortableMouseAligned,
						differentLevel = dragEl.parentNode !== el,
						scrolledPastTop = _isScrolledPast(target, axis === 'vertical' ? 'top' : 'left');

					if (lastTarget !== target) {
						lastMode = null;
						targetBeforeFirstSwap = _getRect(target)[axis === 'vertical' ? 'top' : 'left'];
						pastFirstInvertThresh = false;
					}

					// Reference: https://www.lucidchart.com/documents/view/10fa0e93-e362-4126-aca2-b709ee56bd8b/0
					if (
						_isElInRowColumn(dragEl, target, axis) && aligned ||
						differentLevel ||
						scrolledPastTop ||
						options.invertSwap ||
						lastMode === 'insert' ||
						// Needed, in the case that we are inside target and inserted because not aligned... aligned will stay false while inside
						// and lastMode will change to 'insert', but we must swap
						lastMode === 'swap'
					) {
						// New target that we will be inside
						if (lastMode !== 'swap') {
							isCircumstantialInvert = options.invertSwap || differentLevel || scrolling || scrolledPastTop;
						}

						direction = _getSwapDirection(evt, target, axis,
							options.swapThreshold, options.invertedSwapThreshold == null ? options.swapThreshold : options.invertedSwapThreshold,
							isCircumstantialInvert,
							lastTarget === target);
						lastMode = 'swap';
					} else {
						// Insert at position
						direction = _getInsertDirection(target, options);
						lastMode = 'insert';
					}
					if (direction === 0) return completed();

					realDragElRect = null;
					lastTarget = target;

					lastDirection = direction;

					targetRect = _getRect(target);

					var nextSibling = target.nextElementSibling,
						after = false;

					after = direction === 1;

					var moveVector = _onMove(rootEl, el, dragEl, dragRect, target, targetRect, evt, after);

					if (moveVector !== false) {
						if (moveVector === 1 || moveVector === -1) {
							after = (moveVector === 1);
						}

						_silent = true;
						setTimeout(_unsilent, 30);

						if (isOwner) {
							activeSortable._hideClone();
						} else {
							activeSortable._showClone(this);
						}

						if (after && !nextSibling) {
							el.appendChild(dragEl);
						} else {
							target.parentNode.insertBefore(dragEl, after ? nextSibling : target);
						}

						parentEl = dragEl.parentNode; // actualization

						// must be done before animation
						if (targetBeforeFirstSwap !== undefined && !isCircumstantialInvert) {
							targetMoveDistance = abs(targetBeforeFirstSwap - _getRect(target)[axis === 'vertical' ? 'top' : 'left']);
						}
						changed();
						!differentLevel && this._animate(targetRect, target);
						this._animate(dragRect, dragEl);

						return completed();
					}
				}

				if (el.contains(dragEl)) {
					return completed();
				}
			}

			if (IE11OrLess && !evt.rootEl) {
				_artificalBubble(el, evt, '_onDragOver');
			}

			return false;
		},

		_animate: function (prevRect, target) {
			var ms = this.options.animation;

			if (ms) {
				var currentRect = _getRect(target);

				if (target === dragEl) {
					realDragElRect = currentRect;
				}

				if (prevRect.nodeType === 1) {
					prevRect = _getRect(prevRect);
				}

				// Check if actually moving position
				if ((prevRect.left + prevRect.width / 2) !== (currentRect.left + currentRect.width / 2)
					|| (prevRect.top + prevRect.height / 2) !== (currentRect.top + currentRect.height / 2)
				) {
					var matrix = _matrix(this.el),
						scaleX = matrix && matrix.a,
						scaleY = matrix && matrix.d;

					_css(target, 'transition', 'none');
					_css(target, 'transform', 'translate3d('
						+ (prevRect.left - currentRect.left) / (scaleX ? scaleX : 1) + 'px,'
						+ (prevRect.top - currentRect.top) / (scaleY ? scaleY : 1) + 'px,0)'
					);

					forRepaintDummy = target.offsetWidth; // repaint
					_css(target, 'transition', 'transform ' + ms + 'ms' + (this.options.easing ? ' ' + this.options.easing : ''));
					_css(target, 'transform', 'translate3d(0,0,0)');
				}

				(typeof target.animated === 'number') && clearTimeout(target.animated);
				target.animated = setTimeout(function () {
					_css(target, 'transition', '');
					_css(target, 'transform', '');
					target.animated = false;
				}, ms);
			}
		},

		_offUpEvents: function () {
			var ownerDocument = this.el.ownerDocument;

			_off(document, 'touchmove', _preventScroll);
			_off(document, 'pointermove', _preventScroll);
			_off(document, 'touchmove', this._onTouchMove);
			_off(document, 'pointermove', this._onTouchMove);
			_off(ownerDocument, 'mouseup', this._onDrop);
			_off(ownerDocument, 'touchend', this._onDrop);
			_off(ownerDocument, 'pointerup', this._onDrop);
			_off(ownerDocument, 'touchcancel', this._onDrop);
			_off(ownerDocument, 'pointercancel', this._onDrop);
			_off(document, 'selectstart', this);
		},

		_onDrop: function (/**Event*/evt) {
			var el = this.el,
				options = this.options;

			awaitingDragStarted = false;
			scrolling = false;
			isCircumstantialInvert = false;
			pastFirstInvertThresh = false;

			clearInterval(this._loopId);

			clearInterval(pointerElemChangedInterval);
			_clearAutoScrolls();
			_cancelThrottle();

			clearTimeout(this._dragStartTimer);

			_cancelNextTick(this._cloneId);
			_cancelNextTick(this._dragStartId);

			// Unbind events
			_off(document, 'mousemove', this._onTouchMove);


			if (this.nativeDraggable) {
				_off(document, 'drop', this);
				_off(el, 'dragstart', this._onDragStart);
				_off(document, 'dragover', this._handleAutoScroll);
				_off(document, 'dragover', _checkAlignment);
			}

			this._offUpEvents();

			if (evt) {
				if (moved) {
					evt.cancelable && evt.preventDefault();
					!options.dropBubble && evt.stopPropagation();
				}

				ghostEl && ghostEl.parentNode && ghostEl.parentNode.removeChild(ghostEl);

				if (rootEl === parentEl || (putSortable && putSortable.lastPutMode !== 'clone')) {
					// Remove clone
					cloneEl && cloneEl.parentNode && cloneEl.parentNode.removeChild(cloneEl);
				}

				if (dragEl) {
					if (this.nativeDraggable) {
						_off(dragEl, 'dragend', this);
					}

					_disableDraggable(dragEl);
					dragEl.style['will-change'] = '';

					// Remove class's
					_toggleClass(dragEl, putSortable ? putSortable.options.ghostClass : this.options.ghostClass, false);
					_toggleClass(dragEl, this.options.chosenClass, false);

					// Drag stop event
					_dispatchEvent(this, rootEl, 'unchoose', dragEl, parentEl, rootEl, oldIndex, null, evt);

					if (rootEl !== parentEl) {
						newIndex = _index(dragEl, options.draggable);

						if (newIndex >= 0) {
							// Add event
							_dispatchEvent(null, parentEl, 'add', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

							// Remove event
							_dispatchEvent(this, rootEl, 'remove', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

							// drag from one list and drop into another
							_dispatchEvent(null, parentEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
							_dispatchEvent(this, rootEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
						}

						putSortable && putSortable.save();
					}
					else {
						if (dragEl.nextSibling !== nextEl) {
							// Get the index of the dragged element within its parent
							newIndex = _index(dragEl, options.draggable);

							if (newIndex >= 0) {
								// drag & drop within the same list
								_dispatchEvent(this, rootEl, 'update', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
								_dispatchEvent(this, rootEl, 'sort', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);
							}
						}
					}

					if (Sortable.active) {
						/* jshint eqnull:true */
						if (newIndex == null || newIndex === -1) {
							newIndex = oldIndex;
						}

						_dispatchEvent(this, rootEl, 'end', dragEl, parentEl, rootEl, oldIndex, newIndex, evt);

						// Save sorting
						this.save();
					}
				}

			}
			this._nulling();
		},

		_nulling: function() {
			rootEl =
			dragEl =
			parentEl =
			ghostEl =
			nextEl =
			cloneEl =
			lastDownEl =

			scrollEl =
			scrollParentEl =
			autoScrolls.length =

			pointerElemChangedInterval =
			lastPointerElemX =
			lastPointerElemY =

			tapEvt =
			touchEvt =

			moved =
			newIndex =
			oldIndex =

			lastTarget =
			lastDirection =

			forRepaintDummy =
			realDragElRect =

			putSortable =
			activeGroup =
			Sortable.active = null;

			savedInputChecked.forEach(function (el) {
				el.checked = true;
			});

			savedInputChecked.length = 0;
		},

		handleEvent: function (/**Event*/evt) {
			switch (evt.type) {
				case 'drop':
				case 'dragend':
					this._onDrop(evt);
					break;

				case 'dragenter':
				case 'dragover':
					if (dragEl) {
						this._onDragOver(evt);
						_globalDragOver(evt);
					}
					break;

				case 'selectstart':
					evt.preventDefault();
					break;
			}
		},


		/**
		 * Serializes the item into an array of string.
		 * @returns {String[]}
		 */
		toArray: function () {
			var order = [],
				el,
				children = this.el.children,
				i = 0,
				n = children.length,
				options = this.options;

			for (; i < n; i++) {
				el = children[i];
				if (_closest(el, options.draggable, this.el, false)) {
					order.push(el.getAttribute(options.dataIdAttr) || _generateId(el));
				}
			}

			return order;
		},


		/**
		 * Sorts the elements according to the array.
		 * @param  {String[]}  order  order of the items
		 */
		sort: function (order) {
			var items = {}, rootEl = this.el;

			this.toArray().forEach(function (id, i) {
				var el = rootEl.children[i];

				if (_closest(el, this.options.draggable, rootEl, false)) {
					items[id] = el;
				}
			}, this);

			order.forEach(function (id) {
				if (items[id]) {
					rootEl.removeChild(items[id]);
					rootEl.appendChild(items[id]);
				}
			});
		},


		/**
		 * Save the current sorting
		 */
		save: function () {
			var store = this.options.store;
			store && store.set && store.set(this);
		},


		/**
		 * For each element in the set, get the first element that matches the selector by testing the element itself and traversing up through its ancestors in the DOM tree.
		 * @param   {HTMLElement}  el
		 * @param   {String}       [selector]  default: `options.draggable`
		 * @returns {HTMLElement|null}
		 */
		closest: function (el, selector) {
			return _closest(el, selector || this.options.draggable, this.el, false);
		},


		/**
		 * Set/get option
		 * @param   {string} name
		 * @param   {*}      [value]
		 * @returns {*}
		 */
		option: function (name, value) {
			var options = this.options;

			if (value === void 0) {
				return options[name];
			} else {
				options[name] = value;

				if (name === 'group') {
					_prepareGroup(options);
				}
			}
		},


		/**
		 * Destroy
		 */
		destroy: function () {
			var el = this.el;

			el[expando] = null;

			_off(el, 'mousedown', this._onTapStart);
			_off(el, 'touchstart', this._onTapStart);
			_off(el, 'pointerdown', this._onTapStart);

			if (this.nativeDraggable) {
				_off(el, 'dragover', this);
				_off(el, 'dragenter', this);
			}
			// Remove draggable attributes
			Array.prototype.forEach.call(el.querySelectorAll('[draggable]'), function (el) {
				el.removeAttribute('draggable');
			});

			this._onDrop();

			sortables.splice(sortables.indexOf(this.el), 1);

			this.el = el = null;
		},

		_hideClone: function() {
			if (!cloneEl.cloneHidden) {
				_css(cloneEl, 'display', 'none');
				cloneEl.cloneHidden = true;
				if (cloneEl.parentNode && this.options.removeCloneOnHide) {
					cloneEl.parentNode.removeChild(cloneEl);
				}
			}
		},

		_showClone: function(putSortable) {
			if (putSortable.lastPutMode !== 'clone') {
				this._hideClone();
				return;
			}

			if (cloneEl.cloneHidden) {
				// show clone at dragEl or original position
				if (rootEl.contains(dragEl) && !this.options.group.revertClone) {
					rootEl.insertBefore(cloneEl, dragEl);
				} else if (nextEl) {
					rootEl.insertBefore(cloneEl, nextEl);
				} else {
					rootEl.appendChild(cloneEl);
				}

				if (this.options.group.revertClone) {
					this._animate(dragEl, cloneEl);
				}
				_css(cloneEl, 'display', '');
				cloneEl.cloneHidden = false;
			}
		}
	};

	function _closest(/**HTMLElement*/el, /**String*/selector, /**HTMLElement*/ctx, includeCTX) {
		if (el) {
			ctx = ctx || document;

			do {
				if ((selector === '>*' && el.parentNode === ctx) || _matches(el, selector) || (includeCTX && el === ctx)) {
					return el;
				}

				if (el === ctx) break;
				/* jshint boss:true */
			} while (el = _getParentOrHost(el));
		}

		return null;
	}


	function _getParentOrHost(el) {
		return (el.host && el !== document && el.host.nodeType)
			? el.host
			: el.parentNode;
	}


	function _globalDragOver(/**Event*/evt) {
		if (evt.dataTransfer) {
			evt.dataTransfer.dropEffect = 'move';
		}
		evt.cancelable && evt.preventDefault();
	}


	function _on(el, event, fn) {
		el.addEventListener(event, fn, captureMode);
	}


	function _off(el, event, fn) {
		el.removeEventListener(event, fn, captureMode);
	}


	function _toggleClass(el, name, state) {
		if (el && name) {
			if (el.classList) {
				el.classList[state ? 'add' : 'remove'](name);
			}
			else {
				var className = (' ' + el.className + ' ').replace(R_SPACE, ' ').replace(' ' + name + ' ', ' ');
				el.className = (className + (state ? ' ' + name : '')).replace(R_SPACE, ' ');
			}
		}
	}


	function _css(el, prop, val) {
		var style = el && el.style;

		if (style) {
			if (val === void 0) {
				if (document.defaultView && document.defaultView.getComputedStyle) {
					val = document.defaultView.getComputedStyle(el, '');
				}
				else if (el.currentStyle) {
					val = el.currentStyle;
				}

				return prop === void 0 ? val : val[prop];
			}
			else {
				if (!(prop in style) && prop.indexOf('webkit') === -1) {
					prop = '-webkit-' + prop;
				}

				style[prop] = val + (typeof val === 'string' ? '' : 'px');
			}
		}
	}

	function _matrix(el) {
		var appliedTransforms = '';
		do {
			var transform = _css(el, 'transform');

			if (transform && transform !== 'none') {
				appliedTransforms = transform + ' ' + appliedTransforms;
			}
			/* jshint boss:true */
		} while (el = el.parentNode);

		if (window.DOMMatrix) {
			return new DOMMatrix(appliedTransforms);
		} else if (window.WebKitCSSMatrix) {
			return new WebKitCSSMatrix(appliedTransforms);
		} else if (window.CSSMatrix) {
			return new CSSMatrix(appliedTransforms);
		}
	}


	function _find(ctx, tagName, iterator) {
		if (ctx) {
			var list = ctx.getElementsByTagName(tagName), i = 0, n = list.length;

			if (iterator) {
				for (; i < n; i++) {
					iterator(list[i], i);
				}
			}

			return list;
		}

		return [];
	}



	function _dispatchEvent(sortable, rootEl, name, targetEl, toEl, fromEl, startIndex, newIndex, originalEvt) {
		sortable = (sortable || rootEl[expando]);
		var evt,
			options = sortable.options,
			onName = 'on' + name.charAt(0).toUpperCase() + name.substr(1);
		// Support for new CustomEvent feature
		if (window.CustomEvent && !IE11OrLess && !Edge) {
			evt = new CustomEvent(name, {
				bubbles: true,
				cancelable: true
			});
		} else {
			evt = document.createEvent('Event');
			evt.initEvent(name, true, true);
		}

		evt.to = toEl || rootEl;
		evt.from = fromEl || rootEl;
		evt.item = targetEl || rootEl;
		evt.clone = cloneEl;

		evt.oldIndex = startIndex;
		evt.newIndex = newIndex;

		evt.originalEvent = originalEvt;

		if (rootEl) {
			rootEl.dispatchEvent(evt);
	        }

		if (options[onName]) {
			options[onName].call(sortable, evt);
		}
	}


	function _onMove(fromEl, toEl, dragEl, dragRect, targetEl, targetRect, originalEvt, willInsertAfter) {
		var evt,
			sortable = fromEl[expando],
			onMoveFn = sortable.options.onMove,
			retVal;
		// Support for new CustomEvent feature
		if (window.CustomEvent && !IE11OrLess && !Edge) {
			evt = new CustomEvent('move', {
				bubbles: true,
				cancelable: true
			});
		} else {
			evt = document.createEvent('Event');
			evt.initEvent('move', true, true);
		}

		evt.to = toEl;
		evt.from = fromEl;
		evt.dragged = dragEl;
		evt.draggedRect = dragRect;
		evt.related = targetEl || toEl;
		evt.relatedRect = targetRect || _getRect(toEl);
		evt.willInsertAfter = willInsertAfter;

		evt.originalEvent = originalEvt;

		fromEl.dispatchEvent(evt);

		if (onMoveFn) {
			retVal = onMoveFn.call(sortable, evt, originalEvt);
		}

		return retVal;
	}

	function _disableDraggable(el) {
		el.draggable = false;
	}

	function _unsilent() {
		_silent = false;
	}

	/**
	 * Gets nth child of el, ignoring hidden children, sortable's elements (does not ignore clone if it's visible)
	 * and non-draggable elements
	 * @param  {HTMLElement} el       The parent element
	 * @param  {Number} childNum      The index of the child
	 * @param  {Object} options       Parent Sortable's options
	 * @return {HTMLElement}          The child at index childNum, or null if not found
	 */
	function _getChild(el, childNum, options) {
		var currentChild = 0,
			i = 0,
			children = el.children;

		while (i < children.length) {
			if (
				children[i].style.display !== 'none' &&
				children[i] !== ghostEl &&
				children[i] !== dragEl &&
				_closest(children[i], options.draggable, el, false)
			) {
				if (currentChild === childNum) {
					return children[i];
				}
				currentChild++;
			}

			i++;
		}
		return null;
	}

	/**
	 * Gets the last child in the el, ignoring ghostEl
	 * @param  {HTMLElement} el       Parent element
	 * @return {HTMLElement}          The last child, ignoring ghostEl
	 */
	function _lastChild(el) {
		var last = el.lastElementChild;

		if (last === ghostEl) {
			last = el.children[el.childElementCount - 2];
		}

		return last || null;
	}

	function _ghostIsLast(evt, axis, el) {
		var elRect = _getRect(_lastChild(el)),
			mouseOnAxis = axis === 'vertical' ? evt.clientY : evt.clientX,
			mouseOnOppAxis = axis === 'vertical' ? evt.clientX : evt.clientY,
			targetS2 = axis === 'vertical' ? elRect.bottom : elRect.right,
			targetS1Opp = axis === 'vertical' ? elRect.left : elRect.top,
			targetS2Opp = axis === 'vertical' ? elRect.right : elRect.bottom;

		return (
			mouseOnOppAxis > targetS1Opp &&
			mouseOnOppAxis < targetS2Opp &&
			mouseOnAxis > targetS2
		);
	}

	function _getSwapDirection(evt, target, axis, swapThreshold, invertedSwapThreshold, invertSwap, isLastTarget) {
		var targetRect = _getRect(target),
			mouseOnAxis = axis === 'vertical' ? evt.clientY : evt.clientX,
			targetLength = axis === 'vertical' ? targetRect.height : targetRect.width,
			targetS1 = axis === 'vertical' ? targetRect.top : targetRect.left,
			targetS2 = axis === 'vertical' ? targetRect.bottom : targetRect.right,
			dragRect = _getRect(dragEl),
			invert = false;


		if (!invertSwap) {
			// Never invert or create dragEl shadow when target movemenet causes mouse to move past the end of regular swapThreshold
			if (isLastTarget && targetMoveDistance < targetLength * swapThreshold) { // multiplied only by swapThreshold because mouse will already be inside target by (1 - threshold) * targetLength / 2
				// check if past first invert threshold on side opposite of lastDirection
				if (!pastFirstInvertThresh &&
					(lastDirection === 1 ?
						(
							mouseOnAxis > targetS1 + targetLength * invertedSwapThreshold / 2
						) :
						(
							mouseOnAxis < targetS2 - targetLength * invertedSwapThreshold / 2
						)
					)
				)
				{
					// past first invert threshold, do not restrict inverted threshold to dragEl shadow
					pastFirstInvertThresh = true;
				}

				if (!pastFirstInvertThresh) {
					var dragS1 = axis === 'vertical' ? dragRect.top : dragRect.left,
						dragS2 = axis === 'vertical' ? dragRect.bottom : dragRect.right;
					// dragEl shadow (target move distance shadow)
					if (
						lastDirection === 1 ?
						(
							mouseOnAxis < targetS1 + targetMoveDistance // over dragEl shadow
						) :
						(
							mouseOnAxis > targetS2 - targetMoveDistance
						)
					)
					{
						return lastDirection * -1;
					}
				} else {
					invert = true;
				}
			} else {
				// Regular
				if (
					mouseOnAxis > targetS1 + (targetLength * (1 - swapThreshold) / 2) &&
					mouseOnAxis < targetS2 - (targetLength * (1 - swapThreshold) / 2)
				) {
					return ((mouseOnAxis > targetS1 + targetLength / 2) ? -1 : 1);
				}
			}
		}

		invert = invert || invertSwap;

		if (invert) {
			// Invert of regular
			if (
				mouseOnAxis < targetS1 + (targetLength * invertedSwapThreshold / 2) ||
				mouseOnAxis > targetS2 - (targetLength * invertedSwapThreshold / 2)
			)
			{
				return ((mouseOnAxis > targetS1 + targetLength / 2) ? 1 : -1);
			}
		}

		return 0;
	}

	/**
	 * Gets the direction dragEl must be swapped relative to target in order to make it
	 * seem that dragEl has been "inserted" into that element's position
	 * @param  {HTMLElement} target       The target whose position dragEl is being inserted at
	 * @param  {Object} options           options of the parent sortable
	 * @return {Number}                   Direction dragEl must be swapped
	 */
	function _getInsertDirection(target, options) {
		var dragElIndex = _index(dragEl, options.draggable),
			targetIndex = _index(target, options.draggable);

		if (dragElIndex < targetIndex) {
			return 1;
		} else {
			return -1;
		}
	}


	/**
	 * Generate id
	 * @param   {HTMLElement} el
	 * @returns {String}
	 * @private
	 */
	function _generateId(el) {
		var str = el.tagName + el.className + el.src + el.href + el.textContent,
			i = str.length,
			sum = 0;

		while (i--) {
			sum += str.charCodeAt(i);
		}

		return sum.toString(36);
	}

	/**
	 * Returns the index of an element within its parent for a selected set of
	 * elements
	 * @param  {HTMLElement} el
	 * @param  {selector} selector
	 * @return {number}
	 */
	function _index(el, selector) {
		var index = 0;

		if (!el || !el.parentNode) {
			return -1;
		}

		while (el && (el = el.previousElementSibling)) {
			if ((el.nodeName.toUpperCase() !== 'TEMPLATE') && el !== cloneEl) {
				index++;
			}
		}

		return index;
	}

	function _matches(/**HTMLElement*/el, /**String*/selector) {
		if (el) {
			try {
				if (el.matches) {
					return el.matches(selector);
				} else if (el.msMatchesSelector) {
					return el.msMatchesSelector(selector);
				} else if (el.webkitMatchesSelector) {
					return el.webkitMatchesSelector(selector);
				}
			} catch(_) {
				return false;
			}
		}

		return false;
	}

	var _throttleTimeout;
	function _throttle(callback, ms) {
		return function () {
			if (!_throttleTimeout) {
				var args = arguments,
					_this = this;

				_throttleTimeout = setTimeout(function () {
					if (args.length === 1) {
						callback.call(_this, args[0]);
					} else {
						callback.apply(_this, args);
					}

					_throttleTimeout = void 0;
				}, ms);
			}
		};
	}

	function _cancelThrottle() {
		clearTimeout(_throttleTimeout);
		_throttleTimeout = void 0;
	}

	function _extend(dst, src) {
		if (dst && src) {
			for (var key in src) {
				if (src.hasOwnProperty(key)) {
					dst[key] = src[key];
				}
			}
		}

		return dst;
	}

	function _clone(el) {
		if (Polymer && Polymer.dom) {
			return Polymer.dom(el).cloneNode(true);
		}
		else if ($) {
			return $(el).clone(true)[0];
		}
		else {
			return el.cloneNode(true);
		}
	}

	function _saveInputCheckedState(root) {
		savedInputChecked.length = 0;

		var inputs = root.getElementsByTagName('input');
		var idx = inputs.length;

		while (idx--) {
			var el = inputs[idx];
			el.checked && savedInputChecked.push(el);
		}
	}

	function _nextTick(fn) {
		return setTimeout(fn, 0);
	}

	function _cancelNextTick(id) {
		return clearTimeout(id);
	}


	function _preventScroll(evt) {
		if ((Sortable.active || awaitingDragStarted) && evt.cancelable) {
			evt.preventDefault();
		}
	}

	/**
	 * Returns the "bounding client rect" of given element
	 * @param  {HTMLElement} el                The element whose boundingClientRect is wanted
	 * @param  {[HTMLElement]} container       the parent the element will be placed in
	 * @param  {[Boolean]} adjustForTransform  Whether the rect should compensate for parent's transform
	 * (used for fixed positioning on el)
	 * @return {Object}                        The boundingClientRect of el
	 */
	function _getRect(el, container, adjustForTransform) {
		if (!el.getBoundingClientRect && el !== win) return;

		var elRect,
			top,
			left,
			bottom,
			right,
			height,
			width;

		if (el !== win) {
			elRect = el.getBoundingClientRect();
			top = elRect.top;
			left = elRect.left;
			bottom = elRect.bottom;
			right = elRect.right;
			height = elRect.height;
			width = elRect.width;
		} else {
			top = 0;
			left = 0;
			bottom = window.innerHeight;
			right = window.innerWidth;
			height = window.innerHeight;
			width = window.innerWidth;
		}

		if (adjustForTransform && el !== win) {
			// Adjust for translate()
			container = container || el.parentNode;

			// solves #1123 (see: https://stackoverflow.com/a/37953806/6088312)
			// Not needed on <= IE11
			if (!IE11OrLess) {
				do {
					if (container && container.getBoundingClientRect && _css(container, 'transform') !== 'none') {
						var containerRect = container.getBoundingClientRect();

						// Set relative to edges of padding box of container
						top -= containerRect.top + parseInt(_css(container, 'border-top-width'));
						left -= containerRect.left + parseInt(_css(container, 'border-left-width'));
						bottom = top + elRect.height;
						right = left + elRect.width;

						break;
					}
					/* jshint boss:true */
				} while (container = container.parentNode);
			}

			// Adjust for scale()
			var matrix = _matrix(el),
				scaleX = matrix && matrix.a,
				scaleY = matrix && matrix.d;

			if (matrix) {
				top /= scaleY;
				left /= scaleX;

				width /= scaleX;
				height /= scaleY;

				bottom = top + height;
				right = left + width;
			}
		}

		return {
			top: top,
			left: left,
			bottom: bottom,
			right: right,
			width: width,
			height: height
		};
	}


	/**
	 * Checks if a side of an element is scrolled past a side of it's parents
	 * @param  {HTMLElement}  el       The element who's side being scrolled out of view is in question
	 * @param  {String}       side     Side of the element in question ('top', 'left', 'right', 'bottom')
	 * @return {Boolean}               Whether the element is overflowing the viewport on the given side of it's parent
	 */
	function _isScrolledPast(el, side) {
		var parent = _getParentAutoScrollElement(parent, true),
			elSide = _getRect(el)[side];

		/* jshint boss:true */
		while (parent) {
			var parentSide = _getRect(parent)[side],
				visible;

			if (side === 'top' || side === 'left') {
				visible = elSide >= parentSide;
			} else {
				visible = elSide <= parentSide;
			}

			if (!visible) return true;

			if (parent === win) break;

			parent = _getParentAutoScrollElement(parent, false);
		}

		return false;
	}


	// Export utils
	Sortable.utils = {
		on: _on,
		off: _off,
		css: _css,
		find: _find,
		is: function (el, selector) {
			return !!_closest(el, selector, el, false);
		},
		extend: _extend,
		throttle: _throttle,
		closest: _closest,
		toggleClass: _toggleClass,
		clone: _clone,
		index: _index,
		nextTick: _nextTick,
		cancelNextTick: _cancelNextTick,
		detectDirection: _detectDirection,
		getChild: _getChild
	};


	/**
	 * Create sortable instance
	 * @param {HTMLElement}  el
	 * @param {Object}      [options]
	 */
	Sortable.create = function (el, options) {
		return new Sortable(el, options);
	};


	// Export
	Sortable.version = '1.8.0';
	return Sortable;
});


// FILE: src/config/config.js
(function () {

	window.templateUrls = window.templateUrls || [];
	var packageName = 'zh-grid-configuration';

	var hasPackageRegistered = window.templateUrls.some(function (templateUrl) {
		return templateUrl.package === packageName;
	});

	if (!hasPackageRegistered) {
		window.templateUrls.push({
			'package': packageName,
			'baseUrl': './bower_components/' + packageName + '/dist/templates/'
		});
	}
})();

// FILE: src/controllers/ZhGridConfigController.js
function ZhGridConfigController($scope, $timeout, $filter, ApplicationContext, ScreenService, metaDataFactory) {
	var self = this;
	var widget = $scope.widget;
	var widgetShowCheckbox = widget.showCheckbox;
	var GRID_MODE = 'grid';
	var REPORT_MODE = 'report';

	widget.reportShowOnlyVisibleFields = false;
	widget._selectedColumn = undefined;

	self.configMode = GRID_MODE;

	self.inGridMode = function(){
		return self.configMode == GRID_MODE;
	};
	$scope.inGridMode = self.inGridMode;

	self.inReportMode = function(){
		return self.configMode == REPORT_MODE;
	};
	$scope.inReportMode = self.inReportMode;

	self.setConfigMode = function(mode) {
		self.configMode = mode;
		widget.showCheckbox = self.inGridMode() && widgetShowCheckbox;
		widget._selectedColumn = undefined;

		$timeout(function() {
			$scope.updateFields(false);
		});
	};

	$scope.getConfigMode = function() {
		return self.configMode;
	};

	self.setReportOrientation = function(orientation) {
		widget.reportOrientation = orientation;

		$timeout(function() {
			$scope.calcColumnSize();
			widget.resizeGridColumns(true);
		});
	};

	self.getPageInfo = function() {
		return ScreenService.i18n("Page")+": 1<br />"+(new Date()).toLocaleString();
	};

	$scope.selectColumn = function(column) {
		if (widget._selectedColumn == column) {
			widget._selectedColumn = undefined;
		} else {
			widget._selectedColumn = column;
			if (Util.isUndefined(column.fixed)) {
				column.fixed = false;
			}

			$scope.updateFieldDetailsData(column);
		}
	};

	$scope.updateFieldDetailsData = function(column) {
		self.fieldDetailsWidget.dataSource.data[0] = {
			"LABEL": (self.inReportMode() && column.labelOnReport) || column.label,
			"ALIGNMENT": column.align,
			"EXPRESSION": column.expression
		};
		self.fieldDetailsWidget.moveToFirst();
		self.fieldDetailsWidget.getField("FIXED").setValue(column.fixed);
		self.fieldDetailsWidget.getField("OPENED").setValue(Util.isUndefinedOrNull(column.groupOpened) ? true : column.groupOpened);
		self.fieldDetailsWidget.getField("VISIBLE").setValue(self.inReportMode() ? column.isVisibleOnReport : column.showOnList);

		if (self.inReportMode()) {
			self.fieldDetailsWidget.getField("COLUMNSIZE").label = "Column Width (%)";
			if (!Util.isUndefinedOrNull(column.reportColumnSize)) {
				self.fieldDetailsWidget.getField("COLUMNSIZE").setValue(parseInt(column.reportColumnSize));
			}

			self.fieldDetailsWidget.getField("FIXED").isVisible = false;
		} else {
			self.fieldDetailsWidget.getField("COLUMNSIZE").label = "Column Width (px)";
			self.fieldDetailsWidget.getField("COLUMNSIZE").setValue(column.gridColumnSize);

			self.fieldDetailsWidget.getField("FIXED").isVisible = true;
		}

		self.fieldDetailsWidget.getField('ALIGNMENT').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('EXPRESSION').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('COLUMNSIZE').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('FIXED').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('VISIBLE').isVisible = !column.grouped;
		self.fieldDetailsWidget.getField('OPENED').isVisible = column.grouped && !self.inReportMode();
		self.fieldDetailsWidget.getField('LABEL').class = column.grouped && !self.inReportMode() ? 10 : 12;
	};

	$scope.calcColumnSize = function() {
		var totalWidth = $scope.getTotalWidth();

		widget._filteredFields.forEach(function(field) {
			field.minWidth = field.maxWidth = undefined;

			if (self.inReportMode()) {
				if (field.reportColumnSize) {
					field.minWidth = field.maxWidth = totalWidth * parseInt(field.reportColumnSize) / 100;
				}
			} else if (field.gridColumnSize) {
				field.minWidth = field.maxWidth = field.gridColumnSize;
			}
		});
	};

	$scope.calcColumnWidthProperties = function() {
		var totalWidth = $scope.getTotalWidth();

		widget._filteredFields.forEach(function(field) {
			if (field.minWidth) {
				if (self.inReportMode()) {
					field.reportColumnSize = Math.round(field.minWidth * 100 / totalWidth) + "%";
				} else {
					field.gridColumnSize = field.minWidth;
				}
			}
		});
	};

	$scope.updateFields = function(apply) {
		function _updateFields() {
			widget.updateFilteredFields();
			self.updateGroupedColumns();
			self.updateNotVisibleColumns();
			$scope.calcColumnSize();
		}

		if (apply !== false) {
			$scope.$apply(_updateFields);
		} else {
			_updateFields();
		}
	};

	widget.updateFilteredFields = function() {
		widget._filteredFields = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		widget._filteredFields = widget._filteredFields.filter(function(field) {
			return (self.inReportMode() ? field.isVisibleOnReport !== false : field.showOnList) && field.aclIsVisible && !field.grouped;
		});
	};

	self._groupedColumns = [];
	self.updateGroupedColumns = function() {
		self._groupedColumns = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		self._groupedColumns = self._groupedColumns.filter(function(field) {
			return field.grouped && field.aclIsVisible;
		});
	};

	self._notVisibleColumns = [];
	self.updateNotVisibleColumns = function() {
		self._notVisibleColumns = $filter('orderBy')(widget.fields, $scope.fieldOrder);
		self._notVisibleColumns = self._notVisibleColumns.filter(function(field) {
			return !~widget._filteredFields.indexOf(field) && !~self._groupedColumns.indexOf(field) && field.aclIsVisible;
		});
	};

	$timeout(function() { $scope.updateFields(false); });

	self.layoutNameWidget = metaDataFactory.widgetFactory({
		"id": "grid-config-layout-name",
		"name": "grid-config-layout-name",
		"template": "widget/form-structure.html",
		"container": widget.container,
		"isVisible": true,
		"popupNoBlock": false,
		"showDescriptionOrder": false,
		"floatingControl": false,
		"dataSource": { "data": [] },
		"actions": [],
		"events": [],
		"fields": [
			{
				"label": "Layout name",
				"name": "LABEL",
				"class": "12",
				"showOnForm": true,
				"template": "field/text-edit.html",
				"events": [],
				"validations": {
					"required": {}
				}
			}
		]
	});
	self.layoutNameWidget.dataSource.data.push({ LABEL: widget.layoutName });
	self.layoutNameWidget.currentRow = self.layoutNameWidget.dataSource.data[0];

	$scope.$watch(function() { return self.layoutNameWidget.dataSource.data[0].LABEL; }, function(newValue) {
		if (!Util.isUndefinedOrNull(newValue)) {
			widget.layoutName = newValue;
		}
	});

	self.fieldDetailsWidget = metaDataFactory.widgetFactory({
		"id": "grid-config-field-details",
		"name": "grid-config-field-details",
		"label": "Field Details",
		"template": 'widget/form-structure.html',
		"container": widget.container,
		"isVisible": true,
		"popupNoBlock": false,
		"showDescriptionOrder": false,
		"floatingControl": false,
		"fieldGroups": [{
			"name": "FIELDGROUP",
			"opened": true,
			"collapsible": false,
			"label": "Column properties"
		}],
		"fields": [{
			"label": "Label",
			"name": "LABEL",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 12,
			"template": "field/text-edit.html"
		},{
			"label": "Alignment",
			"name": "ALIGNMENT",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/select-native.html",
			"dataSource": {
				"data": [
					{"value": "left"},
					{"value": "center"},
					{"value": "right"}
				]
			},
			"valueField": "value",
			"hideEmptyValue": true,
			"descriptionField": "value"
		},{
			"label": "Group expression",
			"name": "EXPRESSION",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/select-native.html",
			"dataSource": {
				"data": [
					{"value": "SUM"},
					{"value": "AVG"},
					{"value": "COUNT"},
					{"value": "MAX"},
					{"value": "MIN"}
				]
			},
			"valueField": "value",
			"hideEmptyValue": false,
			"descriptionField": "value"
		},{
			"label": "Column Width (%)",
			"name": "COLUMNSIZE",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 3,
			"template": "field/number.html",
			"spin": true
		},{
			"label": "Fixed",
			"name": "FIXED",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 1,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		},{
			"label": "Visible",
			"name": "VISIBLE",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 2,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		},{
			"label": "Opened",
			"name": "OPENED",
			"showLabel": true,
			"showOnList": true,
			"showOnForm": true,
			"fieldGroup": "FIELDGROUP",
			"class": 2,
			"template": "field/checkbox.html",
			"valueToTrue": true,
			"valueToFalse": false,
		}],
		"dataSource": { "data": [] },
		"actions": [],
		"events": []
	});

	var lastSelectedColumn;
	$scope.$watch(function() { return self.fieldDetailsWidget.dataSource.data[0]; }, function(newValue) {
		if (newValue && widget._selectedColumn && lastSelectedColumn == widget._selectedColumn) {
			var forceResize = false;

			if (self.inReportMode()) {
				widget._selectedColumn.labelOnReport = newValue.LABEL;

				widget._selectedColumn.reportColumnSize = undefined;
				if (!Util.isUndefinedOrNull(newValue.COLUMNSIZE) && widget._selectedColumn.reportColumnSize != newValue.COLUMNSIZE+"%") {
					widget._selectedColumn.reportColumnSize = newValue.COLUMNSIZE+"%";

					$scope.calcColumnSize();
					$scope.setFieldWidthFromElementWidth();
					$scope.calcColumnWidthProperties();
					forceResize = true;
				}

				if (widget._selectedColumn.isVisibleOnReport != newValue.VISIBLE) {
					widget._selectedColumn.isVisibleOnReport = newValue.VISIBLE;

					widget._selectedColumn = undefined;
					widget.updateFilteredFields();
					self.updateNotVisibleColumns();
					$scope.removeInvisibleColumns();
					forceResize = true;
				}
			} else {
				widget._selectedColumn.label = newValue.LABEL;

				widget._selectedColumn.gridColumnSize = undefined;
				if (!Util.isUndefinedOrNull(newValue.COLUMNSIZE) && widget._selectedColumn.gridColumnSize != newValue.COLUMNSIZE) {
					widget._selectedColumn.gridColumnSize = newValue.COLUMNSIZE;

					$scope.calcColumnSize();
					$scope.setFieldWidthFromElementWidth();
					$scope.calcColumnWidthProperties();
					forceResize = true;
				}

				if (widget._selectedColumn.showOnList != newValue.VISIBLE) {
					widget._selectedColumn.showOnList = newValue.VISIBLE;

					widget._selectedColumn = undefined;
					widget.updateFilteredFields();
					self.updateNotVisibleColumns();
					$scope.removeInvisibleColumns();
					forceResize = true;
				}
			}

			if (widget._selectedColumn) {
				widget._selectedColumn.align = newValue.ALIGNMENT;
				widget._selectedColumn.expression = newValue.EXPRESSION;
				widget._selectedColumn.groupOpened = newValue.OPENED;

				if (widget._selectedColumn.fixed != newValue.FIXED) {
					widget._selectedColumn.fixed = newValue.FIXED;

					widget.updateFilteredFields();
					var fixed = true;
					widget._filteredFields.forEach(function(field, idx) {
						field.sequenceInList = idx;
						fixed = fixed && field.fixed;
						field.fixed = fixed;
					});

					forceResize = true;
				}
			}

			if (forceResize) {
				widget.resizeGridColumns(true);
			}
		}
		lastSelectedColumn = widget._selectedColumn;
	}, true);

	self.previewPDF = function() {
		ApplicationContext.ZhReports.openReport(undefined, widget, undefined, undefined, 'PDF');
	};

	self.openSearch = function() {
		self.searchOpened = !self.searchOpened;
		if (self.searchOpened) {
			$('#zh-grid-config-not-visible-search input').focus();
		}
	};

	self.clearSearch = function() {
		delete self.notVisibleSearch;
	};

	self.getNotVisibleColumns = function() {
		return self._notVisibleColumns.filter(function(item) {
			return !self.notVisibleSearch || item.label.indexOfLatin(self.notVisibleSearch) > -1;
		}).sort(function(a, b) {
			a.label = a.label || "";
			b.label = b.label || "";
			var labelA = a.label.toUpperCase();
			var labelB = b.label.toUpperCase();
			if (labelA < labelB) {
				return -1;
			}
			if (labelA > labelB) {
				return 1;
			}
			return 0;
		});
	};

	widget.gridConfigWidget = self.layoutNameWidget;
}


// FILE: src/directives/ZhGridConfigDirective.js

(function () {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhGridConfig', zhGridConfig);

	function zhGridConfig(ApplicationContext) {
		return {
			restrict: 'A',
			link: function($scope, element) {
				var widget = $scope.widget;

				if (Util.isUndefinedOrNull(ApplicationContext.ZhReports)) {
					element.find('span.config-buttons').hide();
				}

				$scope.getTotalWidth = function() {
					return element.find('.header-content').outerWidth() - element.find('.header-content .th.grid-checkbox-column').outerWidth() - 10;
				};

				element.find('.body-content').off('mousewheel wheel DOMMouseScroll');

				var groupedColumns = element.find('.zh-grid-config-groups .sortable-columns')[0];
				var visibleColumns = element.find('.zh-grid-container .sortable-columns')[0];
				var notVisibleColumns = element.find('.zh-grid-config-not-visible .sortable-columns')[0];

				$scope.onStartSortable = function(evt) {
					var el = $(evt.item);
					var fieldName = el.attr('data-zh-field-name');
					var field = widget.getField(fieldName);

					if (field == widget._selectedColumn) {
						delete widget._selectedColumn;
					}
				};

				$scope.onAddSortable = function(evt) {
					var fieldName = $(evt.item).attr('data-zh-field-name');
					var field = widget.getField(fieldName);
					if (field) {
						if ($(evt.to).parent().hasClass('zh-grid-config-groups')) {
							field.showOnList = false;
							field.grouped = true;
							field.groupOpened = Util.isUndefinedOrNull(field.groupOpened) ? true : field.groupOpened;
						} else if ($(evt.to).parent().hasClass('zh-grid-config-not-visible')) {
							if ($scope.inReportMode()) {
								field.isVisibleOnReport = false;
							} else {
								field.showOnList = false;
							}
							field.grouped = false;
						} else {
							if ($scope.inReportMode()) {
								field.isVisibleOnReport = true;
							} else {
								field.showOnList = true;
							}
							field.isVisible = true;
							field.grouped = false;
						}
					}

					$scope.onUpdateSortable(evt);
				};

				$scope.onUpdateSortable = function(evt) {
					var fixed = true;
					$(evt.to).parent().find('.th').each(function(idx, item) {
						var fieldName = $(item).attr('data-zh-field-name');
						var field = widget.getField(fieldName);
						if (field) {
							if (!$(evt.to).parent().hasClass('zh-grid-config-not-visible')) {
								field.sequenceInList = idx;
							}

							if ($(evt.to).parent().hasClass('tr')) {
								fixed = fixed && field.fixed;
								field.fixed = fixed;
							}
						}
					});

					if (evt.type == "add") {
						$(evt.item).remove();
					}

					$scope.updateFields();
				};

				var sortableOptions = {
					group: "sortableList",
					onUpdate: $scope.onUpdateSortable,
					onAdd: $scope.onAddSortable,
					onStart: $scope.onStartSortable,
					animation: 100,
					handle: ".label-text",
					scroll: element.find('.body-scroll-control')[0]
				};

				var sortableItems = [];
				[groupedColumns, visibleColumns, notVisibleColumns].forEach(function(el) {
					sortableItems.push(Sortable.create(el, sortableOptions));
				});

				$scope.$on('$destroy', function() {
					sortableItems.forEach(function(item) {
						item.destroy();
					});
					sortableItems = [];

					delete $scope.onUpdateSortable;
					delete $scope.onAddSortable;
					delete $scope.onStartSortable;
				});

				element.on('touchend touchcancel mouseup pointercancel', function() {
					delete element[0]._tapEvt;
				});

				$scope.setFieldWidthFromElementWidth = function(force) {
					element.find('.header-content .sortable-columns .th').each(function(idx, item) {
						item = $(item);
						var itemField = widget.getField(item.attr('data-zh-field-name'));
						if (Util.isUndefinedOrNull(itemField.minWidth) || force) {
							itemField.minWidth = item.width();
							itemField.maxWidth = item.outerWidth();
						}
					});
				};

				element.on('touchmove mousemove', function(evt) {
					if (element[0]._tapEvt) {
						var tapElement = $(element[0]._tapEvt.target);
						var thElement = tapElement.closest('.th');
						var deltaX = evt.clientX - element[0]._tapEvt.clientX;

						if (deltaX === 0) {
							return;
						}

						if (thElement.offset().left + thElement.outerWidth() + deltaX > thElement.offsetParent().offset().left + thElement.offsetParent().outerWidth()) {
							return;
						}

						var field = widget.getField(thElement.attr('data-zh-field-name'));
						var width = thElement.width();
						var outerWidth = thElement.outerWidth();

						$scope.setFieldWidthFromElementWidth(true);

						if (tapElement.hasClass('resize-handle-left')) {
							if (thElement.prev().length) {
								thElement.width(width - deltaX);

								if (field) {
									field.minWidth = field.maxWidth = outerWidth - deltaX;
								}

								var prevElement = thElement.prev();
								if (prevElement) {
									var prevWidth = prevElement.width();
									var prevOuterWidth = prevElement.outerWidth();

									prevElement.width(prevWidth + deltaX);

									var prevField = widget.getField(prevElement.attr('data-zh-field-name'));
									if (prevField) {
										prevField.minWidth = prevField.maxWidth = prevOuterWidth + deltaX;
									}
								}
							}
						} else {
							thElement.width(width + deltaX);

							if (field) {
								field.minWidth = width + deltaX;
								field.maxWidth = outerWidth + deltaX;
							}
						}

						element[0]._tapEvt.clientX += deltaX;

						$scope.calcColumnWidthProperties();
						$scope.updateFieldDetailsData(field);

						widget.resizeGridColumns(true);
					}
				});

				$scope.removeInvisibleColumns = function() {
					var thItems = element.find('.header-content .sortable-columns .th');
					thItems.each(function(idx, th) {
						th = $(th);
						var fieldName = th.attr('data-zh-field-name');
						var field = widget.getField(fieldName);
						if (!field || widget._filteredFields.indexOf(field) == -1) {
							th.remove();
						}
					});
				};
			}
		};
	}

	ZeedhiDirectives.directive('zhGridConfigResizeHandle', zhGridConfigResizeHandle);

	function zhGridConfigResizeHandle() {
		return {
			restrict: 'A',
			link: function($scope, element) {
				var widget = $scope.widget;

				element.on('touchStart mousedown', function(evt) {
					element.closest('.zh-grid-config-container')[0]._tapEvt = evt;
				});

				element.on('touchend touchcancel mouseup pointercancel', function(evt) {
					delete element.closest('.zh-grid-config-container')[0]._tapEvt;
				});
			}
		};
	}
})();

// FILE: bower_components/zh-preferences/dist/preferences.js
// FILE: src/ZhPreferences.js
// FILE: src/ZhPreferences.js
/**
 * Service to handle preferences storage
 *
 * @class
 * @param {RepositoryFactory} RepositoryFactory Service to factory repositories
 * @param {RequestEngine}     requestEngine     Service to make http requests
 * @param {RequestFactory}    requestFactory    Service to factory requests
 * @param {ZHPromise}         ZHPromise         Service to manipulate promises
 * @param {Query}             Query             Service to build queries
 */
function ZhPreferences(RepositoryFactory, requestEngine, requestFactory, ZHPromise, Query, $q) {

    var repositoryMap = {};

    var userWidgetConfiguration = {};

    /**
     * Register new repository to save preferences
     *
     * @param  {String} name Repository name used to save a preference
     */
    this.registerRepository = function(name) {
        var repositoryName = name.split('#').pop();
        repositoryMap[repositoryName] = RepositoryFactory.factory(name, 'MEMORY');
    };


    /**
     * Persist preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to persist
     *
     * @return {Promise}                 Preferences saved
     */
    this.savePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var returnedIds = response.dataset[preferenceType];
            return preferences.map(function(preference, index) {
                preference.__is_new = false;
                preference.changed = false;
                preference.ID = returnedIds[index].ID;
                return preference;
            });
        }).then(function(preferences) {
            return repositoryMap[preferenceType].save(preferences);
        });
    };

    /**
     * Delete preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to delete
     *
     * @return {Promise}                 Preferences deleted
     */
    this.deletePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var deletedIds = response.dataset[preferenceType].map(function(preference) {
                return preference.ID;
            });
            var query = Query.build().where('ID').in(deletedIds);

            return repositoryMap[preferenceType].remove(query);
        });
    };

    /**
     * Get local saved preferences from repository
     *
     * @param  {String} preferenceType Repository name
     * @param  {Query}  query          Conditions to filter the preferences
     *
     * @return {Promise}               Array of preferences
     */
    this.getPreferences = function(preferenceType, query) {
        var filterRequest = requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType,
            'requestType': 'filterData',
            'filter': query.where()
        });
        return requestEngine.doRequest(filterRequest)
            .then(function(response) {
                var preferences = response.dataset[preferenceType];
                var removePreferences = repositoryMap[preferenceType].remove(query);
                var savePreferences = repositoryMap[preferenceType].save(preferences);
                return ZHPromise.all([removePreferences, savePreferences]).then(function(){
                    return preferences;
                });
            })
            .catch(function(response) {
                throw new Error(response.data.error);
            });
    };

    /**
     * Get local saved preferences from repository filtering by widget and logged user
     *
     * @param  {String} preferenceType Repository name
     * @param  {String} widgetID       Widget identified
     *
     * @return {Promise}               Array of preferences
     */
    this.getWidgetPreferences = function(preferenceType, widgetID) {
        var query = Query.build().where('WIDGET_ID').equals(widgetID);
        return this.getPreferences(preferenceType, query);
    };

    /**
     * Get default preferences for user on specified widget
     *
     * @param  {String} widgetID Widget to get default preferences
     *
     * @return {Promise}         Widget default preferences
     */
    this.getDefaultWidgetPreferences = function(widgetID) {
        if (userWidgetConfiguration[widgetID] === undefined) {
            var query = Query.build().where('WIDGET_ID').equals(widgetID);

            userWidgetConfiguration[widgetID] = requestEngine.doRequest(requestFactory.factory({
                'serviceName': 'zh-preferences#/default-preferences',
                'requestType': 'filterData',
                'filter': query.where()
            })).then(function(response) {
                return response.dataset.user_widget.pop() || {
                    '__is_new': true,
                    'ID': null,
                    'WIDGET_ID': widgetID,
                    'DEFAULT_FILTER': null,
                    'DEFAULT_LAYOUT': null,
                    'DEFAULT_VIEW': null,
                    'LAST_MODIFIED_DATE': null
                };
            });
        }

        return userWidgetConfiguration[widgetID];
    };

    /**
     * Change default preference for user on specified widget
     *
     * @param  {String}  configWidget        Preference configuration widget
     * @param  {String}  property            Property that contains the default preference ID
     * @param  {Integer} preference          Preference configuration widget changed row
     *
     * @return {Promise}                     Widget default preferences
     */
    this.changeDefaultPreferences = function(configWidget, property, preference){
        var widgetID = configWidget.listWidget.id;
        var preferenceID = preference[property] ? preference.ID : null;

        userWidgetConfiguration[widgetID] = this.getDefaultWidgetPreferences(widgetID)
            .then(function(defaultPreferences) {
                defaultPreferences[property] = preferenceID;
                return ZHPromise.all([
                    defaultPreferences,
                    requestEngine.doRequest(requestFactory.factory({
                        'serviceName': 'zh-preferences#/default-preferences/save',
                        'requestType': 'dataSource',
                        'dataSource': [defaultPreferences]
                    }))
                ]);
            }).then(function(args) {
                var defaultPreferences = args[0];
                var response = args[1];
                defaultPreferences.ID = Util.getNestedProperty(response, ['dataset', 'user_widget', '0', 'ID']);

                configWidget.dataSource.data.forEach(function(row){
                    row.__is_new = false;
                    row[property] = row.ID == defaultPreferences[property];
                });
                return defaultPreferences;
            });

        return userWidgetConfiguration[widgetID];
    };

    function isMasterDetail(template){
        return template.indexOf('master_detail/grid.html') !== -1 ||
               template.indexOf('master_detail/list.html') !== -1 ||
               template.indexOf('master_detail/tree/grid.html') !== -1;
    }

    function isMasterDetailView(widget){
        return isMasterDetail(widget.template) && widget.mode == ViewMode.VIEW;
    }

    this.validateWidget = function(widget){
        if (!widget.id) {
            throw new Error('Widget\'s must have "id" to use Preferences. '+
                'Widget with name "'+widget.name+'" does not have an id.');
        }
        if (!widget.container.id) {
            throw new Error('Container\'s must have "id" to use Preferences.');
        }

        return !isMasterDetailView(widget);
    };

    var userPromise = ZHPromise.defer();

    /**
     * Invalidate logged user and clear preferences loaded
     */
    this.unsetUser = function() {
        userPromise = ZHPromise.defer();
        userWidgetConfiguration = {};

        Object.keys(repositoryMap).forEach(function(repositoryName) {
            repositoryMap[repositoryName].clearAll();
        });
    };

    /**
     * Set logged user
     *
     * @param {String} userID Logged user
     */
    this.setUserID = function(userID) {
        userPromise.resolve(userID);
    };

    /**
     * Get logged user ID
     *
     * @return {Promise} User identifier
     */
    this.getUserID = function() {
        return userPromise.promise;
    };


    /**
     * Generate preference file to download
     *
     * @param {Object} preference       Preference to be exported
     * @param {String} fileName         File name
     *
     * @return {Promise<Object>}        Preference exported
     */
    this.downloadPreferenceFile = function(preference, fileName) {
        var dataStr = "data:text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(preference,false,4));

        var linkElement = document.createElement('a');
        linkElement.setAttribute("href",     dataStr);
        linkElement.setAttribute("download", fileName);
        linkElement.click();
        linkElement.remove();

        return ZHPromise.when(preference);
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhPreferences', ZhPreferences);
});

// FILE: src/Config.js
(function(){

window.metaDataUrls = window.metaDataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-preferences';

var hasMetaData = window.metaDataUrls.some(function(metaDataUrl){
	return metaDataUrl.package === PACKAGE_NAME;
});

var hasServiceUrl = window.metaDataUrls.some(function(metaDataUrl){
    return metaDataUrl.package === PACKAGE_NAME;
});

if (!hasMetaData) {
    window.metaDataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-preferences/dist/assets/'
    });
}

if (!hasServiceUrl) {
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: bower_components/zh-filter-preferences/dist/filter-preferences.js
// FILE: src/services/ZhFilterBuilder.js
/**
 * Service to create the filter conditions
 *
 * @class
 * @since 3.0.0
 *
 * @param {QueryBuilder} Query Service to build the query conditions
 */
function ZhFilterBuilder(Query){

    var defaultOperators = {
        'field/between-dates.html': Query.BETWEEN,
        'field/calendar.html': Query.EQ,
        'field/date.html': Query.EQ,
        'field/datetime.html': Query.EQ,
        'field/interval.html': {
            'default': Query.BETWEEN,
            'start': Query.GTE,
            'end': Query.LTE
        },
        'field/number-edit.html': Query.EQ,
        'field/number-keyboard.html': Query.EQ,
        'field/number.html': Query.EQ,
        'field/radio.html': Query.EQ,
        'field/select.html': Query.EQ,
        'field/select-simple.html': Query.EQ,
        'field/select-native.html': Query.EQ,
        'field/select-autocomplete.html': Query.EQ,
        'field/select-multiple.html': Query.IN,
        'default': Query.LIKE_INSENSITIVE
    };

    var buildFilterByTemplate = {
        'field/select.html': buildSelectFilter,
        'field/select-autocomplete.html': buildSelectFilter,
        'field/select-native.html': buildSelectNativeFilter,
        'field/select-multiple.html': buildSelectMultipleFilter,
        'field/interval.html':buildIntervalFilter,
        'default': buildDefaultFilter
    };

    /**
     * Register the default operator for a field
     *
     * @since 3.0.0
     *
     * @param {String} template Template of the field
     * @param {String} operator Operator to use on template
     */
    this.setDefaultOperator = function(template, operator){
        defaultOperators[template] = operator;
    };

    /**
     * Register the method used to create condition for a field
     *
     * @since 3.0.0
     *
     * @param {String}   template Template of the field
     * @param {Function} callback Method to be used to create the condition
     */
    this.setBuildFilterByTemplate = function(template, callback){
        buildFilterByTemplate[template] = callback;
    };

    function getFieldTemplate(field){
        return field.template.split('#').pop();
    }

    function getFilterBuilderByTemplate(template){
        if(!buildFilterByTemplate[template]){
            template = 'default';
        }

        return buildFilterByTemplate[template];
    }

    /**
     * Clear filter with same filter name
     *
     * @since 3.10.0
     *
     * @param  {Object[]} List of filters
     *
     * @return {Object[]} List of filters with unique name
     */
    function clearFilters(filters){
        var clearedFilters = [];
        filters.forEach(function(filter){
            var repeated = clearedFilters.some(function(_filter){
                return _filter.name === filter.name;
            });

            if(!repeated){
                clearedFilters.push(filter);
            }
        });
        return clearedFilters;
    }

    /**
     * Build filter conditions
     *
     * @since 3.0.0
     *
     * @param  {Field[]} fields List of fields to get the filter values
     * @param  {Object}  row    Row to get the values from
     *
     * @return {Object[]}       List of conditions
     */
    this.buildFilter = function (fields, row){
        var query = Query.build();

        fields.forEach(function(field){
            if(!field.skipSearch){
                var template = getFieldTemplate(field);
                getFilterBuilderByTemplate(template)(query, field, row);
            }
        });

        return clearFilters(query.where());
    };

    var operatorFlow = {
        "BETWEEN": function (query, value){
            query.between([value.start, value.end]);
        },
        "NOT BETWEEN": function (query, value){
            query.notBetween([value.start, value.end]);
        },
        "LIKE": function (query, value){
            query.like('%' + value + '%');
        },
        "LIKE_I": function (query, value){
            query.likeInsensitive('%' + value + '%');
        },
        "LIKE_BEGINNING": function (query, value){
            query.likeInsensitive( value + '%');
        },
        "NOT LIKE": function (query, value){
            query.notLike('%' + value + '%');
        },
        "LIKE ALL": function (query, value){
            query.likeAll('%' + value + '%');
        },
        "<>": function (query, value){
            query.notEquals(value);
        },
        "=": function (query, value){
            query.equals(value);
        },
        ">": function (query, value){
            query.greater(value);
        },
        "<": function (query, value){
            query.lower(value);
        },
        ">=": function (query, value){
            query.greaterOrEqual(value);
        },
        "<=": function (query, value){
            query.lowerOrEqual(value);
        },
        "IN": function (query, value){
            query.in(value);
        },
        "NOT IN": function (query, value){
            query.notIn(value);
        }
    };

    function buildFilter(query, fieldName, operator, value){
        query.where(fieldName);
        operatorFlow[operator](query, value);
    }

    function isEmptyArrayOrObject(obj){
        var name;
        for(name in obj){
            return false;
        }
        return true;
    }

    function existValue(value){
        var isObject = typeof value == 'object';
        return isObject ? !isEmptyArrayOrObject(value) : !Util.isEmptyOrBlank(value);
    }

    function buildDefaultFilter(query, field, currentRow){
        var fieldName = field.name;
        var value = currentRow[fieldName];

        if(existValue(value)){
            var operator = getFieldOperator(field);
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildIntervalFilter(query, field, currentRow){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        value = existValue(value) ? Util.clone(value) : {};

        var existStartValue = !Util.isEmptyOrBlank(value.start);
        var existEndValue = !Util.isEmptyOrBlank(value.end);

        if(existStartValue || existEndValue){
            var operator = getFieldOperator(field);
            if(existStartValue && existEndValue){
                operator = operator.default;
            } else if(existStartValue){
                operator = operator.start;
                value = value.start;
            } else {
                operator = operator.end;
                value = value.end;
            }
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.valueField;
        var value = currentRow[fieldName];
        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithValueFieldsFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];

        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithOutDataFilter(query, field, currentRow, operator){
        var outData = field.outData;
        Object.keys(outData).forEach(function(outDataKey){
            var fieldName = outData[outDataKey];
            var value = currentRow[fieldName];
            if(existValue(value)){
                buildFilter(query, fieldName, operator, value);
            }
        });
    }

    function buildSelectFilter(query, field, currentRow){
        var operator = getFieldOperator(field);
        if(field.valueField){
            buildSelectWithValueFieldFilter(query, field, currentRow, operator);
        } else if(field.valueFields){
            buildSelectWithValueFieldsFilter(query, field, currentRow, operator);
        } else if(field.outData){
            buildSelectWithOutDataFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select configuration on field ' + field.name);
        }
    }

    function buildSelectNativeWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectNativeFilter(query, field, currentRow){
        var operator = getFieldOperator(field);

        if(field.valueField){
            buildSelectNativeWithValueFieldFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select-native configuration on field ' + field.name);
        }

    }

    function buildSelectMultipleWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        if(currentRow[fieldName + '_EXCEPT']){
            value = currentRow[fieldName + '_EXCEPT'];
            operator = 'NOT IN';
        }

        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectMultipleWithValueFieldsFilter(query, field, currentRow, operator){
        throw new Error('Filter on field select-multiple with property valuesFields is not supported yet. ' +
            'Field "'+field.name+'"');
    }

    function buildSelectMultipleFilter(query, field, currentRow){
        var operator = getFieldOperator(field);

        if(field.valueField){
            buildSelectMultipleWithValueFieldFilter(query, field, currentRow, operator);
        } else if(field.valueFields){
            buildSelectMultipleWithValueFieldsFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select-multiple configuration on field ' + field.name);
        }
    }

    function getFieldOperator(field){
        var template = getFieldTemplate(field);
        return field.operator || defaultOperators[template] || defaultOperators['default'];
    }

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterBuilder', ZhFilterBuilder);
});

// FILE: src/services/ZhFilterDescriptionBuilder.js
/**
 * Service to create the applied filter description
 *
 * @class
 * @since 3.0.0
 *
 * @param {DecodeValueFieldService} DecodeValueFieldService Service to retrieve field value
 */
function ZhFilterDescriptionBuilder(DecodeValueFieldService, ZhUtil, templateManager){

    /**
     * Build filter description for a filter preference
     *
     * @since 3.0.0
     *
     * @param  {String} preferenceName Filter preference label
     *
     * @return {Object[]}              List containing the applied preference description
     */
    this.buildFilterAppliedInfo = function(preferenceName){
        return [{
            'label': 'Filter',
            'filterName': preferenceName
        }];
    };

    function getFilterWidget(id){
        return templateManager.container.factoredWidgets.filter(function(widget){ return widget._filteredWidgetId == id; })[0];
    }

    /**
     * Build filter description for a list of filter conditions
     *
     * @since 3.0.0
     *
     * @param  {Field[]} fields         List of fields on the list widget
     * @param  {Object}  currentRow     Row of the applied filter
     * @param  {Object}  searchCriteria Search criteria currently applied
     *
     * @return {Object[]}               List of descriptions for the applied conditions
     */
    this.buildFilterInfo = function(fields, currentRow, searchCriteria, widget){
        var filterInfo = [];

        var filterWidget = getFilterWidget(widget && widget.id);

        fields.forEach(function(field){
            buildDefaultFilterInfo(filterInfo, field, currentRow, filterWidget);
        });

        buildFilterInfoForSearchCriteria(fields, filterInfo, searchCriteria);

        return filterInfo;
    };

    function buildFilterInfoForSearchCriteria(fields, filterInfo, searchCriteria){
        if(searchCriteria){
            Object.keys(searchCriteria).forEach(function(fieldName){
                if(searchCriteria[fieldName] !== ''){
                    var fieldLabel;
                    if(fieldName == '_ALL_MAP'){
                        fieldLabel = 'All fields';
                    } else {
                        fieldLabel = getFieldByName(fields, fieldName).label;
                    }

                    filterInfo.push(buildFilterInfo(fieldLabel, searchCriteria[fieldName]));
                }
            });
        }
    }

    function getFieldByName(fields, fieldName){
        return fields.filter(function(field){
            return field.name === fieldName;
        }).shift();
    }

    function buildFilterInfo(label, value){
        return {
            'label': label,
            'value': value
        };
    }

    function buildDefaultFilterInfo(filterInfo, field, currentRow, widget){
        var condition = !Util.isEmptyOrBlank(currentRow[field.name]);
        var filterField = widget && widget.getField(field.name);
        var showInfo = filterField && filterField.isVisible;

        if(condition && showInfo){
            var value = DecodeValueFieldService.model(field, currentRow[field.name]);
            value = ZhUtil.evaluate(value);
            filterInfo.push(buildFilterInfo(field.label, value));
        }
    }

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterDescriptionBuilder', ZhFilterDescriptionBuilder);
});

// FILE: src/services/ZhFilterPreferences.js
/**
 * A filter preference
 * @typedef {Object} Filter
 * @property {String} ID                  Filter unique identifier
 * @property {String} USER_ID             User identifier that created the filter
 * @property {String} ORGANIZATION_ID     Organization the user belongs to
 * @property {String} LABEL_TO_PREFERENCE Filter label
 * @property {String} WIDGET_ID           Widget identifier the filter belongs to
 * @property {String} FILTER_ROW          Row indicating the filter settings
 * @property {String} LAST_MODIFIED_DATE  Last date the filter was modified
 */

/**
 * Service to handle filters storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences              Service to handle preferences storage
 * @param {ZhFilterBuilder}            ZhFilterBuilder            Service to create filters conditions
 * @param {ZhFilterDescriptionBuilder} ZhFilterDescriptionBuilder Service to conditions description
 * @param {TemplateManager}            templateManager            Service to handle template
 * @param {EventAggregator}            eventAggregator            Service to use project events
 * @param {ZhUtil}                     ZhUtil                     Service to provice utilities methods
 * @param {EventEngine}                eventEngine                Service to use models events
 * @param {MetaDataFactory}            metaDataFactory            Service to factory models from metadata
 * @param {ZHPromise}                  ZHPromise                  Service to handle promises
 * @param {MessageService}             MessageService             Service to handle messages
 * @param {ScreenService}              ScreenService              Service to handle screen
 * @param {ZhKeyboardService}          ZhKeyboardService          Service to handle keyboard binds
 * @param {ApplicationContext}         ApplicationContext         Service to retrieve Zeedhi services
 */
function ZhFilterPreferences(ZhPreferences, ZhFilterBuilder, ZhFilterDescriptionBuilder, templateManager, eventAggregator, ZhUtil, eventEngine, metaDataFactory, ZHPromise, MessageService, ScreenService, ZhKeyboardService, ApplicationContext, $timeout) {
    var self = this;
    var originalFilterRows = {};

    var originalFilter = {
        'ID': 'ORIGINAL',
        'USER_ID': 0,
        'ORGANIZATION_ID': 0,
        'LABEL_TO_PREFERENCE': 'Original'
    };

    function shouldShowFilterAction(widget){
        return Util.get(widget, ['floatingControl', 'searchAction']) !== false ||
            Util.get(widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function(){
        ZhPreferences.registerRepository('zh-filter-preferences#filter');

        templateManager.project.customFloatingActions.push({
            'name': 'Filter Action',
            'label': 'Search',
            'type': 'filter',
            'order': 20,
            'icon': 'search',
            'template': 'zh-filter-preferences#floating-filter-card.html',
            'expandable': true,
            'isVisible': shouldShowFilterAction
        });
    });

    eventEngine.registerEvent('FilterBeforeApply', 'beforeFilterApply');
    eventEngine.registerEvent('FilterAfterApply', 'afterFilterApply');

    var filterRegisteredDataSources = {};

    /**
     * Register filters datasource
     *
     * @since 3.0.0
     *
     * @param {String} widgetID         Widget identifier
     * @param {Object} datasource       Datasource that contains the filters
     *
     */
    this.registerFilterDataSource = function(widgetID, dataSource){
        if(widgetID){
            filterRegisteredDataSources[widgetID] = dataSource;
        }

        return filterRegisteredDataSources;
    };

    /**
     * Update dataSource with local saved filters
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded filters
     *
     * @return {Promise}               Promise resolved after the filters are loaded into the dataSource
     */
    this.updateFilters = function(widgetID, dataSource){
        if(!dataSource){
            dataSource = filterRegisteredDataSources[widgetID];
        }

        return this.getWidgetFilters(widgetID).then(function(filters){
            return this.addIsDefaultFilterToFilters(filters, widgetID).then(function(filters){
                dataSource.data = filters;
            });
        }.bind(this));
    };

    function addIsDefaultFilterToFilter(defaultFilter, filter){
        filter.DEFAULT_FILTER = filter.ID === defaultFilter;
        return filter;
    }

    /**
     * Add property to identify if the filter is selected as default
     *
     * @since 3.0.0
     *
     * @param {Filter[]} filters  List of filters to update the default filter property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultFilterToFilters = function(filters, widgetID){
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){
            var defaultFilter = defaultPreferences.DEFAULT_FILTER;
            return filters.map(addIsDefaultFilterToFilter.bind(this, defaultFilter));
        });
    };

    /**
     * Retrieve local cached filters for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<Filter[]>} Filter preferences
     */
    this.getWidgetFilters = function(widgetID){
        return ZhPreferences.getWidgetPreferences('filter', widgetID).then(function parseFilterRow(preferences){
            preferences.forEach(function(preference){
                preference.FILTER_ROW = JSON.parse(preference.FILTER_ROW);
            });
            return preferences;
        });
    };

    /**
     * Retrieve local cached filters
     *
     * @since 3.0.0
     *
     * @param  {Query} query       Conditions to filter the preferences
     *
     * @return {Promise<Filter[]>} Filter preferences
     */
    this.getFilters = function(query){
        return ZhPreferences.getPreferences('filter', query);
    };

    /**
     * Persist filters
     *
     * @since 3.0.0
     *
     * @param  {Filter[]} filters  Filters to persist
     *
     * @return {Promise<Filter[]>} List of persisted filters
     */
    this.saveFilters = function(filters){
        var cloneFilters = filters.map(function(filter){ return ZhUtil.clearRow(filter); });
        return ZhPreferences.savePreferences('filter', cloneFilters).then(function(response){
            cloneFilters.forEach(function(clone, index){
                filters[index].ID = clone.ID;
                filters[index].__is_new = clone.__is_new;
                filters[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete filters
     *
     * @since 3.0.0
     *
     * @param  {Filter[]} filters  Filters to delete
     *
     * @return {Promise<Filter[]>} List of deleted filters
     */
    this.deleteFilters = function(filters){
        var cloneFilters = filters.map(function(filter){ return ZhUtil.clearRow(filter); });
        return ZhPreferences.deletePreferences('filter', cloneFilters);
    };

    function applyOriginalFilterRow(widget, filter){
        filter.FILTER_ROW = Util.clone(originalFilterRows[widget.id] || {});
    }

    /**
     * Apply original filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original filter on
     *
     * @return {Object[]}      Rows returned using the filter
     */
    this.applyOriginalFilter = function(widget){
        applyOriginalFilterRow(widget, originalFilter);
        return this.applyFilter(widget, originalFilter);
    };

    /**
     * Set the filter row used when applying the Original filter
     *
     * @since 3.4.0
     *
     * @param {String} widgetId Widget identifier to set the original filter row
     * @param {Object} filter   Filter row to create the original filter
     */
    this.setOriginalFilter = function(widgetId, filter){
        originalFilterRows[widgetId] = Util.clone(filter);
    };

    /**
     * Get the filter row used when applying the Original filter
     *
     * @since 3.4.0
     *
     * @return {Object} filter  Filter row used to create the original filter
     */
    this.getOriginalFilter = function(widgetId, widget){
        var clearedFilter = ZhUtil.clearRow(originalFilterRows[widgetId] || {});
        var originalFilter = {
            ID: 'ORIGINAL',
            LABEL_TO_PREFERENCE: 'Original',
            FILTER_ROW: Util.clone(clearedFilter)
        };
        return originalFilter;
    };

    function getFilterConditions(filters){
        return filters.filter(function(filter){
            return filter.name.indexOf('SEARCHLIST_') === -1;
        }).map(function(filter){
            filter.isCustomFilter = true;
            return filter;
        });
    }

    function getSearchConditions(filters){
        var searchFields = Util.clone(filters);
        searchFields = searchFields.filter(function(filter){
            return ~filter.name.indexOf('SEARCHLIST_');
        });

        return searchFields.map(function(filter){
            filter.name = filter.name.replace('SEARCHLIST_', '');
            return filter;
        });
    }

    function triggerBeforeFilterApplyEvent(widget, preference){
        if(widget.beforeFilterApply){
            return widget.beforeFilterApply({
                'preference': preference
            });
        }
    }

    function triggerAfterFilterApplyEvent(widget, preference, filters){
        if(widget.afterFilterApply){
            widget.afterFilterApply({
                'preference': preference,
                'filters': filters
            });
        }
    }

    function prepareFilterRow(preference){
        var filterRow = preference.FILTER_ROW || {};
        filterRow = typeof preference.FILTER_ROW === 'string'?
            JSON.parse(preference.FILTER_ROW) : preference.FILTER_ROW;
        preference.FILTER_ROW = filterRow;
    }

    function applyFilterInFilterPopup(widget, filterRow){
        self.createFilterWidget(widget, filterRow);
    }

    function mergeFilters(baseFilters, newFilters){
        var result = baseFilters.concat(newFilters).reduce(function(prev, current, index){
            var fieldName = current.name;
            if(fieldName in prev.keys){
                var replaceIndex = prev.keys[fieldName];
                prev.filters[replaceIndex] = current;
            } else {
                prev.keys[fieldName] = index;
                prev.filters.push(current);
            }
            return prev;
        }, {
            filters: [],
            keys: {}
        });
        return result.filters;
    }

    function buildDataSourceFilter(widget, filters){
        var filterConditions = getFilterConditions(filters);
        var originalFilters = getOriginalDataSourceFilters(widget.dataSourceFilter);
        return mergeFilters(originalFilters, filterConditions);
    }

    function applyFilterInDataSource(widget, filters){
        widget.dataSourceFilter = buildDataSourceFilter(widget, filters);

        var searchConditions = getSearchConditions(filters);
        if(widget.setSearchCriteria){
            widget.setSearchCriteria(searchConditions);
        }
        return searchConditions;
    }

    function setAppliedFilter(widget, preference){
        widget.appliedFilter = preference;
    }

    function applyFilterInWidget(widget, fields, filterRow, preference){
        self.updateFilterInfo(widget, fields, filterRow, preference, {});
        setAppliedFilter(widget, preference);
    }

    /**
     * Apply filter
     *
     * @since 3.0.0
     *
     * @param  {Widget}  widget              Widget to apply the original filter on
     * @param  {Filter}  preference          Preference to be applied on widget
     * @param  {Boolean} [reloadWidget=true] Flag to cancel reload in widget
     *
     * @return {Object[]}          Rows returned using the filter
     */
    this.applyFilter = function(widget, preference, reloadWidget){
        reloadWidget = reloadWidget !== false;
        prepareFilterRow(preference);
        var event;

        if(reloadWidget){
            event = triggerBeforeFilterApplyEvent(widget, preference);
        }

        var filterRow = preference.CUSTOM_FILTER_ROW || preference.FILTER_ROW;

        var fields = self.factoryFields(null, widget.fields).filter(function removeInvisibleFields(field){
            return field.isVisible;
        });
        var filters = ZhFilterBuilder.buildFilter(fields, filterRow);

        if(reloadWidget){
            if(!event || !event.isDefaultPrevented()){
                var searchConditions = applyFilterInDataSource(widget, filters);
                var searchCriteria = buildSearchCriteria(searchConditions);

                return widget.reload(ZhUtil.getLikeAllFilter(widget, searchCriteria)).then(function(){
                    if (widget.dataSource) {
                        widget.dataSource.clearCheckedRows();
                    }
                    applyFilterInWidget(widget, fields, filterRow, preference);
                    triggerAfterFilterApplyEvent(widget, preference, filters);
                    applyFilterInFilterPopup(widget, preference.FILTER_ROW);

                    return ZHPromise.when();
                }.bind(this));
            } else {
                return ZHPromise.reject();
            }
        } else {
            applyFilterInDataSource(widget, filters);
            applyFilterInWidget(widget, fields, filterRow, preference);
            applyFilterInFilterPopup(widget, preference.FILTER_ROW);
            return ZHPromise.when();
        }
    };

    /**
     * Update the applied filter information
     *
     * @since 3.0.0
     *
     * @param  {Widget}  widget         Widget to update the filter information
     * @param  {Field[]} fields         List of fields used to create the filter
     * @param  {Object}  filterRow      Row representing the filter configuration
     * @param  {Filter}  preference     Filter currently applied
     * @param  {Object}  searchCriteria Search criteria currently applied
     */
    this.updateFilterInfo = function(widget, fields, filterRow, preference, searchCriteria){
        var filterInfo;
        var filterName = preference.LABEL_TO_PREFERENCE;

        if(((preference.ID === 'ORIGINAL' || preference.ID === 'EDITED') && !filterName) || preference.forceBuild){
            filterInfo = ZhFilterDescriptionBuilder.buildFilterInfo(fields, filterRow, searchCriteria, widget);
        } else {
            filterInfo = ZhFilterDescriptionBuilder.buildFilterAppliedInfo(filterName);
        }

        widget._filterInfo = filterInfo;
    };

    function trimPercentageSymbol(value){
        if(value.charAt(0) === '%'){
            value = value.substr(1);
        }

        if(value.charAt(value.length - 1) === '%'){
            value = value.substr(0, value.length - 1);
        }

        return value;
    }

    function buildSearchCriteria(conditions){
        var searchCriteria = {};
        conditions.forEach(function(condition){
            searchCriteria[condition.name] = trimPercentageSymbol(condition.value);
        });

        return searchCriteria;
    }

    function getOriginalDataSourceFilters(dataSourceFilter){
        return dataSourceFilter.filter(function(filter){
            return !filter.isCustomFilter;
        });
    }

    function removeConflictingProperties(field){
        var conflictingProperties = ['selectWidget', 'bindedEvents'];
        conflictingProperties.forEach(function(property){
            delete field[property];
        });
    }

    function factoryFields(gridFields, editing){
        var cloneField = function(field){
            var fieldClone = self.cloneObj(field);
            if(editing){
                fieldClone.fieldGroup = 'FILTER_FIELDS';
            }
            fieldClone.dataSource = field.dataSource;
            fieldClone.dataSourceFilter = field.dataSourceFilter;
            return fieldClone;
        };
        return gridFields.filter(function(field){
            return !Util.isUndefined(field.template);
        }).map(cloneField);
    }

    function factorySearchFields(gridFields){
        var factoredFields = [
            {
                'id': 'SEARCHLIST__ALL_MAP',
                'label': 'All Fields',
                'name': 'SEARCHLIST__ALL_MAP',
                'class': 12,
                'template': 'field/text-edit.html',
                'isVisible': true,
                'showOnList': true,
                'descending': true,
                'fieldGroup': 'MORE_FIELDS'
            }
        ];

        var buildSearchListField = function(field){
            var fieldClone = self.cloneObj(field);
            fieldClone.id = 'SEARCHLIST_' + fieldClone.id;
            fieldClone.name = 'SEARCHLIST_' + fieldClone.name;
            fieldClone.class = 6;
            fieldClone.isVisible = true;
            fieldClone.readOnly = false;
            fieldClone.validations = {};
            fieldClone.template = 'field/text-edit.html';
            fieldClone.fieldGroup = 'MORE_FIELDS';
            return fieldClone;
        };

        return factoredFields.concat(gridFields.filter(function(field){
            return !field.skipSearch && Util.isUndefined(field.template);
        }).map(buildSearchListField));
    }

    function mergeFieldNewProperties(field, filterProperties){
        var keys = Object.keys(filterProperties);
        keys.forEach(function(property){
            field[property] = filterProperties[property];
        });
        if(field.dataSource && !(field.dataSource instanceof DataSource)){
            field.dataSource = metaDataFactory.factoryDataSource(field.dataSource, field.dataSourceFilter, field);
        }
        delete field.filterProperties;
        return field;
    }

    function removeEventListener(field, eventName, eventIndex){
        eventIndex = eventEngine.bindEvent(field, eventName);
        eventEngine.unbindEvent(field, eventName, eventIndex);
        eventIndex--;
        eventEngine.unbindEvent(field, eventName, eventIndex);
    }

    /**
     * Factory fields used on filter widget
     *
     * @since 3.0.0
     *
     * @param  {Widget}   parentWidget Widget to be the parent of the fields
     * @param  {Fields[]} gridFields   Fields from the widget to use the filter
     * @param  {Boolean}  editing      Filter configuration being edited
     *
     * @return {Fields[]}              Factored fields
     */
    this.factoryFields = function(parentWidget, gridFields, editing){
        var fields = factoryFields(gridFields, editing);
        var searchFields = factorySearchFields(gridFields);
        var buildFilterProperties = function(field){
            if(field.filterProperties){
                field = mergeFieldNewProperties(field, field.filterProperties);
            }
            removeConflictingProperties(field);
            removeEventListener(field, "FieldOnChange");
            return metaDataFactory.fieldFactory(field, parentWidget);
        };

        return fields.concat(searchFields).map(buildFilterProperties);
    };

    /**
     * Export filter file
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter  Filter to export
     *
     * @return {Promise<Filter>} Filter exported
     */
    this.downloadPreferenceFile = function(filter){
        var fileName = filter.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index){
            if(+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.filter';

        var filterToExport = {
            'ID': filter.ID,
            'LABEL_TO_PREFERENCE': filter.LABEL_TO_PREFERENCE,
            'WIDGET_ID': filter.WIDGET_ID,
            'FILTER_ROW': ZhUtil.clearRow(filter.FILTER_ROW),
            'LAST_MODIFIED_DATE': filter.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(filterToExport, fileName);
    };

    /**
     * Validade filter to import
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter              Filter to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} filterDataSource  Existing filters data (optional)
     *
     * @return {Boolean} Filter valid
     */
    this.validateFilterImport = function(filter, configWidget, filterDataSource){
        filterDataSource = filterDataSource || configWidget.dataSource.data;

        if(configWidget.listWidget.id != filter.WIDGET_ID){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_widget_id');
            return false;
        }

        var filtersWithSameId = filterDataSource.filter(function(row){
            return row.ID == filter.ID;
        });

        if(filtersWithSameId.length > 0){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_id_violation');
            return false;
        }

        var filtersWithSameLabel = filterDataSource.filter(function(row){
            return row.LABEL_TO_PREFERENCE == filter.LABEL_TO_PREFERENCE;
        });

        if(filtersWithSameLabel.length > 0){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_name_violation');
            return false;
        }

        return true;
    };

    function openFilterWidget(widget, rowToBeSet) {
        return createFilterWidget(widget, rowToBeSet).then(function(widgetToBeShown) {
            widget.activate();
            ScreenService.openPopup(widgetToBeShown);
        });
    }
    this.openFilterWidget = openFilterWidget;

    var filterWidgets = {};

    function forceSelectDataSourceToLoad(field, row){
        var data = Util.getNestedProperty(field, ['dataSource', 'data', 'length']);
        var currentRow = row[field.name];

        if(field.dataSource && !data && currentRow){
            var fieldName = field.valueField || field.name;
            field.dataSource.load([{name: fieldName, value: currentRow, operator: "IN"}]);
        }
    }

    function forceFieldDataSourceToLoad(widget){
        widget.fields.forEach(function(field){
            forceSelectDataSourceToLoad(field, widget.currentRow);
        });
    }

    function createFilterWidget(widget, rowToBeSet) {
        return ApplicationContext.ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForFilterPopup(widget).then(function(filterWidget) {
            if(!widget.filterInitialized){
                filterWidget.listWidget = widget;
                widget.filterInitialized = true;
            }

            filterWidget.label = ScreenService.i18n('Filter');

            var currentRow = rowToBeSet || widget.filterRowData;
            if(!currentRow) {
                currentRow = widget.appliedFilter ? widget.appliedFilter.FILTER_ROW : {};
            }

            filterWidget.currentRow = filterWidget.currentRow || {};
            Object.keys(currentRow).forEach(function(fieldName){
                filterWidget.currentRow[fieldName] = ZhUtil.evaluate(currentRow[fieldName]);
            });

            forceFieldDataSourceToLoad(filterWidget);

            filterWidget.actions[0].isVisible = true;
            filterWidget.actions[1].isVisible = true;
            filterWidget.actions[2].isVisible = false;
            filterWidget.actions[3].isVisible = false;

            filterWidgets[widget.id] = filterWidget;
            return ZHPromise.when(filterWidget);
        }.bind(this));
    }
    this.createFilterWidget = createFilterWidget;

    function isCopiable(object){
        var isCopiable = Util.isUndefinedOrNull(object);
        if(!isCopiable){
            var constr = object.constructor;
            var types = [String, Number, Array, Object, Boolean];
            isCopiable = types.some(function(type){
                return type().constructor === constr;
            });
        }
        return isCopiable;
    }

    /**
     * Clone the object avoiding first level non JavaScript basic and undefined types.
     *
     * @since ^3.14
     *
     * @param {Object} object The object to be copied.
     *
     * @return {Object} Returns the new object.
     */
    this.cloneObj = function(object){
        var clone = {};
        Object.keys(object).forEach(function(property){
            if(isCopiable(object[property])){
                clone[property] = ZhUtil.clearRow(object[property]);
            }
        });
        return clone;
    };

    function hasPopupOpened(){
        var isMobiScrollOpen = $('.mbsc-mobiscroll').length > 0;
        return !ScreenService.isSideMenuOpen() && !isMobiScrollOpen && !ScreenService.isAlertShown();
    }

    function isFilterPreferencesActive(widget){
        return Util.get(widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    function isSearchActionActive(widget){
        return Util.get(widget, ['floatingControl', 'searchAction']) !== false;
    }

    function openWidgetFilter(){
        var activeWidget = templateManager.container.getActiveWidget();
        if(hasPopupOpened() && isFilterPreferencesActive(activeWidget) && ZhPreferences.validateWidget(activeWidget)){
            openFilterWidget(activeWidget);
        }

        return false;
    }

    function openWidgetSearch(){
        var activeWidget = templateManager.container.getActiveWidget();
        var searchIsEnabled = isFilterPreferencesActive(activeWidget) || isSearchActionActive(activeWidget);
        if(searchIsEnabled && hasPopupOpened() && ZhPreferences.validateWidget(activeWidget)){
            activeWidget.openSearchAction();
        }

        return false;
    }

    templateManager.projectPromise.then(function(){
        templateManager.project.shortcutKeys = templateManager.project.shortcutKeys || {};
        var openFilterKey = templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l';
        var openSearchKey = templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f';

        ZhKeyboardService.unbind(openFilterKey);
        ZhKeyboardService.unbind(openSearchKey);

        ZhKeyboardService.bind(openFilterKey, openWidgetFilter);
        ZhKeyboardService.bind(openSearchKey, openWidgetSearch);
    });
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterPreferences', ZhFilterPreferences);
});

// FILE: src/controllers/ZhFilterWidgetsBuilder.js
/**
 * Service to create widgets to handle filters
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function ZhFilterWidgetsBuilder(restEngine, templateManager, metaDataFactory, ZhFilterPreferences){
    var widgetsJson = {};

    function getWidgetJSON(widgetName){
        var widgetPath = 'zh-filter-preferences#json/widgets/'+widgetName+'.json';
        if(!widgetsJson[widgetPath]){
            widgetsJson[widgetPath] = restEngine.requestMetaData(widgetPath).then(function(response){
                return response.data;
            });
        }
        return widgetsJson[widgetPath];
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName){
        return getWidgetJSON(widgetName).then(function(widget){

            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored filter configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Filter configuration widget
     */
    this.factoryFilterConfigurationWidget = function(){
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create filter widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create filter widget
     */
    this.factoryCreateFilterWidget = function(){
        return this.factoryWidget('createFilter');
    };

    /**
     * Retrieve factored import filter widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import filter widget
     */
    this.factoryImportFilterWidget = function(){
        return this.factoryWidget('importFilter');
    };

    function copyFieldGroups(fieldGroups){
        var copiedObj = (fieldGroups || []).map(function(fieldGroup){
            return Util.clone(fieldGroup, ['__proto__', 'rows', 'widget']);
        });
        return copiedObj;
    }

    function buildFilterWidgetFieldGroups(filterWidget, widget){
        var fieldGroups = (filterWidget.fieldGroups || []).concat(copyFieldGroups(widget.fieldGroups));
        var moreFieldsFieldGroup = [];
        fieldGroups = fieldGroups.filter(function(fieldGroup){
            var isMoreFieldFieldGroup = fieldGroup.name === 'MORE_FIELDS';
            if(isMoreFieldFieldGroup){
                moreFieldsFieldGroup = fieldGroup;
                moreFieldsFieldGroup.isVisible = !widget.hideMoreFieldsOnFilter;
            }
            return !isMoreFieldFieldGroup;
        });
        filterWidget.fieldGroups = fieldGroups.concat(moreFieldsFieldGroup);
    }

    /**
     * Retrieve factored build filters conditions widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build filters conditions widget
     */
    this.factoryBuildFiltersConditionsWidget = function(baseWidget){
        return getWidgetJSON('createFilterConditions').then(function(widgetJson){
            buildFilterWidgetFieldGroups(widgetJson, baseWidget);
            var widget = metaDataFactory.widgetFactory(widgetJson, templateManager.container);
            widget.fields = widget.fields.concat(ZhFilterPreferences.factoryFields(widget, baseWidget.fields));
            widget._filteredWidgetId = baseWidget.id;
            return widget;
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterWidgetsBuilder', ZhFilterWidgetsBuilder);
});

// FILE: src/controllers/ZhFilterCreateFilterConditionsController.js
/**
 * Create filter conditions widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhFilterPreferences}    ZhFilterPreferences    Service to handle filter preferences methods
 * @param {ZhFilterWidgetsBuilder} ZhFilterWidgetsBuilder Service to create widgets to handle filters
 * @param {MessageService}         MessageService         Service to show messages
 * @param {ScreenService}          ScreenService          Service to manipulate screen
 * @param {ZHPromise}              ZHPromise              Service to handle promises
 * @param {ZhUtil}                 ZhUtil                 Service that contains utilities methods
 */
function ZhFilterCreateFilterConditionsController(ZhFilterPreferences, ZhFilterWidgetsBuilder, MessageService, ScreenService, ZHPromise, ZhUtil){

    var createFilterConditionsWidgetPromise = {};

    /**
     * Retrieve widget to create filter conditions
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidget = function(widget){
        var widgetId = widget.id;
        if(!createFilterConditionsWidgetPromise[widgetId]){
            createFilterConditionsWidgetPromise[widgetId] = ZhFilterWidgetsBuilder.factoryBuildFiltersConditionsWidget(widget);
        }

        return createFilterConditionsWidgetPromise[widgetId];
    };

    function saveFilterFields(widget){
        var allowedFields = widget.fields.map(function(field){
            return ZhFilterPreferences.cloneObj(field);
        });

        widget.__originalFilterStatus = allowedFields;
        widget.__originalCurrentRow = ZhFilterPreferences.cloneObj(ZhUtil.clearRow(widget.currentRow));
    }

    /**
     * Retrieve widget to create filter conditions with filter popup configuration
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidgetForFilterPopup = function(widget){
        return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){
            createFilterConditionsWidget.actions[0].isVisible = true;
            createFilterConditionsWidget.actions[1].isVisible = true;
            createFilterConditionsWidget.actions[2].isVisible = false;
            createFilterConditionsWidget.actions[3].isVisible = false;
            var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');

            if(labelField){
                labelField.isVisible = false;
            }

            saveFilterFields(createFilterConditionsWidget);

            return createFilterConditionsWidget;
        });
    };


    /**
     * Retrieve widget to create filter conditions with edit preference configuration
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidgetForEditPreference = function(widget){
        return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){
            createFilterConditionsWidget.actions[0].isVisible = false;
            createFilterConditionsWidget.actions[1].isVisible = false;
            createFilterConditionsWidget.actions[2].isVisible = true;
            createFilterConditionsWidget.actions[3].isVisible = true;

            var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');

            if(labelField){
                labelField.isVisible = true;
            }

            saveFilterFields(createFilterConditionsWidget);

            return createFilterConditionsWidget;
        });
    };

    /**
     * Save filter and apply
     *
     * @since 3.0.0
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be saved
     *
     * @return {Promise}                             Promise resolved when saved filter is applied
     */
    this.saveAndApplyFilter = function(createFilterConditionsWidget){

        var clearRequiredFieldErrors = function(invalidFields){
            invalidFields.shift();
            invalidFields.forEach(function(invalidField){
                var fieldName;
                invalidField = invalidField.filter(function(invalidItem){
                    fieldName = invalidItem.fieldName;
                    return invalidItem.errorMessage != "Required field";
                });
                if(!invalidField.length){
                    delete createFilterConditionsWidget.currentRow.$error[fieldName];
                }
            });
        };

        if(createFilterConditionsWidget.isValid(clearRequiredFieldErrors)){
            var currentRow = createFilterConditionsWidget.currentRow;
            var filter = createFilterConditionsWidget.preference;

            filter.LABEL_TO_PREFERENCE = currentRow.LABEL_TO_PREFERENCE;
            filter.FILTER_ROW = ZhUtil.clearRow(currentRow);
            delete filter.FILTER_ROW.LABEL_TO_PREFERENCE;
            filter.FILTER_ROW = JSON.stringify(filter.FILTER_ROW);

            var configWidget = createFilterConditionsWidget.configWidget;
            var listWidget = configWidget.listWidget;

            return ZhFilterPreferences.saveFilters([filter]).then(function(){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                MessageService.successNotificationByCode('zh-filter-preferences#save_success');
                ScreenService.closePopup(true);

                if(createFilterConditionsWidget.isValid()){
                    filter.forceBuild = true;
                    listWidget.filterRowData = currentRow;
                    ZhFilterPreferences.applyFilter(listWidget, filter);
                    listWidget.activate();
                } else {
                    listWidget.openFilterWidget(createFilterConditionsWidget.currentRow);
                }

                ZhFilterPreferences.updateFilters(listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                MessageService.showMessageByCode('zh-filter-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        } else {
            return ZHPromise.reject("Invalid params");
        }
    };

    function getRowToApply(appliedFilter, currentRow){
        var appliedRow = (appliedFilter && appliedFilter.FILTER_ROW) || {};
        var row = ZhUtil.clearRow(currentRow);
        Object.keys(appliedRow).forEach(function(key){
            // Do not replace evaluated values
            if(Util.isString(appliedRow[key]) && appliedRow[key].indexOf('@') === 0){
                row[key] = appliedRow[key];
            }
        });
        return row;
    }

    this.clearFilter = function(widget){
        widget.fields.forEach(function(field){
            field.clearValue();
            if(field.change){
                field.change();
            }
        });
    };

    /**
     * Apply filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be saved
     * @param  {String} [filterName] - The name of the filter being applied
     *
     * @return {Promise}                             Promise resolved when saved filter is applied
     */
    this.applyFilter = function(createFilterConditionsWidget, filterName){
        if(createFilterConditionsWidget.isValid()){
            var currentRow = createFilterConditionsWidget.currentRow;
            var listWidget = createFilterConditionsWidget.listWidget;
            delete currentRow.LABEL_TO_PREFERENCE;

            var filter = {
                'ID': 'EDITED',
                'FILTER_ROW': getRowToApply(listWidget.appliedFilter, currentRow)
            };

            if(filterName){
                filter.LABEL_TO_PREFERENCE = filterName;
            }

            return ZhFilterPreferences.applyFilter(listWidget, filter).then(function(){
                if(ScreenService.getPopupOpen() == createFilterConditionsWidget){
                    ScreenService.closePopup(true);
                    listWidget.activate();
                }
            });
        } else {
            if(ScreenService.getPopupOpen() != createFilterConditionsWidget){
                ScreenService.openPopup(createFilterConditionsWidget);
            }
            return ZHPromise.reject();
        }
    };

    function restoreFilterFields(widget){
        if(widget.__originalFilterStatus){
            widget.fields.forEach(function(field, index){
                var __field = widget.__originalFilterStatus.filter(function(fieldOriginal){
                    return fieldOriginal.name === field.name;
                })[0];
                Object.keys(__field).forEach(function(key){
                    widget.fields[index][key] = __field[key];
                });
            });
            widget.currentRow = widget.__originalCurrentRow;

            delete widget.__originalCurrentRow;
            delete widget.__originalFilterStatus;
        }
    }

    /**
     * Close widget popup and restore original status of filter widget
     *
     * @since 3.14.x
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be restored
     *
     * @return {undefined}
     */
    this.closePopup = function(widget){
        restoreFilterFields(widget);
        ScreenService.closePopup();
        widget.listWidget.activate();
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterCreateFilterConditionsController', ZhFilterCreateFilterConditionsController);
});

// FILE: src/controllers/ZhFilterImportFilterController.js
/**
 * Import filter widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhFilterPreferences}                   ZhFilterPreferences                      Service to handle filter preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhFilterWidgetsBuilder}                ZhFilterWidgetsBuilder Service to create widgets to handle filters
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 */
function ZhFilterImportFilterController(MessageService, ZhFilterPreferences, ZhPreferences, ZhFilterWidgetsBuilder, ZHPromise, ScreenService){

    var importFilterWidgetPromise = null;

    /**
     * Retrieve widget to import filter
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportFilterWidget = function(){
        if(importFilterWidgetPromise === null){
            importFilterWidgetPromise = ZhFilterWidgetsBuilder.factoryImportFilterWidget();
        }

        return importFilterWidgetPromise;
    };

    /**
     * Import filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} importFilterWidget Widget with the filter configuration
     *
     * @return {Promise}                      Promise resolved after the filter is imported
     */
    this.importPreference = function(importFilterWidget){
        if(!importFilterWidget.isValid()){
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }

        var filterFileData = importFilterWidget.fields[0].value()[0].b64File;
        filterFileData = filterFileData.split('base64,')[1];
        filterFileData = window.atob(filterFileData);
        filterFileData = decodeURIComponent(escape(filterFileData));

        var filter = JSON.parse(filterFileData);

        var configWidget = importFilterWidget.configWidget;
        var listWidget = configWidget.listWidget;

        if(!ZhFilterPreferences.validateFilterImport(filter, configWidget)){
            return ZHPromise.reject({err: 'FILTER_FILE_INVALID'});
        }

        filter.__is_new = true;
        filter.__generate_id = false;
        filter.FILTER_ROW = JSON.stringify(filter.FILTER_ROW);

        return ZhPreferences.getUserID().then(function(userID){
            filter.USER_ID = userID;

            return ZhFilterPreferences.saveFilters([filter]).then(function(){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                ScreenService.closePopup();
                ZhFilterPreferences.updateFilters(listWidget.id, configWidget.dataSource);
                configWidget.refreshWidget();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-filter-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterImportFilterController', ZhFilterImportFilterController);
});

// FILE: src/controllers/ZhFilterConfigWidgetController.js
/**
 * Filter configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}                            ZhPreferences                            Service to handle preferences persistance
 * @param {ZhFilterPreferences}                      ZhFilterPreferences                      Service to handle filter preferences methods
 * @param {ZhFilterWidgetsBuilder}                   ZhFilterWidgetsBuilder                   Service to create widgets to handle filters
 * @param {ScreenService}                            ScreenService                            Service to manipulate screen
 * @param {MessageService}                           MessageService                           Service to show messages
 * @param {ZHPromise}                                ZHPromise                                Service to handle promises
 * @param {ZhFilterCreateFilterConditionsController} ZhFilterCreateFilterConditionsController Controller to manipulate create filter conditions widget
 * @param {ZhFilterImportFilterController}           ZhFilterImportFilterController           Controller to manipulate import filter widget
 */
function ZhFilterConfigWidgetController(ZhPreferences, ZhFilterPreferences, ZhFilterWidgetsBuilder, ScreenService, MessageService, ZHPromise, ZhFilterCreateFilterConditionsController, ZhFilterImportFilterController){

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles filters configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create filters for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles filters configuration
     */
    this.getConfigWidget = function(listWidget, dataSource){
        if(configWidgetPromise === null){
            configWidgetPromise = ZhFilterWidgetsBuilder.factoryFilterConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args){
            var userID = args[0];
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id}
            ];
            dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == (listWidget.appliedFilter ? listWidget.appliedFilter.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    var hideMoreFields = function(groups, hide){
        var moreFieldsFieldGroup= [];
        return groups.filter(function(fieldGroup){
            delete fieldGroup.rows;
            var isMoreFieldFieldGroup = fieldGroup.name === 'MORE_FIELDS';
            if(isMoreFieldFieldGroup){
                moreFieldsFieldGroup = fieldGroup;
                moreFieldsFieldGroup.isVisible = !hide;
            }
            return !isMoreFieldFieldGroup;
        }).concat(moreFieldsFieldGroup);
    };

    /**
     * Open widget to create a new filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateFilterWidget = function(configWidget){
        var label = configWidget.currentRow.LABEL_TO_PREFERENCE;
        return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget).then(function(createFilterConditionsWidget){
            var listWidget = configWidget.listWidget;

            createFilterConditionsWidget.label = ScreenService.i18n('Filter');
            createFilterConditionsWidget.configWidget = configWidget;

            if(!listWidget.filterInitialized){
                var fieldGroups = (createFilterConditionsWidget.fieldGroups || []).concat((listWidget.fieldGroups || []));
                createFilterConditionsWidget.fieldGroups = fieldGroups;
                listWidget.filterInitialized = true;
            }
            createFilterConditionsWidget.fieldGroups = hideMoreFields(createFilterConditionsWidget.fieldGroups, listWidget.hideMoreFieldsOnFilter);

            return ZhPreferences.getUserID().then(function(userID){
                var preference = {
                    '__is_new': true,
                    'ID': null,
                    'USER_ID': userID,
                    'LABEL_TO_PREFERENCE': label,
                    'WIDGET_ID': listWidget.id,
                    'LAST_MODIFIED_DATE': null
                };

                createFilterConditionsWidget.preference = preference;

                var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');
                var fields = ZhFilterPreferences.factoryFields(createFilterConditionsWidget, listWidget.fields, true);
                createFilterConditionsWidget.fields = [labelField].concat(fields);
                labelField.isVisible = true;
                createFilterConditionsWidget.newRow();
                return ScreenService.openPopup(createFilterConditionsWidget);
            });
        });
    };


    /**
     * Open popup to edit a saved filter
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter   Filter to edit
     *
     * @return {Promise<Widget>} Widget used to edit the filter
     */
    this.editFilter = function(configWidget, filter){
        var listWidget = configWidget.listWidget;
        return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget)
            .then(function(createFilterConditionsWidget){
                createFilterConditionsWidget.preference = filter;
                createFilterConditionsWidget.label = ScreenService.i18n('Filter') + ' - ' + filter.LABEL_TO_PREFERENCE;
                createFilterConditionsWidget.configWidget = configWidget;
                createFilterConditionsWidget.fieldGroups= hideMoreFields(createFilterConditionsWidget.fieldGroups || [], listWidget.hideMoreFieldsOnFilter);

                var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');
                var fields = ZhFilterPreferences.factoryFields(createFilterConditionsWidget, listWidget.fields, true);
                createFilterConditionsWidget.fields = [labelField].concat(fields);
                labelField.isVisible = true;

                createFilterConditionsWidget.currentRow = filter.FILTER_ROW;
                createFilterConditionsWidget.currentRow.LABEL_TO_PREFERENCE = filter.LABEL_TO_PREFERENCE;
                return ScreenService.openPopup(createFilterConditionsWidget);
            });
    };

    /**
     * Delete filters
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the filters to delete
     *
     * @return {Promise}             Promise resolved after filters are deleted
     */
    this.deleteFilters = function(configWidget){
        var filtersToDelete = configWidget.getCheckedRows();

        var defaultFilter = filtersToDelete.filter(function(filter){
            return filter.DEFAULT_FILTER;
        });
        if(defaultFilter.length){
            MessageService.showMessageByCode('zh-filter-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-filter-preferences#delete', {
            'NR_OF_FILTERS': filtersToDelete.length
        }).then(function(){
            return ZhFilterPreferences.deleteFilters(filtersToDelete).then(function(){
                configWidget.dataSource.clearCheckedRows();
                return ZhFilterPreferences.updateFilters(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                return MessageService.getMessageByCode('zh-filter-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-filter-preferences#delete_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        });
    };

    /**
     * Set a filter as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     * @param  {Filter} filter       Filter that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultFilter = function(configWidget, filter){
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_FILTER', filter);
    };

    /**
     * Apply Filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     * @param  {Filter} filter       Filter to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedFilter = function(configWidget, filter){
        var listWidget = configWidget.listWidget;

        return ZhFilterPreferences.applyFilter(listWidget, filter).then(function(){
            configWidget.dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == filter.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter       Filter to export
     *
     * @return {Promise<Filter>}     Filter exportado
     */
    this.downloadPreferenceFile = function(filter){
        return ZhFilterPreferences.downloadPreferenceFile(filter);
    };

    /**
     * Open widget to import a new filter from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filter configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget){
        return ZhFilterImportFilterController.getImportFilterWidget().then(function(importFilterWidget){
            importFilterWidget.newRow();
            importFilterWidget.configWidget = configWidget;
            return ScreenService.openPopup(importFilterWidget);
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterConfigWidgetController', ZhFilterConfigWidgetController);
});

// FILE: src/controllers/ZhFilterFloatingCardController.js
/**
 * Floating window controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {$scope}              $scope              Floating window scope
 * @param {$timeout}            $timeout            AngularJS's wrapper for window.setTimeout
 * @param {ApplicationContext}  ApplicationContext  Service to retrieve Zeedhi services
 * @param {templateManager}     templateManager     Service to handle templates
 * @param {metaDataFactory}     metaDataFactory     Service to factory models from metadata
 * @param {ScreenService}       ScreenService       Service to handle screen
 * @param {Query}               Query               Service to build queries
 * @param {eventEngine}         eventEngine         Service to handle events
 * @param {ZHPromise}           ZHPromise           Service to handle promises
 * @param {ZhUtil}              ZhUtil              Service to provice utilities methods
 */
function ZhFilterFloatingCardController($scope, ApplicationContext, templateManager, metaDataFactory, ScreenService, Query, eventEngine, ZHPromise, ZhUtil){

    var widget = $scope.widget;
    var ZhFilterPreferences = ApplicationContext.ZhFilterPreferences;
    var ZhPreferences = ApplicationContext.ZhPreferences;
    var ZhFilterConfigWidgetController = ApplicationContext.ZhFilterConfigWidgetController;
    var ZhFilterCreateFilterConditionsController = ApplicationContext.ZhFilterCreateFilterConditionsController;
    var filterWidget;

    var dataSource = metaDataFactory.factoryDataSource({
        'name': 'zh-filter-preferences#/filter',
        'localStorage': false,
        'lazyLoad': true,
        'rest': true,
        'data': []
    });

    dataSource.afterLoadDataSource(function(){
        return ZhFilterPreferences.addIsDefaultFilterToFilters(dataSource.data, widget.id).then(function(filters){
            dataSource.data = filters.map(function(filter){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                return filter;
            });

        });
    });

    $scope.customizationCard = {
        'label': 'Filters',
        'type': 'filter',
        'openFilterConfiguration': openFilterConfiguration,
        'refreshFilters': refreshFilters,
        'openFilterWidget': openFilterWidget,
        'applyOriginalFilter': applyOriginalFilter,
        'applyFilter': applyPreference,
        'dataSource': dataSource
    };

    ZhFilterPreferences.registerFilterDataSource(widget.id, $scope.customizationCard.dataSource);

    widget.openFilterWidget = openFilterWidget;
    widget.refreshFilters = refreshFilters;

    function updateFilterRow(filterRow, searchCriteria){
        var changed = false;
        filterRow = filterRow || {};
        Object.keys(searchCriteria).forEach(function(key){
            var name = 'SEARCHLIST_' + key;
            var filterValue = filterRow[name];
            if(!Util.isEmptyOrBlank(filterValue) && searchCriteria[key] !== filterValue){
                changed = true;
            }
            filterRow[name] = searchCriteria[key];
        });

        filterRow.changed = changed;
        return filterRow;
    }

    $scope.$watchCollection('widget.searchCriteria', function(searchCriteria){
        if(ZhPreferences.validateWidget(widget)){
            var filterRow = widget.appliedFilter && widget.appliedFilter.FILTER_ROW;
            filterRow = updateFilterRow(filterRow, searchCriteria || {});

            if(filterRow.changed){
                widget.appliedFilter = {
                    ID: 'EDITED',
                    FILTER_ROW: filterRow
                };
            }
            delete filterRow.changed;

            if(widget.appliedFilter){
                ZhFilterPreferences.updateFilterInfo(widget, widget.fields, filterRow, widget.appliedFilter, searchCriteria);
            }
        }
    });

    function openFilterConfiguration(){
        var dataSource = $scope.customizationCard.dataSource;
        return ZhFilterConfigWidgetController.getConfigWidget(widget, dataSource).then(function(configWidget){
            return ScreenService.openPopup(configWidget);
        });
    }

    function refreshFilters(){
        return ZhPreferences.getUserID().then(function(){
            var query = Query.build()
                .where('WIDGET_ID').equals(widget.id);

            return dataSource.load(query.where());
        });
    }

    function openFilterWidget(rowToBeSet){
        return ZhFilterPreferences.openFilterWidget(widget, rowToBeSet);
    }

    function applyOriginalFilter(){
        return ZhFilterPreferences.applyOriginalFilter(widget);
    }

    function applyPreference(filter, reloadWidget){
        ZhFilterPreferences.applyFilter(widget, filter, reloadWidget);
    }

    function getDefaultFilter(){
        return dataSource.data.filter(function(filter){
            return filter.DEFAULT_FILTER;
        }).shift();
    }

    function applyDefaultFilter(){
        var isLazyLoad = Util.getNestedProperty(widget, ['dataSource', 'lazyLoad']);
        var filter = getDefaultFilter() || ZhFilterPreferences.getOriginalFilter(widget.id);

        applyPreference(filter, !isLazyLoad);
    }

    function getDataSourceFilterRow(){
        var filterRow = {};
        (widget.dataSourceFilter || [])
            .filter(function removeCustoms(filter){
                return !filter.isCustomFilter;
            })
            .forEach(function parseToFilterRow(filter){
                filterRow[filter.name] = filter.value;
            });
        return filterRow;
    }

    function initOriginalFilter(){
        var widgetId = widget.id;
        var filter = ZhFilterPreferences.getOriginalFilter(widgetId);
        if(Util.isEmptyObject(filter.FILTER_ROW)){
            var filterRow = getDataSourceFilterRow();
            widget.fields.forEach(function(field){
                var hasFilterDefaultValue = field.filterProperties && field.filterProperties.hasOwnProperty('defaultValue');
                var defaultValue = hasFilterDefaultValue ? field.filterProperties.defaultValue : field.defaultValue;
                if(!Util.isUndefinedOrNull(defaultValue)){
                    filterRow[field.name] = defaultValue;
                }
            });
            ZhFilterPreferences.setOriginalFilter(widgetId, filterRow);
        }

    }

    function isFilterPreferencesActive(activeWidget){
        return Util.get(activeWidget || widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    $scope.$on('$destroy', function() {
        delete widget.openFilterWidget;
        delete widget.refreshFilters;
        delete dataSource.afterLoadDataSourcelistener;
    });

    (function initFilterPreferences(){
        var filterIsActive = isFilterPreferencesActive();
        if(filterIsActive && ZhPreferences.validateWidget(widget)){
            ZhFilterPreferences.updateFilters(widget.id, $scope.customizationCard.dataSource).then(function(){
                if(Util.isUndefinedOrNull(widget.appliedFilter) && widget.filterInitialized !== true){
                    initOriginalFilter();
                    applyDefaultFilter();
                }
            });
        }
    })();

}


// FILE: src/directives/ZhFilterFloatingControl.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

ZeedhiDirectives.directive('zhFloatingControl', ['$timeout', 'templateManager', ZhFilterFloatingControl]);

function ZhFilterFloatingControl($timeout, templateManager){

    function checkFloatingCardOverflow(e){
        var actionElement = $(e.target).closest('.float-action');
        var floatingControl = actionElement.parents('.zh-floating-control');
        var widgetContainer = actionElement.closest('.container');
        var actionTop, actionBottom;
        var widgetTop, widgetBottom;

        if(!actionElement.hasClass('opened')){
            actionElement.removeClass('fix-position-top');
            actionElement.removeClass('fix-position-bottom');

            actionTop = actionElement.offset().top;
            widgetTop = widgetContainer.offset().top;
            widgetBottom = widgetTop + widgetContainer.height();
            if(floatingControl.hasClass('pos-top')){
                actionTop = actionTop - 200 + actionElement.height();
                if(actionTop < widgetTop){
                    actionElement.addClass('fix-position-top');
                }
            } else {
                actionBottom = actionTop + 200;
                if(actionBottom > widgetBottom){
                    actionElement.addClass('fix-position-bottom');
                }
            }
        }
    }

    return {
        priority: 10,
        link: function($scope, element, attrs, controller){
            var widget = $scope.widget;
            if(widget.floatingControl === false){
                return;
            }

            var isFilterApplied = function(){
                return widget && widget.appliedFilter && (Object.keys(widget.appliedFilter.FILTER_ROW).length > 0);
            };

            var defaultOptions = {
                searchAction: Util.isUndefined(element.parent().attr('data-hide-search')) && Util.isUndefined(element.parent().attr('hide-search')),
                openSearch: false
            };
            widget.floatingControl = Util.extend(defaultOptions, widget.floatingControl);

            var isFilterAppliedWithoutOriginal = function(){
                return widget._filterInfo && widget._filterInfo.length > 0 && widget.appliedFilter && (widget.appliedFilter.ID != 'ORIGINAL');
            };

            widget.floatingControl.addStatus({
                priority: 20,
                classString: 'searching',
                icon: 'more',
                conditionFunction: function(){
                    return isFilterAppliedWithoutOriginal();
                },
                infoTemplate: function(){
                    var hideFilterValue = templateManager.project.hideFilterValue || widget.hideFilterValue;
                    var filterInfoValue = hideFilterValue ? '</b> {{info.filterName}}': ':</b> {{info.value}}';
                    if(isFilterApplied()){
                        widget.floatingControl.statuses[1].icon = 'search';
                        if(widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter){
                            icon = 'filter';
                            method = 'openFilter';
                        } else {
                            icon = 'close-x';
                            method = 'clearFilter';
                        }
                        return '<div data-zh-touchstart="'+method+'()"> <div class="searching-control-info"><span data-zh-icon="'+icon+' no-border icon-white"></span><div class="searching-control-info-details"><span ng-repeat="info in widget._filterInfo"><b>{{info.label|i18n}}'+filterInfoValue+'</span></div></div><div class="searching-control-info-text" data-zh-bind-translate="\'Filter applied\'"></div></div>';
                    }
                    widget.floatingControl.statuses[1].icon = 'more';
                },
                scope: $scope
            });

            if(templateManager.project.showEmptyFilter){
                widget.floatingControl.addStatus({
                    priority: 30,
                    classString: 'emptyFilter',
                    icon: 'more',
                    conditionFunction: function(){
                        return !isFilterAppliedWithoutOriginal();
                    },
                    infoTemplate: '<div data-zh-touchstart="openFilter()"> <div class="searching-control-info"><span data-zh-icon="filter no-border icon-white"></span></div><div class="searching-control-info-text"></div></div>',
                    scope: $scope
                });
            }

            $scope.openSearchAction = function(){
                $timeout.cancel($scope._closeTimeout);
                if(!element.hasClass('opened')){
                    $scope.toggleActionMenu();
                }
                $scope.closeOpenedActions();
                var searchAction = element.find('.search-action').closest('.float-action');
                if(searchAction && searchAction.length > 0){
                    searchAction.find("span.search-action-icon").click();
                    Util.selectElement(searchAction.find('input'));
                }
            };
            widget.openSearchAction = $scope.openSearchAction;

            $scope.$on('$destroy', function() {
               delete widget.openSearchAction;
               widget.floatingControl.statuses = [];
            });

            $scope.openFilter = function(){
                $scope.widget.openFilterWidget();
            };

            $scope.clearFilter = function(){
                $scope.widget.searchCriteria = {};
            };

            $scope.togglePreferencesCard = function(e){
                checkFloatingCardOverflow(e);
                $scope.toggleAction(e);
            };

            $scope.clearSearch = function(event, fieldName){
                if(!fieldName){
                    widget.searchCriteria = {};
                } else {
                    delete widget.searchCriteria[fieldName];
                }

                if(event){
                    Util.selectElement($(event.target).closest(".floating-card-input").find("input"));
                }
            };

            $scope.toggleSearchFieldsList = function(event){
                if(!widget.floatingControl.customizationAction || !widget.floatingControl.customizationAction.filter){
                    var el = $(event.target).closest(".floating-card").find('.floating-card-search-field-select');
                    el.toggleClass("opened");

                    if(el.hasClass("opened")){
                        $(event.target).closest(".floating-card").find('.floating-card-select-field-options').removeClass("opened");
                        $(event.target).closest(".floating-card").find('.zh-select-search-floating').removeClass("opened");
                    }
                }
            };

            $scope.selectSearchField = function(field){
                $scope.searchField = field;
                element.find('.floating-card-search-field-select').removeClass('opened');
                element.find('input.zh-input-search-floating').focus();
            };

            $scope.filterFields = function(field){
                return field.aclIsVisible && !field.skipSearch && ((field.isVisible && field.showOnList) || field.showOnFilter);
            };

            $scope.getFields = function(widget) {
                return (widget.fields || []).filter($scope.filterFields);
            };

            if(widget.floatingControl.defaultSearchField){
                $scope.searchField = widget.getField(widget.floatingControl.defaultSearchField);
            }

            if (widget.floatingControl.openSearch) {
                $timeout(widget.openSearchAction, 0, false);
            }
        }
    };

}

// FILE: src/Config.js
(function(){

window.templateUrls = window.templateUrls || [];
window.metadataUrls = window.metadataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-filter-preferences';

var hasTemplateUrl = Boolean(window.templateUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasMetadataUrl = Boolean(window.metadataUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasServiceUrl = Boolean(window.serviceUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

if(!hasTemplateUrl){
    window.templateUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-filter-preferences/dist/templates/'
    });
}

if(!hasMetadataUrl){
    window.metadataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-filter-preferences/dist/assets/'
    });
}

if(!hasServiceUrl){
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: bower_components/zh-layout-preferences/dist/layout-preferences.js
// FILE: src/services/ZhLayoutPreferences.js
/**
 * Layout
 * @typedef {Object} Layout
 *
 * @since 3.0.0
 *
 * @property {String} ID                  Layout unique identifier
 * @property {String} USER_ID             User identifier that created the layout
 * @property {String} ORGANIZATION_ID     Organization the user belongs to
 * @property {String} LABEL_TO_PREFERENCE Layout label
 * @property {String} WIDGET_ID           Widget identifier what the layout belongs to
 * @property {String} LAYOUT_CHANGES      Widget changes saved on the layout
 * @property {String} LAST_MODIFIED_DATE  Last date the layout was modified
 */

/**
 * Service to handle preferences storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences   Service to handle preferences storage
 * @param {TemplateManager}            templateManager Service to handle template
 * @param {EventAggregator}            eventAggregator Service to use project events
 * @param {ContainerMerger}            containerMerger Service to handle layout changes
 * @param {ZHPromise}                  ZHPromise       Service to handle promises
 * @param {EventEngine}                eventEngine     Service to use models events
 * @param {ZhUtil}                     ZhUtil          Service to provice utilities methods
 * @param {MessageService}             MessageService  Service to handle messages
 */
function ZhLayoutPreferences(ZhPreferences, templateManager, eventAggregator, containerMerger, ZHPromise, eventEngine, ZhUtil, MessageService){

    var originalLayout = {
        'ID': 'ORIGINAL',
        'USER_ID': 0,
        'ORGANIZATION_ID': 0,
        'LAYOUT_CHANGES': '{}',
        'LABEL_TO_PREFERENCE': 'Original'
    };

    function shouldShowLayoutAction(widget){
        return Util.get(widget, ['floatingControl', 'customizationAction', 'layout']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function(){
        ZhPreferences.registerRepository('zh-layout-preferences#layout');

        templateManager.project.customFloatingActions.push({
            'name': 'Layout Action',
            'label': 'Layout',
            'type': 'layout',
            'order': 20,
            'icon': 'layout',
            'template': 'zh-layout-preferences#floating-layout-card.html',
            'expandable': true,
            'isVisible': shouldShowLayoutAction
        });
    });

    eventEngine.registerEvent('LayoutBeforeApply', 'beforeLayoutApply');
    eventEngine.registerEvent('LayoutAfterApply', 'afterLayoutApply');

    var layoutRegisteredDataSources = {};

    /**
     * Register layouts datasource
     *
     * @since 3.0.0
     *
     * @param {String} widgetID         Widget identifier
     * @param {Object} datasource       Datasource that contains the layouts
     *
     */
    this.registerLayoutDataSource = function(widgetID, dataSource){
        if(widgetID){
            layoutRegisteredDataSources[widgetID] = dataSource;
        }

        return layoutRegisteredDataSources;
    };

    /**
     * Update dataSource with local saved layouts
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded layouts
     *
     * @return {Promise}               Promise resolved after the layouts are loaded into the dataSource
     */
    this.updateLayouts = function(widgetID, dataSource){
        if(!dataSource){
            dataSource = layoutRegisteredDataSources[widgetID];
        }

        return this.getWidgetLayouts(widgetID).then(function(layouts){
            return layouts;
        }).then(function(layouts){
            return this.addIsDefaultLayoutToLayouts(layouts, widgetID).then(function(layouts){
                dataSource.data = layouts;
            });
        }.bind(this));
    };

    /**
     * Add property to identify if the layout is selected as default
     *
     * @since 3.0.0
     *
     * @param {Layout[]} layouts  List of layouts to update the default layout property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultLayoutToLayouts = function(layouts, widgetID){
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){
            var defaultLayout = defaultPreferences.DEFAULT_LAYOUT;
            return layouts.map(addIsDefaultLayoutToLayout.bind(this, defaultLayout));
        });
    };

    function addIsDefaultLayoutToLayout(defaultLayout, layout){
        layout.DEFAULT_LAYOUT = layout.ID === defaultLayout;
        return layout;
    }

    /**
     * Retrieve local cached layouts for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getWidgetLayouts = function(widgetID){
        return ZhPreferences.getWidgetPreferences('layout', widgetID);
    };

    /**
     * Retrieve local cached layouts
     *
     * @since 3.0.0
     *
     * @param  {Query} query       Conditions to filter the preferences
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getLayouts = function(query){
        return ZhPreferences.getPreferences('layout', query);
    };

    /**
     * Persist layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts  Layouts to persist
     *
     * @return {Promise<Layout[]>} List of persisted layouts
     */
    this.saveLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.savePreferences('layout', cloneLayouts).then(function(response){
            cloneLayouts.forEach(function(clone, index){
                layouts[index].ID = clone.ID;
                layouts[index].__is_new = clone.__is_new;
                layouts[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts Layouts to delete
     *
     * @return {Promise<Layout[]>} List of deleted layouts
     */
    this.deleteLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.deletePreferences('layout', cloneLayouts);
    };

    function triggerBeforeLayoutApplyEvent(widget, layout){
        if(widget.beforeLayoutApply){
            widget.beforeLayoutApply({
                'layout': layout
            });
        }
    }

    function triggerAfterLayoutApplyEvent(widget, layout){
        if(widget.afterLayoutApply){
            widget.afterLayoutApply({
                'layout': layout
            });
        }
    }

    var appliedPreference = {ID: 'ORIGINAL'};

    /**
     * Apply original layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original layout on
     *
     * @return {Promise}       Promise resolved after the original layout is applied
     */
    this.applyOriginalLayout = function(widget){
        triggerBeforeLayoutApplyEvent(widget, originalLayout);

        containerMerger.restore(widget, 'ORIGINAL');
        widget.redraw();
        widget.setAppliedLayout(originalLayout);
        appliedPreference = originalLayout;

        triggerAfterLayoutApplyEvent(widget, originalLayout);

        return ZHPromise.when();
    };

    /**
     * Apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget     Widget to apply the layout on
     * @param  {Layout} preference Layout preference to be applied
     *
     * @return {Promise}           Promise that is resolved after the layout is applied
     */
    this.applyLayout = function(widget, preference){
        triggerBeforeLayoutApplyEvent(widget, preference);

        if(widget.backupPDFOrientation === undefined){
            widget.backupPDFOrientation = widget.reportOrientation || 'PORTRAIT';
        }
        widget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

        var layoutChanges = prepareLayout(JSON.parse(preference.LAYOUT_CHANGES));
        containerMerger.merge(templateManager.container, layoutChanges, appliedPreference.ID);

        widget.redraw(true);

        widget.setAppliedLayout(preference);
        appliedPreference = preference;

        triggerAfterLayoutApplyEvent(widget, preference);

        return ZHPromise.when();
    };

    var propertyToBeVerified = {
        container: ['widgets'],
        widget: ['widgets', 'fields']
    };

    function prepareLayout(objToBeMerged, baseObject){
        baseObject = baseObject || templateManager.container;
        var childProperties = [];

        if(baseObject instanceof Container){
            childProperties = propertyToBeVerified.container;
        } else if(baseObject instanceof Widget){
            childProperties = propertyToBeVerified.widget;
        }

        childProperties.forEach(function prepareChildLayout(propertyName){
            var childBaseObj = baseObject[propertyName];
            var childObjToBeMerged = objToBeMerged[propertyName];
            if(childObjToBeMerged){
                if(childBaseObj && childBaseObj.length){
                    var existentIds = childBaseObj.map(function returnObjId(obj){
                        return obj.id;
                    });
                    objToBeMerged[propertyName] = childObjToBeMerged.filter(function filterChilds(obj){
                        var objIndex = existentIds.indexOf(obj.id);
                        if(objIndex != -1){
                            prepareLayout(obj, childBaseObj[objIndex]);
                        }
                        return ~objIndex;
                    });
                } else {
                    objToBeMerged[propertyName] = [];
                }
            }
        });

        return objToBeMerged;
    }

    /**
     * Export layout file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout  Layout to export
     *
     * @return {Promise<Layout>} Layout exported
     */
    this.downloadPreferenceFile = function(layout){
        var fileName = layout.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index){
            if(+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.layout';

        var layoutToExport = {
            'ID': layout.ID,
            'LABEL_TO_PREFERENCE': layout.LABEL_TO_PREFERENCE,
            'WIDGET_ID': layout.WIDGET_ID,
            'PDF_REPORT_ORIENTATION': layout.PDF_REPORT_ORIENTATION,
            'LAYOUT_CHANGES': JSON.parse(layout.LAYOUT_CHANGES),
            'LAST_MODIFIED_DATE': layout.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(layoutToExport, fileName);
    };

    /**
     * Validade layout to import
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout              Layout to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} layoutDataSource  Existing layouts data (optional)
     *
     * @return {Boolean} Layout valid
     */
    this.validateLayoutImport = function(layout, configWidget, layoutDataSource){
        layoutDataSource = layoutDataSource || configWidget.dataSource.data;

        if(configWidget.listWidget.id != layout.WIDGET_ID){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_widget_id');
            return false;
        }

        var layoutsWithSameId = layoutDataSource.filter(function(row){
            return row.ID == layout.ID;
        });

        if(layoutsWithSameId.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_id_violation');
            return false;
        }

        var layoutsWithSameLabel = layoutDataSource.filter(function(row){
            return row.LABEL_TO_PREFERENCE == layout.LABEL_TO_PREFERENCE;
        });

        if(layoutsWithSameLabel.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_name_violation');
            return false;
        }

        return true;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutPreferences', ZhLayoutPreferences);
});

// FILE: src/controllers/ZhLayoutWidgetsBuilder.js
/**
 * Service to create widgets to handle layout
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function ZhLayoutWidgetsBuilder(restEngine, templateManager, metaDataFactory){

    function getWidgetJSON(widgetName){
        var widgetPath = 'zh-layout-preferences#json/widgets/'+widgetName+'.json';
        return restEngine.requestMetaData(widgetPath).then(function(response){
            return response.data;
        });
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName){
        return getWidgetJSON(widgetName).then(function(widget){
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored layout configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Layout configuration widget
     */
    this.factoryLayoutConfigurationWidget = function(){
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create layout widget
     */
    this.factoryCreateLayoutWidget = function(){
        return this.factoryWidget('createLayout');
    };

    /**
     * Retrieve factored import layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import layout widget
     */
    this.factoryImportLayoutWidget = function(){
        return this.factoryWidget('importLayout');
    };

    /**
     * Retrieve factored build layout options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout options widget
     */
    this.factoryBuildLayoutOptionsWidget = function(){
        return this.factoryWidget('createLayoutOptions');
    };

    /**
     * Retrieve factored build layout field options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout field options widget
     */
    this.factoryBuildLayoutFieldOptionsWidget = function(){
        return this.factoryWidget('createLayoutFieldOptions');
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutWidgetsBuilder', ZhLayoutWidgetsBuilder);
});

// FILE: src/controllers/ZhLayoutCreateLayoutOptionsController.js
/**
 * Controller to manipulate create layout options widget
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                             MessageService                             Service to show messages
 * @param {ZhLayoutPreferences}                        ZhLayoutPreferences                        Service to handle layout preferences methods
 * @param {ScreenService}                              ScreenService                              Service to manipulate screen
 * @param {ZhLayoutWidgetsBuilder}                     ZhLayoutWidgetsBuilder                     Service to create widgets to handle layouts
 * @param {ZHPromise}                                  ZHPromise              Service to handle promises
 */
function ZhLayoutCreateLayoutOptionsController(MessageService, ZhLayoutPreferences, ScreenService, ZhLayoutWidgetsBuilder, metaDataFactory, ZHPromise){

    var fieldDefaultProperties = {
        isVisible: true,
        showOnList: true,
        visibility: true,
        grouped: false,
        fixed: false
    };

    var createLayoutOptionsWidgetPromise = null;

    function removeConflictingProperties(field){
        var conflictingProperties = ['selectWidget', 'inlineEditable'];
        conflictingProperties.forEach(function(property){
            delete field[property];
        });
    }

    function factoryFields(parentWidget, gridFields){
        return gridFields.filter(function(field){
            return !field.skipLayout;
        }).map(function(field){
            var factoredField = metaDataFactory.fieldFactory(field, parentWidget);
            removeConflictingProperties(factoredField);
            return factoredField;
        });
    }

    /**
     * Retrieve widget that handles create layout options
     *
     * @since 3.0.0
     *
     * @param  {Layout} preference   Layout to be changed
     * @param  {Widget} configWidget Widget that handles layouts configuration
     *
     * @return {Widget}              Widget that handles create layout options
     */
    this.getCreateLayoutOptionsWidget = function(preference, configWidget){
        if(createLayoutOptionsWidgetPromise === null){
            createLayoutOptionsWidgetPromise = ZhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget();
        }

        return createLayoutOptionsWidgetPromise.then(function(layoutOptionsWidget){
            var label = preference.LABEL_TO_PREFERENCE;

            var listWidget = configWidget.listWidget;

            layoutOptionsWidget.preference = preference;
            layoutOptionsWidget.label = ScreenService.i18n('Layout');
            if(label){
                layoutOptionsWidget.label = layoutOptionsWidget.label + ' - ' + label;
            }
            layoutOptionsWidget.layoutName = label;
            layoutOptionsWidget.reportTitle = listWidget.label;
            layoutOptionsWidget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

            // get only the first two rows from listWidget
            layoutOptionsWidget.dataSource.data = Util.clone(listWidget.dataSource.data.slice(0,2));

            // get fields from listWidget
            layoutOptionsWidget.fields = factoryFields(layoutOptionsWidget, listWidget.fields);

            var layoutChanges = JSON.parse(preference.LAYOUT_CHANGES);
            var widgetDifferences = getEditingWidgetPath(layoutChanges.widgets, listWidget);

            layoutOptionsWidget.fields = layoutOptionsWidget.fields.map(function(field, index){
                delete field.__isSelected;
                delete field.__leftActions;
                delete field.__notSelectable;
                delete field.__rightActions;

                if(!Util.isEmptyOrBlank(widgetDifferences.fields)){
                    var fieldPreference = filterObject(widgetDifferences.fields, field.id, "id");
                    if(fieldPreference.obj && Object.keys(fieldPreference.obj).length){
                        return Util.extend(field, fieldPreference.obj);
                    }
                }

                return Util.extend(fieldDefaultProperties, field, true);
            });

            return layoutOptionsWidget;
        });
    };

    /**
     * Save and apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} createLayoutOptionsWidget Widget that handles layout options
     *
     * @return {Promise}                          Promise resolved when the layout is applied
     */
    this.saveAndApplyLayout = function(createLayoutOptionsWidget){
        if(createLayoutOptionsWidget.gridConfigWidget.isValid()){
            var configWidget = createLayoutOptionsWidget.createNewLayoutWidget.configWidget;
            var listWidget = configWidget.listWidget;

            var fields = getFieldEditingWidgetAsDataSource(createLayoutOptionsWidget);
            var layout = createLayoutOptionsWidget.preference;

            var layoutChanges = JSON.parse(layout.LAYOUT_CHANGES);
            var editingWidget = getEditingWidgetPath(layoutChanges.widgets, listWidget);
            editingWidget.fields = fields;

            layout.PDF_REPORT_ORIENTATION = createLayoutOptionsWidget.reportOrientation;
            layout.LABEL_TO_PREFERENCE = createLayoutOptionsWidget.layoutName;
            layout.LAYOUT_CHANGES = JSON.stringify(layoutChanges);

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(layouts){
                return ZhLayoutPreferences.applyLayout(listWidget, layouts[0]);
            }).then(function(){
                ScreenService.closePopup(true);
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                listWidget.activate();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        } else {
            return ZHPromise.reject("Invalid params");
        }
    };

    /**
     * Change field visibility
     *
     * @since 3.0.0
     *
     * @param {Field} field Field to change the visibility
     */
    this.updateFieldVisibility = function(field){
        field.isVisible = true;
        field.showOnList = field.isVisibleOnReport = field.visibility;
    };

    function getFieldEditingWidgetAsDataSource(listWidget){
        return listWidget.fields.map(function(field){
            var data = {
                'id': field.id,
                'name': field.name,
                'label': field.label,
                'labelOnReport': field.labelOnReport || null,
                'expression': field.expression || null,
                'align': field.align || null,
                'isVisible': field.isVisible,
                'showOnList': field.showOnList,
                'minWidth': field.gridColumnSize || null,
                'maxWidth': field.gridColumnSize || null,
                'gridColumnSize': field.gridColumnSize || null,
                'reportColumnSize': field.reportColumnSize || null,
                'isVisibleOnReport': field.isVisibleOnReport,
                'sequenceInList': field.sequenceInList,
                'visibility': field.isVisible && field.showOnList,
                'grouped': field.grouped || false,
                'groupOpened': field.groupOpened,
                'fixed': field.fixed || false,
                'skipLayout': field.skipLayout || false
            };
            return data;
        }).filter(function(field){
            return !field.skipLayout;
        });
    }

    function getEditingWidgetPath(widgetsArray, listWidget){
        var foundWidget = getWidgetObjectByID(widgetsArray, listWidget.id);
        if(!foundWidget){
            widgetsArray.some(function(eachWidget){
                foundWidget = getEditingWidgetPath(eachWidget.widgets, listWidget);
                return foundWidget;
            });
        }
        return foundWidget;
    }

    function filterObject(obj, value, key){
        var filtered = obj.filter(function(preference){
            return preference[key] == value;
        });

        var toReturn = {};
        toReturn.index = obj.indexOf(filtered[0]);
        toReturn.obj = filtered[0];
        return toReturn;
    }

    function getWidgetObjectByID(widgets, widgetID){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetID;
        }).shift();
    }

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutCreateLayoutOptionsController', ZhLayoutCreateLayoutOptionsController);
});

// FILE: src/controllers/ZhLayoutImportLayoutController.js
/**
 * Import layout widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                      Service to handle layout preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder Service to create widgets to handle layouts
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 */
function ZhLayoutImportLayoutController(MessageService, ZhLayoutPreferences, ZhPreferences, ZhLayoutWidgetsBuilder, ZHPromise, ScreenService){

    var importLayoutWidgetPromise = null;

    /**
     * Retrieve widget to import layout
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportLayoutWidget = function(){
        if(importLayoutWidgetPromise === null){
            importLayoutWidgetPromise = ZhLayoutWidgetsBuilder.factoryImportLayoutWidget();
        }

        return importLayoutWidgetPromise;
    };

    /**
     * Import layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} importLayoutWidget Widget with the layout configuration
     *
     * @return {Promise}                      Promise resolved after the layout is imported
     */
    this.importPreference = function(importLayoutWidget){
        if(!importLayoutWidget.isValid()){
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }

        var layoutFileData = importLayoutWidget.fields[0].value()[0].b64File;
        layoutFileData = layoutFileData.split('base64,')[1];
        layoutFileData = window.atob(layoutFileData);
        layoutFileData = decodeURIComponent(escape(layoutFileData));

        var layout = JSON.parse(layoutFileData);

        var configWidget = importLayoutWidget.configWidget;
        var listWidget = configWidget.listWidget;

        if(!ZhLayoutPreferences.validateLayoutImport(layout, configWidget)){
            return ZHPromise.reject({err: 'LAYOUT_FILE_INVALID'});
        }

        layout.__is_new = true;
        layout.__generate_id = false;
        layout.LAYOUT_CHANGES = JSON.stringify(layout.LAYOUT_CHANGES);

        return ZhPreferences.getUserID().then(function(userID){
            layout.USER_ID = userID;

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(){
                ScreenService.closePopup();
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                configWidget.refreshWidget();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutImportLayoutController', ZhLayoutImportLayoutController);
});

// FILE: src/controllers/ZhLayoutConfigController.js
/**
 * Layout configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}                         ZhPreferences                         Service to handle preferences persistance
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                   Service to handle layout preferences methods
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder                Service to create widgets to handle layouts
 * @param {ScreenService}                         ScreenService                         Service to manipulate screen
 * @param {MessageService}                        MessageService                        Service to show messages
 * @param {ZHPromise}                             ZHPromise                             Service to handle promises
 * @param {$timeout}                              $timeout                              Service to handle timeouts
 * @param {ZhLayoutCreateLayoutOptionsController} ZhLayoutCreateLayoutOptionsController Controller to manipulate create layout options widget
 * @param {ZhLayoutImportLayoutController}        ZhLayoutImportLayoutController        Controller to manipulate import layout widget
 */
function ZhLayoutConfigController(ZhPreferences, ZhLayoutPreferences, ZhLayoutWidgetsBuilder, ScreenService, MessageService, ZHPromise, $timeout, ZhLayoutCreateLayoutOptionsController, ZhLayoutImportLayoutController){

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles layouts configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create layouts for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles layouts configuration
     */
    this.getConfigWidget = function(listWidget, dataSource){
        if(configWidgetPromise === null){
            configWidgetPromise = ZhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args){
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id}
            ];
            dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == (listWidget.appliedLayout ? listWidget.appliedLayout.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    /**
     * Open widget to create a new layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateLayoutWidget = function(configWidget){
        var listWidget = configWidget.listWidget;
        var pdfReportOrientation = listWidget.reportOrientation || "PORTRAIT";

        return ZhPreferences.getUserID().then(function(userID){
            return {
                '__is_new': true,
                'ID': null,
                'WIDGET_ID': listWidget.id,
                'USER_ID': userID,
                'LAYOUT_CHANGES': JSON.stringify(createDiffDataStructure(listWidget.container, listWidget)),
                'PDF_REPORT_ORIENTATION': pdfReportOrientation,
                'LAST_MODIFIED_DATE': null
            };
        }).then(function(preference){
            return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(createLayoutOptionsWidget){
                createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

                $timeout(function(){
                    ScreenService.openPopup(createLayoutOptionsWidget);
                });
            });
        });

    };

    function createDiffDataStructure(container, widget){
        var containerDiff = {};
        createWidgetsDataStructure(widget.id, container.widgets, containerDiff);
        return containerDiff;
    }

    function createWidgetsDataStructure(widgetID, widgets, containerDiff){
        containerDiff.widgets = [];
        createWidgetsObjects(containerDiff.widgets, widgets, widgetID);
        if(Util.isArray(widgets) && widgets.length > 0){
            widgets.forEach(function(eachWidget){
                createWidgetsDataStructure(widgetID, eachWidget.widgets, getWidgetObjectByID(containerDiff.widgets, eachWidget.id));
            });
        }
    }

    function createWidgetsObjects(widgetsDiff, widgetsArray, widgetID){
        if(Util.isArray(widgetsArray) && widgetsArray.length > 0){
            widgetsArray.forEach(function(eachWidget){
                var properties = {
                    "id": eachWidget.id
                };
                if(eachWidget.id == widgetID){
                    properties.fields = [];

                }
                widgetsDiff.push(properties);
            });
        }
    }

    function getWidgetObjectByID(widgets, widgetId){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetId;
        }).shift();
    }

    /**
     * Open popup to edit a saved layout
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout   Layout to edit
     *
     * @return {Promise<Widget>} Widget used to edit the layout
     */
    this.editLayout = function(configWidget, layout){
        return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(layout, configWidget).then(function(createLayoutOptionsWidget){
            createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

            $timeout(function(){
                ScreenService.openPopup(createLayoutOptionsWidget);
            });
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the layouts to delete
     *
     * @return {Promise}             Promise resolved after layouts are deleted
     */
    this.deleteLayouts = function(configWidget){
        var layoutsToDelete = configWidget.getCheckedRows();

        var defaultLayout = layoutsToDelete.filter(function(layout){
            return layout.DEFAULT_LAYOUT;
        });
        if(defaultLayout.length){
            MessageService.showMessageByCode('zh-layout-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-layout-preferences#delete', {
            'NR_OF_LAYOUTS': layoutsToDelete.length
        }).then(function(){
            return ZhLayoutPreferences.deleteLayouts(layoutsToDelete).then(function(){
                configWidget.dataSource.clearCheckedRows();
                return ZhLayoutPreferences.updateLayouts(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                return MessageService.getMessageByCode('zh-layout-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-layout-preferences#delete_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        });
    };

    /**
     * Set a layout as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultLayout = function(configWidget, layout){
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_LAYOUT', layout);
    };

    /**
     * Apply Layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedLayout = function(configWidget, layout){
        var listWidget = configWidget.listWidget;

        return ZhLayoutPreferences.applyLayout(listWidget, layout).then(function(){
            configWidget.dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == layout.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout       Layout to export
     *
     * @return {Promise<Layout>}     Layout exportado
     */
    this.downloadPreferenceFile = function(layout){
        return ZhLayoutPreferences.downloadPreferenceFile(layout);
    };

    /**
     * Open widget to import a new layout from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget){
        return ZhLayoutImportLayoutController.getImportLayoutWidget().then(function(importLayoutWidget){
            importLayoutWidget.newRow();
            importLayoutWidget.configWidget = configWidget;
            return ScreenService.openPopup(importLayoutWidget);
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutConfigController', ZhLayoutConfigController);
});

// FILE: src/controllers/ZhLayoutFloatingCardController.js
/**
 * Floating window controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {$scope}              $scope              Floating window scope
 * @param {ApplicationContext}  ApplicationContext  Service to retrieve Zeedhi services
 * @param {Query}               Query               Service to build queries
 * @param {ScreenService}       ScreenService       Service to handle screen
 * @param {metaDataFactory}     metaDataFactory     Service to factory models from metadata
 */
function ZhLayoutFloatingCardController($scope, ApplicationContext, Query, ScreenService, metaDataFactory){

    var widget = $scope.widget;
    $scope.appliedPreference = null;

    var ZhLayoutPreferences = ApplicationContext.ZhLayoutPreferences;
    var ZhPreferences = ApplicationContext.ZhPreferences;
    var ZhLayoutConfigController = ApplicationContext.ZhLayoutConfigController;

    var fieldDefaultProperties = {
        isVisible: true,
        showOnList: true,
        visibility: true,
        grouped: false,
        fixed: false
    };

    var dataSource = metaDataFactory.factoryDataSource({
        'name': 'zh-layout-preferences#/layout',
        'localStorage': false,
        'lazyLoad': true,
        'rest': true,
        'data': []
    });

    dataSource.afterLoadDataSource(function(){
        return ZhLayoutPreferences.addIsDefaultLayoutToLayouts(dataSource.data, widget.id).then(function(layouts){
            dataSource.data = layouts;
        });
    });

    $scope.customizationCard = {
        'label': 'Layouts',
        'type': 'layout',
        'openLayoutConfiguration': openLayoutConfiguration,
        'refreshLayouts': refreshLayouts,
        'applyOriginalLayout': applyOriginalLayout,
        'applyLayout': applyPreference,
        'dataSource': dataSource
    };

    ZhLayoutPreferences.registerLayoutDataSource(widget.id, $scope.customizationCard.dataSource);

    widget.setAppliedLayout = setAppliedLayout;

    $scope.$on('$destroy', function() {
        delete dataSource.afterLoadDataSourcelistener;
    });

    function setAppliedLayout(layout){
        $scope.appliedPreference = layout;
        widget.appliedLayout = layout;
    }

    function openLayoutConfiguration(){
        var dataSource = $scope.customizationCard.dataSource;
        return ZhLayoutConfigController.getConfigWidget(widget, dataSource).then(function(configWidget){
            return ScreenService.openPopup(configWidget);
        });
    }

    function refreshLayouts(){
        return ZhPreferences.getUserID().then(function(){
            var query = Query.build()
                .where('WIDGET_ID').equals(widget.id);

            return $scope.customizationCard.dataSource.load(query.where());
        });
    }

    function applyOriginalLayout(){
        return ZhLayoutPreferences.applyOriginalLayout(widget);
    }

    function applyPreference(layout){
        return ZhLayoutPreferences.applyLayout(widget, layout);
    }

    function getDefaultLayout(){
        return dataSource.data.filter(function(layout){
            return layout.DEFAULT_LAYOUT;
        }).shift();
    }

    function applyDefaultLayout(){
        var layout = getDefaultLayout();
        if(layout){
            applyPreference(layout);
        } else {
            applyOriginalLayout();
        }
    }

    (function initLayoutPreferences(){
        if(ZhPreferences.validateWidget(widget)){
            ZhLayoutPreferences.updateLayouts(widget.id, dataSource).then(function(){
                if(Util.isUndefinedOrNull(widget.appliedLayout)){
                    applyDefaultLayout();
                } else {
                    applyPreference(widget.appliedLayout);
                }
            });
        }
    })();

}

// FILE: src/Config.js
(function(){

window.templateUrls = window.templateUrls || [];
window.metadataUrls = window.metadataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-layout-preferences';

var hasTemplateUrl = Boolean(window.templateUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasMetadataUrl = Boolean(window.metadataUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasServiceUrl = Boolean(window.serviceUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

if(!hasTemplateUrl){
    window.templateUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-layout-preferences/dist/templates/'
    });
}

if(!hasMetadataUrl){
    window.metadataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-layout-preferences/dist/assets/'
    });
}

if(!hasServiceUrl){
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: bower_components/zh-customization-preferences/dist/view-preferences.js
// FILE: src/services/ZhViewPreferences.js
/**
 * A view preference
 * @typedef {Object} View
 * @property {String}  ID                  Filter unique identifier
 * @property {String}  USER_ID             User identifier that created the filter
 * @property {String}  ORGANIZATION_ID     Organization the user belongs to
 * @property {String}  LABEL_TO_PREFERENCE Filter label
 * @property {String}  WIDGET_ID           Widget identifier the filter belongs to
 * @property {Integer} FILTER_ID           Filter identifies
 * @property {Integer} LAYOUT_ID           Layout identifies
 * @property {String}  LAST_MODIFIED_DATE  Last date the filter was modified
 */

/**
 * Service to handle layouts storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}       ZhPreferences       Service to handle preferences storage
 * @param {ZhFilterPreferences} ZhFilterPreferences Service to handle filters
 * @param {ZhLayoutPreferences} ZhLayoutPreferences Service to handle layouts
 * @param {TemplateManager}     templateManager     Service to handle template
 * @param {EventAggregator}     eventAggregator     Service to use project events
 * @param {ZHPromise}           ZHPromise           Service to handle promises
 * @param {Query}               Query               Service to build queries
 * @param {EventEngine}         eventEngine         Service to handle events
 * @param {ZhUtil}              ZhUtil              Service to provice utilities methods
 * @param {MessageService}      MessageService      Service to handle messages
 */
function ZhViewPreferences(ZhPreferences, ZhFilterPreferences, ZhLayoutPreferences, templateManager, eventAggregator, ZHPromise, Query, eventEngine, ZhUtil, MessageService) {

    var originalView = {
        'ID': 'ORIGINAL',
        'LABEL_TO_PREFERENCE': 'Original',
        'FILTER_ID': 'ORIGINAL',
        'LAYOUT_ID': 'ORIGINAL'
    };

    function shouldShowViewAction(widget) {
        return Util.get(widget, ['floatingControl', 'customizationAction', 'view']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function() {
        ZhPreferences.registerRepository('zh-view-preferences#view');

        templateManager.project.customFloatingActions.push({
            'name': 'View Action',
            'label': 'View',
            'type': 'view',
            'order': 30,
            'icon': 'view',
            'template': 'zh-view-preferences#floating-view-card.html',
            'expandable': true,
            'isVisible': shouldShowViewAction
        });
    });

    eventEngine.registerEvent('ViewBeforeApply', 'beforeViewApply');
    eventEngine.registerEvent('ViewAfterApply', 'afterViewApply');

    /**
     * Update dataSource with local saved views
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded views
     *
     * @return {Promise}               Promise resolved after the views are loaded into the dataSource
     */
    this.updateViews = function(widgetID, dataSource) {
        return this.getWidgetViews(widgetID).then(function(views) {
            return views;
        }).then(function(views) {
            return this.addIsDefaultViewToViews(views, widgetID).then(function(views) {
                dataSource.data = views;
            });
        }.bind(this));
    };

    /**
     * Add property to identify if the view is selected as default
     *
     * @since 3.0.0
     *
     * @param {View[]}   views    List of views to update the default view property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultViewToViews = function(views, widgetID) {
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences) {
            var defaultView = defaultPreferences.DEFAULT_VIEW;
            return views.map(addIsDefaultViewToView.bind(this, defaultView));
        });
    };

    function addIsDefaultViewToView(defaultView, view) {
        view.DEFAULT_VIEW = view.ID === defaultView;
        return view;
    }

    /**
     * Retrieve local cached views for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<View[]>}   View preferences
     */
    this.getWidgetViews = function(widgetID) {
        return ZhPreferences.getWidgetPreferences('view', widgetID);
    };

    /**
     * Persist views
     *
     * @since 3.0.0
     *
     * @param  {View[]} views    Views to persist
     *
     * @return {Promise<view[]>} List of persisted views
     */
    this.saveViews = function(views) {
        var cloneViews = views.map(function(view) { return ZhUtil.clearRow(view); });
        return ZhPreferences.savePreferences('view', cloneViews).then(function(response) {
            cloneViews.forEach(function(clone, index) {
                views[index].ID = clone.ID;
                views[index].__is_new = clone.__is_new;
                views[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete views
     *
     * @since 3.0.0
     *
     * @param  {View[]} views    Views to delete
     *
     * @return {Promise<View[]>} List of deleted views
     */
    this.deleteViews = function(views) {
        var cloneViews = views.map(function(view) { return ZhUtil.clearRow(view); });
        return ZhPreferences.deletePreferences('view', cloneViews);
    };

    function triggerBeforeViewApplyEvent(widget, view) {
        if (widget.beforeViewApply) {
            widget.beforeViewApply({
                'view': view
            });
        }
    }

    function triggerAfterViewApplyEvent(widget, view) {
        if (widget.afterViewApply) {
            widget.afterViewApply({
                'view': view
            });
        }
    }

    /**
     * Apply original view
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original view on
     *
     * @return {Promise}       Promise resolved after the filter and layout are applied
     */
    this.applyOriginalView = function(widget) {
        var filter = ZhFilterPreferences.applyOriginalFilter(widget);
        var layout = ZhLayoutPreferences.applyOriginalLayout(widget);

        triggerBeforeViewApplyEvent(widget, originalView);

        return ZHPromise.all([filter, layout]).then(function() {
            widget.setAppliedView(originalView);

            triggerAfterViewApplyEvent(widget, originalView);
        });
    };

    /**
     * Apply view
     *
     * @since 3.0.0
     *
     * @param  {Widget}  widget              Widget to apply the original view on
     * @param  {View}    preference          Preference to be applied on widget
     * @param  {Boolean} [reloadWidget=true] Preference to be applied on widget
     *
     * @return {Promise}           Promise resolved after the filter and layout are applied
     */
    this.applyView = function(widget, preference, reloadWidget) {
        reloadWidget = reloadWidget !== false;
        var filterQuery = Query.build()
            .where('ID').equals(preference.FILTER_ID)
            .where('WIDGET_ID').equals(widget.id);
        var layoutQuery = Query.build()
            .where('ID').equals(preference.LAYOUT_ID)
            .where('WIDGET_ID').equals(widget.id);

        var filter = ZhFilterPreferences.getFilters(filterQuery);
        var layout = ZhLayoutPreferences.getLayouts(layoutQuery);

        triggerBeforeViewApplyEvent(widget, preference);

        return ZHPromise.all([filter, layout]).then(function(args) {
            var filter = args[0].pop();
            var layout = args[1].pop();

            var filterStatus = ZhFilterPreferences.applyFilter(widget, filter, reloadWidget);
            var layoutStatus = ZhLayoutPreferences.applyLayout(widget, layout);

            return ZHPromise.all([filterStatus, layoutStatus]);
        }).then(function() {
            widget.setAppliedView(preference);

            triggerAfterViewApplyEvent(widget, preference);
        });
    };

    /**
     * Export view file
     *
     * @since 3.0.0
     *
     * @param  {View} view  View to export
     *
     * @return {Promise<View>} view exported
     */
    this.downloadPreferenceFile = function(view) {
        var fileName = view.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index) {
            if (+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.view';

        var viewToExport = {
            'ID': view.ID,
            'LABEL_TO_PREFERENCE': view.LABEL_TO_PREFERENCE,
            'WIDGET_ID': view.WIDGET_ID,
            'FILTER_ID': view.FILTER_ID,
            'LAYOUT_ID': view.LAYOUT_ID,
            'LAST_MODIFIED_DATE': view.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(viewToExport, fileName);
    };

    function getObjectFromBase64(b64) {
        b64 = b64.split('base64,')[1];
        b64 = window.atob(b64);
        b64 = decodeURIComponent(escape(b64));

        return JSON.parse(b64);
    }

    /**
     * Validade view to import
     *
     * @since 3.0.0
     *
     * @param  {View} view                  View to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} layouts           Existing layouts data
     * @param  {Object[]} filters           Existing filters data
     * @param  {Object[]} viewDataSource    Existing views data (optional)
     *
     * @return {Boolean} View valid
     */
    this.validateViewImport = function(view, importViewWidget, layouts, filters, viewDataSource) {
        var configWidget = importViewWidget.configWidget;

        viewDataSource = viewDataSource || configWidget.dataSource.data;

        var layoutFileField = importViewWidget.getField('layoutFile');
        var filterFileField = importViewWidget.getField('filterFile');
        var layoutFileFieldGroup = importViewWidget.fieldGroups[1];
        var filterFileFieldGroup = importViewWidget.fieldGroups[2];

        if (configWidget.listWidget.id != view.WIDGET_ID) {
            MessageService.showMessageByCode('zh-view-preferences#import_error_widget_id');
            return false;
        }

        var viewsWithSameId = viewDataSource.filter(function(row) {
            return row.ID == view.ID;
        });

        if (viewsWithSameId.length > 0) {
            MessageService.showMessageByCode('zh-view-preferences#import_error_id_violation');
            return false;
        }

        var viewsWithSameLabel = viewDataSource.filter(function(row) {
            return row.LABEL_TO_PREFERENCE == view.LABEL_TO_PREFERENCE;
        });

        if (viewsWithSameLabel.length > 0) {
            MessageService.showMessageByCode('zh-view-preferences#import_error_name_violation');
            return false;
        }

        var selectedLayout = layouts.filter(function(layout) {
           return layout.ID == view.LAYOUT_ID;
        });

        if (selectedLayout.length == 0 && layoutFileField.value().length == 0) {
            MessageService.showMessageByCode('zh-view-preferences#import_error_layout_dependency');
            layoutFileFieldGroup.opened = true;
            return false;
        }

        if (layoutFileField.value().length > 0) {
            var layout = getObjectFromBase64(layoutFileField.value()[0].b64File);
            if (layout.ID != view.LAYOUT_ID) {
                MessageService.showMessageByCode('zh-view-preferences#import_error_layout_dependency2');
                return false;
            }
        }

        var selectedFilter = filters.filter(function(filter) {
           return filter.ID == view.FILTER_ID;
        });

        if (selectedFilter.length == 0 && filterFileField.value().length == 0) {
            MessageService.showMessageByCode('zh-view-preferences#import_error_filter_dependency');
            filterFileFieldGroup.opened = true;
            return false;
        }

        if (filterFileField.value().length > 0) {
            var filter = getObjectFromBase64(filterFileField.value()[0].b64File);
            if (filter.ID != view.FILTER_ID) {
                MessageService.showMessageByCode('zh-view-preferences#import_error_filter_dependency2');
                return false;
            }
        }

        return true;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhViewPreferences', ZhViewPreferences);
});

// FILE: src/controllers/ZhViewWidgetsBuilder.js
/**
 * Service to create widgets to handle views
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function ZhViewWidgetsBuilder(restEngine, templateManager, metaDataFactory) {

    function getWidgetJSON(widgetName) {
        var widgetPath = 'zh-view-preferences#json/widgets/'+widgetName+'.json';
        return restEngine.requestMetaData(widgetPath).then(function(response) {
            return response.data;
        });
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName) {
        return getWidgetJSON(widgetName).then(function(widget) {
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored view configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} View configuration widget
     */
    this.factoryViewConfigurationWidget = function(methods) {
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create view widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create view widget
     */
    this.factoryCreateViewWidget = function(methods) {
        return this.factoryWidget('create-view');
    };

    /**
     * Retrieve factored import view widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import view widget
     */
    this.factoryImportViewWidget = function() {
        return this.factoryWidget('importView');
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhViewWidgetsBuilder', ZhViewWidgetsBuilder);
});

// FILE: src/controllers/ZhViewCreateViewController.js
/**
 * Create new view widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhViewPreferences}    ZhViewPreferences    Service to handle filter views methods
 * @param {ZhFilterPreferences}  ZhFilterPreferences  Service to handle filter filters methods
 * @param {ZhLayoutPreferences}  ZhLayoutPreferences  Service to handle filter layouts methods
 * @param {ZhViewWidgetsBuilder} ZhViewWidgetsBuilder Service to create widgets to handle filters
 * @param {ScreenService}        ScreenService        Service to manipulate screen
 * @param {MessageService}       MessageService       Service to handle messages
 * @param {ZHPromise}            ZHPromise            Service to handle promises
 * @param {ZhUtil}               ZhUtil               Service to provice utilities methods
 */
function ZhViewCreateViewController(ZhViewPreferences, ZhFilterPreferences, ZhLayoutPreferences, ZhViewWidgetsBuilder, ScreenService, MessageService, ZHPromise, ZhUtil) {

    var createViewWidgetPromise = null;

    /**
     * Retrieve widget to create a new filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles views configuration
     *
     * @return {Promise<Widget>}     Widget used to create a new view
     */
    this.getCreateViewWidget = function(configWidget) {
        if (createViewWidgetPromise === null) {
            createViewWidgetPromise = ZhViewWidgetsBuilder.factoryCreateViewWidget();
        }

        var listWidget = configWidget.listWidget;

        var filters = ZhFilterPreferences.getWidgetFilters(listWidget.id);
        var layouts = ZhLayoutPreferences.getWidgetLayouts(listWidget.id);

        return ZHPromise.all([createViewWidgetPromise, filters, layouts]).then(function(args) {
            var createViewWidget = args[0];
            var filters = args[1];
            var layouts = args[2];

            createViewWidget.configWidget = configWidget;

            createViewWidget.getField('FILTER_ID').dataSource.data = filters;
            createViewWidget.getField('LAYOUT_ID').dataSource.data = layouts;
            return createViewWidget;
        });
    };

    /**
     * Cancel the creation or edition of a new view
     *
     * @since 3.0.0
     *
     * @param  {Widget} createViewWidget Widget used to create or edit a view
     *
     * @return {Promise}                 Promise that is resolved after the widget to edit the view is closed
     */
    this.cancelCreateView = function(createViewWidget) {
        var configWidget = createViewWidget.configWidget;
        Util.extend(configWidget.currentRow, configWidget.backupRow);
        return ScreenService.closePopup();
    };

    /**
     * Save filter and view
     *
     * @since 3.0.0
     *
     * @param  {Widget} createViewWidget Widget that has the view to be saved
     *
     * @return {Promise}                 Promise resolved when saved view is applied
     */
    this.saveAndApplyView = function(createViewWidget) {
        if (createViewWidget.isValid()) {
            var view = ZhUtil.clearRow(createViewWidget.currentRow);

            var configWidget = createViewWidget.configWidget;
            var listWidget = configWidget.listWidget;

            return ZhViewPreferences.saveViews([view]).then(function() {
                return ZhViewPreferences.applyView(listWidget, view);
            }).then(function() {
                ZhViewPreferences.updateViews(listWidget.id, configWidget.dataSource);
                ScreenService.closePopup(true);
                return listWidget.activate();
            }).catch(function(error) {
                return MessageService.getMessageByCode('zh-view-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-view-preferences#save_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        } else {
            return ZHPromise.when();
        }
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhViewCreateViewController', ZhViewCreateViewController);
});

// FILE: src/controllers/ZhViewImportViewController.js
/**
 * Import view widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhViewPreferences}                     ZhViewPreferences                        Service to handle view preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhFilterPreferences}                   ZhFilterPreferences                      Service to handle filters
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                      Service to handle layouts
 * @param {ZhViewWidgetsBuilder}                  ZhViewWidgetsBuilder                     Service to create widgets to handle views
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 * @param {$timeout}                              $timeout                                 Angular timeout 
 */
function ZhViewImportViewController(MessageService, ZhViewPreferences, ZhPreferences, ZhFilterPreferences, ZhLayoutPreferences, ZhViewWidgetsBuilder, ZHPromise, ScreenService, $timeout) {

    var importViewWidgetPromise = null;

    /**
     * Retrieve widget to import view
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportViewWidget = function() {
        if (importViewWidgetPromise === null) {
            importViewWidgetPromise = ZhViewWidgetsBuilder.factoryImportViewWidget();
        }

        return importViewWidgetPromise;
    };
    
    function getObjectFromBase64(b64) {
        b64 = b64.split('base64,')[1];
        b64 = window.atob(b64);
        b64 = decodeURIComponent(escape(b64));
        
        return JSON.parse(b64);
    }

    function buildImportPromises(importViewWidget, userID, layouts, filters) {
        var layoutFileField = importViewWidget.getField('layoutFile');
        var filterFileField = importViewWidget.getField('filterFile');
        var promises = [];
        
        if (layoutFileField.value().length > 0) {
            var layout = getObjectFromBase64(layoutFileField.value()[0].b64File);

            if (!ZhLayoutPreferences.validateLayoutImport(layout, importViewWidget.configWidget, layouts)) {
                return [ZHPromise.reject()];
            }
            
            promises.push(function() {
                layout.__is_new = true;
                layout.__generate_id = false;
                layout.USER_ID = userID;
                layout.LAYOUT_CHANGES = JSON.stringify(layout.LAYOUT_CHANGES);
                
                return ZhLayoutPreferences.saveLayouts([layout]).then(function() {
                    ZhLayoutPreferences.updateLayouts(importViewWidget.configWidget.listWidget.id);
                }).catch(function(error) {
                    MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                        'ERROR': error.message
                    });
                    
                    return ZHPromise.reject();
                });                
            }());
        }
        
        if (filterFileField.value().length > 0) {
            var filter = getObjectFromBase64(filterFileField.value()[0].b64File);
            
            if (!ZhFilterPreferences.validateFilterImport(filter, importViewWidget.configWidget, filters)) {
                return [ZHPromise.reject()];
            }
    
            promises.push(function() {
                filter.__is_new = true;
                filter.__generate_id = false;
                filter.USER_ID = userID;
                filter.FILTER_ROW = JSON.stringify(filter.FILTER_ROW);
                
                return ZhFilterPreferences.saveFilters([filter]).then(function() {
                    ZhFilterPreferences.updateFilters(importViewWidget.configWidget.listWidget.id);
                }).catch(function(error) {
                    MessageService.showMessageByCode('zh-filter-preferences#save_error', {
                        'ERROR': error.message
                    });
                    
                    return ZHPromise.reject();
                });                
            }());
        }
        
        return promises;
    }
    
    /**
     * Import view
     *
     * @since 3.0.0
     *
     * @param  {Widget} importViewWidget Widget with the view configuration
     *
     * @return {Promise}                      Promise resolved after the view is imported
     */
    this.importPreference = function(importViewWidget) {
        if (!importViewWidget.isValid()) {
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }
        
        var viewFileField = importViewWidget.getField('viewFile');

        var view = getObjectFromBase64(viewFileField.value()[0].b64File);
        
        var configWidget = importViewWidget.configWidget;
        var listWidget = configWidget.listWidget;

        var filters = ZhFilterPreferences.getWidgetFilters(listWidget.id);
        var layouts = ZhLayoutPreferences.getWidgetLayouts(listWidget.id);

        return ZHPromise.all([filters, layouts]).then(function(args) {
            filters = args[0]; 
            layouts = args[1]; 
            
            if (!ZhViewPreferences.validateViewImport(view, importViewWidget, layouts, filters)) {
                return ZHPromise.reject({err: 'VIEW_FILE_INVALID'});
            }        
    
            return ZhPreferences.getUserID().then(function(userID) {
                view.USER_ID = userID;
                view.__is_new = true;
                view.__generate_id = false;
                
                var importPromises = buildImportPromises(importViewWidget, userID, layouts, filters);
            
                return ZHPromise.all(importPromises).then(function() {
                    return ZhViewPreferences.saveViews([view]).then(function() {
                        ScreenService.closePopup();
                        ZhViewPreferences.updateViews(listWidget.id, configWidget.dataSource);
                        configWidget.refreshWidget();
                    }).catch(function(error) {
                        MessageService.showMessageByCode('zh-view-preferences#save_error', {
                            'ERROR': error.message
                        });
                    });
                });
    
            });
        });

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhViewImportViewController', ZhViewImportViewController);
});

// FILE: src/controllers/ZhViewConfigController.js
/**
 * View configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences              Service to handle preferences persistance
 * @param {ZhViewPreferences}          ZhViewPreferences          Service to handle view preferences methods
 * @param {ZhViewWidgetsBuilder}       ZhViewWidgetsBuilder       Service to create widgets to handle views
 * @param {ScreenService}              ScreenService              Service to manipulate screen
 * @param {MessageService}             MessageService             Service to show messages
 * @param {ZHPromise}                  ZHPromise                  Service to handle promises
 * @param {ZhViewCreateViewController} ZhViewCreateViewController Controller to manipulate create view widget
 * @param {ZhViewImportViewController} ZhViewImportViewController Controller to manipulate import view widget
 */
function ZhViewConfigController(ZhPreferences, ZhViewPreferences, ZhViewWidgetsBuilder, ScreenService, MessageService, ZHPromise, ZhViewCreateViewController, ZhViewImportViewController) {

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles views configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create views for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles views configuration
     */
    this.getConfigWidget = function(listWidget, dataSource) {
        if (configWidgetPromise === null) {
            configWidgetPromise = ZhViewWidgetsBuilder.factoryViewConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args) {
            var userID = args[0];
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id},
                {'name': 'USER_ID', 'operator': '=', 'value': userID}
            ];
            dataSource.data.forEach(function(row) {
                row.APPLIED = row.ID == (listWidget.appliedView ? listWidget.appliedView.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    /**
     * Open widget to create a new view
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles views configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateViewWidget = function(configWidget) {
        var createViewWidget = ZhViewCreateViewController.getCreateViewWidget(configWidget);
        var userID = ZhPreferences.getUserID();

        var listWidget = configWidget.listWidget;

        return ZHPromise.all([createViewWidget, userID]).then(function(args) {
            var createViewWidget = args[0];
            var userID = args[1];

            createViewWidget.newRow();
            createViewWidget.currentRow.ID =  null;
            createViewWidget.currentRow.USER_ID =  userID;
            createViewWidget.currentRow.WIDGET_ID =  listWidget.id;
            createViewWidget.currentRow.LAST_MODIFIED_DATE =  null;
            createViewWidget.label = ScreenService.i18n('View');

            return ScreenService.openPopup(createViewWidget);
        });
    };

    /**
     * Open popup to edit a saved view
     *
     * @since 3.0.0
     *
     * @param  {View} view       View to edit
     *
     * @return {Promise<Widget>} Widget used to edit the view
     */
    this.editView = function(configWidget, view) {
        return ZhViewCreateViewController.getCreateViewWidget(configWidget).then(function(createViewWidget) {
            configWidget.backupRow = Util.clone(view, ['__rightActions', '__leftActions']);
            createViewWidget.currentRow = view;
            createViewWidget.label = ScreenService.i18n('View') + ' - ' + view.LABEL_TO_PREFERENCE;

            return ScreenService.openPopup(createViewWidget);
        });
    };

    /**
     * Delete views
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the views to delete
     *
     * @return {Promise}             Promise resolved after views are deleted
     */
    this.deleteViews = function(configWidget) {
        var viewsToDelete = configWidget.getCheckedRows();

        var defaultView = viewsToDelete.filter(function(view) {
            return view.DEFAULT_VIEW;
        });
        if (defaultView.length) {
            MessageService.showMessageByCode('zh-view-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-view-preferences#delete', {
            'NR_OF_VIEWS': viewsToDelete.length
        }).then(function() {
            return ZhViewPreferences.deleteViews(viewsToDelete).then(function() {
                configWidget.dataSource.clearCheckedRows();
                ZhViewPreferences.updateViews(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error) {
                MessageService.showMessageByCode('zh-view-preferences#delete_error', {
                    'ERROR': error.message
                });
            });
        });
    };

    /**
     * Set a view as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles views configuration
     * @param  {View} view           View that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultView = function(configWidget, view) {
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_VIEW', view);
    };

    /**
     * Apply View
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles views configuration
     * @param  {View} view           View to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedView = function(configWidget, view) {
        var listWidget = configWidget.listWidget;

        return ZhViewPreferences.applyView(listWidget, view).then(function() {
            configWidget.dataSource.data.forEach(function(row) {
                row.APPLIED = row.ID == view.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {View} view       view to export
     *
     * @return {Promise<View>}   view exportado
     */
    this.downloadPreferenceFile = function(view) {
        return ZhViewPreferences.downloadPreferenceFile(view);
    };

    /**
     * Open widget to import a new view from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles view configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget) {
        return ZhViewImportViewController.getImportViewWidget().then(function(importViewWidget) {
            importViewWidget.newRow();
            importViewWidget.configWidget = configWidget;
            return ScreenService.openPopup(importViewWidget);
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhViewConfigController', ZhViewConfigController);
});

// FILE: src/controllers/ZhViewFloatingCardController.js
/**
 * View floating card controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {$scope}             $scope             Floating card scope
 * @param {ApplicationContext} ApplicationContext Service to retrieve Zeedhi services
 * @param {ScreenService}      ScreenService      Service to handle screen
 * @param {EventEngine}        eventEngine        Service to handle events
 * @param {Query}              Query              Service to build queries
 * @param {metaDataFactory}    metaDataFactory    Service to factory models from metadata
 */
function ZhViewFloatingCardController($scope, ApplicationContext, ScreenService, eventEngine, Query, metaDataFactory) {
    var widget = $scope.widget;
    $scope.appliedPreference = null;

    var ZhViewConfigController = ApplicationContext.ZhViewConfigController;
    var ZhViewPreferences = ApplicationContext.ZhViewPreferences;
    var ZhPreferences = ApplicationContext.ZhPreferences;

    var dataSource = metaDataFactory.factoryDataSource({
        'name': 'zh-view-preferences#/view',
        'localStorage': false,
        'lazyLoad': true,
        'rest': true,
        'data': []
    });

    dataSource.afterLoadDataSource(function() {
        return ZhViewPreferences.addIsDefaultViewToViews(dataSource.data, widget.id).then(function(views) {
            dataSource.data = views;
        });
    });

    $scope.customizationCard = {
        'label': 'Views',
        'type': 'view',
        'openViewConfiguration': openViewConfiguration,
        'refreshViews': refreshViews,
        'applyOriginalView': applyOriginalView,
        'applyView': applyPreference,
        'dataSource': dataSource
    };

    var filterApplyEventIndex = eventEngine.bindEvent(widget, 'FilterAfterApply', function() {
        $scope.appliedPreference = null;
        widget.appliedView = null;

        if(widget.reportCustomData) {
            delete widget.reportCustomData.name;
        }
    });

    var layoutApplyEventIndex = eventEngine.bindEvent(widget, 'LayoutAfterApply', function() {
        $scope.appliedPreference = null;
        widget.appliedView = null;

        if(widget.reportCustomData) {
            delete widget.reportCustomData.name;
        }
    });

    widget.setAppliedView = setAppliedView;

    $scope.$on('$destroy', function() {
        delete widget.setAppliedView;

        eventEngine.unbindEvent(widget, 'FilterAfterApply', filterApplyEventIndex);
        eventEngine.unbindEvent(widget, 'LayoutAfterApply', layoutApplyEventIndex);

        delete dataSource.afterLoadDataSourcelistener;
    });

    function setAppliedView(view) {
        $scope.appliedPreference = view;
        widget.appliedView = view;

        if(view.LABEL_TO_PREFERENCE){
            sessionStorage.setItem('view_' + widget.id, view.LABEL_TO_PREFERENCE);
        }

        widget.reportCustomData = widget.reportCustomData || {};

        if (view.ID != 'ORIGINAL') {
            widget.reportCustomData.name = view.LABEL_TO_PREFERENCE;
        } else {
            delete widget.reportCustomData.name;
        }

    }

    function openViewConfiguration() {
        return ZhViewConfigController.getConfigWidget(widget, dataSource).then(function(configWidget) {
            return ScreenService.openPopup(configWidget);
        });
    }

    function refreshViews() {
        return ZhPreferences.getUserID().then(function(userID) {
            var query = Query.build()
                .where('WIDGET_ID').equals(widget.id);

            return $scope.customizationCard.dataSource.load(query.where());
        });
    }

    function applyOriginalView() {
        return ZhViewPreferences.applyOriginalView(widget);
    }

    function applyPreference(preference, reloadWidget) {
        return ZhViewPreferences.applyView(widget, preference, reloadWidget);
    }

    function getDefaultView() {
        return dataSource.data.filter(function(view) {
            return view.DEFAULT_VIEW;
        }).shift();
    }

    function applyDefaultView() {
        var view = getDefaultView();
        if (view) {
            var filterIsLazyLoad = Util.getNestedProperty(widget, ['dataSource', 'lazyLoad']);
            applyPreference(view, !filterIsLazyLoad);
        }
    }

    (function initViewPreferences() {
        if(ZhPreferences.validateWidget(widget)){
            ZhViewPreferences.updateViews(widget.id, dataSource).then(function() {
                if(Util.isUndefinedOrNull(widget.appliedView)){
                    applyDefaultView();
                } else {
                    applyPreference(widget.appliedView);
                }
            });
        }
    })();

}

// FILE: src/Config.js
(function() {

window.templateUrls = window.templateUrls || [];
window.metadataUrls = window.metadataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-view-preferences';

var hasTemplateUrl = Boolean(window.templateUrls.find(function(value) {
    return value.package === PACKAGE_NAME;
}));

var hasMetadataUrl = Boolean(window.metadataUrls.find(function(value) {
    return value.package === PACKAGE_NAME;
}));

var hasServiceUrl = Boolean(window.serviceUrls.find(function(value) {
    return value.package === PACKAGE_NAME;
}));

if (!hasTemplateUrl) {
    window.templateUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-view-preferences/dist/templates/'
    });
}

if (!hasMetadataUrl) {
    window.metadataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-view-preferences/dist/assets/'
    });
}

if (!hasServiceUrl) {
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: bower_components/zh-cache-bypass/src/CacheBypass.js
(function() {
    angular.module("ZeedhiApp").config(function($httpProvider) {
        if (Boolean(window.applicationVersion)) {
            $httpProvider.interceptors.push(function() {
                return {
                    'request': function cacheBypass(config) {
                        if (config && config.url && shouldApplyBypass(config.url)) {
                            config.url += '?v='+window.applicationVersion;
                        }
                        return config;
                    }
                };
            });
        } else {
            console.warn('Missing "applicationVersion" global property for CacheBypass.');
        }
    });

    function shouldApplyBypass(url) {
        return (
            Util.stringEndsWith(url, '.json') ||
            Util.stringEndsWith(url, '.html') ||
            Util.stringEndsWith(url, '.svg')
        );
    }
})();

// FILE: bower_components/zh-reports/dist/zhReports.js

// FILE: src/FloatingReportCardController.js
function FloatingReportCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {

	//------ Control variables
	var editingWidget = $scope.$parent.widget;

    /**
     * Checks if the card is already opened and reconfigure the state of it if it not.
     *
     * @param  {jquery.Event} e   Event that triggered the click.
     */
	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			}
			else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}
			}
		}
	}

	/**
	 * Toggle the report action when it's template is configured as card.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 */
	$scope.toggleFloatReportCard = function(e) {
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};

	editingWidget.toggleFloatReportCard = $scope.toggleFloatReportCard;

	/**
	 * Change the report orientation.
	 *
	 * @param  {jquery.Event} e   Event that triggered the toggle.
	 * @param  {String} orientation   Report orientation.
	 */
	$scope.setOrientation = function(e, orientation) {
		$scope.widget.reportOrientation = orientation;
		e.stopPropagation();
	};
}

// FILE: src/ZhReports.js
/* global Util, Configuration*/
function ZhReports(templateManager, ZHPromise, ScreenService, restEngine, ZhUtil, eventAggregator, $filter, DecodeValueFieldService, ZhLogger, eventEngine) {

    var reportConfig = {
        PDF:{
            uri: '/generateReport',
            type: 'PDF',
            action: 'pdfAction'
        },
        XLS:{
            uri: '/generateXLS',
            type: 'XLS',
            action: 'xlsAction'
        },
        CSV:{
            uri: '/generateCSV',
            type: 'CSV',
            action: 'csvAction'
        }
    };

    var maxVisibleColumns = {
        'LANDSCAPE': 15,
        'PORTRAIT': 10
    };

    var reportNameToReportFunction = {
        "=": "is equal to",
        "IN": "is among",
        "NOT_IN": "is not among",
        "!=": "is different of",
        ">": "is bigger than",
        "<": "is lower than",
        ">=": "is not lower than",
        "<=": "is not bigger than",
        "BETWEEN": ["is between", "and"],
        "NOT_BETWEEN": ["is lower than", "and bigger than"],
        "LIKE": "has",
        "LIKE_I": "has",
        "LIKE_ALL": "has any of",
        "NOT_LIKE": "does not have",
        "IS_NULL": "is not defined",
        "IS_NOT_NULL": "is defined",
        "ORDER_DESC": "ordered descending",
        "ORDER_ASC": "ordered ascending",
        "ORDER_BY": "ordered",
        "GROUP_BY": "grouped"
    };

    eventEngine.registerEvent('BeforeGenerateReport', 'beforeGenerateReport');

    function triggerBeforeGenerateReport(widget, metaData){
        if(widget.beforeGenerateReport){
            return widget.beforeGenerateReport({
                'metadata': metaData
            });
        }
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Field}  field  - Field with the data related to the filter.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    function getFieldFilterValue(field, filter) {
        return getFieldFilterDefaultText(filter, filter.value, ScreenService.i18n(reportNameToReportFunction[filter.operator]));
    }

    /**
     * Configure the default value for the variable if it's not defined.
     *
     * @param {Mixed} value        - The variable to be tested.
     * @param {Mixed} defaultValue - The value attributed the variable if it's not defined.
     *
     * @return {Mixed} The value that the variable should recieve.
     */
    function notUndefinedOrDefault(value, defaultValue) {
        return value === undefined ? defaultValue : value;
    }

    /**
     * Tests if the given type is PDF.
     *
     * @param {String} type - The type to be tested.
     *
     * @return {Boolean} Tells if the given type is PDF.
     */
    function isPdfFormat(type) {
        return type === reportConfig.PDF.type;
    }

     /**
     * Check which type of template need to be returned.
     *
     * @return {String} Template type to be used.
     */
    function checkZhReportActionDisplayMode() {
        if( !templateManager.project.zhReportActionDisplayMode ) {
            return 'zh-reports#floating-report-card.html';
        } else if (templateManager.project.zhReportActionDisplayMode === 'buttons') {
            return 'zh-reports#floating-report.html';
        }
    }

    eventAggregator.subscribe('beforeProjectStartup', function() {
        templateManager.project.customFloatingActions.push({
            'id': '#reportAction#',
            'name': 'reportAction',
            'label': ScreenService.i18n('Export'),
            "expandable": !templateManager.project.zhReportActionDisplayMode,
            'order': 10,
            'icon': 'file-export',
            'template': checkZhReportActionDisplayMode(),
            'isVisible': function(widget) {
                var show = true;
                if (widget.floatingControl) {
                    var showPdfAction = notUndefinedOrDefault(widget.floatingControl.pdfAction, true);
                    var showXlsAction = notUndefinedOrDefault(widget.floatingControl.xlsAction, true);
                    var showCsvAction = notUndefinedOrDefault(widget.floatingControl.csvAction, true);
                    show = showPdfAction || showXlsAction || showCsvAction;
                }

                return show;
            }
        });
    });

    /**
     * Checks if the propertie 'reportsUrl' is configured on the environment.json.
     *
     * @throws {Error} Message warning that the propertie should be configured at the file.
     */
    var checkConfiguration = function() {
        if (!(templateManager.project && templateManager.project.reportsUrl)) {
            throw (ScreenService.i18n(
                'Need report configuration in "environment.json": "reportsUrl"!'));
        }
    };

    /**
     * Returns the format of the column.
     *
     * @param {Field} column - The Field to test it's format.
     *
     * @return {Object} The Field's format.
     */
    var getFormatOfColumn = function(column) {
        var format = column.format;
        if (!format && column.mask) {
            column.mask.params = column.mask.params || {};
            format = {};
            format.params = column.mask.params;
            format.type = column.mask.type;
            if (Util.isString(format.params.mask) && Util.stringStartsWith(format.params.mask, '@') && Util.stringEndsWith(format.params.mask, '@')) {
                format.params.mask = ZhUtil.evaluate(format.params.mask);
            }
        }
        return format;
    };

    /**
     * Build a column to the metadata based on the given field.
     *
     * @param {Field}  column - Field base to create the column to the report.
     * @param {String} size   - The size of the Field, in percentage inside the report.
     *
     * @return {Object} Column to the metadata, with the necessary data.
     */
    var buildColumn = function(column, size) {
        if (Util.isUndefined(column.reportAlign)) {
            column.reportAlign = column.align || 'left';
        }
        return {
            align: column.reportAlign,
            description: ScreenService.i18n(column.labelOnReport || column.label),
            expression: column.expression,
            format: getFormatOfColumn(column),
            sequence: column.sequenceInList,
            size: size,
            xlsType: column.xlsType
        };
    };

    /**
     * Filter the widgetColumns by just the visible fields.
     *
     * @param {Array.<Field>}  widgetColumns               - Array to be filtered.
     * @param {Boolean}        reportShowOnlyVisibleFields - Tells if the report should show only visible fields.
     * @param {String}         [reportType='PDF']          - The type of the report.
     * @param {Array.<String>} [groups]                    - List of the field names that are grouped.
     *
     * @return {Array.<Field>} Filtered array with all fields that are visible on the report.
     */
    var filterVisibleColumns = function(widgetColumns, reportShowOnlyVisibleFields, reportType, groups) {
        var showGroupedColumns = !isPdfFormat(reportType);

        return widgetColumns.filter(function(column) {
            var isGroupedColumn = groups ? groups.indexOf(column.name) != -1 : column.grouped;
            var columnGroupVisibilitity = showGroupedColumns && isGroupedColumn;

            var isVisibleOnReport = Util.isUndefinedOrNull(column.isVisibleOnReport) || column.isVisibleOnReport;
            var columnVisibility = isVisibleOnReport && (!reportShowOnlyVisibleFields || (column.isVisible && column.showOnList));

            return column.aclIsVisible && (columnGroupVisibilitity || columnVisibility);
        });
    };

    /**
     * Returns the max visible columns on the report.
     *
     * @param  {Widget}    widget       -   Widget where the report is being generated.
     * @param  {String}    reportType   -   The type of the report.
     *
     * @return {Integer} The max of visible columns on the report.
     */
    var getMaxVisibleColumns = function(widget, reportType) {
        var maxColumns;
        if (reportType==reportConfig.PDF.type) {
            maxColumns = widget.maxColumns;
        } else {
            maxColumns = 99999;
        }
        return maxColumns;
    };

    /**
     * Builds the configuration of the field's display.
     *
     * @param {Widget}         widget             - Widget where the report is being generated.
     * @param {Array.<String>} [columnsToUse]     - The name of the fields that will be used on the
     * report generation.
     * @param {String}         [reportType='PDF'] - The type of the report.
     * @param {Array.<String>} [groups]           - List of the field names that are grouped.
     *
     * @return {Object} Object where each propertie is a object with the configuration of its display
     * on the report.
     */
    var _buildMetaDataColumns = function(widget, columnsToUse, reportType, groups) {
        var columns = {};
        var widgetColumns = widget.fields;
        var maxVisibleColCount = getMaxVisibleColumns(widget, reportType);
        if (widgetColumns) {
            widgetColumns = filterVisibleColumns(widgetColumns, widget.reportShowOnlyVisibleFields, reportType, groups);
            if (maxVisibleColCount && widgetColumns.length > maxVisibleColCount) {
                ZhLogger.warning(ScreenService.i18n(
                    "Number of columns too big to render the report correctly. It was limited to __maxVisibleColCount__ columns.",
                    {
                        'maxVisibleColCount': maxVisibleColCount
                    }
                ));
            }

            var remainingSize = 100;
            var columnsWithoutColumnSize = 0;

            widgetColumns.forEach(function(column) {
                if (column.reportColumnSize) {
                    remainingSize -= parseInt(column.reportColumnSize);
                }
                else {
                    columnsWithoutColumnSize++;
                }
            });

            if (remainingSize < 0) {
                ZhLogger.warning(ScreenService.i18n('Sum of columns width is bigger than 100%.'));
            }

            var size = Math.max(2, Math.floor(remainingSize / columnsWithoutColumnSize));
            var sum = 0;
            widgetColumns.forEach(function(column, index) {
                if (!columnsToUse || !columnsToUse[column.name]) {
                    var columnSize;
                    if (column.reportColumnSize) {
                        columnSize = column.reportColumnSize;
                    }
                    else {
                        if (index === (widgetColumns.length - 1)) {
                            columnSize = Math.max(2, remainingSize - sum);
                        }
                        else {
                            columnSize = size;
                            sum += size;
                        }
                    }

                    if (!columnSizeHasUnity(columnSize)) {
                        columnSize += '%';
                    }

                    columns[column.valueFieldOnReport || column.name] = buildColumn(column, columnSize);
                }
            });

        }
        return columns;
    };

    /**
     * Returns an array containing only the displayed value from field.
     *
     * @param {Object} object - Object containing the dataSource.data's row.
     *
     * @return {Object} Array containing only a object as {field.descriptionField: <displayedValue>}.
     */
    function getStaticPropertyValueFromObject(row) {
        var toReturn = {};
        toReturn[row[this.valueField]] = row[this.descriptionField];
        return toReturn;
    }

    /**
     * Builds the configuration of the field's staticData.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     * @param {Object} object - Object were the configuration field's display are gathered.
     *
     * @return {Object} Object where all the staticData from fields with DataSource are
     * gathered to be treated.
     */
    var _buildStaticDataColumns = function(widget, columns) {
        var widgetColumns = widget.fields;
        var staticData = {};
        if (widgetColumns) {
            widgetColumns.forEach(function(column) {
                if (columns[column.name] && column.dataSource && column.dataSource.data) {
                    staticData[column.name] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
                else if (columns[column.valueFieldOnReport] && column.dataSource && column.dataSource.data) {
                    staticData[column.valueFieldOnReport] = column.dataSource.data.map(getStaticPropertyValueFromObject, column);
                }
            });

            if (Object.keys(staticData).length) {
                for (var data in staticData) {
                    staticData[data] = staticData[data].reduce(function(o1, o2) {
                        for (var key in o2) {
                            o1[key] = o2[key];
                        }
                        return o1;
                    }, {});
                }
            }
        }

        return staticData;
    };

    /**
     * Tests if the column size has a measurement unit.
     *
     * @param {String|Number} columnSize - The column size to be tested.
     *
     * @return {Boolean} Tells if the column size has a measurement unit.
     */
    function columnSizeHasUnity(columnSize) {
        return (/[\%px]/g).test(columnSize.toString());
    }

    /**
     * Build a report's group based on the given Field.
     *
     * @param {Field} column - The Field to be based on the cration of the column.
     *
     * @return {Object} Report's group.
     */
    var buildGroup = function(column) {
        return {
            name: column.name,
            field: column.name,
            isNumber: column.isNumber,
            descending: column.descending,
            format: getFormatOfColumn(column),
            internalSorts: column.internalSorts,
            header: {
                label: column.headerLabel || ScreenService.i18n(column.label),
                displayfield: column.name
            }
        };
    };

    /**
     * Build the metadata of the fields grouped.
     *
     * @param {Widget} widget - Widget used to create the grouped metadata.
     * @param {Array.<String>} [customGroups] - List of the field names that are grouped.
     *
     * @return {Array.<Object>} Metadata of grouped columns.
     */
    var _buildMetaDataGroups = function(widget, customGroups) {
        var buildedGroups = [];
        var groups = widget.groups;
        if (customGroups) {
            groups = customGroups.map(function(name) {
                return widget.getField(name);
            });
        }
        buildedGroups = groups.map(function(group) {
            return buildGroup(group);
        });
        return buildedGroups;
    };

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function getFieldFilterDefaultTextWithoutValue(text) {
        return ScreenService.i18n(text) + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function likeIFieldText(value, text) {
        return ScreenService.i18n(text) + ' \'' + value.replace(/%/g, '') + '\' case sensitive.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     * @param {Object} filter - Filter used to create the report.
     *
     * @return {String} Text to be shown in the header.
     */
    function valueFieldText(filter, value, text) {
        if (value[0] === "%" && value[value.length - 1] === "%") {
            value = value.substring(1, value.length - 1);
        }
        else {

            if (value[0] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(1, value.length);
                    text = "ends with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(1, value.length);
                    text = "does not ends with";
                }
            }
            if (value[value.length - 1] === "%") {
                if (filter.operator == 'LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "starts with";
                }
                else if (filter.operator == 'NOT_LIKE') {
                    value = value.substring(0, (value.length - 1));
                    text = "does not starts with";
                }
            }
        }
        return ScreenService.i18n(text) + ' ' + value + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} value- Value of the Filter.
     * @param {String} text - Text used to create the report header.
     *
     * @return {String} Text to be shown in the header.
     */
    function oneOrMoreValuesFieldText(value, text) {
        if (Array.isArray(text)) {
            return ScreenService.i18n(text[0]) + ' ' + value[0] + ' ' + ScreenService.i18n(text[1]) + ' ' + value[1] + '.';
        }

        return ScreenService.i18n(text) + ' ' + value.join(', ') + '.';
    }

    /**
     * Get the text related to the filter that is being used.
     *
     * @param {Object} filter- Filter used to create the report.
     * @param {Object} value - Value of the filter.
     * @param {String} text  - Text used to create the header of the report.
     *
     * @return {String} Text to be shown woth data related to the given filter.
     */
    var getFieldFilterDefaultText = function(filter, value, text) {
        if (Array.isArray(value)) {

            return oneOrMoreValuesFieldText(value, text);
        }

        if (filter.operator === "LIKE_I") {
            return likeIFieldText(value, text);
        }

        if (!value) {
            return getFieldFilterDefaultTextWithoutValue(text);
        }

        return valueFieldText(filter, value, text);


    };

    /**
     * Checks if the filter related to the given item should be displayed on the header of
     * the report.
     *
     * @param  {Field} field -  Field to be filter to be tested.
     *
     * @return {Boolean} Boolean telling if the filter related to the field should be displayed.
     */
    function filterIsVisibleOnHeader(field) {
        return field && field.isVisibleOnReport;
    }

    /**
     * Builds the metadata of the filter.
     *
     * @param {Widget} widget - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The metadatas of each filter in the widget.
     */
    var _buildMetaDataFilter = function(widget) {
        var filterData = [];
        if (widget.dataSourceFilter) {
            widget.dataSourceFilter.forEach(function(item) {
                var field = widget.getField(item.name);
                if (filterIsVisibleOnHeader(field)) {
                    filterData.push({
                        label: field.label,
                        value: ZhUtil.evaluate(getFieldFilterValue(field, item))
                    });
                }
            });
        }

        return filterData;
    };

    /**
     * Retrieves the image from the url and converts it to base64
     *
     * @param {String}   url    - The source of the image
     * @param {Function} widget - Widget where the report is being generated.
     */
    function convertImageToB64(url, callback) {
        var xhr = new XMLHttpRequest();

        xhr.onload = function() {
            var reader = new FileReader();
            reader.onloadend = function() {
                callback(reader.result);
            };
            reader.readAsDataURL(xhr.response);
        };

        xhr.open('GET', url);
        xhr.responseType = 'blob';
        xhr.send();
    }

    /**
     * Fetches the images if they exist.
     *
     * @param {Function} callback  - The callback to be executed after all the images where fetched.
     * @param {Widget}   widget    - Widget where the report is being generated.
     */
    function getAllImages(container, callback) {
        var imgs = {};
        var imgSrc = ['reportClientLogo', 'reportProductLogo', 'reportEnterpriseLogo'];
        var index = 0;

        if(!container[imgSrc[0]] && !container[imgSrc[1]] && !container[imgSrc[2]]){
            callback({});
            return;
        }

        imgSrc.forEach(function(imgName){
            if(container[imgName]){
                index++;
                convertImageToB64(container[imgName], function(imageData){
                    imgs[imgName] = imageData;
                    index--;

                    if(index === 0){
                        callback(imgs);
                    }
                });
            }
        });
    }

    /**
     * Builds the widget metadata.
     *
     * @param {Object} [customData]       - Customization data for the generation of the report.
     * @param {Widget} sourceWidget       - Widget where the report is being generated.
     * @param {String} [reportType='PDF'] - The type of the report.
     *
     * @return {Object} The widget's metadata for the generation of the report.
     */
    var buildMetaData = function(customData, sourceWidget, reportType, callback) {
        var widget = sourceWidget;
        var columns = _buildMetaDataColumns(widget, customData.columns, reportType, customData.groups);
        var staticColumns = _buildStaticDataColumns(widget, columns);
        var widgetLabel = ScreenService.i18n(widget.label);

        getAllImages(templateManager.project, function(images) {
            var widgetMetaData = {
                name: widget.name,
                title: customData.name || widgetLabel,
                reportName: customData.reportName || widget.name,
                reportClientLogo: images.reportClientLogo,
                reportProductLogo: images.reportProductLogo,
                reportEnterpriseLogo: images.reportEnterpriseLogo,
                widgetLabel: widgetLabel,
                version: '1.0',
                clientLogo: templateManager.project.reportClientLogo,
                productLogo: templateManager.project.reportProductLogo,
                enterpriseLogo: templateManager.project.reportEnterpriseLogo,
                columns: columns,
                staticData: staticColumns,
                groups: _buildMetaDataGroups(widget, customData.groups),
                filter: _buildMetaDataFilter(widget),
                xlsDefaultType: widget.xlsDefaultType,
                orientation: widget.reportOrientation || 'PORTRAIT'
            };

            callback(widgetMetaData);
        });
    };


    /**
     * Tells if the given field needs to have it's value decoded.
     *
     * @param {Field} field - Fild to test if it's value need to be decoded.
     *
     * @return {Boolean} Tells if the field's value need to be decoded
     */
    var shouldUseDecodeValueField = function(field) {
        var masksTypesToAvoid = ['currency', 'hour', 'float', 'percent', 'fix'];
        return !field.mask || masksTypesToAvoid.indexOf(field.mask.type) == -1;
    };

    /**
     * Get the grouped columns from the given metadata of grouped fields.
     *
     * @param {Object} groupsMetadata - Metadata of the grouped on the report.
     *
     * @return {Array.<Field>} The fields of the group.
     */
    var getGroupedColumns = function(groupsMetadata) {
        var columns = [];
        groupsMetadata.forEach(function(group) {
            columns.push(group.field);
        });

        return columns;
    };

    /**
     * Format the values of the dataset based on the field of the widget configured
     * the metaData.
     *
     * @param {Array.<Object>} dataSet  - The widget's dataset.
     * @param {Object}         metaData - Metadata of the report.
     * @param {Widget}         widget   - Widget where the report is being generated.
     *
     * @return {Array.<Object>} The given dataset formatted for be used on the report.
     */
    var formatValuesForReport = function(dataSet, metaData, widget) {
        var columns = Object.keys(metaData.columns);
        columns = columns.concat(getGroupedColumns(metaData.groups));

        return dataSet.map(function(row) {
            if(row.__groupHeader || row.__groupFooter) return row;

            var formattedRow = {};
            columns.forEach(function(column) {
                var field = widget.getField(column);
                if (field && shouldUseDecodeValueField(field)) {
                    formattedRow[column] = DecodeValueFieldService.model(field, row[column]);
                }
                else {
                    formattedRow[column] = row[column];
                }
            });

            return formattedRow;
        });
    };

    /**
     * Builds the dataSet based on the given widget,
     *
     * @param {Object} metaData   - The metadata of the report.
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Array.<Object>}  Dataset from the widget formatted to the report.
     */
    var buildDataSet = function(metaData, widget, reportType) {
        return loadAllData(widget, reportType).then(function(dataSet) {
            return isPdfFormat(reportType) ? formatValuesForReport(dataSet, metaData, widget) : dataSet;
        });
    };

    /**
     * Get the datasorce's name without the initial '/'.
     *
     * @param {DataSource} dataSource - Widget's dataSource to get the name.
     *
     * @return {String} Datasorce name without the initial '/'.
     */
    function getDataSourceName(dataSource) {
        return dataSource.name.split('/').pop();
    }

    /**
     * Load all data from the widget's dataSource.
     *
     * @param {Widget} widget     - Widget where the report is being generated.
     * @param {String} reportType - The report type to generate.
     *
     * @return {Promise.<Array>} Promise that will be resolved with the widget's datasource.
     */
    function loadAllData(widget, reportType) {
        var dataSource = widget.dataSource;
        var storageStrategy = dataSource.getStorageStrategy();
        var result;

        if (storageStrategy) {
            result = storageStrategy.getDataSource(dataSource.name, widget.getDataSourceFilter(), null).then(function(dataSet) {
                if (Util.isObject(dataSet)) {
                    dataToReturn = dataSet.dataset[getDataSourceName(dataSource)];
                }
                else {
                    dataToReturn = dataSet;
                }

                return isPdfFormat(reportType) ? widget.getUpdatedGroupData(dataToReturn) : dataToReturn;
            });
        }
        else {
            result = ZHPromise.when(widget.groupedData);
        }

        return result;
    }

    /**
     * Build the path where the file was created.
     *
     * @param {String} fileName - Name of the created file.
     *
     * @return {Object} Object with the ful path of the created file.
     */
    var buildFilePath = function(fileName) {
        var reportsUrl = templateManager.project.reportsUrl;
        return {
            path: reportsUrl + '/reports/' + fileName
        };
    };

    /**
     * Builds a successs callback.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccess = function(callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePath(data.messages[0].message));
            }
        };
    };

    /**
     * Build the file path of the generated file for the dataExporter
     *
     * @param {String} fileName - Name of the generated file.
     *
     * @return {Object} Object with the fukk path of the generated file.
     */
    var buildFilePathForDataExporter = function(exporterUrl, fileName) {
        return {
            path: exporterUrl + fileName
        };
    };

    /**
     * Builds a successs callback for the dataExporter.
     *
     * @param {Function} callback - Function to be executed with the FilePath.
     *
     * @return {Function} Callback with the given callback.
     */
    var buildCallbackOfSuccessForDataExporter = function(exporterUrl, callback) {
        return function(data) {
            if (callback) {
                callback(buildFilePathForDataExporter(exporterUrl, data.messages[0].message));
            }
        };
    };

    /**
     * Returns the uri from a given report type.
     *
     * @param {String} reportType - Report type that will be used to get the uri.
     *
     * @return {String} - Uri from the given report type.
     */
    var getReportUri = function(reportType) {
        return reportConfig[reportType].uri;
    };

    /**
     * Format the dataset for the XLS and CSV files.
     *
     * @param {Object} columns - The metadata of each column of the report.
     * @param {Array}  dataSet - The datasource's data of the widget.
     *
     * @return {Array.<Object>} The dataSet to the XLS and CSV files
     */
    var formatDataSet = function(columns, dataSet) {
        var columnsName = Object.keys(columns);
        var rowValues = [];
        var formatedDataSet = [];

        dataSet.forEach(function(row) {
            columnsName.forEach(function(column) {
                rowValues.push(row[column]);
            });

            formatedDataSet.push(rowValues);
            rowValues = [];
        });

        return [columnsName].concat(formatedDataSet);
    };

    /**
     * Build the dataSet to be used on the backend to generate the report.
     *
     * @param {String} reportType - Defines what report type will be generate.
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} dataSet - Data used to create the reports.
     *
     * @return {Object} - Builded dataSet.
     */
    var buildDataSetToReportRow = function(reportType, metaData, dataSet) {
        if(reportType == reportConfig.PDF.type) {
            return dataSet.map(ZhUtil.clearRow);
        } else {
            return formatDataSet(metaData.columns, dataSet);
        }
    };

    /**
     * Build the reportRow to be used on the backend to generate the report.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} reportRow - Builded reportRow.
     */
    var buildReportRow = function(metaData, reportRowDataSet, widget) {
        var reportRow = {};
        reportRow.metaData = JSON.stringify(metaData);
        reportRow.dataSet = JSON.stringify(reportRowDataSet);
        reportRow.filter = JSON.stringify(widget.dataSourceFilter);
        return reportRow;
    };

    /**
     * Build the params to be used on the request.
     *
     * @param {Object} metaData - Defines the report's structure.
     * @param {Object} reportRowDataSet - Data used to create the reports.
     * @param {Widget} widget - Widget to report.
     *
     * @return {Object} - Builded params.
     */
    var buildParamsToRequest = function(metaData, reportRowDataSet, widget) {
        return {
            'requestType': 'Row',
            'row': buildReportRow(metaData, reportRowDataSet, widget)
        };
    };

    /**
     * Build the request to generate the report.
     *
     * @param {String} reportType         - Defines what report type will be generate.
     * @param {Object} metaData           - Defines the report's structure.
     * @param {Object} reportRowDataSet   - Data used to create the reports.
     * @param {function} callback         - Callback called after sucess.
     * @param {function} callbackError    - Callback called after error.
     * @param {Widget} widget             - Widget to report.
     *
     * @return {Promisse} Request's promisse.
     */
    var buildRequest = function(reportType, metaData, reportRowDataSet, callback, callbackError, widget) {
        var uri = getReportUri(reportType);
        var params = buildParamsToRequest(metaData, reportRowDataSet, widget);
        var callbackSuccess = buildCallbackOfSuccess(callback);
        var requestType = 'post';
        var reportsUrl = templateManager.project.reportsUrl;

        return restEngine.request(uri,params,callbackSuccess,requestType,callbackError,reportsUrl);
    };

    /**
     * Process to be executed after the request.
     *
     * @param {Object}      response        - Request result.
     * @param {Promisse}    deferred        - Promisse that defines the result of report generation.
     * @param {String}      reportType      - Defines what report type will be generated.
     *
     */
    var afterRequest = function(widget, response, deferred, reportType) {
        if (response) {
            var file = buildFilePath(response.messages[0].message);

            deferred.resolve(file);

            if(widget.afterOpenreport){
                widget.afterOpenreport();
            }
        }
        else {
            deferred.reject();
            publishErrorEvent();
        }
    };

    /**
     * Generates the report making a requests to the server to generate the file.
     *
     * @param {Object}          metaData        - Metadata for the construction of the report.
     * @param {Array.<Object>}  dataSet         - The widget's dataset.
     * @param {Function}        callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}        callbackError   - Callback that will be executed if the request failed.
     * @param {String}          reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateReport = function(widget, metaData, dataSet, callback, callbackError, reportType) {
        var deferred = ZHPromise.defer();
        var reportRowDataSet = buildDataSetToReportRow(reportType, metaData, dataSet);

        buildRequest(reportType, metaData, reportRowDataSet, callback, callbackError, widget).then(function(response) {
            afterRequest(widget, response, deferred, reportType);
        }).catch(function(response) {
            deferred.reject(response);
            publishErrorEvent();
        });

        return deferred.promise;
    };

    /**
     * Returns the params to make the request.
     *
     * @param {Object} actionControl - The widget's actionControl.
     * @param {Widget} widget        - Widget where the report is being generated.
     * @param {Object} metaData      - Metadata for the construction of the report.
     *
     * @return {Object} Params to the request.
     */
    function getParams(actionControl, widget, metaData){
        var params = {
            'requestType': 'FilterData',
            'filter': widget.dataSourceFilter,
            'metaData': metaData
        };
        if(actionControl.dataSourceName){
            params.dataSourceName = actionControl.dataSourceName;
        } else if(actionControl.collectionName){
            params.collectionName = actionControl.collectionName;
        } else {
            var reportRow = {};
            reportRow.metaData = JSON.stringify(metaData);
            reportRow.dataSet = JSON.stringify(widget.dataSource.data);
            reportRow.filter = JSON.stringify(widget.dataSourceFilter);

            params = {
                'requestType': 'Row',
                'row': reportRow,
            };
        }
        return params;
    }

    /**
     * Generates the report making only one request to the server generate the file.
     *
     * @param {Object}   metaData           - Metadata for the construction of the report.
     * @param {Object}   actionControl      - The widget's actionControl.
     * @param {Object}   widget             - Widget where the report is being generated.
     * @param {Function} callback           - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function} callbackError      - Callback that will be executed if the request failed.
     * @param {String}   [reportType='PDF'] - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created of rejected if somethig went wrong.
     */
    var _generateServerSideReport = function(metaData, actionControl, widget, callback, callbackError, reportType) {
        var exporterUrl = templateManager.project.exporterUrl;
        var deferred = ZHPromise.defer();
        var uri = getReportUri(reportType);
        var params = getParams(actionControl, widget, metaData);

        restEngine.request(uri, params, buildCallbackOfSuccessForDataExporter(exporterUrl, callback), 'post', callbackError, exporterUrl)
            .then(
                function(response) {
                    afterRequest(widget, response, deferred, reportType);
                })
            .catch(
                function(response) {
                    deferred.reject(response);
                    publishErrorEvent();
                }
            );

        return deferred.promise;
    };

    /**
     * Publish a new event. Lauched after the report failed.
     */
    var publishErrorEvent = function() {
        eventAggregator.publish('afterReportError');
    };

    /**
     * Creates a metaData for the translate of fixed words on the report.
     *
     * @param {Object} metaData - Metada of the report.
     *
     * @return {Object} The metadata with the configuration of the i18n
     */
    var _i18nMetaData = function(metaData) {
        var words = {
            'Page': ScreenService.i18n('Page'),
            'Report': ScreenService.i18n('Report'),
            'Version': ScreenService.i18n('Version')
        };

        metaData.words = words;
        metaData.lang = templateManager.lang;

        Object.keys(metaData.columns).forEach(function(key) {
            var column = metaData.columns[key];

            column.description = ScreenService.i18n(column.description);
        });

        metaData.groups.forEach(function(group) {
            Object.keys(group).forEach(function(key) {
                var grp = group[key];
                if (grp) {
                    grp.label = ScreenService.i18n(grp.label);
                }
            });
        });

        return metaData;
    };

    /**
     * Get the action control's configuration.
     *
     * @param  {Widget} widget     - Widget where the report is being generated.
     * @param  {String} reportType - The type of the report.
     *
     * @return {Object | Boolean} A object containing the configuration of the floating action or true.
     */
    var getActionControl = function(widget, reportType) {
        return Util.getNestedProperty(widget, ['floatingControl', reportConfig[reportType].action]);
    };

    /**
     * Generates the report.
     *
     * @param {Object}      customData      - Informations to customize the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Function to be executed after the file be generated, reciving the name of the generated file.
     * @param {Function}    callbackError   - Function to be executed if happen a error while generating the file.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created o rejected.
     */
    this.generateReport = function generateReport(customData, sourceWidget, callback, callbackError, reportType) {
        if (Util.getNestedProperty(sourceWidget, ['dataSource', 'data', 'length'])) {
            customData = customData || {};
            checkConfiguration();
            var widget = sourceWidget;
            var promise;

            buildMetaData(customData, widget, reportType, function(metaDataResolved) {
                var metaDataInternationalized = _i18nMetaData(metaDataResolved);

                triggerBeforeGenerateReport(widget, metaDataInternationalized);

                var actionControl = getActionControl(widget, reportType);

                if (typeof actionControl === "object" && actionControl.runAt == 'server') {
                    promise = _generateServerSideReport(metaDataInternationalized, actionControl, widget, callback, callbackError, reportType);
                }
                else {
                    promise = buildDataSet(metaDataInternationalized, widget, reportType).then(function(dataSet) {
                        return _generateReport(widget, metaDataInternationalized, dataSet, callback, callbackError, reportType);
                    });
                }
            });

            return promise;
        }
        else {
            ScreenService.errorNotification("Can't create a report without data!");
        }
    };

    /**
     * Returns a callback to be executed if the report is created succefully.
     *
     * @param {Function} callback - Function executed after the test.
     *
     * @return {Function} Callback that tests if the screen is able to open the popup creating a fake popup and, warning the user if the windows's popups are blocked.
     */
    var _previewCallback = function(callback) {
        return function(data) {
            var popup_window = window.open(data.path, '_blank');
            if (!popup_window) {
                ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
            }
            if (callback) {
                callback(data);
            }
        };
    };

    /**
     * Creates the report and open it rigth after.
     *
     * @param {Object}      customData      - Custom data for the generation of the report.
     * @param {Widget}      sourceWidget    - Widget where the report is being generated.
     * @param {Function}    callback        - Callback used to create a success callback that will be executed if the request does not failed.
     * @param {Function}    callbackError   - Callback that will be executed if the request failed.
     * @param {String}      reportType      - The type of the report.
     *
     * @return {Promise} Promise resolved when the file is created or rejected
     */
    this.openReport = function openReport(customData, sourceWidget, callback, callbackError, reportType) {
        return this.generateReport(customData, sourceWidget, _previewCallback(callback), callbackError, reportType);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhReports', ZhReports);
});


// FILE: src/ZhReportsDirective.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

function ReportsDirective(ApplicationContext){
    return {
        priority: 2,
        link: function($scope) {
            /**
             * Generate the report of the given widget.
             * @param  {Widget}   widget  Widget where the report is being generated.
             * @param  {String}   type    The type of the report.
             */
            $scope.generateReport = function(widget, type) {
                ApplicationContext.ZhReports.openReport(widget.reportCustomData, widget, undefined, undefined, type);
            };
        }
    };
}

ZeedhiDirectives.directive(
    'zhFloatingControl', ['ApplicationContext', ReportsDirective]
);


// FILE: bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js

// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-select-autocomplete';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
    return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
    window.templateUrls.push({
        'package': packageName,
        'baseUrl': './bower_components/' + packageName + '/dist/templates/'
    });
}

// FILE: src/controllers/zh-select-autocomplete.controller.js
function ZhSelectAutoCompleteController($scope, $rootScope, $timeout, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, EventChildPropagation, DecodeValueFieldService, $filter, metaDataFactory) {

	var self = this;
    var lastSearch = "";
    var propertyToSearch = [];
	var isMakingRequest = false;
	var shouldSelectAfterLoad = false;

	$scope.field.lazyLoadEvent = $scope.field.lazyLoadEvent || templateManager.project.lazyLoadEvent;

	$scope.isRequestBeingMade = function() {
		return isMakingRequest;
	};

	$scope.selectAfterLoad = function() {
		shouldSelectAfterLoad = true;
	};

    var dataInitialized = Util.isUndefinedOrNull($scope.field.dataSource.lazyLoad) ?
            true : !$scope.field.dataSource.lazyLoad;

	$scope.viewMore = ScreenService.i18n("View more...");
	$scope.filteredData = [];
	$scope.showDropdown = true;
	var filterUpdated = false;

	var oldFieldToggleDropdown = $scope.field.toggleDropdown;
    $scope.field.toggleDropdown = function(){
        $scope.showDropdown = !$scope.showDropdown;
    };

	var oldFieldInitializeField = $scope.field.initializeField;
	$scope.field.initializeField = function() {
		if(!dataInitialized){
			dataInitialized = true;
			updateDatasource(lastSearch, true);
		}
	};

	var parentReload = $scope.field.reload.bind($scope.field);

	/**
	 * Override the field function reload to initialize the field when called.
	 * @param {Object[]} [addictionalFilters] - An object of filters to be applied when reloading the field
	 * @returns {Promise.<Array>} Field data with the given fields.
	 */
	$scope.field.reload = function(addictionalFilters){
		$scope.field.initializeField();
		return parentReload(addictionalFilters).then(function(response){
			updateDatasource();
			return response;
		});
	};

	/**
	 * Get a row from dataSets
	 *
	 * @function getNewData
	 * @private
	 * @param   {Object} keys - DataSets returned by request
	 * @returns {Array} Returns a data array.
	 */
	function getNewData(dataSets){
		var data;
		result = dataSets.dataset;
		var keys = Object.keys(result);

		if (keys && keys.length > 0 && keys[0] !== '0') {
			var dataSetName = $scope.field.dataSource.name.split('/').pop();
			if (result[dataSetName]) {
				data = result[dataSetName];
			}
		} else {
			data = result;
		}

		return data;
	}

	/**
	 * Make a request to get an specific value.
	 *
	 * @function findNewValue
	 * @private
	 * @param   {Array} keys - CurrentRow's keys.
	 * @param   {Field} field - Field.
	 * @returns {Array} Returns a row.
	 */
	function findNewValue(keys, field, value){
		var filter = [];
		keys.forEach(function(key){
			filter.push({
				"name": key,
				"operator": '=',
				"value": value[key]
			});
		});
		var storageStrategy = field.dataSource.getStorageStrategy();
		return storageStrategy.getDataSource(field.dataSource.name, function(){}, filter, 1, 2, undefined, 0);
	}

	/**
	 * Prepares the value.
	 *
	 * @function findNewValue
	 * @private
	 * @param   {Array} valueFields - The field's valueFields
	 * @param   {String|Object} value - the value to be inserted.
	 * @returns {Object} The prepared value.
	 */
	function prepareValue(valueFields, value) {
		var valueBuilt = {};

		if(valueFields.length === 1 && typeof value !== 'object'){
			valueBuilt[valueFields[0]] =  value;
		} else {
			valueBuilt = value;
		}

		return valueBuilt;
	}

	/**
	 * Override the field function value to use valueFields.
	 * @param {Object} - The value to be set.
	 */
	var oldFieldValue = $scope.field.value;
	$scope.field.value = function(value){
		if(value){
			var shouldMakeRequest = $scope.field.dataSource.rest && $scope.field.hasPagination;
			currentRow = $scope.field.widget.currentRow;
			valueFields = $scope.field.valueFields;

			value = prepareValue(valueFields, value);

			if(shouldMakeRequest){
				findNewValue(valueFields, $scope.field, value).then(function(result){
					result = getNewData(result);
					if(result.length === 1){
						self.addItemAsSelected(result[0], $scope.field);
					} else {
						currentRow[$scope.field.name] = "";
					}
				});
			} else {
				var row = $scope.field.dataSource.data.filter(function(row){
					var result = true;

					valueFields.forEach(function(key){
						if(row[key] !== value[key]){
							result = false;
						}
					});

					return result;
				});

				if(row.length === 1){
					self.addItemAsSelected(row[0], $scope.field);
				} else {
					currentRow[$scope.field.name] = "";
				}
			}
		} else {
			return this.widget.currentRow[this.name];
		}
	};

	/**
	 * Extends the behavior of it's parent (ListController).
	 */
	function extendBehavior() {
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

    /**
     * Retrieves the code to be executed on FieldOnAdd event.
     * @returns {String} - Code to be executed.
     */
	function getOnAddCode(){
		var onAddEventCode = $scope.field.events.filter(function(event) {
			return event.name === "FieldOnAdd";
		});

		if(onAddEventCode.length > 0){
			return onAddEventCode[0].code;
		} else {
			return "args.owner.widget.onAdd()";
		}
	}

	var onAddCode = getOnAddCode();

	/**
	 * Build all actions of the widget that appears when the field's popup is opened.
	 *
	 * @function buildActions
	 * @private
	 * @param   {Field} field - Widget owner
	 * @returns {Array} - Actions of the widget
	 */
	function buildActions(field) {
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": onAddCode,
					"id": 9999
				}]
			});
		}

		return actions;
	}

    /**
     * Initialize the field.
     *
     * @function initialize
     */
    (function initialize() {
        initializeValuesOnCurrentRow();
        initializeFieldProperties();
        return $scope.field.dataSource.data;
    })();

	/**
     * Config which template of widget will appear when the field's popup is opened.
     *
     * @function configTemplate
     * @private
     * @param   {Field} field - Widget owner
     * @returns {String} template - Widget template
     */
	function configTemplate(field){
		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		}
		else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		}
		else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		return template;
	}

	/**
     * Create the fields to be inserted
     *
     * @function buildFields
     * @private
     * @param   {descriptionField} displayedField - Used to create the fields
     * @param   {Number} key - Array position
     */
	function buildFields(displayedField, key){
        this[key] = {
            "name": displayedField,
            "showLabel": false,
            "showOnList": true,
            "mask": this.mask
        };
    }

    /**
     * Insert labels on the fields.
     *
     * @function insertLabels
     * @private
     * @param   {String} label - label to be inserted
     * @param   {Number} key - Array position
     */
    function insertLabels(label, key){
        this[key].label = label;
        this[key].showLabel = true;
    }

	/**
     * Build all fields of the widget that appears when the field's popup is opened.
     *
     * @function buildSelectWidgetFields
     * @private
     * @param   {Field} field - Widget owner
     */
	function buildSelectWidgetFields(field) {
        var fields = [];

        if (field.fields && field.fields.length) {
            fields = field.fields;
        } else {
            field.descriptionFields.forEach(buildFields, fields);
        }

        if(field.displayedFieldsLabels){
            field.displayedFieldsLabels.forEach(insertLabels, fields);
        }

        fields.forEach(function(currentElement) {
            metaDataFactory.fieldFactory(currentElement, field.selectWidget, true);
        });
	}

	/**
     * Build all events of the widget that appears when the field's popup is opened.
     *
     * @function buildEvents
     * @private
     * @returns {Array} - Events of the widget
     */
	function buildEvents(){
		return [
			{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			},
			{
				"name": "WidgetAfterLoadDataSource",
				"code": "args.owner.updateDatasource()",
				"id": 9999
			}
		];
	}

	/**
	 * Set the args' row as selected.
	 *
	 * @function setSelected
	 * @private
	 * @param {Object} args - The window args.
	 */
	function setSelected(args) {
		if (!$scope.field.allowSelectOnlyLastChild || ($scope.field.allowSelectOnlyLastChild && this.isTree($scope.field.selectWidget) && isLastChild(args.row))) {
			ScreenService.closePopup();
			self.onAutoCompleteItemClick(args.row);
		}
	}

	/**
	 * Routine executed after the field's popup be closed.
	 *
	 * @function afterClosePopup
	 * @private
	 * @param {Field} field - The field that had it's popup closed.
	 */
	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	/**
	 * Build the widget that appears when the field's popup is opened.
	 *
	 * @function buildSelectWidget
	 * @private
	 * @param {Field} args - Widget owner.
	 * @param {Object} widgetConfig - Widget's propertys
	 */
	function buildSelectWidget(field, widgetConfig) {
		if (!field.selectWidget){
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 9000,
				"name": "Widget",
				"label": field.label,
				"template": widgetConfig.template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField,
					"customizationAction": {
						"layout": false,
						"filter": false,
						"view": false
					},
					"xlsAction": false,
					"pdfAction": false,
					"csvAction": false
				},
				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,
				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": [],
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage || 30,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"actions": widgetConfig.actions,
				"events": EventChildPropagation.propagate(widgetConfig.widgetEvents, field),
				"searchCriteria": field.searchCriteria
			});
			buildSelectWidgetFields(field);
		}
		field.selectWidget.setSelected = setSelected;
		field.selectWidget.updateDatasource = updateDatasource;
		field.selectWidget.afterClosePopup = afterClosePopup;
	}

	/**
     * Reload the field
     *
     * @function processReload
     * @private
     * @param   {boolean} isToReload
     * @param   {Field} field
     * @returns {Promise} - A resolved promise
     */
	function processReload(isToReload, field){
		var defer = ZHPromise.defer();

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.reload().then(defer.resolve, defer.resolve);
		}
		else {
			defer.resolve();
		}

		return defer.promise;
	}

	/**
     * Change the dataSource's data to the filtered by the given value.
     *
     * @function innerSearch
     * @private
     * @param {String} selectListSearch - The value to be searched on the field.
     * @param {Field} field - The field where will be searched the values and changed the data of the dataSource.
     */
    function innerSearch(selectListSearch, field) {
        if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
            var dataSource = field.dataSource;
            var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
            dataSource.filter(filter, function(data) {
                dataSource.data = data;
                templateManager.updateTemplate();
            });
        }
    }

    /**
     * Removes all filters from the dataSource's data
     *
     * @function  cleanFieldSearch
     * @private
     * @param {Field} field - The field where will be removed the filters of the dataSource's data.
     */
    function cleanFieldSearch(field) {
        innerSearch("", field);
    }

	/**
     * Build the Widget to be open
     *
     * @function buildFieldWidget
     * @private
     * @param   {Field} field - Widget owner
     * @returns {Promise} - A resolved promise
     */
	function buildFieldWidget(field) {
		cleanFieldSearch(field);

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));

		if (!field.selectWidget) {
			initializeSelectWidget(field);
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		return processReload(isToReload, field);
	}

	/**
     * Build configs to be used on the widget
     *
     * @function buildwidgetConfig
     * @private
     * @param   {Field} field - field used to build the configs
     * @returns {Object} Object with the configs
     */
	function buildWidgetConfig(field) {
		var widgetConfig = {};
		widgetConfig.template = configTemplate(field);
		widgetConfig.actions = buildActions(field);
		widgetConfig.widgetEvents = buildEvents();
		return widgetConfig;
	}

	/**
     * Initialize the field's SelectWidget propertys
     *
     * @function initializeSelectWidget
     * @private
     * @param   {Field} field - SelectWidget owner
     */
	function initializeSelectWidget(field) {
		buildSelectWidget(field, buildWidgetConfig(field));
		overrideReload(field);
	}

	var oldReload;
	function overrideReload(field) {
		oldReload = field.selectWidget.reload;
		field.selectWidget.reload = function() {
			if(dataInitialized){
				return oldReload.call(field.selectWidget);
			} else {
				return ZHPromise.when();
			}
		};
	}

	/**
	 * Call the beforeSelectOpen event if defined
	 *
	 * @function processBeforeSelectOpenEvent
	 * @private
	 * @param   {Field} field - Widget owner
	 */
	$scope.processBeforeSelectOpenEvent = function(field){
		if (field.beforeSelectOpen) {
			field.beforeSelectOpen();
		}
	};

	/**
     * Call the afterSelectOpen event if defined
     *
     * @function processAfterSelectOpenEvent
     * @param   {Field} field - Widget owner
     */
	$scope.processAfterSelectOpenEvent = function(field){
		if (field.afterSelectOpen) {
			field.afterSelectOpen();
		}
	};

	/**
     * Remove element Focus
     *
     * @function removeElementFocus
     * @private
     * @param   {Field} field - Field where the focus will be removed
     */
	function removeElementFocus(field) {
		if (field.element) {
			field.element.blur();
			window.getSelection().removeAllRanges(); //bug webkit
		}
	}

	/**
     * Open the popup of the field
     *
     * @function openField
     *
     * @param   {Field} field - Popup owner
     */
	$scope.openField = function(field, $event) {
        $event.stopPropagation();
        $scope.hideList();
		if (!field.isReadOnly()) {
			$scope.processBeforeSelectOpenEvent(field);
			removeElementFocus(field);
            $scope.inputWithEditableContent.click();
			buildFieldWidget(field).then(function() {
				$scope.clearSearchFilter();
				ScreenService.openPopup(field.selectWidget);
				$scope.processAfterSelectOpenEvent(field);
			});
		}
	};

	/**
	 * Open the popup of the field on the scope.
	 */
	var oldFieldOpenField = $scope.field.openField;
	$scope.field.openField = function() {
		$scope.openField($scope.field);
	};

    /**
     * Config the property valueField because the controller works only with valueFields.
     *
     * @function configValueFields
     * @private
     */
    function configValueFields(){
        var field = $scope.field;

        if(!field.valueFields){
            field.valueFields = [field.valueField];
        }
    }

    /**
     * Set the property descriptionField to avoid component break if not defined and config the property
     * descriptionField because the controller works only with descriptionFields.
     *
     * @function configDescriptionFields
     * @private
     */
    function configDescriptionFields() {
        var field = $scope.field;
        field.descriptionFields = field.descriptionFields ? field.descriptionFields : [field.descriptionField || field.name];
    }

	/**
	 * Clear the outdata related to the component.
	 */
	var oldFieldClearByOutData = $scope.field.clearByOutData;
	$scope.field.clearByOutData = (function() {
		for (var prop in this.outData) {
			this.widget.currentRow[this.outData[prop]] = "";
		}
	}).bind($scope.field);

	/**
	 * Clear the value of the selected row by the component from the current row.
	 *
	 * @function clearByName
	 * @public
	 */
	var oldFieldClearByName = $scope.field.clearByName;
	$scope.field.clearByName = (function () {
		this.widget.currentRow[this.name] = "";
	}).bind($scope.field);

	/**
	 * Clear the outDescription related to the component
	 *
	 * @function clearByOutDescription
	 * @public
	 */
	var oldFieldClearByOutDescription = $scope.field.clearByOutDescription;
	$scope.field.clearByOutDescription = (function() {
		this.widget.currentRow[this.outDescription] = "";
	}).bind($scope.field);

	if ($scope.selectField) {
		var oldSelectFieldOpenField = $scope.selectField.openField;
		$scope.selectField.openField = function() {
			$scope.openField($scope.selectField);
		};
	}

	/**
	 * Set the widget of the component as changed.
	 *
	 * @function setChanged
	 * @private
	 * @param {Field} field - The field to have it's current row set as changed.
	 */
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

    /**
     * Clear the filter of the component, and the values related to him from the current row.
     *
     * @function clearField
     * @public
     * @param {Field} field - The field to have it's values cleared.
     */
    $scope.clearField = function(field, $event) {
        $event.stopPropagation();
		$scope.resetPlaceholderValue();
        if (field.isReadOnly()) {
            return;
        }

        field.clearByOutData();
		field.clearByName();
		field.clearByOutDescription();
		field.dataSource.clearCheckedRows();
		setChanged(field);
		if (field.change) {
			field.change();
		}
		$scope.clearAutoCompleteListData();
		$scope.hideList();
		templateManager.updateTemplate();
		$scope.removeFocusOfInput();
	};

	$scope.hasValueSetted = function() {
		var field = $scope.field;
		var currentRow = field.widget.currentRow;
		return !!currentRow[field.name];
	};

    $scope.$watch($scope.hasValueSetted, function(newVal){
    	$scope.showSpan = newVal;
    });

    /**
     * Searches for the given value if the value is valid.
     *
     * @function search
     * @public
     * @param {String} selectListSearch - The value to be searched. Should be bigger than 0.
     * @param {Field} field - The field that will have the data that will be searched.
     */
    $rootScope.search = function(selectListSearch, field) {
        var minLength = field.minimumInputLength || 0;
        if (selectListSearch.length > minLength) {
            innerSearch(selectListSearch, field);
        }
    };

	/**
	 * Add the given item as selected
	 *
	 * @function addItemAsSelected
	 * @public
	 * @param {Object} item - The selected item.
	 * @param {Field} field - The item's field, where the item will be set as selected.
	 */
	this.addItemAsSelected = function(item, field) {
		(field.outData ? changeByOut : changeByName)(field, item);
		if(field.outDescription){
			changeByDescription(field, item);
		}
		item.__isSelected = true;
		field.dataSource.checkedRows = [item];
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(item);
		}
		if (field.change) field.change();
	};

	/**
	 * Returns the mounted description of the selected value.
	 *
	 * @function getDescriptionFromSelectedValue
	 * @public
	 * @returns {String} The value that should be displayed of the selected row.
	 */
	$scope.getDescriptionFromSelectedValue = function() {
		var fieldName = $scope.field.name;
		var currentRow = $scope.field.widget.currentRow;

		return DecodeValueFieldService.model($scope.field, currentRow[fieldName]);
	};

	/**
	 * Returns the item's identifier.
	 *
	 * @function getItemIdentifier
	 * @public
	 * @param   {Object} item - The item to get it's identifier.
	 * @returns {String} The item's identifier.
	 */
	this.getItemIdentifier = function(item) {
		var identifier = [];
		var field = $scope.field;
		var valueFields = field.valueFields;

		for(var valueField in valueFields){
			identifier.push(item[valueFields[valueField]]);
		}

		return identifier.join(" | ");
	};

	/**
	 * Return the ordered data.
	 *
	 * @function orderDropdown
	 * @private
	 * @param {Object} data - The data to be ordered.
	 * @param {String} order - The selectWidget's order.
	 * @returns {Object} The ordered data.
	 */
	function orderDropdown(data, order){
		return $filter('orderBy')(data, order);
	}

    /**
     * Updates the dataSource, applying any typed filter.
     *
     * @function updateDatasourceRemote
     * @private
     * @param {DataSourceFilter} itemSearch - The field's input value, used to filter the dataSource.
     */
    function updateDatasourceRemote(itemSearch, isToReload){
        if($scope.field.dataSource.rest){
            if((itemSearch !== undefined) && itemSearch !== lastSearch || isToReload){
                if($scope.field.searchableFields){
					var filterSet = false;
					var fieldName = $scope.field.searchableFields.join("|");
					if(!$scope.filterToAdd){
						$scope.filterToAdd = {};
						filterSet = true;
					}
					$scope.filterToAdd.name = fieldName;
					$scope.filterToAdd.operator = "LIKE_ALL";
					$scope.filterToAdd.value = buildSearchValue(itemSearch);

					if(filterSet){
						$scope.field.dataSourceFilter = $scope.field.dataSourceFilter || [];
						$scope.field.dataSourceFilter.push($scope.filterToAdd);
					}
					isMakingRequest = true;
					if($scope.field.dataSource.disableLoader && $scope.showLoader){
						$scope.showLoader();
					}
					$scope.field.dataSource.load($scope.field.dataSourceFilter, 1, $scope.field.selectWidget.itemsPerPage).then(function(){
						if($scope.field.selectWidget.order){
							$scope.field.dataSource.data = orderDropdown($scope.field.dataSource.data, $scope.field.selectWidget.order);
							$scope.filteredData = $scope.field.dataSource.data;
						}

						if(shouldSelectAfterLoad && $scope.field.dataSource.data.length === 1){
							self.addItemAsSelected($scope.field.dataSource.data[0], $scope.field);
							shouldSelectAfterLoad = false;
						}

						isMakingRequest = false;
						if($scope.field.dataSource.disableLoader && $scope.hideLoader){
							$scope.hideLoader();
						}
					});
                    resetDropdown();
                } else {
                    throw new Error('Field ' + $scope.field.name + ' must have searchableFields. ');
                }
            }
        }
	}

	/**
	 * Deletes the filter add by searching in selectAutocomplete
	 */
	$scope.$on("$destroy", function(){
		if($scope.field.dataSourceFilter){
			$scope.field.dataSourceFilter.forEach(function(filter){
				if(angular.equals(filter, $scope.filterToAdd)){
					$scope.field.dataSourceFilter.splice($scope.field.dataSourceFilter.indexOf(filter), 1);
				}
			});
		}

		if ($scope.field.selectWidget) {
			if (oldReload) {
				$scope.field.selectWidget.reload = oldReload;
			}
		}

	    $scope.field.toggleDropdown = oldFieldToggleDropdown;
		$scope.field.initializeField = oldFieldInitializeField;
		$scope.field.value = oldFieldValue;
		$scope.field.openField = oldFieldOpenField;
		$scope.field.clearByOutData = oldFieldClearByOutData;
		$scope.field.clearByName = oldFieldClearByName;
		$scope.field.clearByOutDescription = oldFieldClearByOutDescription;
		$scope.field.reload = parentReload;
		if ($scope.selectField) {
			$scope.selectField.openField = oldSelectFieldOpenField;
		}
	});


    /**
     * Creates a filtered local copy of the Datasource.
     *
     * @function updateDatasourceLocal
     * @private
     * @param {DataSourceFilter} itemSearch - The field's input value, used to filter the dataSource copy.
     */
    function updateDatasourceLocal(itemSearch) {
        var expression = function(row) {
            var result = false;
            $scope.field.searchableFields.forEach(function(fieldName) {
                var rowValue = String(row[fieldName]).toLowerCase();
                switch ($scope.field.searchIn){
                    case 'beginning':
                        result = rowValue.startsWith(itemSearch.toLowerCase()) ? true : result;
                        break;
                    case 'end':
                        result = rowValue.endsWith(itemSearch.toLowerCase()) ? true : result;
                        break;
                    default:
                        result = rowValue.indexOf(itemSearch.toLowerCase()) !== -1 ? true : result;
                        break;
                }
            });

            return result;
        };

		$scope.filteredData = $filter('filter')($scope.field.dataSource.data, expression);
		if($scope.field.selectWidget.order){
			$scope.filteredData = orderDropdown($scope.filteredData, $scope.field.selectWidget.order);
		}
		resetDropdown();
		$timeout(function(){
            $scope.$digest();
        }, 0, false);
    }

    /**
     * Chose which method will be used to filter the data according with
     * the searchWithPagination property
     *
     * @function    updateDatasource
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
    function updateDatasource(itemSearch, isToReload){
        itemSearch = itemSearch || $scope.itemSearch || "";

        if($scope.shouldMakeRequest(itemSearch, isToReload)){
            updateDatasourceRemote(itemSearch, isToReload);
        }
        else {
            updateDatasourceLocal(itemSearch);
        }

        lastSearch = itemSearch;
    }

	/**
     * Updates the dataSource and updates the selected item in dropdown.
     *
     * @function onItemSearchChanged
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
	function onItemSearchChanged(itemSearch, isToReload){
		updateDatasource(itemSearch, isToReload);
		if($scope.updateDropdown){
			$scope.updateDropdown();
		}
	}

    /**
     * Verifies if the request should be made.
     *
     * @function shouldMakeRequest
     * @private
     * @param {String} itemSearch - value used to filter the data
     */
	$scope.shouldMakeRequest = function(itemSearch, isToReload) {
		/* Checks if datasource is currently making requests */
		if($scope.field.searchWithPagination && dataInitialized && !isMakingRequest){
			/* Checks if the request has already been made */
			if(itemSearch == lastSearch && !filterUpdated){
				return false;
			} else {
				filterUpdated = false;
				/* Checks if the request is a more specific version of last request.
				*  If so, checks if the last request has already found all the data.
				*/
				return (!(itemSearch.length > 0 && itemSearch.indexOf(lastSearch) !== -1 && self.getData() && self.getData().length < 30) || isToReload);
			}
		} else {
			return false;
		}
    };

    updateDatasource();

    $scope.$watch('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget, true);
    $scope.$watch('itemSearch', Util.buildDebounceMethod(onItemSearchChanged, 500, false));

    /**
     * Propagate the field's DataSourceFilter to the selectWidget.
     *
     * @function propagateFieldDataSourceFilterToSelectWidget
     * @private
     * @param {DataSourceFilter} fieldDataSourceFilter - The field's DataSourceFilter to be propagated.
     */
    function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		filterUpdated = true;
		if ($scope.field.selectWidget) {
            $scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
        }
		updateDatasource($scope.itemSearch, true);
    }

    /**
     * Defines the operator that will be used by filter.
     *
     * @function setOperator
     */
    function setOperator() {
        $scope.field.operator = $scope.field.operator || "=";
    }


    /**
     * Set hasPagination's default value if not set
     */
    function configHasPagination() {
        var field = $scope.field;
		if(field.hasPagination === undefined){
			field.hasPagination = true;
		}
    }

    /**
     * Initialize the field properties.
     *
     * @function initializeFieldProperties
     */
    function initializeFieldProperties() {
        configValueFields();
        configDescriptionFields();
        configHasPagination();
        buildPropertiesToSearch();
        setOperator();
        initializeSelectWidget($scope.field);
    }

    /**
     * Test the row to check if it should be displayed on the field's dropdown.
     *
     * @function searchFilter
     * @public
     * @param   {Object} row - The row to be tested.
     * @returns {Boolean} Result telling if the field should be displayed.
     */
    $scope.searchFilter = function(row) {
        return Util.isUndefinedOrNull($scope.itemSearch) || propertyToSearch.some(function(data) {
            return ~row[data].indexOfLatin($scope.itemSearch);
        });
    };

	/**
	 * Get the item with the given identifier.
	 *
	 * @function searchItem
	 * @private
	 * @param   {String} itemIdentifier - The desired item's identifier.
	 * @returns {Object} The item with the given identifier.
	 */
	function searchItem(itemIdentifier){
		var field = $scope.field;
		var data = field.dataSource.data;
		var valueFields = field.valueFields;
		var itemIdentifierArr = itemIdentifier.split(" | ");

		function findItem(item){
			var result = true;
			for(var valueField in valueFields){
				result &= item[valueFields[valueField]] == itemIdentifierArr[valueField];
			}
			return result;
		}

		item = data.find(findItem);
		return item;
	}

	/**
	 * Field's routine executed after one row be selected.
	 *
	 * @function onAutoCompleteItemSelect
	 * @public
	 * @param {String} selectedItemIdentifier - The identifier of the selected row.
	 */
	$scope.onAutoCompleteItemSelect = function(selectedItemIdentifier) {
		var item = searchItem(selectedItemIdentifier);
		self.clearEditableInput();

		if(!Util.isUndefinedOrNull(item))
			self.addItemAsSelected(item, $scope.field);

		$scope.removeFocusOfInput();
	};

	/**
	 * Set the item passed as clicked, clearing the component's last input and hiding it's list.
	 *
	 * @function onAutoCompleteItemClick
	 * @public
	 * @param {Object} clickedItem - The clicked item, with it's data.
	 */
	this.onAutoCompleteItemClick = function(clickedItem) {
		self.clearEditableInput();
		self.addItemAsSelected(clickedItem, $scope.field);
		$scope.removeFocusOfInput();
	};

	/**
	 * Tells if the given row is the last child.
	 *
	 * @function isLastChild
	 * @private
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result of the test of the row being the last child.
	 */
	var isLastChild = function(row) {
		return row._childCount === 0;
	};

	/**
	 * Change the values on the current row used to identify the selected row and, print the fields value on the screen.
	 *
	 * @function changeByName
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByName(field, row) {
		var valueFields = field.valueFields;
		var currentRow = field.widget.currentRow;
		var fieldName = field.name;
		currentRow[fieldName] = "";

		function buildCurrentRowValueFields(value, key){
			currentRow[value] = row[value];
		}

		valueFields.forEach(buildCurrentRowValueFields);
		currentRow[fieldName] = self.getItemDescription(row);
	}

	/**
	 * Change the given field current row value based on the configured outdata.
	 *
	 * @function changeByOut
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByOut(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				}
				else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}

	/**
	 * Change the given field current row value based on the configured outDescription.
	 *
	 * @function changeByDescription
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByDescription(field, row) {
		field.widget.currentRow[field.outDescription] = self.getItemDescription(row);
	}

	/**
	 * Get the component's actual value, on it's placeholder, returning the fieldRow, if it is passed.
	 *
	 * @function getSelectionText
	 * @public
	 * @param {Object} [fieldRow] - The clicked item, with it's data.
	 */
	this.getSelectionText = function getSelectionText(fieldRow) {
		if ($scope.field.placeholder && !fieldRow) {
			selectionText = $scope.field.placeholder;
		}
		else {
			selectionText = fieldRow || ScreenService.i18n("Search");
		}
		return selectionText;
	};

	/**
	 * Open a new window.
	 *
	 * @param  {Field} field - The field with the container's name that will be opened.
	 */
	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	/**
	 * Build the value to be used as filter.
	 *
	 * @function buildSearchValue
	 * @private
	 * @param {String} itemSearch - The field's input value, used to build the filter.
	 */
	function buildSearchValue(itemSearch) {
		switch ($scope.field.searchIn){
			case 'beginning':
				return itemSearch + "%";
			case 'end':
				return "%" + itemSearch;
			default:
				return "%" + itemSearch + "%";
		}
	}

	/**
	 * Clear the variables related to the filter.
	 *
	 * @function clearSearchFilter
	 * @public
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result telling if the field should be displayed.
	 */
	$scope.clearSearchFilter = function(){
		$scope.itemSearch = '';
	};

	/**
	 * Insert into the propertysToSearch the fields that the user can search on the field's dropdown.
	 *
	 * @function addFieldNamesToPropertyToSearch
	 * @private
	 * @param {Array.<Fields>} fields - The fields that should be avaible to search.
	 */
	function addFieldNamesToPropertyToSearch(fields) {
		fields.forEach(function(field){
			propertyToSearch.push(field);
		});
	}

	/**
	 * Build the properties that the user can search on the field's  dropdown.
	 *
	 * @function buildPropertiesToSearch
	 * @private
	 */
	function buildPropertiesToSearch(){
		var field = $scope.field;
		$scope.field.searchWithPagination = Util.isUndefinedOrNull($scope.field.searchWithPagination) ? true : $scope.field.searchWithPagination;
		var propertyToAdd = null;
		if (!Util.isUndefined(field.searchableFields)) {
			propertyToAdd = (field.searchableFields);
		} else if (!Util.isUndefined(field.displayedFields)) {
			propertyToAdd = (field.displayedFields);
		} else {
			propertyToAdd = (field.descriptionFields);
		}

		addFieldNamesToPropertyToSearch(propertyToAdd);
	}

	/**
	 * Initilize the field.name value on it's currentRow.
	 *
	 * @function initializeValuesOnCurrentRow
	 * @private
	 */
	function initializeValuesOnCurrentRow() {
		var currentRow = $scope.field.widget.currentRow;
		var fieldName = $scope.field.name;
		currentRow[fieldName] = currentRow[fieldName] || "";
	}

	/**
	 * Remove the item from the current row by it's index.
	 *
	 * @function removeSelectedItemByIndex
	 * @private
	 */
	function removeSelectedItemByIndex(index) {
		var widgetParent = $scope.field.widget;
		var fieldName = $scope.field.name;
		widgetParent.currentRow[fieldName].splice(index, 1);
	}

	/**
	 * Functions used by template
	 **/

	/**
	 * Get the component's label.
	 *
	 * @function getFieldLabel
	 * @public
	 * @returns {String} The component's label.
	 */
	this.getFieldLabel = function() {
		return $scope.field.label;
	};

	/**
	 * Get the selected item's values.
	 *
	 * @function getSelectedValues
	 * @public
	 * @returns {Object} The values of the selected item.
	 */
	this.getSelectedValues = function() {
		return $scope.field.widget.currentRow[$scope.field.name];
	};

	/**
	 * Get the selected item's description field.
	 *
	 * @function getSelectedItemDescription
	 * @public
	 * @returns {String} The component's data.
	 */
	this.getSelectedItemDescription = function(item) {
		return item[$scope.field.descriptionField];
	};

	/**
	 * Get the component's data.
	 *
	 * @function getData
	 * @public
	 * @returns {Array<Object>} The component's data.
	 */
	this.getData = function() {
		return $scope.filteredData;
	};

	$scope.$watchCollection(function() {
		return self.getData();
	}, function(newValues, oldValues) {
		if (newValues !== oldValues) {
			if($scope.changeDropdownListPosition){
				$scope.changeDropdownListPosition();
			}
		}
	});

	$scope.dropDownLimit = 10;

	/**
	 * Updates the number of items in the dropdown.
	 *
	 * @function loadMore
	 * @public
	 */
	$scope.loadMore = function($event){
		if($scope.dropDownLimit === 20){
			$scope.dropDownLimit += 10;
			$scope.viewMore = ScreenService.i18n("Open search widget");
			$scope.openList();
			$event.stopPropagation();
		} else if($scope.dropDownLimit >= 30){
			$timeout(function(){$scope.inputWithEditableContent.blur();}, 0, false);
			resetDropdown();
			$scope.openField($scope.field, $event);
		} else {
			$scope.dropDownLimit += 10;
			$scope.openList();
			$event.stopPropagation();
		}
	};

	/**
	 * Set the dropdown to it's initial state.
	 *
	 * @function resetDropdown
	 * @private
	 */
	function resetDropdown() {
		$scope.dropDownLimit = 10;
		$scope.viewMore = ScreenService.i18n("View more...");
	}

	/**
	 * Verify if the LoadMore should be shown on dropDown.
	 *
	 * @function shouldShowLoadMore
	 * @public
	 */
	$scope.shouldShowLoadMore = function(){
		if($scope.field.hasPagination) {
			return $scope.filteredData.length >= $scope.dropDownLimit;
		} else {
			return false;
		}
	};

	/**
	 * Translate the text of the item according to field's option and i18n.
	 *
	 * @param  {Field}  field The field with the options.
	 * @param  {Object} item  The item With the text to be translated.
	 * @return {String}       The translated text.
	 */
	function translateText(field, item){
		if(!field.getTemplateSimpleName){
			Util.extend(field, new Templatable());
		}
		var text = item[field.name];
		return DecodeValueFieldService.model(field, text);
	}

	/**
	 * Returns the given item's text to be shown on the dropdown.
	 *
	 * @function getItemText
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemText = function(item) {
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var displayedFields = $scope.field.displayedFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(displayedFields){
			if(fields){
				displayedFields.forEach(buildTranslatedItemDescription);
			}
			else {
				displayedFields.forEach(buildItemDescription);
			}
		}
		else {
			if(fields){
				descriptionFields.forEach(buildTranslatedItemDescription);
			}
			else {
				descriptionFields.forEach(buildItemDescription);
			}
		}
		return itemDescription.join(textSpacer);
	};


	/**
	 * Returns the given item's descriptionField if the field has an descriptionField.
	 *
	 * @function getItemDescription
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemDescription = function(item){
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(fields){
			descriptionFields.forEach(buildTranslatedItemDescription);
		}
		else {
			descriptionFields.forEach(buildItemDescription);
		}
		itemDescription = itemDescription.join(textSpacer);

		return itemDescription;
	};

	/**
	 * Remove the item with the given index from the parent field's current row.
	 *
	 * @function onRemoveSelectedItem
	 * @public
	 * @param {Integer} item - The index of the item to be removed.
	 */
	this.onRemoveSelectedItem = function(removedItemIndex) {
		removeSelectedItemByIndex(removedItemIndex);
	};

	/**
	 * Functions called from directive
	 *
	 **/

	/**
	 * Functions used internally
	 **/

	/**
	 * Clear the field's search parameter and it's list data.
	 *
	 * @function clearAutoCompleteListData
	 */
	$scope.clearAutoCompleteListData = function() {
		$scope.itemSearch = '';
		$scope.clearInputFieldText();
		$scope.autoCompleteListData = [];
		$scope.clearSelectedItem();
		$scope.showSpan = true;
	};

	/**
	 * Clear the value on the field's input.
	 *
	 * @function clearEditableInput
	 * @public
	 */
	this.clearEditableInput = function() {
		$scope.inputWithEditableContent.val('');
	};

	/**
	 * Tells if the component should display the span with the currentRow value.
	 *
	 * @function shouldShowSpan
	 * @public
	 * @returns {Boolean} Value telling if the span should be displayed.
	 */
	this.shouldShowSpan = function() {
		var isReadOnly = $scope.field.isReadOnly();
		return isReadOnly || $scope.showSpan;
	};
}

// FILE: src/directives/zh-select-autocomplete.directive.js
// register jQuery extension to select by TAB and Navation by press TAB
jQuery.extend(jQuery.expr[':'], {
    focusable: function(el, index, selector) {
        return $(el).is('a, button, :input, [tabindex]');
    }
});

/**
 *
 */
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zhSelectAutocomplete', ['$timeout', 'eventEngine', zhSelectAutocomplete]);

    function zhSelectAutocomplete($timeout, eventEngine) {

        var EDITABLE_INPUT_MIN_WIDTH = 130;
        var ENTER = 13;
        var SPACE = 32;
        var DOWN = 40;
        var ESC = 27;
        var TAB = 9;
        var UP = 38;


        var linkFunction = function(theScope, theElement) {

            var $scope = theScope;
            var element = theElement;
            var selectedList = '';
            var lastMouseDownElement;
            var loaderElement = $('img.autocomplete-loading-icon');
            var namespace = $scope.$id;
            var documentEvents = [];
            var lazyLoadEventIndex;

            $scope.hideLoader = function() {
                loaderElement.hide();
            };
            $scope.showLoader = function() {
                loaderElement.css('display', 'initial');
            };

            $scope.itemSearch = $scope.itemSearch || "";
            $scope.selectAutoCompleteElement = $(element);
            $scope.inputWithEditableContent = $(element).find('.editable');

            var initialPlaceholderValue;

            $scope.spanValueField = $(element).find('span');

            $scope.inputWithEditableContent.change(function(e) {
                e.stopPropagation();
            });

            $scope.field.element.click(function(e) {
                e.stopPropagation();
            });

            if($scope.field.lazyLoadEvent){
                lazyLoadEventIndex = eventEngine.bindEvent($scope.inputWithEditableContent, $scope.field.lazyLoadEvent, function(){
                    $scope.field.initializeField();
                });
            }

            /*
            *   Prevent loader from canceling element click
            */
            $('div.zh-background-loading').mouseup( function(e){
            	if(lastMouseDownElement && lastMouseDownElement === $scope.inputWithEditableContent[0]){
                    $scope.inputWithEditableContent.click();
                }
            });

            $(document).on('mousedown.'+namespace, function(e) {
                lastMouseDownElement = e.target;
            });
            documentEvents.push('mousedown');

            createFocusListener();
            clickListener();
            KeyPressNavegationListener();

            /**
             * Creates the click clickListener.
             *
             * @function clickListener
             * @private
             */
            function clickListener() {
                var showSelect = $(element).find(".show-select");
                showSelect.click(function(event) {
                    event.stopPropagation();
                });
            }

            /**
             * Resets the placeholder value.
             *
             * @function resetPlaceholderValue
             * @private
             */
            $scope.resetPlaceholderValue = function() {
                if(initialPlaceholderValue){
                    $scope.inputWithEditableContent[0].placeholder = initialPlaceholderValue;
                } else {
                    initialPlaceholderValue = $scope.inputWithEditableContent[0].placeholder;
                }
            };

            /**
             * Creates the focus listener.
             *
             * @function createFocusListener
             * @private
             */
            function createFocusListener() {
                $scope.inputWithEditableContent.focus(function(event) {
                    $scope.inputWithEditableContent.val($scope.itemSearch);
                    $scope.resetPlaceholderValue();
                    $scope.inputWithEditableContent.select();
                    $scope.inputWithEditableContent.parent().scrollTop(0);
                    $scope.showSpan = false;
                    $scope.changeDropdownListPosition();
                    $scope.$apply();
                    $scope.processBeforeSelectOpenEvent($scope.field);
                    $scope.openList();
                    $scope.updateDropdown();
                    $scope.processAfterSelectOpenEvent($scope.field);
                    $scope.selectAutoCompleteElement.closest("[zh-scroller]").on("scroll." + $scope.$id, function(){
                        $scope.hideList();
                    });
                });
            }

            /**
             * Updates the dropdown and selects the first item
             *
             * @function updateDropdown
             *
             * @returns {Array.<Object>} The selected items list.
             */
            $scope.updateDropdown = function(){
                if($scope.selectAutoCompleteElement.find('.selected').length === 0 && $scope.itemSearch){
                    var spanList = $scope.selectAutoCompleteElement.find('.show-select span');

                    selectedList = spanList.eq(0).addClass('selected');
                }

                return selectedList;
            };

            /**
             * Creates the key press navigation listener.
             *
             * @function KeyPressNavegationListener
             * @private
             */
            function KeyPressNavegationListener() {
                var next;
                var navegationKeyPress = $scope.selectAutoCompleteElement;
                navegationKeyPress.on('keydown', 'input,select', function(event) {

                    var EVENTKEY = event;
                    var spanList = $scope.selectAutoCompleteElement.find('.show-select span');

                    //Fix cursor input 'auto' to change position
                    if(EVENTKEY.which === DOWN || EVENTKEY.which === UP ) {
                        EVENTKEY.preventDefault();
                    }

                    //Enter, Space or Tab
                    if (EVENTKEY.which === ENTER || EVENTKEY.which === TAB) {
                        EVENTKEY.preventDefault();

                        if (isShowMore(selectedList)) {
                            $scope.loadMore(event);
                        }
                        else {
                            if($scope.shouldMakeRequest($scope.itemSearch)) {
                                $scope.selectAfterLoad();
                            } else if(canSelect(selectedList)){
                                $scope.selectItem(selectedList);
                                $scope.inputWithEditableContent.val("");
                            }
                            if (getFocusableItems().length > 1) {
                                $scope.removeFocusOfInput();

                                $scope.focusOnNextItem();
                                $scope.hideList();
                            }
                        }

                    }
                    //Down
                    else if (EVENTKEY.which === DOWN) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.next();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.eq(0).addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.eq(0).addClass('selected');
                        }
                    }
                    //Up
                    else if (EVENTKEY.which === UP) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.prev();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.last().addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.last().addClass('selected');
                        }
                    }
                    //Esc
                    else if (EVENTKEY.which === ESC) {
                        $scope.hideList();
                        selectedList = '';
                        return false;
                    }
                    if (navegationKeyPress) {
                        var itemList = $scope.selectAutoCompleteElement.find('.list-items');
                        var item = navegationKeyPress.find('.selected');
                        var index;

                        if (item.offset() !== undefined) {
                            if (EVENTKEY.which === ENTER) {
                                $timeout(function() {
                                    spanList = $scope.selectAutoCompleteElement.find('.show-select span');
                                    index = $scope.dropDownLimit - 10;
                                    movimentation = (index + 3.7) * 32 + item.outerHeight() - itemList.innerHeight();
                                    animateItemFocus(itemList, movimentation);
                                    selectedList = spanList.eq(index);
                                    selectedList.addClass('selected');
                                }, 1000, false);
                            }
                            else {
                                index = parseInt(item.attr('data-item-index')) || parseInt(item.prev().attr('data-item-index')) + 1;
                                var movimentation = 0;
                                if (item.position().top + item.outerHeight() > itemList.innerHeight()) {
                                    movimentation = (index * 32 + item.outerHeight() - itemList.innerHeight());
                                    animateItemFocus(itemList, movimentation);
                                }
                                else if (item.position().top < 0) {
                                    movimentation = index * 32;
                                    animateItemFocus(itemList, movimentation);
                                }
                            }
                        }
                    }
                });
            }

            /**
             * Makes the animation to scroll the item for the one who has focus.
             *
             * @function animateItemFocus
             * @private
             * @param  {Object} itemList      - The element witch has the list of items
             * @param  {Number} movimentation - The value in pixels of the scrolling distance from the top.
             */
            function animateItemFocus(itemList, movimentation) {
                itemList.finish().animate({
                    scrollTop: movimentation
                }, 500);
            }

            /**
             * Get all the focusabel items.
             *
             * @function getFocusableItems
             * @private
             * @returns {Array.<Object>} The focusable items on the screen.
             */
            function getFocusableItems() {
                return $(':focusable');
            }

            /**
             * Test if the given selectedList is a valid row to be selected.
             *
             * @function canSelect
             * @private
             * @returns {Boolean} Result telling if the selectedList is a valid row to be selected.
             */
            function canSelect(selectedList) {
                return (typeof selectedList === "string" && selectedList !== "") ||
                    (typeof selectedList === "object" && selectedList.text() !== "") &&
                    selectedList.attr("data-item-identifier");
            }

            function isShowMore(selectedItem) {
                return selectedItem && selectedItem.hasClass("view-more");
            }

            /**
             * Open the field's dropdown list.
             *
             * @function openList
             */
            $scope.openList = function() {
                $(".zh-select-autocomplete .show-select").css("display", "none");
                $scope.selectAutoCompleteElement.find(".show-select").css("display", "block");
                $scope.selectAutoCompleteElement.find('.list-items').scrollTop(0);
            };


            /**
             * Change the dropdown list position on field.
             *
             * @function changeDropdownListPosition
             */
            $scope.changeDropdownListPosition = function() {
                var dropdown = $scope.selectAutoCompleteElement.find('.zh-select-autocomplete');
                var dropdownOffset = dropdown.offset().top;
                var listItemsElement = $scope.selectAutoCompleteElement.find('.list-items');

                updateListWidth(listItemsElement, dropdown);
                updateListPosition(listItemsElement, dropdownOffset, dropdown);
            };

            /**
             * Updates the position of the dropdown list.
             *
             * @function updateListPosition
             * @param {Element} listItemsElement - The list's DOM.
             * @param {Element} dropdownOffset - The dropdown's offset.
             * @param {Element} dropdown - The dropdown's DOM.
             */
            function updateListPosition(listItemsElement, dropdownOffset, dropdown) {
                var popupModal = $('.popup-modal');
                if (popupModal.length) {
                    dropdownOffset -= popupModal.offset().top;
                }
                $timeout(function() {
                    listItemsElement.css('top', dropdownOffset + dropdown.outerHeight() - getParentFixedTop(dropdown));
                    if (dropdown.offset().top + dropdown.outerHeight() + listItemsElement.outerHeight() + 10 > $(window).height()) {
                        listItemsElement.css('margin-top', -1 * (dropdown.outerHeight() + listItemsElement.outerHeight()));
                    }
                    else {
                        listItemsElement.css('margin-top', 0);
                    }
                }, 0, false);
            }

            /**
             * Updates the width of the dropdown list.
             *
             * @function updateListPosition
             * @param {Element} listItemsElement - The list's DOM.
             * @param {Element} dropdown - The dropdown's DOM.
             */
            function updateListWidth(listItemsElement, dropdown) {
                //Set the correct width field
                if(listItemsElement.width() !== dropdown.innerWidth()){
                    listItemsElement.width(dropdown.innerWidth());
                }
            }

            /**
             * Get the parent fixed top position element.
             *
             * @function getParentFixedTop
             * @param {String} el - A element of the DOM.
             */
            function getParentFixedTop(el) {
                var parents = el.parents();
                var top = 0;
                parents.each(function(idx, item) {
                    item = $(item);
                    if ((item.css('position') === "fixed" || item.css('position') === "absolute") &&
                        item.parent() && (item.parent().css('position') === "fixed" || item.parent().css('position') === "absolute")) {
                        top = item.offset().top;
                        return false;
                    }
                });

                return top;
            }

            /**
             * Clear the field input's text.
             *
             * @function clearInputFieldText
             */
            $scope.clearInputFieldText = function() {
                $scope.selectAutoCompleteElement.find('input:text').val('');
            };

            /**
             * Hide the fields's dropdown list.
             *
             * @function hideList
             */
            $scope.hideList = function() {
                $scope.selectAutoCompleteElement.find(".show-select").hide();
                $scope.selectAutoCompleteElement.find('.show-select span').removeClass('selected');
                $scope.selectAutoCompleteElement.closest("[zh-scroller]").off("scroll." + $scope.$id);
            };

            /**
             * Set as selected the given item.
             *
             * @function selectItem
             * @param {Object} selectedList - The DOM element of the selected row.
             */
            $scope.selectItem = function(selectedList) {
                if (!selectedList) return;
                selectedItem = selectedList.text();
                var itemIdentifier = selectedList.attr("data-item-identifier");
                $scope.onAutoCompleteItemSelect(itemIdentifier);
            };

            /**
             * Focus on the next item of the screen.
             *
             * @function focusOnNextItem
             */
            $scope.focusOnNextItem = function() {
                $scope.inputHasFocus = false;
                $canfocus = getFocusableItems();
                var index = $canfocus.index(document.activeElement) + 1;
                if (index >= $canfocus.length) index = 0;
                $canfocus.eq(index).focus();
            };

            $scope.focusOnInput = function() {
                $timeout(function() {
                    $scope.inputWithEditableContent.focus();
                }, 0, false);
            };

            $scope.removeFocusOfInput = function() {

                $timeout(function() {
                    $scope.inputWithEditableContent.blur();

                    if($scope.itemSearch){
                        $scope.inputWithEditableContent[0].placeholder = $scope.itemSearch;
                    }

                    $scope.hideList();
                    $scope.showSpan = $scope.hasValueSetted();
                    $scope.clearInputFieldText();
                });
            };

            $scope.clearSelectedItem = function() {
                var item = $scope.inputWithEditableContent.find('.selected');
                selectedList = '';
                item.removeClass('selected');
            };

            /**
             * Treats the click outside the component.
             */
            $(document).on("click." + namespace, function(event) {
                if (!$.contains($scope.selectAutoCompleteElement[0], event.target)){
                    $scope.removeFocusOfInput();
                }
            });
            documentEvents.push('click');

            $scope.$on("$destroy", function(){
                documentEvents.forEach(function(eventName){
                    $(document).off(eventName + "." + namespace);
                });

                $('div.zh-background-loading').off('mouseup');
                $scope.selectAutoCompleteElement.closest("[zh-scroller]").off("scroll." + $scope.$id);
                eventEngine.unbindEvent($scope.inputWithEditableContent, $scope.field.lazyLoadEvent, lazyLoadEventIndex);
            });

        };
        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();

// FILE: src/registers/zh-select-autocomplete.register.js
(function () {

	Configuration(registerSelectAutocomplete);

	function registerSelectAutocomplete(decodeTemplateFieldService) {
		decodeTemplateFieldService.registerViewTemplate(
			"zh-select-autocomplete#field/select-autocomplete.html",
			"zh-select-autocomplete#field/select-autocomplete-view.html"
		);
	}

})();

// FILE: bower_components/zhuaclfrontend/assets/dist/main.js
// FILE: assets/js/util/ZhuAclConfig.js
/* global Configuration */
function ZhuAclConfig() {
	var self = this;
	this.RULE_ALLOW = 'A';
	this.RULE_READONLY = 'R';
	this.RULE_INVISIBLE = 'I';

	this.menuElementsListWidgetName = 'elementsListWidget';
	this.containerElementsListWidgetName = 'containerElementsModal';
	this.containerElementsTreeWidgetName = 'containerElementsTree';

	this.containerType = 'container';

	this.getDefaultDataSourceFilter = function(groupIds, projectId, organizationId){
		var dataFilter = [];
		var groupFilter = {
			"name": "GROUP_ID",
			"operator": "=",
			"value": groupIds
		};
		var projectFilter = {
			"name": "PROJECT_ID",
			"operator": "=",
			"value": projectId
		};

		if(organizationId !== undefined){
			var organizationFilter = {
				"name": "ORGANIZATION_ID",
				"operator": "=",
				"value": organizationId
			};
			dataFilter.push(organizationFilter);
		}

		dataFilter.push(groupFilter);
		dataFilter.push(projectFilter);
		return dataFilter;
	};

	this.getAclMap = function(type){
		var aclMap = {
			"container": {
				"aclTypeCod": "C",
				"childs": ["widgets"]
			},
			"widgets": {
				"aclTypeCod": "W",
				"childs": ["widgets", "fields", "actions"]
			},
			"fields": {
				"aclTypeCod": "F",
				"childs": []
			},
			"actions": {
				"aclTypeCod": "A",
				"childs": []
			}
		};
		return aclMap[type] || {};
	};

	this.getRuleType = function(isVisible, isEditable){
		var ruleType;
		if(!isVisible){
			ruleType = self.RULE_INVISIBLE;
		} else if(!isEditable){
			ruleType = self.RULE_READONLY;
		} else {
			ruleType = self.RULE_ALLOW;
		}
		return ruleType;
	};

	this.getRuleConfiguration = function(ruleType){
		var ruleConfig = {
			isVisible: ruleType != self.RULE_INVISIBLE,
			isEditable: ruleType == self.RULE_ALLOW
		};
		return ruleConfig;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ZhuAclConfig', ZhuAclConfig);
});

// FILE: assets/js/util/ZhuAclMessages.js
var ZhuAclMessages = {
	CANCEL: 'Cancelar',
	YES: 'Sim',
	NO: 'Não',
	VISIBLE: 'visíveis',
	EDITABLE: 'editáveis',
	INVISIBLE: 'invisíveis'
};
(function(){
	var mappedMessages = [
		"ACCESS_DENIED",
		"RULE_NOT_SAVED",
		"SUC_RULE_INSERTED",
		"ERR_RULE_INSERTED",
		"CANNOT_LOAD_RULES",
		"CONFIRMATION_MESSAGE",
		"CONFIRM_CHILDS_QUESTION",
		"NSHOW_WITHOUT_ID",
		"NSHOW_WITHOUT_CONTAINER_CONFIG",
		"NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG"
	];
	mappedMessages.forEach(function addMappedMessages(code){
		ZhuAclMessages[code] = 'zhuacl#' + code;
	});
})();

// FILE: assets/js/services/ZhuAccessButtonService.js
/* global _Notification, Util, ZeedhiServices */
function ZhuAccessButtonService(){
	this.insertButtonInProject = function(project){
		var notification = {
			"label": "Access Control",
			"name": "accessControl",
			"icon": "block",
			"isVisible": false,
			"tooltip": "Access Control",
			"isAclNotification": true,
			"events": [
				{
					"name": "NotificationEvent",
					"code": "ZhuAclController.openAclModalWidget();"
				}
			]
		};
		try {
			notification = Util.extend(new _Notification(), notification);
			project.notifications = project.notifications || [];
			project.notifications.push(notification);
		} catch (e){
			console.error('O Controle de acesso à elementos só funciona em versões acima da 1.40.x do Zeedhi Framework');
		}
	};

	this.disableButtonInProject = function(project){
		if(project.notifications){
			project.notifications = project.notifications.filter(function(notification){
				return !notification.isAclNotification;
			});
		}
	};

	this.enableButtonInProject = function(project){
		if(project.notifications){
			project.notifications.forEach(function(notification){
				if(notification.isAclNotification){
					notification.isVisible = true;
				}
			});
		}
	};
}

ZeedhiServices.service("ZhuAccessButtonService", function(ApplicationContext){
	var instance = new ZhuAccessButtonService();
	ApplicationContext.registerInstance("ZhuAccessButtonService", instance);
	return instance;
});


// FILE: assets/js/services/ZhuAclService.js
/* global Util, Configuration, ZhuAclMessages*/
function ZhuAclService(requestDataSourceEngine, restEngine, ZhuAclConfig){

	this.getZhuAclRules = function(projectId, groupIds){
		return requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', ZhuAclConfig.getDefaultDataSourceFilter(groupIds, projectId), 1, undefined, undefined, undefined, 120000).then(function(response){
			var dataset = response.dataset || {};
			var rules = dataset.zhu_access_control_rule_login;
			if(Util.isArray(rules)){
				var objToReturn = {
					aclRules: rules,
					defaultRule: dataset.zhu_access_control_default_rule.defaultRule,
				};
				return objToReturn;
			} else {
				throw {code: ZhuAclMessages.CANNOT_LOAD_RULES};
			}
		});
	};

	this.getWidgetMetadata = function(widgetName){
		var widgetUrl = 'zhuacl#json/'+ widgetName +'.json';
		return restEngine.requestMetaData(widgetUrl).then(function(metadata){
			return metadata.data;
		});
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclService', ZhuAclService);
});

// FILE: assets/js/controllers/ZhuAclController.js
/* global openWindow, Util, Configuration, ZhuAclMessages */
function ZhuAclController(ScreenService, templateManager, metaDataFactory, ZHPromise, ZhuAclService, ZhuAccessButtonService, ZhuAclConfig, MessageService, ApplicationContext, eventAggregator){
	var self = this;
	var user;
	var validProperties = ['widgets', 'fields', 'actions'];
	var loadedContainers = {};

	function User(userId, projectId, organizationId, defaultRule){
		var currentGroupId;
		var userDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;

		this.setCurrentGroupId = function(id){
			currentGroupId = id;
		};

		this.getDefaultRule = function(){
			return userDefaultRule;
		};

		this.getCurrentGroupId = function(){
			return currentGroupId;
		};

		this.getProjectId = function(){
			return projectId;
		};

		this.getUserId = function(){
			return userId;
		};

		this.getOrganizationId = function(){
			return organizationId;
		};
	}

	this.getDefaultRule = function(){
		return user.getDefaultRule();
	};

	this.getCurrentGroupId = function(){
		return user.getCurrentGroupId();
	};

	this.getProjectId = function(){
		return user.getProjectId();
	};

	var prepareLabelToName = function(label){
		var name = label.toLowerCase();
		name = name.replace(/\ /g, "_");
		return name.replace(/[^\w\s]/gi, '');
	};

	var translateLabel = function(label){
		return ScreenService.i18n(label);
	};

	this.getRulesFromDatasource = function(rulesData){
		var userId = user.getUserId();
		var rules = {};
		rulesData.forEach(function(row){
			var obj = {
				ruleRow: row,
				userGroups: {}
			};
			obj.userGroups[userId] = {
				rule: row.RULE
			};
			rules[row.ELEMENT_UNIQUE_ID] = obj;
		});
		return rules;
	};

	this.getElementRule = function(rules, itemId){
		var itemRule = {};
		if(rules[itemId]){
			itemRule = rules[itemId].ruleRow;
		}
		return itemRule;
	};

	this.getPlannedItemToInsertInDataSet = function(rules, item, itemType, parentId){
		var uniqueId = item.id;
		var itemRule = self.getElementRule(rules, uniqueId);
		var itemToInsert = {
			"UNIQUE_ID": uniqueId,
			"NAME": item.name || prepareLabelToName(item.label),
			"LABEL": translateLabel(item.label || item.name),
			"TYPE": itemType,
			"RULE": itemRule.RULE || this.getDefaultRule(),
			"RULE_ID": itemRule.ID,
			"PARENT_ID": parentId,
			"USER_ID": user.getUserId(),
			"PROJECT_ID": user.getProjectId(),
			"ORGANIZATION_ID": user.getOrganizationId()
		};

		itemToInsert.isVisible = item.aclIsVisible;
		itemToInsert.isEditable = !item.aclReadOnly && item.aclIsVisible;

		return itemToInsert;
	};

	var insertContainerElementsListWidget = function(container){
		return getFactoredWidget(container, ZhuAclConfig.containerElementsListWidgetName, true).then(function(factoredWidget){
			container.actions = container.actions || [];
			return factoredWidget;
		});
	};

	this.openAclModalWidget = function(){
		insertContainerElementsListWidget(templateManager.container).then(function(widget){
			ScreenService.openPopup(widget);
		});
	};

	var redirectToMainWindow = function(){
		var dashboardMenu = templateManager.project.getMenu('dashboard');
		var openLinkByMenu = ApplicationContext.LibUtilities && ApplicationContext.LibUtilities.openLinkByMenu;
		if(dashboardMenu && openLinkByMenu){
			openLinkByMenu(dashboardMenu);
		} else {
			openWindow(templateManager.project.mainWindow);
		}
	};

	var verifyContainerRule = function(rule){
		var containerIsRestrict = rule === ZhuAclConfig.RULE_INVISIBLE;
		if(containerIsRestrict){
			var actions = [{
				label: ScreenService.i18n("Ok"),
				code: redirectToMainWindow
			}];
			var dialogConfig = {
				type: 'ALERT',
				icon: 'exclamation',
				actions: actions
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.ACCESS_DENIED, {}, dialogConfig);
		}
		return !containerIsRestrict;
	};

	var mergeUndefinedRules = function(groupIds, itemRule, parentRules){
		parentRules = parentRules || {};

		var definedRule = itemRule.rule;
		var groupRules = parentRules.groupRules || {};

		if(!definedRule){
			definedRule = parentRules.rule || templateManager.project.aclDefaultRule;
		} else {
			var undefinedGroups = itemRule.undefinedGroups || [];
			groupIds.forEach(function(groupId){
				var parentRule = groupRules[groupId] || templateManager.project.aclDefaultRule;
				var isUndefinedGroup = undefinedGroups.indexOf(groupId) !== -1;
				var groupRule = isUndefinedGroup ? parentRule : definedRule;
				groupRules[groupId] = groupRule;

				if(definedRule !== ZhuAclConfig.RULE_ALLOW && groupRule !== ZhuAclConfig.RULE_INVISIBLE){
					definedRule = groupRule;
				}
			});
			itemRule.rule = definedRule;
		}

		return {rule: definedRule, groupRules: groupRules};
	};

	var updateContainerAcl = function(rules, groupIds, userId){
		var applyRule = function(item, parentRules){
			var id = item.id;
			var userRule = Util.getNestedProperty(rules, [id, 'userGroups', userId]) || {};
			var mergedItem = mergeUndefinedRules(groupIds, userRule, parentRules);
			var itemRule = mergedItem.rule;

			item.aclIsVisible = itemRule != ZhuAclConfig.RULE_INVISIBLE;
			item.aclReadOnly = itemRule != ZhuAclConfig.RULE_ALLOW;

			return mergedItem;
		};

		var updateItemAcl = function(item, type, parentRules){
			var mergedItem = applyRule(item, parentRules);

			ZhuAclConfig.getAclMap(type).childs.forEach(function(type){
				if(Util.isArray(item[type])){
					item[type].forEach(function(child){
						updateItemAcl(child, type, mergedItem);
					});
				}
			});
		};

		var changeNotificationVisibility = function(isVisible){
			templateManager.project.notifications.some(function(notification){
				var isAclNotification = notification.name === "accessControl";
				if(isAclNotification){
					notification.isVisible = isVisible;
				}
				return isAclNotification;
			});
		};

		var container = templateManager.container;
		var aclEnable = container.aclEnable;
		aclEnable = aclEnable !== undefined ? aclEnable : true;
		changeNotificationVisibility(aclEnable);

		if(aclEnable){
			var containerRule = applyRule(container);
			if(verifyContainerRule(containerRule.rule)){
				updateItemAcl(container, ZhuAclConfig.containerType);
			}
			loadedContainers[container.id] = true;
		}
	};

	var convertObjectRulesToArray = function(userId, objRules){
		var arrayRules = [];
		Object.keys(objRules).forEach(function(ruleId){
			var objRule = objRules[ruleId];
			var userRule = objRule.userGroups[userId];
			if(userRule){
				var rule = {
					"UNIQUE_ID": ruleId,
					"NAME": objRule.itemName,
					"RULE": userRule.rule
				};
				arrayRules.push(rule);
			}
		});
		return arrayRules;
	};

	var convertArrayRulesToObject = function(userId, arrayRules){
		var objRules = {};
		arrayRules.forEach(function(rule){
			var profileObjects = {};
			profileObjects[userId] = {
				'rule': rule.RULE,
				'undefinedGroups': rule.GROUP_ID || []
			};
			objRules[rule.UNIQUE_ID] = {
				'itemName': rule.NAME,
				'userGroups': profileObjects
			};
		});
		return objRules;
	};

	var subscribeContainerEvent = function(rules, groupIds, userId){
		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function applyACL(){
			var containerId = templateManager.container.id;
			if(!loadedContainers[containerId]){
				updateContainerAcl(rules, groupIds, userId);
			}
		});

	};

	var updateProjectAcl = function(loginRules, defaultRule, groupIds){
		var rules = {};
		var userId = user.getUserId();
		if(Util.isObject(loginRules)){
			rules = loginRules;
			loginRules = convertObjectRulesToArray(userId, rules);
		} else if(Util.isArray(loginRules)){
			rules = convertArrayRulesToObject(userId, loginRules);
		}

		templateManager.project.aclDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;
		templateManager.userId = userId;

		templateManager.aclRules = rules;
		updateContainerAcl(rules, groupIds, userId);
		subscribeContainerEvent(rules, groupIds, userId);
	};

	var createUser = function createUser(userId, projectId, organizationId, defaultRule){
		user = new User(userId, projectId, organizationId, defaultRule);
	};

	this.setUserRules = function(groupIds, projectId, userId, organizationId, options){
		loadedContainers = {};
		options = options || {};

		if(options.canConfigureAcl){
			ZhuAccessButtonService.enableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		} else {
			ZhuAccessButtonService.disableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		}

		if(options.rules){
			createUser(userId, projectId, organizationId, options.defaultRule);
			updateProjectAcl(options.rules, options.defaultRule, groupIds);
		} else {
			ZhuAclService.getZhuAclRules(projectId, groupIds).then(function(response){
				createUser(userId, projectId, organizationId, response.defaultRule);
				updateProjectAcl(response.aclRules, response.defaultRule, groupIds);
			}).catch(function(err){
				MessageService.showMessageByCode(err.code);
			});
		}
	};

	this.setElementsListFilter = function(container, groupId, widgetName){
		widgetName = widgetName || ZhuAclConfig.menuElementsListWidgetName;
		var elementsWidget = container.getWidget(widgetName);
		elementsWidget.dataSourceFilter = ZhuAclConfig.getDefaultDataSourceFilter(groupId, user.getProjectId(), user.getOrganizationId());
		user.setCurrentGroupId(groupId);
	};

	var getFactoredWidget = function(container, widgetName, pushInContainer){
		var widget = container.getWidget(widgetName);
		if(widget){
			return ZHPromise.when(widget);
		} else {
			return ZhuAclService.getWidgetMetadata(widgetName).then(function(metadata){
				var factoredWidget = metaDataFactory.widgetFactory(metadata, container);
				if(pushInContainer){
					container.widgets.push(factoredWidget);
				}
				return factoredWidget;
			});
		}
	};

	this.insertElementsListWidget = function(container, parentWidget){
		getFactoredWidget(container, ZhuAclConfig.menuElementsListWidgetName).then(function(factoredWidget) {
			if(parentWidget && !factoredWidget.parent){
				factoredWidget.parent = parentWidget;
				parentWidget.widgets = parentWidget.widgets || [];
				parentWidget.widgets.push(factoredWidget);
			}
		});
	};

}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclController', ZhuAclController);
});


// FILE: assets/js/controllers/ZhuAclCRUDController.js
/* global Configuration, ZhuAclMessages */
function ZhuAclCRUDController(ScreenService, ZHPromise, ZhuAclConfig, ZhuAclController, MessageService){

	function defineQuestionToUse(rule){
		var question = ZhuAclMessages.VISIBLE;
		if(rule == ZhuAclConfig.RULE_INVISIBLE){
			question = ZhuAclMessages.INVISIBLE;
		} else if(rule == ZhuAclConfig.RULE_ALLOW){
			question = ZhuAclMessages.EDITABLE;
		}
		return {newVisibility: ScreenService.i18n(question)};
	}

	function deleteChildrensRules(elementsWidget, parentRow, groupId, ruleToContainer){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					row.changed = true;
					row.__isDeleted = true;
					row.GROUP_ID = groupId;
					row.RULE_SAVE_CONTAINER = ruleToContainer;
					deleteChildrensRules(elementsWidget, row, groupId, ruleToContainer);
				}
			});
		}
	}

	function confirmChildsVisibility(elementsWidget, ruleType){
		var returnPromise = ZHPromise.when();
		if(elementsWidget.currentRow._childCount){
			var currentRow = elementsWidget.currentRow;
			var defer = ZHPromise.defer();
			var actions = [
				{
					label: ScreenService.i18n(ZhuAclMessages.YES),
					code: function(){
						currentRow.changed = true;
						deleteChildrensRules(elementsWidget, currentRow, currentRow.GROUP_ID, currentRow.RULE);
						defer.resolve();
					}
				},
				{
					label: ScreenService.i18n(ZhuAclMessages.NO),
					code: function(){
						var messageConfig = {
							type: 'confirmation',
							icon: 'question'
						};
						MessageService.showMessageByCode(ZhuAclMessages.CONFIRMATION_MESSAGE, {}, messageConfig);
						defer.reject();
					}
				}
			];
			var dialogConfig = {
				type: 'confirmation',
				icon: 'question',
				actions: actions
			};

			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_CHILDS_QUESTION, defineQuestionToUse(ruleType), dialogConfig);
			returnPromise = defer.promise;
		}

		return returnPromise;
	}

	function reloadChildrensRules(elementsWidget, parentRow, updatedRules, deletedRules){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					var rowId = row.UNIQUE_ID;
					row.isVisible = parentRow.isVisible;
					row.isEditable = parentRow.isEditable;

					if(deletedRules.indexOf(rowId) != -1){
						delete row.RULE_ID;
					}
					if(updatedRules[rowId] && updatedRules[rowId].containerId){
						row.CONTAINER_RULE_ID = updatedRules[rowId].containerId;
					}

					reloadChildrensRules(elementsWidget, row, updatedRules, deletedRules);
				}
				delete row.__isDeleted;
				delete row.changed;
			});
		}
	}

	function fixZeedhiBehavior(widget){
		widget.dataSource.data.forEach(function(data){
			data.isEditable = (data._childCount && data.isVisible) || data.isEditable;
		});
	}

	function isFatherVisible(parentRow, widget){
		return widget.dataSource.data.some(function(row){
			return (row.PARENT_ID == parentRow.UNIQUE_ID) && (row.isVisible || row.isEditable);
		});
	}

	function reloadParentsRules(row, widget){
		var parentRow = row._parent;
		if(parentRow){
			if(row.isVisible){
				parentRow.isEditable = parentRow.isEditable || row.isEditable || row.isVisible;
				parentRow.isVisible = true;
			} else {
				var updateFather = isFatherVisible(parentRow, widget);
				parentRow.isEditable = updateFather;
				parentRow.isVisible = updateFather;
			}
			reloadParentsRules(row._parent, widget);
		}
	}

	function reloadDataSourceAfterSave(elementsWidget, newRules){
		var currentRow = elementsWidget.currentRow;
		var currentRowId = currentRow.UNIQUE_ID;
		var updatedRule = newRules.updated[currentRowId];
		var deletedRules = newRules.deleted || [];
		if(updatedRule){
			if(Util.isObject(updatedRule)){
				currentRow.RULE_ID = updatedRule.menuId;
				currentRow.CONTAINER_RULE_ID = updatedRule.containerId;
			} else {
				currentRow.RULE_ID = updatedRule;
			}
			reloadChildrensRules(elementsWidget, currentRow, newRules.updated, deletedRules);
			reloadParentsRules(currentRow, elementsWidget);
			fixZeedhiBehavior(elementsWidget);
		}
	}

	function cancelSave(widget){
		widget.reload();
	}

	function changeElementRule(currentRow, isVisible, isEditable){
		currentRow.RULE = ZhuAclConfig.getRuleType(isVisible, isEditable);
		currentRow.isVisible = isVisible;
		currentRow.isEditable = isEditable;
		currentRow.GROUP_ID = currentRow.GROUP_ID || ZhuAclController.getCurrentGroupId();
	}

	function clearInlineChangedStatus(row){
		delete row.changed_isEditable;
		delete row.changed_isVisible;
	}

	function saveRule(widget, isEditable, isVisible){
		var currentRow = widget.currentRow;
		var ruleType = ZhuAclConfig.getRuleType(isVisible, isEditable);

		clearInlineChangedStatus(currentRow);
		changeElementRule(currentRow, isVisible, isEditable);

		confirmChildsVisibility(widget, ruleType).then(function(){
			widget.saveRow().then(function(response){
				reloadDataSourceAfterSave(widget, response.dataset.aclZhuAccessControlRule);
				MessageService.successNotificationByCode(ZhuAclMessages.SUC_RULE_INSERTED);
			}).catch(function(){
				cancelSave(widget);
				MessageService.notificationMessageByCode(ZhuAclMessages.ERR_RULE_INSERTED, {}, {type: "alert"});
			});
		}).catch(function(){
			cancelSave(widget);
			MessageService.notificationMessageByCode(ZhuAclMessages.RULE_NOT_SAVED, {}, {type: "alert"});
		});
	}

	this.changeIsEditable = function changeIsEditable(widget, row){
		var isEditable = row.isEditable;
		var isVisible = isEditable || row.isVisible;
		saveRule(widget, isEditable, isVisible);
	};

	this.changeIsVisible = function changeIsVisible(widget, row){
		var isVisible = row.isVisible;
		var isEditable = isVisible && row.isEditable;
		saveRule(widget, isEditable, isVisible);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclCRUDController', ZhuAclCRUDController);
});


// FILE: assets/js/controllers/ZhuContainerElementsController.js
/* global Util, Configuration, ZhuAclMessages */
function ZhuContainerElementsController(templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var self = this;
	var idErrorMessages = "";

	var getContainerElementsTreeWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsTreeWidgetName);
	};

	var getContainerElementsListWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = "";
	};

	var showIdErrorsMessage = function(){
		if(idErrorMessages){
			var messageConfig = {
				type: 'alert',
				icon: 'exclamation'
			};
			MessageService.showMessageByCode(ZhuAclMessages.NSHOW_WITHOUT_ID, {elementsIds: idErrorMessages}, messageConfig);
		}
	};

	var getPlannedContainer = function(rules, item, itemType, parentId){
		var plannedItems = [];
		itemType = itemType || ZhuAclConfig.containerType;
		if(item && !item.isZhuAclElement){
			var uniqueId = item.id;
			if(uniqueId){
				var aclType = ZhuAclConfig.getAclMap(itemType);
				var itemToInsert = ZhuAclController.getPlannedItemToInsertInDataSet(rules, item, aclType.aclTypeCod, parentId);
				plannedItems.push(itemToInsert);
				aclType.childs.forEach(function(type){
					if(Util.isArray(item[type])){
						item[type].forEach(function(child){
							plannedItems = plannedItems.concat(getPlannedContainer(rules, child, type, uniqueId));
						});
					}
				});
			} else {
				var nameToShow = item.label || item.name;
				idErrorMessages += "<br> " + nameToShow + ";";
			}
		}
		return plannedItems;
	};

	this.reloadContainerElementsModal = function(){
		getContainerElementsListWidget().reload();
	};

	this.reloadContainerElementsTree = function(){
		getContainerElementsTreeWidget().reload();
	};

	this.selectAccessOnChange = function(selectedRow){
		if(selectedRow && selectedRow.SELECT_ACCESS){
			ZhuAclController.setElementsListFilter(templateManager.container, selectedRow.ID, ZhuAclConfig.containerElementsTreeWidgetName);
		}
		self.reloadContainerElementsTree();
	};

	var copyItem = function(itemToCopy, itemType, parentId){
		var item;
		if(itemToCopy && !itemToCopy.isZhuAclElement && !itemToCopy.ignoreAclElementsConfig){
			var uniqueId = itemToCopy.id;
			item = {
				id: uniqueId,
				name: itemToCopy.name,
				label: itemToCopy.label,
			};
			itemType = itemType || ZhuAclConfig.containerType;
			var aclType = ZhuAclConfig.getAclMap(itemType);
			aclType.childs.forEach(function(type){
				if(Util.isArray(itemToCopy[type])){
					item[type] = itemToCopy[type].map(function(child){
						return copyItem(child, type, uniqueId);
					}).filter(function(copiedChild){
						return copiedChild;
					});
				}
			});
		}
		return item;
	};

	this.containerElementsTreeAfterLoadDataSource = function(){
		clearIdErrorsMessage();
		var widget = getContainerElementsTreeWidget();
		var elementsListWidget = getContainerElementsListWidget();
		var elementsListRow = elementsListWidget && elementsListWidget.currentRow;
		var rulesData = Util.clone(widget.dataSource.data);
		widget.dataSource.data = [];

		if(elementsListRow && elementsListRow.SELECT_ACCESS){
			var rules = ZhuAclController.getRulesFromDatasource(rulesData);
			var container = copyItem(templateManager.container);
			if(container){
				aclEngine.processAcl(container, rules, templateManager.userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule());
				widget.dataSource.data = getPlannedContainer(rules, container);
				showIdErrorsMessage();
			}
		}
	};

	this.isEditableOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsEditable(widget, currentRow);
	};

	this.isVisibleOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsVisible(widget, currentRow);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuContainerElementsController', ZhuContainerElementsController);
});

// FILE: assets/js/controllers/ZhuMenuElementsController.js
/* global Configuration Util ZhuAclMessages */
function ZhuMenuElementsController(ScreenService, templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var MENU = "M";
	var GROUP_MENU = "G";
	var IGNORED_MENUS = ['zhHistory'];
	var idErrorMessages = {"errorMenus": "", "errorContainers": ""};

	var getWidgetElements = function(){
		return templateManager.container.getWidget(ZhuAclConfig.menuElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = {"errorMenus": "", "errorContainers": ""};
	};

	var showIdErrorsMessage = function(){
		var messageCode;
		var errorMenus = idErrorMessages.errorMenus;
		var errorContainers = idErrorMessages.errorContainers;
		var messageParameters = {
			elementsIds: errorMenus,
			elementsWithoutContainer: errorContainers
		};

		if(errorMenus && errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG;
		} else if(errorMenus){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID;
		} else if(errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_CONTAINER_CONFIG;
		}

		if(messageCode){
			MessageService.showMessageByCode(messageCode, messageParameters, { type: 'alert' });
		}
	};

	var addErrorMenuId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorMenus += "<br> " + nameToShow + ";";
	};

	var addErrorContainerId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorContainers += "<br> " + nameToShow + ";";
	};

	this.elementsActionEventEdit = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			currentRow.isEditable = !currentRow.isEditable;
			ZhuAclCRUDController.changeIsEditable(elementsWidget, currentRow);
		}
	};

	this.elementsActionEventView = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			currentRow.isVisible = !currentRow.isVisible;
			currentRow.isEditable = currentRow.isVisible;
			ZhuAclCRUDController.changeIsVisible(elementsWidget, currentRow);
		}
	};

	this.elementsActionEventBack = function(){
		var elementsWidgetParent = getWidgetElements().parent;
		return ScreenService.closeSwipe().then(function(){
			elementsWidgetParent.list();
			elementsWidgetParent.activate();
		});
	};

	var addContainerProperties = function(item, menuItem, rules){
		var containerId = menuItem.containerId;
		var containerLabel = menuItem.containerLabel || menuItem.containerName;
		if(containerId && containerLabel){
			var containerRule = ZhuAclController.getElementRule(rules, containerId);
			item.CONTAINER_ID = containerId;
			item.CONTAINER_NAME = menuItem.containerName || ZhuAclController.prepareLabelToName(containerLabel);
			item.CONTAINER_LABEL = containerLabel;
			item.CONTAINER_RULE_ID = containerRule.RULE_ID;
		} else if(menuItem.windowName && !menuItem.buildedMenu){
			addErrorContainerId(menuItem);
		}
	};

	var planningMenuItem = function(menuItem, parentMenuItem, rules){
		var item;
		var uniqueId = menuItem.id;
		if(uniqueId){
			var parentId = parentMenuItem && parentMenuItem.id;
			var itemType = !parentId ? GROUP_MENU : MENU;
			item = ZhuAclController.getPlannedItemToInsertInDataSet(rules, menuItem, itemType, parentId);
			item.buildedMenu = menuItem.buildedMenu;
			addContainerProperties(item, menuItem, rules);
		} else {
			addErrorMenuId(menuItem);
		}

		return item;
	};

	var addMenusInDataSource = function(widget, rules, menus, parentMenu){
		menus.forEach(function(menu, index){
			var item = planningMenuItem(menu, parentMenu, rules);
			if(item){
				widget.dataSource.data.push(item);
				if (menu.menus && menu.menus.length > 0){
					addMenusInDataSource(widget, rules, menu.menus, menu);
				}
			}
		});
	};

	var getProjectMenu = function(){
		var projectMenu = Util.clone(templateManager.project.groupMenu);
		return projectMenu.filter(function(menu){
			return IGNORED_MENUS.indexOf(menu.name) == -1;
		});
	};

	this.loadPlanningTree = function(){
		var elementsWidget = getWidgetElements();
		var userId = templateManager.userId;
		var rules = ZhuAclController.getRulesFromDatasource(elementsWidget.dataSource.data);
		var newMenu = getProjectMenu();
		clearIdErrorsMessage();
		aclEngine.processMenuAcl(newMenu, rules, templateManager.containers, userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule());
		elementsWidget.dataSource.data = [];
		addMenusInDataSource(elementsWidget, rules, newMenu);
		showIdErrorsMessage();
	};

	this.reloadElementsWidget = function(){
		getWidgetElements().reload();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuMenuElementsController', ZhuMenuElementsController);
});

// FILE: assets/js/adapter/config.js
ZeedhiApp.run(function(templateManager, ZhuAccessButtonService) {
	templateManager.projectPromise.then(function(project){
		if(templateManager.project.aclEnable){
			ZhuAccessButtonService.insertButtonInProject(templateManager.project);
		}
	});
});


// FILE: bower_components/zhulog/dist/main.js

// FILE: src/services/ZhuLogService.js
/* global Util, Configuration */
function ZhuLogService() {

    this.getPlainContentRows = function(value) {
        var data = [];
        var objValue = JSON.parse(value);

        createRows(data, objValue);
        return data;
    };

    function createRows(data, value, parent) {
        Object.keys(value).forEach(function(key) {
            createRow(data, key, value[key], parent);
        });
    }

    function createRow(data, key, value, parent) {
        var method;
        if (Util.isObject(value)) {
            method = factoryCreateObjectRowsMethod();
        } else if (Array.isArray(value)) {
            method = factoryCreateArrayRowsMethod();
        } else {
            method = createSimpleRow;
        }

        method(data, key, value, parent);
    }

    function factoryCreateObjectRowsMethod() {
        return factoryCreateObjectRows(': {}');
    }

    function factoryCreateArrayRowsMethod() {
        return factoryCreateObjectRows(': []');
    }

    function createSimpleRow(data, key, value, parent) {
        if (typeof value === 'string') {
            value = '"' + value + '"';
        }

        value = key + ': ' + value;
        addData(key, value, parent, data);
    }

    function addData(key, value, parent, data) {
        var id = parent + '#' + key;
        data.push({
            'ID': id,
            'VALUE': value,
            'PARENT': parent
        });

        return id;
    }

    function factoryCreateObjectRows(emptySufix) {
        return function createObjectRows(data, key, rowContent, parent) {
            var rowValue = key;
            if (Object.keys(rowContent).length === 0) {
                rowValue += emptySufix;
            }

            var id = addData(key, rowValue, parent, data);
            createRows(data, rowContent, id);
        };
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogService', ZhuLogService);
});

// FILE: src/controllers/ZhuLogController.js
/* global Util, Configuration */
function ZhuLogController(ZhuLogService, ScreenService, templateManager, metaDataFactory, ZhPreferences, ZhFilterPreferences, $timeout) {

    var LOG_WIDGET_NAME = 'zhuLog';
    var CONTENT_WIDGET = 'content';
    var CONTENT_FORM_WIDGET = 'contentForm';
    var CONTENT_GRID_WIDGET = 'contentGrid';
    var CONTENT_FILTER_WIDGET = 'select';

    function getContentWidget() {
        return templateManager.container.getWidget(CONTENT_WIDGET);
    }

    function getContentFormWidget() {
        return templateManager.container.getWidget(CONTENT_FORM_WIDGET);
    }

    function getContentGridWidget() {
        return templateManager.container.getWidget(CONTENT_GRID_WIDGET);
    }

    function getFilterWidget() {
        return templateManager.container.getWidget(CONTENT_FILTER_WIDGET);
    }

    function getZhuLogWidget() {
        return templateManager.container.getWidget(LOG_WIDGET_NAME);
    }

    this.openContent = function(widget) {
        var contentFormWidget = getContentFormWidget();
        contentFormWidget.currentRow = widget.currentRow;

        var rows = ZhuLogService.getPlainContentRows(widget.currentRow.CONTENT);
        getContentGridWidget().dataSource.data = rows;

        ScreenService.openSwipe(getContentWidget());
    };

    var startFilter = function(widget, currentRow) {
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result) {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    var isAppliedFilter = function(widget){
        return !!widget.zhuLogAppliedFilter;
    };

    this.zhuLogWidgetOnEnter = function() {
        var widget = getZhuLogWidget();
        $timeout(function() {
            if(widget.inListMode()){
                if (isAppliedFilter(widget)) {
                    widget.reload();
                } else {
                    var currentRow = widget.currentRowDefaultFilter || widget.currentRow;
                    ZhFilterPreferences.setOriginalFilter(widget.id, currentRow);
                    startFilter(widget, currentRow);
                }
            }
        }, 500, false);
    };

    this.zhuLogFilterAfterApply = function () {
        getZhuLogWidget().zhuLogAppliedFilter = true;
    };

    this.openFilterPopUp = function() {
        ScreenService.openPopup(getFilterWidget());
    };

    this.setSelectValues = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var select2 = templateManager.container.getWidget('select').fields[1];

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            select2.label = 'Resquest type';
            select2.readOnly = false;

            select2.dataSource.data = [];
            select2.dataSource.data.push(
                {'value': 'Dataset'},
                {'value': 'Row'},
                {'value': 'Empty'},
                {'value': 'Filter'}
            );
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                select2.label = 'Response type';
                select2.readOnly = false;

                select2.dataSource.data = [];
                select2.dataSource.data.push(
                    {
                        'value': 'Dataset'
                    },
                    {
                        'value': 'Message'
                    }
                );
            } else {
                select2.dataSource.data = [];
                this.clearFieldsValue(1);
                select2.readOnly = true;
            }
        }
    };

    this.setTextEdit = function() {
        var selectWidget = templateManager.container.getWidget('select');

        selectWidget.actions[2].isVisible = true;
        selectWidget.actions[3].isVisible = true;
        // selectWidget.fields[2].isVisible = true;
        // selectWidget.fields[3].isVisible = true;

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            if (selectWidget.currentRow.SNATIVE2 === 'Dataset' || selectWidget.currentRow.SNATIVE2 === 'Row') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';
                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Empty') {
                selectWidget.fields[2].label = 'Parameter';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Filter') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Column value';
                selectWidget.fields[3].template = 'field/text-edit.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === null) {
                this.removeExtraFields();
            }
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                if (selectWidget.currentRow.SNATIVE2 === 'Dataset') {
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 == 'Message'){
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 === null) {
                    this.removeExtraFields();
                }
            }
        }
    };

    this.removeExtraFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = (fields.length - 1); i > 6; i--) {
            if (!Util.isUndefined(fields[i])) {
                delete selectWidget.currentRow[fields[i].name];
                delete fields[i];
            }
        }
    };

    this.addFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = templateManager.container.getWidget('select').fields;
        var field;
        var dataSource;

        for (var i = 2; i <= 6; i++) {
            if (i !== 5) {
                var data;

                if (!Util.isUndefined(fields[i].dataSource)) {
                    data = fields[i].dataSource.data;
                    dataSource = {
                        'data': data
                    };
                }

                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: true,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            } else {
                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: false,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            }
        }
    };

    this.clearAllFields = function() {
        this.removeExtraFields();
        this.clearFieldsValue(0);
    };

    this.clearFieldsValue = function(startingValue) {
        var selectWidget = templateManager.container.getWidget('select');

        for (var i = startingValue; i < selectWidget.fields.length; i++) {
            if(!Util.isUndefined(selectWidget.fields[i])){
                if(selectWidget.fields[i].value !== ''){
                    selectWidget.fields[i].value('');
                }
            }
        }

        selectWidget.fields[1].readOnly = true;
    };

    this.clearFields = function(args) {
        var fieldIndex = args.owner.field.sequenceInList;
        fieldIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = fieldIndex; i >= fieldIndex - 4; i--) {

            delete selectWidget.currentRow[fields[i].name];
            delete fields[i];
        }

        templateManager.updateTemplate();
    };

    this.manageFields = function(args) {
        var operatorIndex = args.owner.field.sequenceInList;
        operatorIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var selectedfield = selectWidget.fields[operatorIndex];
        var selectedValue = selectedfield.value();

        if (selectedValue === 'IS_NULL' || selectedValue === 'IS_NOT_NULL'){
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 2].isVisible = false;
            selectWidget.fields[operatorIndex + 1].readOnly = true;
        } else if (selectedValue === 'BETWEEN' || selectedValue === 'NOT_BETWEEN') {
            selectWidget.fields[operatorIndex + 1].class = 2;
            selectWidget.fields[operatorIndex + 1].label = 'Begin';
            selectWidget.fields[operatorIndex + 2].isVisible = true;
        } else {
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 1].label = 'Value';
            selectWidget.fields[operatorIndex + 2].isVisible = false;
        }
    };

    this.startFiltering = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        var i = 9;
        var j = 0;
        var params = [];

        while (i < fields.length) {
            if (!Util.isUndefined(fields[i])) {
                var name = fields[i].value();
                i++;
                var operator = fields[i].value();
                i++;

                var value;
                if(operator === 'IS_NULL' || operator === 'IS_NOT_NULL') {

                    value = '';
                    i += 3;

                } else if(operator === 'BETWEEN' || operator === 'NOT_BETWEEN') {
                    value = [];
                    value[0] = fields[i].value();
                    i++;
                    value[1] = fields[i].value();
                    i+= 2;
                } else {
                    value = fields[i].value();
                    i += 3;
                }

                params[j] = {
                    name: name,
                    operator: operator,
                    value: value
                };
                j++;
            } else {
                i+= 5;
            }
        }

    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogController', ZhuLogController);
});

// FILE: src/ZhuLogDecorator.js
/* global Configuration */
function ZhuLogDecorator(ApplicationContext, templateManager) {

    var LOG_CONTAINER_NAME = 'zhulog#log';
    var GRID_WIDGET_INDEX = 0;

    var logDecorator;
    function getLogDecorator() {
        if (logDecorator === undefined) {
            logDecorator = templateManager.projectPromise.then(function() {
                var decoratorServiceName = templateManager.project.zhuLogDecoratorService;
                var logDecorator = ApplicationContext[decoratorServiceName];

                return logDecorator;
            });
        }
        return logDecorator;
    }

    this.afterLoadDataSource = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var dataSource = args.owner.dataSource;
                dataSource.data = logDecorator.normalizeRows(dataSource.data);
            }
        });
    };

    this.beforeGridFilterApply = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var _event = args.e;
                _event.filters = logDecorator.normalizeFilters(_event.preference.FILTER_ROW);
            }
        });
    };

    this.apply = function(containerName, container) {
        return getLogDecorator().then(function(logDecorator) {
            if (logDecorator && containerName === LOG_CONTAINER_NAME) {
                var logWidget = container.widgets[GRID_WIDGET_INDEX];

                logWidget.events.push({
                    "name": "WidgetAfterLoadDataSource",
                    "code": "ZhuLogDecorator.afterLoadDataSource(args);"
                });

                logWidget.events.push({
                    "name": "FilterBeforeApply",
                    "code": "ZhuLogDecorator.beforeGridFilterApply(args);"
                });

                logDecorator.decorateWidget(logWidget);
            }

            return container;
        });
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ZhuLogDecorator', ZhuLogDecorator);
    ContainerDecoratorProvider.registerProvider(new ZhuLogDecorator(ApplicationContext, templateManager));
});

// FILE: src/ZhuLogHelper.js
/* global Util, Configuration */
function ZhuLogHelper() {

    this.moveField = function(widget, fieldName, position) {
        var index = -1;
        widget.fields.find(function(field) {
            index++;
            return field.name === fieldName;
        });
        widget.fields.splice(position - 1, 0, widget.fields.splice(index, 1)[0]);
    };

    this.getField = function(widget, fieldName) {
        var field = widget.fields.find(function(field) {
            return field.name === fieldName;
        });

        if (field === null) {
            throw 'Field "'+fieldName+'" not found';
        }

        return field;
    };

    this.addField = function(widget, field, position) {
        position = (position || widget.fields.length) - 1;
        widget.fields.splice(position, 0, field);
    };

    this.removeField = function(widget, fieldName) {
        widget.fields = widget.fields.filter(function(field) {
            return field.name !== fieldName;
        });
    };

    this.editField = function(widget, fieldName, editings) {
        var field = this.getField(widget, fieldName);
        Util.extend(field, editings);
    };

    this.getFilters = function(filters, filterName) {
        return filters.filter(function(filter) {
            return filter.name === filterName;
        });
    };

    this.addFilter = function(filters, filter) {
        filters.push(filter);
    };

    this.removeFilter = function(filters, filterName) {
        filters = filters.filter(function(filter) {
            return filter.name !== filterName;
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhuLogHelper', ZhuLogHelper);
});

// FILE: bower_components/tek-libraries/dist/main.js

// FILE: js/Util/Colors.js
function Colors() {

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (rgb) {
	    return "#" + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1);
	};
	
	this.RGBToTColor = function(r, g, b){
		var Tcolor = r | (g << 8) | (b << 16);
		return Tcolor;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Colors', Colors);
});

// FILE: js/Util/Endereco.js
var Endereco = function (restEngine, ScreenService, MessageService, ZHPromise) {
    var self = this;

    //Busca pelo CEP na API dos correios e retorna o resultado por promise
    self.buscarCep = function(CEP) {
        CEP = CEP.replace(/[\D]+/g,'');

        if(CEP.length == 8) {
            return restEngine.post('/lib_findAddress', {
                    'requestType':'Row',
                    'row': {'CEP': CEP}
            });
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    /* Busca pelo CEP na API dos correios e trata o retorno.
     * Usar apenas de exemplo para fazer um método próprio!
     */
    self.validarCep = function(widget, CEP) {
        self.buscarCep(CEP).then(function(response) {
            var endereco = response.dataset.ENDERECO;

            if(endereco) {
                widget.currentRow.CEP = CEP;
                widget.currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                widget.currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                widget.currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                widget.currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                widget.currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                widget.currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                widget.currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                widget.currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                widget.currentRow.CDLOGRADOURO = (endereco.CDLOGRADOURO)? endereco.CDLOGRADOURO : '';
                widget.currentRow.DSLOGRADOURO = (endereco.DSLOGRADOURO)? endereco.DSLOGRADOURO : '';
                widget.currentRow.DSENDERECO = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
            } else {
                widget.currentRow.CEP = '';
            }
        }, function(error) {            
            MessageService.getMessageByCode("PROXY_CONNECTION_FAIL").then(function(errorMessage) {
                var row = {
                    "CEP": CEP, 
                    "errorMessage": errorMessage
                };
                widget.currentRow.CEP = '';

                if(error.data)
                    errorMessage = error.data.error;
            
                ScreenService.showMessageByCode("REQUEST_CEP_ERROR", row, Constants.errorMessage);
            });
        });
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('Endereco', Endereco);
});

// FILE: js/Util/General.js
var General = function (ScreenService, templateManager, localStorageEngine, restEngine, maskEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory, MessageService) {
	var searchListDeleted;
	var parent = this;
	var cacheLoaded = false;

	//@TODO Criar objeto de mensagens para tradução.

	/*
	 * Função que atualiza o dataSourceFilter de um field que depende do resultado de
	 * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
	 * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
	 * ao alterar o valor do field "pai".
	 *
	 * @param field dependentField
	 * @param array parameters
	 *
	 * Exemplo:
	 * General.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
	 */
	this.onClickDependentField = function (dependentField, parameters) {

		var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

		var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

		if (changedDataSourceFilter) {
			changeDependetFieldFilter(dependentField, dataSourceFilterNew);
		}

		if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
			dependentField.reload();
		}
	};

	this.buildDataSourceFilter = function (dataSourceFilter, parameters) {
		if (!dataSourceFilter) {
			throw "DataSourceFilter is not defined!";
		}

		for (var position in dataSourceFilter) {
			if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
				dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
			}
		}

		return dataSourceFilter;
	};

	var changeDependetFieldFilter = function (dependentField, dataSourceFilterNew) {

		dependentField.dataSourceFilter = dataSourceFilterNew;
		dependentField.dataSource.lazyLoad = false;
		if (dependentField.selectWidget) {
			dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
		}

		if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

			searchListDeleted = true;
			delete dependentField.selectWidget.searchList;

		} else if (dependentField.selectWidget) {

			searchListDeleted = false;

		}

	};

	var compareDataSourceFilters = function (selectWidget, dataSourceFilterNew, parameters) {
		var changedDataSourceFilter = false;
		if (selectWidget) {
			var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
			for (var position = 0; position < dataSourceFilterNew.length; position++) {
				if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
					if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
						changedDataSourceFilter = true;
					}
				}
			}
		} else {
			changedDataSourceFilter = true;
		}
		return changedDataSourceFilter;
	};

	/*
	 * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
	 * ou produtos correspondentes à máscara utilizada. Através de cada nível
	 * da máscara, é gerado o número correspondente à ele.
	 *
	 * @param string maskName
	 * @param int firstLevel
	 *
	 * @return array
	 *
	 * Exemplo:
	 * General.getLevelByMask('CDPICTPRAT', 1);
	 */
	this.getLevelByMask = function (maskName, firstLevel) {

		firstLevel = toNumber(firstLevel);

		var mask = getLocalVar(maskName);
		var splitMask = mask.split('.');
		var biggestLevel = splitMask.length;

		var level = [];
		var objLevel = {};

		for (var i = 0; i < biggestLevel; i++) {

			objLevel = {
				nivel: i + firstLevel,
				caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
			};

			objLevel.nivel = objLevel.nivel.toString();
			objLevel.caracteres = objLevel.caracteres.toString();

			level.push(objLevel);
		}

		return level;
	};

	var toNumber = function (value) {

		if (!value && value !== 0) {

			throw 'firstLevel is not defined';

		} else if (typeof value !== 'number' && value) {

			value = parseInt(value);

			if (isNaN(parseInt(value))) {

				throw 'Invalid firstLevel';

			} else {

				return value;
			}
		}
		return value;
	};

	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
		templateManager.updateTemplate();
	};

	this.setLoggedUnit = function (widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
	};

	var currentIdgrupoper;
	var currentCallback;
	var currentParameters;
	var gruposDeOperador = {
		LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
	};
	var fields = [
		{
			"name": "email",
			"label": "E-mail",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"template": "field/text-edit.html",
			"events": [],
			"fieldGroup": "AUTH",
			"validations": {
				"required": true,
				"mail": true
			}
        },
		{
			"name": "senha",
			"label": "Senha",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"fieldGroup": "AUTH",
			"template": "field/password-edit.html",
			"events": [],
			"validations": {
				"required": true
			}
        }
    ];
	var actions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.validateUser(args.owner.widget);"
                }
            ]
        }
    ];

	this.openAuthPopup = function (idgrupoper, message, callback, parameters) {

		if (!idgrupoper) {
			throw 'idgrupoper is not defined';
		}

		initAuthVariables(idgrupoper, callback, parameters);

		if (!message) {
			message = gruposDeOperador[currentIdgrupoper] || '';
		}

		var widgetAuth = metaDataFactory.widgetFactory({
			"id": 0,
			"name": "authWidget",
			"label": "Autenticação",
			"template": "widget/form.html",
			"container": templateManager.container,
			"isVisible": true,
			"popupNoBlock": false,
			"fieldGroups": [
				{
					"id": 100000,
					"name": "AUTH",
					"opened": true,
					"collapsible": false,
					"label": message
                }
            ],
			"fields": fields,
			"actions": actions,
			"events": []
		});

		ScreenService.openPopup(widgetAuth);

	};

	var initAuthVariables = function (idgrupoper, callback, parameters) {
		currentIdgrupoper = idgrupoper;
		currentCallback = callback || null;
		currentParameters = parameters || [];
	};

	this.validateUser = function (widget) {

		var self = this;

		if (widget.isValid()) {

			var email = widget.currentRow.email;
			var senha = widget.currentRow.senha;

			autenticar(email, senha).then(function (callback) {

				getPermission(callback.dataset.user.cdoperador).then(function (permission) {

					if (permission) {

						if (typeof currentCallback == 'function') {

							ScreenService.closePopup();

							currentCallback.apply(null, currentParameters);

						} else {

							throw 'callback is not a function';

						}

					} else {

						MessageService.showMessageByCode('lib#GEN_INVALID_OPERGROUP', null, { type: 'ERROR' } );

					}

				}, function (callback) {

					this.callbackError(callback);

				}.bind(self));

			}, function (callback) {

				this.callbackError(callback);

			}.bind(self));
		}

	};

	var autenticar = function (email, senha) {

		return restEngine.request('/Autenticar', {
			"requestType": "FilterData",
			"filter": [
				{
					name: "email",
					operator: "=",
					value: email
                },
				{
					name: "senha",
					operator: "=",
					value: senha
                }
            ]
		});

	};

	var getPermission = function (cdoperador) {

		return restEngine.request('/validateUserGroup', {
			"requestType": "Row",
			"row": {
				"idgrupoper": currentIdgrupoper,
				"cdoperador": cdoperador
			}
		}).then(function (callback) {

			return callback.dataset.permission.isValid;
		});

	};

	this.callbackError = function (callback) {

		if (callback.data) {

			ScreenService.showMessage(ScreenService.i18n(callback.data.error));

		} else {
			
			MessageService.showMessageByCode('lib#GEN_TIMEOUT', null, { type: 'ERROR' } );
		}
	};

	this.comparaPratoProduto = function (pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var menornv;
		if (nvinicial > nvfinal) {
			menornv = nvfinal;
		} else {
			menornv = nvinicial;
		}

		var resultado = true;
		var i = 0;

		while (pinicial[i] == pfinal[i] && i < menornv) {
			i++;
		}

		if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
			resultado = false;
		} else {
			resultado = true;
		}

		return resultado;
	};

	var reportWidget = {
		"id": 0,
		"name": "reportWidget",
		"label": "Tipo de Relatório",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};
	var reportFields = [
		{
			"id": 99999999,
			"name": "QR2TYPE",
			"label": "",
			"class": 12,
			"readOnly": false,
			"isVisible": true,
			"isVisibleForm": true,
			"isVisibleGrid": false,
			"template": "field/radio-group.html",
			"events": [],
			"valueField": "value",
			"descriptionField": "name",
			"defaultOption": 0,
			"dataSource": {
				"data": [
					{
						name: "pdf",
						value: "PDF"
					},
					{
						name: "xls",
						value: "XLS"
					},
					{
						name: "rtf",
						value: "RTF"
					},
					{
						name: "txt",
						value: "TXT"
					}
				]
			}
        }
    ];
	var reportActions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.generateReport(args.owner.widget, args);"
                }
            ]
        }
	];
	
	var reportCallback;
	var reportParameters;
	var reportWidgetParentName;

	this.openReportPopup = function (widget, callback, parameters, reportFormats) {

		reportCallback = callback;
		reportParameters = parameters || [];
		reportWidgetParentName = widget.name;
		reportWidget.container = widget.container;
		reportWidget.fields = reportFields;
		reportWidget.actions = reportActions;

		var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

		updateReportWidgetFormats(reportWidgetInitialized, reportFormats);

		ScreenService.openPopup(reportWidgetInitialized);
	};

	var updateReportWidgetFormats = function(reportWidget, reportFormats) {
		if (Array.isArray(reportFormats) && reportFormats.length) {
			var formatField = reportWidget.getField("QR2TYPE");
			formatField.dataSource.data = reportFormats;
		}
	};

	this.generateReport = function (widget, args) {

		var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

		reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

		args.row = reportWidgetParentCurrentRow;

		if (typeof reportCallback == 'function') {

			reportCallback.apply(null, reportParameters);
		}

		ScreenService.closePopup();

	};

	this.saveRow = function (args, closeSwipe) {
		args.owner.widget.saveRow().then(function () {
			var currentRow = args.owner.widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			args
				.owner
				.widget
				.reload();
			if (closeSwipe) {
				ScreenService.closeSwipe();
			}
		});
	};

	this.openSelectionOnce = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		if (!filterWidget.wasOpened) {
			filterWidget.edit();
			ScreenService.openPopup(filterWidget);
			filterWidget.wasOpened = true;
		}
	};

	this.openSelection = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		filterWidget.edit();
		ScreenService.openPopup(filterWidget);
	};

	this.getContainerName = function () {
		return templateManager.container.name;
	};

	this.backOrCancel = function (args) {
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(true);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}
		ScreenService.goBack();
		return deferred.promise;
	};

	this.edit = function (widget) {
		widget.backupRow = angular.copy(widget.currentRow);
		widget.edit();
	};

	this.modEdit = function (widget) {
		parent.edit(widget);
		var widgetParent = widget.widgets;
		if (widgetParent.length)
			parent.setVisibleforTabbed(widgetParent, false, widget);
		// widget.template = widget.formTemplate = widget.templateToForm;
	};

	this.cancelModEdit = function (args, widgetParent) {
		parent.backOrCancel(args).then(function (success) {
			var widget = args.owner.widget;
			var widgetParent = widget.widgets;
			if (widgetParent.length)
				parent.setVisibleforTabbed(widgetParent, true);
			// widget.template = widget.formTemplate = widget.templateToView;
		});
	};

	this.setVisibleforTabbed = function (widgets, isVisible, exception) {
		widgets.forEach(function (widget) {
			if (exception) {
				if (exception.name != widget.name)
					widget.isVisible = isVisible;
			} else {
				widget.isVisible = isVisible;
			}
		});
	};

	this.setContextVar = function (name, containerName, value) {
		localStorageEngine.setLocalVar(containerName + '_' + name, value);
	};

	this.getContextVar = function (name, containerName) {
		return localStorageEngine.getLocalVar(containerName + '_' + name);
	};

	var _getWidget = function (widgetName) {
		var isPopup = arguments[1] || null;
		if (!!templateManager.container) {
			var widget;
			if (!isPopup) {
				widget = templateManager.container.getWidget(widgetName);
			} else {
				widget = templateManager.container.getWidget(widgetName);
				if(widget)
					if(widget.hasOwnProperty('__filterPopup')) 
						widget = widget.__filterPopup;
					else{
						widget = widget.appliedFilter;
					}
			}
			if (widget) {
				return widget;
			}
		}
		return null;
	};

	var _getField = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var field = widget.getField(name);
		if (field) {
			return field;
		}
		return null;
	};

	var _getAction = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var action = widget.getAction(name);
		if (action) {
			return action;
		}
		return null;
	};

	this.getFieldValue = function (name, widgetName) {
		var isPopup = arguments[2] || null;
		var widget = _getWidget(widgetName, isPopup);
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		} else {
			return "";
		}
	};

	this.setEnableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = false;
	};

	this.setDisableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = true;
	};

	this.setVisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = true;
	};

	this.setInvisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = false;
	};

	this.setEnableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = false;
	};

	this.setDisableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = true;
	};

	this.setVisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = true;
	};

	this.setInvisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = false;
	};

	this.now = function () {
		return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
	};

	this.today = function () {
		return moment(new Date()).format('DD/MM/YYYY');
	};

	this.openReport = function (route, reportName, params) {
		params.__report_name = reportName;
		
		var lastActiveWidget = templateManager.container.getActiveWidget().lastActiveWidget;
	
		if (lastActiveWidget && lastActiveWidget.beforeOpenReport) {
		    lastActiveWidget.beforeOpenReport({
		        reportParams: params
		    });
		}
		
		return restEngine.post(route, {
			"requestType": "Row",
			"row": params
		}, null, null, null, null, 3000000).then(function (response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				var reportOpened = window.open(report.message);
				if(!reportOpened){
					MessageService.showMessageByCode('lib#GEN_POPUP_BLOCKER', null, { type: 'ERROR' } );
				}
			}
		}).catch(function (response) {
			if (response.data && response.data.error) {
				ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
			}
		});
	};

	// CRUD
	/**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function (widget, afterValid) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
					widget.reload().then(function () {
						deferred.resolve(success);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function (widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
					" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		} catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function (arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			} else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.deleteCrud = function (widget) {
		var deferred = ZHPromise.defer();
		MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};

    this.deleteCrudCheckedRows = function (widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

	this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}

	function preventEvent(e) {
		e.preventDefault();
	}

	function padLeft(value, length) {
		return (value.toString().length < length) ? padLeft("0" + value, length) : value;
	}

	function padRight(value, length) {
		return (value.toString().length < length) ? padRight(value + "0", length) : value;
	}

	this.getCurrentDate = function (format) {
		return moment().format(format);
	};

	this.setDateAttrToVarcharOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
	};

	this.setAttrOnCurrentRow = function (currentRow, attrName, value) {
		currentRow[attrName] = value;
	};

	this.setFloatAttrOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
	};

	this.valueToFloat = function (value) {
		if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
			if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
				return parseFloat(value);
			} else if (value.indexOf(',') > value.indexOf('.')) {
				return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
			} else if (value.indexOf(',') < value.indexOf('.')) {
				return parseFloat(value.replace(/\,/g, ''));
			}
		} else {
			return value;
		}
	};

	this.toLocalDate = function (element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	this.toLocalDateFormat = function (value, params) {
		if (value) {
			var valueDate = moment(value, params.format);
			value = valueDate.format(params.toFormat);
		}

		return value;
	};

	maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

	var maskFunction = function (element, viewValue, params, callBack) {
		// do something
	};

	var maskFormatFunction = function (value, params) {
		if (value) {
			var length = value.length;
			if (length === 11)
				value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
			else if (length === 14)
				value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
			return value;
		}
	};

	maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

	/*
	 * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
	 * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
	 * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
	 */

	var constAllWidgets;

	this.setVisibilityByProjectID = function (widgets) {
		if (widgets.length > 1) {
			widgets.forEach(function (childWidget) {
				parent.setVisibilityByProjectID(childWidget.widgets);
				if (childWidget.projects) {
					childWidget.projects.forEach(function (proj) {
						childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
					});
				}
			});

		}

	};

	this.validateFilterPopup = function (args) {
		if(args.owner.hasOwnProperty('__filterPopup')){
			var filterPopup = args.owner.__filterPopup;
		
		if (!filterPopup.isValid())
			args.e.preventDefault();
		}	
	};

	var timeFormats = {
		"hh": isValidHour,
		"mm": isValidMinutesOrSeconds,
		"ss": isValidMinutesOrSeconds,
		"hh:mm": isValidHourAndMinutes,
		"mm:ss": isValidMinutesAndSeconds,
		"hh:mm:ss": isValidTime,
	};

	function isValidHour(hour) {
		if (parseInt(hour) > 23 || parseInt(hour) < 0)
			return false;
		else
			return true;
	}

	function isValidMinutesOrSeconds(minuteOrSecond) {
		if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
			return false;
		else
			return true;
	}

	function isValidHourAndMinutes(hourAndMinute) {
		if (hourAndMinute) {
			var timeArray = hourAndMinute.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidMinutesAndSeconds(minutesAndSeconds) {
		if (minutesAndSeconds) {
			var timeArray = minutesAndSeconds.split(':');
			return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidTime(time) {
		if (time) {
			var timeArray = time.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
		}
		return true;
	}

	var validTime = function (value, format) {
		var validFunction;
		if (!format)
			throw 'format is not defined';
		if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
			validFunction = timeFormats[format.toLowerCase()];
		else
			throw 'time format is not registered';
		return {
			"valid": validFunction(value),
			"message": ScreenService.i18n("Não é um tempo válido.")
		};
	};

	this.registerTimeFormat = function (format, formatFunction) {
		if (typeof format !== 'string')
			throw 'time format must be an string';
		if (timeFormats.hasOwnProperty(format.toLowerCase()))
			throw 'time format already registered';
		timeFormats[format] = formatFunction;
	};

	ValidationEngine.registerValidation('isValidTime', validTime);

	/*
	 * Função que preenche um valor com um único caractere até um certo tamanho.
	 * Faz zerofill pela esquerda por default.
	 *
	 * @param string value
	 * @param int length
	 * @param boolean toRight
	 * @param char filler
	 *
	 * @return string
	 *
	 * Exemplos:
	 * General.fill('123', 10);            //'0000000123'
	 * General.fill('123', 10, true);      //'1230000000'
	 * General.fill('123', 10, null, '_'); //'_______123'
	 */
	this.fill = function (value, length, toRight, filler) {
		if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight)
				while (value.length + filler.length <= length) value = value + filler;
			else
				while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
	};

	/*
	 * Função que faz a validação de CNPJ.
	 *
	 * @param string CNPJ
	 *
	 * @return boolean
	 */
	function validCNPJ(CNPJ) {
		if (CNPJ === undefined || CNPJ === null)
			return false;

		CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

		if (CNPJ.length != 14)
			return false;

		if (CNPJ == "00000000000000" ||
			CNPJ == "11111111111111" ||
			CNPJ == "22222222222222" ||
			CNPJ == "33333333333333" ||
			CNPJ == "44444444444444" ||
			CNPJ == "55555555555555" ||
			CNPJ == "66666666666666" ||
			CNPJ == "77777777777777" ||
			CNPJ == "88888888888888" ||
			CNPJ == "99999999999999")
			return false;

		var tamanho = CNPJ.length - 2;
		var numeros = CNPJ.substring(0, tamanho);
		var digitos = CNPJ.substring(tamanho);
		var soma = 0;
		var pos = tamanho - 7;

		for (var i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(0)) //debugger;
			return false; //'resultado' é o 1º digito de validação do CNPJ

		tamanho = tamanho + 1;
		numeros = CNPJ.substring(0, tamanho);
		soma = 0;
		pos = tamanho - 7;

		for (i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(1)) //debugger;
			return false; //'resultado' é o 2º digito de validação do CNPJ

		return true;
	}

	this.validateCNPJ = validCNPJ;

	/*
	 * Função que faz a validação de CPF.
	 *
	 * @param string CPF
	 *
	 * @return boolean
	 */
	function validCPF(CPF) {
		if (CPF === undefined || CPF === null)
			return false;

		CPF = CPF.toString().replace(/[\D]+/g, '');

		if (CPF.length != 11)
			return false;

		if (CPF == "00000000000" ||
			CPF == "11111111111" ||
			CPF == "22222222222" ||
			CPF == "33333333333" ||
			CPF == "44444444444" ||
			CPF == "55555555555" ||
			CPF == "66666666666" ||
			CPF == "77777777777" ||
			CPF == "88888888888" ||
			CPF == "99999999999")
			return false;

		var soma = 0;
		var resto;

		for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(9, 10))) //debugger;
			return false; //'resto' é o 1º dígito de validação do CPF

		soma = 0;
		for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(10, 11))) //debugger;
			return false; //'resto' é o 2º dígito de validação do CPF

		return true;
	}

	this.validateCPF = validCPF;

	/*
	 * Função que faz a formatação de uma inscrição,
	 * seguindo o padrão de código genérico, CPF, CNPJ e
	 * inscrição genérica. Caso o parâmetro "force" for
	 * true, o método tenta forçar uma inscrição válida.
	 *
	 * Retorna um objeto com as seguintes propriedades:
	 *  * parameter //string de entrada contendo apenas números
	 *  * value     //string formatada conforme o tipo da inscrição
	 *  * tipo.cod  //char do código padrão do tipo da inscrição
	 *  * tipo.desc //string de descrição do tipo da inscrição
	 *  * valid     //boolean
	 *
	 * @param string inscricao
	 * @param boolean force
	 *
	 * @return object
	 */
	this.inscricaoFormat = function (inscricao, force) {
		if (inscricao === undefined || inscricao === null || inscricao === '')
			return {
				'parameter': '',
				'value': '',
				'tipo': {
					'cod': null,
					'desc': null
				},
				'valid': false
			};

		if (force === undefined || force === null)
			force = false;
		inscricao = inscricao.toString().replace(/[\D]+/g, '');

		var obj = {};
		obj.tipo = {};

		if (inscricao.length <= 5) {
			obj.tipo.cod = 'O';
			obj.tipo.desc = 'Código Genérico';
			obj.valid = true;
			obj.value = this.fill(inscricao, 5);
		} else {
			if (inscricao.length == 11) {
				obj.tipo.cod = 'F';
				obj.tipo.desc = 'Pessoa Física';
				obj.valid = this.validateCPF(inscricao);
				obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
			} else if (inscricao.length == 14) {
				obj.tipo.cod = 'J';
				obj.tipo.desc = 'Pessoa Jurídica';
				obj.valid = this.validateCNPJ(inscricao);
				obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
			} else force = true;

			if (force && obj.valid !== true) {
				obj.tipo.cod = 'I';
				obj.tipo.desc = 'Inscrição Internacional';

				if (inscricao.length <= 20) {
					obj.valid = true;
					obj.value = this.fill(inscricao, 20);
				} else {
					obj.valid = false;
					obj.value = inscricao.substr(0, 20);
				}
			}
		}

		obj.parameter = inscricao;
		return obj;
	};

	var CPFValidation = function (value, validationAttrs) {
		return {
			"valid": validCPF(value),
			"message": ScreenService.i18n('CPF Inválido')
		};
	};

	ValidationEngine.registerValidation('CPF', CPFValidation);

	var CNPJValidation = function (value, validationAttrs) {
		return {
			"valid": validCNPJ(value),
			"message": ScreenService.i18n('CNPJ Inválido')
		};
	};

	ValidationEngine.registerValidation('CNPJ', CNPJValidation);

	this.callRestEngine = function (dataSource, requestType, requestValue, disableLoader) {
		var defer = ZHPromise.defer();
		var params = {};
		
		switch (requestType) {
		case "Row":
			params = {
				"requestType": "Row",
				"row": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "DataSet":
			params = {
				"requestType": "DataSet",
				"dataset": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "FilterData":
			params = {
				"requestType": "FilterData",
				"filter": requestValue,
				"disableLoader": disableLoader
			};
			break;

		default:
			return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
		}

		return restEngine.request(dataSource, params);
	};

	/*
	 * Função que muda a acessibilidade de uma ou mais
	 * widgets.
	 *
	 * @param array(objects) widgets
	 * @param boolean enabled
	 *
	 * Exemplos:
	 * General.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
	 * General.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
	 */
	this.changeWidgetAcessibility = function (widgets, enabled) {
		widgets.forEach(function (widget) {
			widget.enabled = enabled;
		});

		templateManager.updateTemplate();
	};

	/*
	 * Função que altera o template de um form
	 * entre edição e visualização. Retorna um boolean
	 * que indica se a widget era do tipo form e a
	 * alteração foi realizada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.formWidgetSwitchTemplate = function (widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	/*
	 * Função que coloca uma widget em modo de edição,
	 * bloqueando as demais widgets. Retorna um boolean
	 * que indica se a alteração de modo foi executada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.widgetEdit = function (widget) {
		if (!widget.editing) {
			//Trata a edição de uma widget filha do tipo form
			if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
				var widgetsToDeactivate = widget.parent.widgets;

				this.changeWidgetAcessibility(
					widgetsToDeactivate.filter(function (subwidget) {
						return subwidget.name != widget.name;
					}).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
					false);
			} else //trata a edição dos demais casos
				this.changeWidgetAcessibility(widget.widgets, false);

			//propriedades a serem ignoradas na criação do backup
			var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

			//faz backup da currentRow para permitir a recuperação dos dados
			widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
			widget.edit();
			return true;
		} else return false;
	};

	/*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * própria widget. Provavelmente usado na action de
	 * 'voltar' do back_view.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToList = function (widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * widget parent. Provavelmente usado na action de
	 * 'voltar' do back_list.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToParent = function (widget) {
		ScreenService.closeSwipe().then(function () {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	/*
	 * Procedimento que habilita inserção em uma widget,
	 * bloqueando as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetNewRow = function (widget) {
		this.changeWidgetAcessibility(widget.widgets, false);

		widget.newRow();
	};

	/*
	 * Função que salva os dados de uma widget e retorna
	 * para o modo adequado de visualização ou listagem.
	 * Faz exibição de mensagens de aviso para o usuário
	 * conforme padronização e desbloqueia as demais
	 * widgets.
	 *
	 * Basicamente, um saveCrud adaptado ao padrão.
	 *
	 * @param object widget
	 *
	 * @return promise
	 */
	this.widgetSave = function (widget) {
		var deferred = ZHPromise.defer();

		if (widget.isValid()) {
			widget.currentRow.changed = true;

			if (!widget.currentRow.__is_new)
				widget.currentRow.__is_new = false;
			else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = widget.paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                	widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
			}

			widget.saveRow().then(function (response) {
				widget.currentRow.changed = false;
				delete widget.backupRow;

				if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
					widget.view();

					var widgets = widget.parent.widgets;
					this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
				} else {
					if (widget.currentRow.__is_new) {
						ScreenService.closeSwipe();
						widget.list();
					} else widget.view();

					this.changeWidgetAcessibility(widget.widgets, true);
				}

				if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
				else widget.reload();

				deferred.resolve(response);
			}.bind(this), function (error) {
				var errorMessage = 'Timeout';

				if (error.data) errorMessage = error.data.error;

				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
	};

	/*
	 * Função para validar containers abertos a partir de outros containers.
	 * Se o container atual não foi aberto a partir do container pai, a
	 * callback é executada. Se nenhuma callback for passada nos parâmetros,
	 * o container pai é aberto.
	 *
	 * @param: object {parentContainerName, callback, scope, params}
	 *
	 * Exemplos:
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
	 */
	this.checkParentContainer = function (containerProperties) {
		containers = templateManager.containers;
		if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
			if (angular.isFunction(containerProperties.callback)) {
				containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
			} else {
				ScreenService.openWindow(containerProperties.parentContainerName);
			}
		}
	};

	var messagesWidget = {
		"id": 0,
		"name": "messagesWidget",
		"label": "Mensagens",
		"messagesDataset": "",
		"template": "widget/form.html",
		"floatingControl": {
			"customizationAction": {
				"filter": false,
				"view": false,
				"layout": false
			}
		},
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};

	var messagesFields = [{
		"label": "",
		"name": "CONTENT",
		"readOnly": true,
		"template": "field/textarea.html",
		"textRows": 14,
		"isVisible": true,
		"class": 12,
		"descriptionOrder": true,
		"showOnView": true,
		"showOnForm": true,
		"showOnList": true
    }];

	var messagesActions = [{
		"name": "cancelar",
		"label": "Cancelar",
		"idVisible": true,
		"hideIcon": true,
		"showAsAction": "back",
		"events": [{
			"name": "ActionEvent",
			"code": "ScreenService.closePopup();"
        }]
    }, {
		"name": "exportar",
		"label": "Exportar",
		"idVisible": true,
		"icon": "export",
		"showAsAction": "always",
		"events": [{
			"name": "ActionEvent",
			"code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
        }]
    }];

	/* Função abre um popup com um Text Area a título de mostrar erros em lista.
	 * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
	 * @params: widget, dataset, popupName
	 */
	this.showErrorMessages = function (widget, dataset, popupName) {
		messagesWidget.container = widget.container;
		messagesWidget.fields = messagesFields;
		messagesWidget.actions = messagesActions;

		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

	/* Gera arquivo básico baseado em linhas de dataSet.
	 * @params: dataset, object {fileName}
	 * !!! fileExtension: por enquanto somente TXT;
	 * Exemplo:
	 * General.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
	 */

	this.generateFileByDataSet = function (dataset, fileProperties) {
		var content = "";
		var name = 'content';
		var extension = 'text/plain';

		if (fileProperties !== undefined) {
			if (fileProperties.fileName !== undefined)
				name = fileProperties.fileName;
		}

		dataset.forEach(function (line) {
			content += line + "\r\n";
		});

		var element = document.createElement('a');
		element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
		element.setAttribute('download', name);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

	/**
	 * Este método verifica se a widget foi filtrada usando o zh-grid-filter
	 * @param Widget widget
	 * @return boolean
	 */
	this.isWidgetFiltered = function (widget) {
		var isFiltered = true;
		var currentFilter;
		if(widget.hasOwnProperty('__filterPopup'))
			currentFilter = widget.__filterPopup.currentFilter;
		else
			currentFilter = widget.appliedFilter.FILTER_ROW;
			
		var requiredFields = widget.fields.filter(function (field){
			return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
		});
		if(requiredFields.length && Array.isArray(currentFilter)) {
			isFiltered = false;
		}
		return isFiltered;
	};

	function isRequiredFieldInFilter(field) {
		return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
	}

	function isRequiredFieldWithoutFilterProperties(field) {
		return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
	}

	/**
	 * Este método valida a inserção de registros na widget
	 * @param Widget widget
	 */
	this.newRow = function (widget) {
		if(this.isWidgetFiltered(widget)) {
			widget.newRow();
		} else {
			MessageService.showMessageByCode('lib#GEN_NOT_FILTER', null, { type: 'ERROR' } );
		}
	};

    /*
     * Função que adiciona à currentRow o(s) valor(es)
     * da widget parent necessários para salvar o registro.
     *
     * @param object widget
     * @param object parents
     *
     * @return promise
     */

    this.saveWithParent = function(widget, parents) {
        if(Array.isArray(parents)) {
            parents.forEach(function(parent){
                widget.currentRow[parent] = widget.parent.currentRow[parent];
            });
            return this.widgetSave(widget);
        }
        else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
    };
    
    /*
     * Aceita [- . , 0 1 2 3 4 5 6 7 8 9]
     *
     */
    
     this.justDigits = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonDigits;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justDigits(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonDigits(e) {
        var digits = [44, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var digitCode = e.which;
        if (digits.indexOf(digitCode) < 0) {
            preventEvent(e);
        }
    }

    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousands, precision) {
    	precision = precision || 2;
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousands": thousands,
            "precision": precision
        };
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    /**
     * Função responsável por aplicar uma máscara do type currency
     * no field.
     * 
     * @param {String} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.' || value[0] === ',')
                value = '0' + value;
            if (params.decimal === ',')
                value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            var counter = angular.copy(int.length / 3);

            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');
            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('maskCurrency', maskFunction, this.maskCurrency);
    
     /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} value - Valor numérico.
     */

    this.parseFloat = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/\,/g.test(value))
            value = value.toString().replace(',', '.');
        return parseFloat(value);
    };
    
    /**
     * Função que formata números float vindos do banco, ou em
     * padrão americano e passa para o formato brasileiro.
     * 
     * @param {String} value - Valor numérico.
     */
    
    var numberFormatBr = function(value) {
        if (/^(.*)(\,+).*(\.+)(.*)$/gm.test(value)) {
            var loop = value.toString().split(',');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace(',', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
		else if(/^(\d*)([\.]*)((\d|\.)*)$/.test(value)){
			value = value.split('.');
			value[value.length-1] = ','+value[value.length-1];
			value = value.join('');
		}
        return value.toString();
    };
    
    /**
     * Função criada para corrigir o erro da maxlength.
     * Em vez do maxlength deve ser usada a propriedade maxLengthCurrency,
     * que considera apenas os números inteiros como length.
     * 
     * Função deve ser usada no evento FieldOnKeydown.
     * 
     * @param {Args} args - objeto args.
     * @param {Field} field - field com a propriedade maxLengthCurrency.
     */
    
    this.lengthCurrency = function(args, field) {
        var value;
        var fieldValue;
        
        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = field.widget.currentRow[field.name + '_inline'];
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        } else {
            fieldValue = field.value();
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        }
    
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        
        if (realLength >= field.maxLengthCurrency && !/^(\d+)(\,){1}(\d*)$/gm.test(value)) {
            var allowedKeys = [8, 37, 38, 39, 40, 46, 110, 188, 190, 194];
            if (!allowedKeys.includes(args.e.keyCode))
                args.e.preventDefault();
        }
    };
    
    var treatResponse = function(response, Authenticator) { //
		var url = response.responseURL;
	    var responseURL = null;
	    var responseData = response.response;
	    
	    if (url.indexOf('index.php/') > -1) {
	        responseURL = url.slice(url.indexOf('index.php/') + 9);
	    }
	    
		if(responseURL){
			var responseObj = JSON.parse(responseData);
	        var errorCode = responseObj.error;
			if(Authenticator){
				Authenticator.run(responseURL, responseObj, errorCode);
			} else if(errorCode && errorCode.indexOf('AUTH_') > -1){// Exception: AUTH_INVALID_UNIT 
	            var cleanError = parent.getCleanError(errorCode);
				MessageService.showMessageByCode(cleanError);
			}
		}
	};
	
	this.getCleanError = function(msg){
		return msg.match(/AUTH_[^\s]+/g)[0];
	};
	
	/**
	 * Overwrite XMLHttpRequest.prototype.open
	 * and add a listener on it.
	 */
	this.initRoutesListener = function(Authenticator) {
	    (function(open) {
	        XMLHttpRequest.prototype.open = function(method, url, async, user, pass) {
	            this.addEventListener("readystatechange", function() {
	            	try{
	                	treatResponse(this, Authenticator);
	            	} catch(e){
	            		return false;
	            	}
	            }, false);
	
	            open.call(this, method, url, async, user, pass);
	        };
	    })(XMLHttpRequest.prototype.open);
	};
	
	this.isValidFiliaisValuesCP = function(values){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
        if(arrFilialValida && cacheLoaded){ 
        	var arrValues = [];
        	Array.isArray(values)? arrValues = values : arrValues.push(values);
        	var isInvalid = arrValues.some(function(value){return !arrFilialValida.includes(value)}.bind(this));
        	return !isInvalid;
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then( function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.isValidFiliaisValuesCP(values);
        		}
        	}.bind(this));
        }
	};
	
	this.validateFieldFiliaisCP = function(field){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
		var filialValues;
        if(arrFilialValida && cacheLoaded){ 
            if(field.name == 'CDFILIAL')
                filialValues = field.value();
            else{
                if(field.outData.CDFILIAL){
                    filialValues = field.widget.currentRow[field.outData.CDFILIAL];
                }else{
                    return console.log('Não existe o campo CDFILIAL neste FIELD, usar a função validateFieldFiliaisCP(field)');   
                }
                if(filialValues && filialValues.length > 0){
                	var arrValues = [];
		        	Array.isArray(filialValues)? arrValues = filialValues : arrValues.push(filialValues);
		            var filtered = arrValues.filter(function(filial){return arrFilialValida.includes(filial)}.bind(this));
		            if(filtered.length == 0 || !this.isValidFiliaisValuesCP(arrValues)){
		            	MessageService.showMessageByCode('lib#CP_BRANCH_NOT_ALLOWED', null, { type: 'ERROR' });
		            	/*
		            	Descomentar essa parte quando for para cloquear totalmente o uso de unidades nao cadastradas
		            	
		            	field.setValue('');
		            	field.widget.currentRow[field.name] = '';
			            //remove valores nao liberados do datasource.data
			            var filteredDataSet = field.dataSource.data.filter(function(dataSetRow){return arrFilialValida.some(function(row){return row == dataSetRow.CDFILIAL}.bind(this))}.bind(this));
			            field.dataSource.data = filteredDataSet;
			            
			            //Remove valores nao liberados do spam do autocomplete.
			            if(field.template == 'zh-select-autocomplete#field/select-autocomplete.html'){
				            var spamList = field.element.context.parentElement.getElementsByClassName('list-items')[0].children;
				            for (var i = 0; i < spamList.length -1; i++) {
				            	if(spamList[i].className != 'view-more' && !arrFilialValida.some(function(row){return spamList[i].getAttribute('data-item-identifier').indexOf(row) >= 0}.bind(this))){
							    	spamList[i].remove();
				            		i--
				            	}
							}
			            }//fim if
			            */
		            }
                }
            }
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then(function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.validateFieldFiliaisCP(field);
        		}
        	}.bind(this));
        }
	};
	
	this.setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            if(Array.isArray(outDataValue[key])){
                var newArray = [];
                outDataValue[key].forEach(function(value){ newArray[value] = value}.bind(this)); // jshint ignore:line
                this.setCurrentRowWithOutData(currentRow, valuesToSetOnCurrentRow, newArray);
            }
            else
                currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };
	
};

Configuration(function (ContextRegister) {
	ContextRegister.register('General', General);
});

// FILE: js/Util/LogDecorator.js
function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const PRODUCT_ID_FIELD_NAME         = 'PRODUCT_ID';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const TYPE_FIELD_NAME               = 'TYPE';
    const CONTAINER_FIELD_NAME_NAME     = 'CONTAINER_NAME';
    const WIDGET_NAME_FIELD_NAME        = 'WIDGET_NAME';
    const ORGANIZATION_ID_FIELD_NAME    = 'ORGANIZATION_ID';

    /* Field Labels */
    const USER_NAME_FIELD_LABEL    = 'User Name';
    const ROUTE_LABEL_FIELD_LABEL  = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE       = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE   = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';
    const DATE_INTERVAL_TEMPLATE   = 'zh-interval-component#field/interval.html';
    const DATETIME_TEMPLATE        = 'field/datetime.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate          = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userNameField       = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userNameField.name     = USER_NAME_FIELD_NAME;
        userNameField.label    = USER_NAME_FIELD_LABEL;
        routeLabelField.name   = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label  = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.moveField(widget, USER_ID_FIELD_NAME,        1);
        ZhuLogHelper.addField (widget, userNameField,             2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,           3);
        ZhuLogHelper.moveField(widget, CONTAINER_FIELD_NAME_NAME, 4);
        ZhuLogHelper.moveField(widget, WIDGET_NAME_FIELD_NAME,    5);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,          6);
        ZhuLogHelper.addField (widget, routeLabelField,           7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            },
            showOnList: true,
            maxWidth: '12%',
            class: 3
        });

        ZhuLogHelper.editField(widget, USER_ID_FIELD_NAME,        { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, USER_NAME_FIELD_NAME,      { showOnList: true, maxWidth: '15%', class: 4 });
        ZhuLogHelper.editField(widget, CONTAINER_FIELD_NAME_NAME, { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, WIDGET_NAME_FIELD_NAME,    { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TIME_FIELD_NAME,           { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, ROUTE_FIELD_NAME,          { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TYPE_FIELD_NAME,           { showOnList: true, maxWidth: '8%' , class: 3 });

        ZhuLogHelper.removeField(widget, PRODUCT_ID_FIELD_NAME);
        ZhuLogHelper.removeField(widget, ORGANIZATION_ID_FIELD_NAME);
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }
            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.editField(gridWidget, TIME_FIELD_NAME, {
            template: DATETIME_TEMPLATE,
            dateFormat: "DD/MM/YYYY",
            timeFormat: "HH:mm:ss",
            displayFormat: "DD/MM/YYYY HH:mm:ss",
            "datetime-dateFormat": "dd/mm/yy",
            "datetime-timeFormat": "HH:ii:ss",
            filterProperties: {
                isVisible: true,
                showOnForm: true,
                label: 'Date',
                template: DATE_INTERVAL_TEMPLATE,
                dateFormat: "DD/MM/YYYY",
                displayFormat: "DD/MM/YYYY",
                intervals: [ {
                    "label": "Última semana",
                    "period": "last-week"
                }, {
                    "label": "Último mês",
                    "period": "last-month"
                } ]
            }
        });

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, { class: 4 });
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, { class: 2 });
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       { class: 3 });
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP: {
                start : moment().add(-1, 'days').format('DD/MM/YYYY'),
                end   : moment().format('DD/MM/YYYY'),
                period: undefined
            }
        };
    };

    this.normalizeRows    = function(rows) { return rows; };
    this.normalizeFilters = function(filters) {};
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});


// FILE: js/Util/MenuBuilder.js
var MenuBuilder = function (restEngine, ZHPromise, templateManager, metaDataFactory, eventAggregator, $window, requestDataSourceEngine, ApplicationContext) {
    var localProductMenu;
    var attrToReplace = [
        "label",
        "showMenu",
        "showHeader",
        "showDescriptionOrder",
        "onRequestIdleTimeout",
        "mainWindow",
        "logLevel",
        "aclEnable",
        "showMenuSearch",
        "userDataActions",
        "reportServiceUrl",
        "reportsPath",
        "inDeveloperMode",
        "showDevtoolsInSameWindow",
        "shouldBackToMainWindowOnReload",
        "shortcutKeys"
    ];

    var requestProductMenu = function () {
        var modulesPackage = $window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/containers.json").then(function (response) {
            var productObjMenu = response.data;
            var keys = Object.keys(productObjMenu);
            // get first project menu found
            return productObjMenu[keys[0]];
        });
    };

    var removeEmptyMenuGroups = function(groupMenus) {
        return groupMenus.filter(function(group) {
            return (group.menus && group.menus.length != 0) || group.name == "zhHistory";
        });
    };

    var orderMenus = function(menus) {
        menus.map(function(menu) {
            if(menu.menus) {
                menu.menus = orderMenus(menu.menus);
            }
            return menu;
        });

        return sortMenus(menus);
    };

    var sortMenus = function(menus) {
        return menus.sort(function(menuA, menuB) {
            if (menuA.order && menuB.order) {
                if (menuA.order < menuB.order) {
                    return -1;
                }
                if (menuA.order > menuB.order) {
                    return 1;
                }
            }
            if (!menuA.order && menuB.order) {
                return -1;
            }
            if (menuA.order && menuB.order) {
                return 1;
            }
            return 0;
        });
    };

    var prepareMenus = function (response) {
        var groupMenus = response.dataset.menu;
        window.routesTranslate = response.dataset.routesTranslate;
        var zhuLogWidget = templateManager.container ? templateManager.container.getWidget('zhuLog') : false;
        if(zhuLogWidget) {
            zhuLogWidget.getField('ROUTE_LABEL').dataSource.data = window.routesTranslate;
            $(window).trigger('resize.grid');
        }
        groupMenus = orderMenus(removeEmptyMenuGroups(groupMenus));
        var groupMenuHistory = [templateManager.project.getMenu("zhHistory")];
        templateManager.project.groupMenu = metaDataFactory.groupMenuFactory(groupMenus.concat(groupMenuHistory));

        //Force framework to apply ACL Rules
        var aux = {};
        Object.keys(templateManager.aclRules).forEach(function(ruleKey){
            aux[ruleKey] = templateManager.aclRules[ruleKey];
        });
        templateManager.aclRules = aux;
        //Force framework to apply ACL Rules

        eventAggregator.publish('menuBuilded');
    };

    var replaceProductAttrOnMenu = function (productMenu) {
        attrToReplace.forEach(function (attr) {
            templateManager.project[attr] = productMenu[attr];
        });
    };

    var setLocalProductMenu = function () {
        if (!$window.modulesPackage) {
            requestProductMenu().then(function (productMenu) {
                localProductMenu = productMenu;
            });
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    if(!window.notUseMicroservice) {
        eventAggregator.subscribe('loginSuccess', function (loginResponse) {
            eventAggregator.publish('AboutOnClick');
            if (!$window.modulesPackage && !$window.keepTitle) {
                replaceProductAttrOnMenu(localProductMenu);
                templateManager.updateTitle(templateManager.project.label);
            }
            var filter = [{
                "name": "CONCATENATE_GROUP_MENU",
                "operator" : "=",
                "value" :  true
            }, {
                "name": "VISIBLE_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.VISIBLE_ELEMENTS || []
            }, {
                "name": "EXPIRED_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.EXPIRED_ELEMENTS || []
            }, {
                "name": "URL_ORIGIN",
                "operator" : "=",
                "value" :  window.location.origin
            }, {
                "name": "CURRENT_MODULE",
                "operator" : "=",
                "value" :  getCurrentModule()
            }, {
                "name": "CURRENT_LANGUAGE",
                "operator" : "=",
                "value" :  window.lang
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_buildMenu', filter)
                .then(prepareMenus)
                .catch(ApplicationContext.LibUtilities.showErrorMessage);
        });

        setLocalProductMenu();
    }
};

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuBuilder', MenuBuilder);
});

// FILE: js/Util/Messages.js
function LibMessages(){
    this.ALE_EXPIRED_SESSION = 'Sua sessão expirou.<br/>Você será redirecionado para efetuar login novamente.';
    this.ALE_MIGRATION_IS_PENDENT = 'ATENÇÃO!<br/><br/>Esse produto foi atualizado recentemente, com necessidade de alterações na estrutura do banco de dados, sem que essas alterações tenham sido aplicadas.<br/>A utilização do produto nessa situação poderá acarretar erros.<br/><br/>Solicite ao administrador do sistema a regularização da alteração na estrutura do bando de dados.', 'INFORMATION';

    this.ERR_ATTEMPT_END = 'Tentativas esgotadas. Redirecionando...';
    this.ERR_ATTEMPT_PARTIAL = 'Senha incorreta.<br/>Tentativa %1/3';
    this.ERR_EXPIRED_SESSION_LOGIN = 'Sessão expirada. Redirecionando...';
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibMessages', LibMessages);
});


// FILE: js/Util/ReportDecorator.js
function ReportDecorator(eventAggregator, restEngine, templateManager, ZhLogger, ScreenService, eventEngine) {
    
    this.onContainerOpen = function() {
        try {
            var container = templateManager.container;
            validContainer(container);
            if (container.customReportsConfig) {
                var customReportsConfig = container.customReportsConfig;
                validCustomReportsConfig(customReportsConfig);
                var widget = selectWidget(container, customReportsConfig.widgetName);
                var containerName = container.customReportsConfig ? container.customReportsConfig.reportCodeName : container.name;
                
                var fieldToChangeData = selectField(widget, customReportsConfig.fieldName);
                
                if (customReportsConfig.reportIsTheMainFunction === undefined || customReportsConfig.reportIsTheMainFunction === true) {
                    buildData(containerName, fieldToChangeData);
                } else {
                    const separator = "', '";
                    const eventCode = "ReportDecorator.triggerBuildData('" + containerName + separator + customReportsConfig.widgetName + separator + customReportsConfig.fieldName + "')";
                    
                    addEvent({
                        entity: widget,
                        event: {
                            name: 'WidgetOnEnterOnce',
                            code: eventCode,
                            bind: true,
                        }
                    });
                }
                widget.beforeOpenReport = beforeOpenReport;
            }
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    eventAggregator.subscribe('onContainerOpen',this.onContainerOpen);
    
    this.triggerBuildData = function(containerName, widgetName, fieldName) {
        const container = templateManager.container;
        
        validContainer(container);
        
        const widget = selectWidget(container, widgetName);
        const fieldToChangeData = selectField(widget, fieldName);
        
        try {
            buildData(containerName, fieldToChangeData);
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    
    function buildData(containerName, fieldToChangeData) {
        const route = getServiceUrlBasedOnTriggerPackage() + '#/lib_getDataReportField';
        
        restEngine.post(route, {
            "requestType": "Row",
            "row": [containerName.substr(0, 6)]
        }).then(function (callback) {
            try {
                var data = callback.dataset.dataReport;
                
                handleErrors(data);
                
                const formattedData = getFormattedData(fieldToChangeData, data);
                
                setDataSource(fieldToChangeData, formattedData);
                applyDefaultValue(fieldToChangeData);
                
                /**
                 * Força o select do item para que haja alteração visual.
                 * Operações visuais como updateTemplate e redraw não surtiram efeitos.
                 */
                setTimeout(function() {
                    clickOnSelectedOption(fieldToChangeData.name);
                }, 500);
                
                addEvent({
                    entity: fieldToChangeData.getParent(),
                    event: {
                        name: 'WidgetOnEnter',
                        code: "ReportDecorator.onEnterReportWidget('" + fieldToChangeData.name + "')",
                        bind: true,
                    }
                });
            } catch (e) {
                ZhLogger.warning(e);
            }
        }.bind(this), function(error) {
            if (error.data && error.data.error) {
                throw error.data.error;
            } else {
                throw 'Ocorreu um erro ao buscar a lista de relátorios da tela.';
            }
        });
    }
    
    function getServiceUrlBasedOnTriggerPackage() {
        const fileName = templateManager.container.fileName;
        const aliasSplited = fileName.split('#').shift().split('_');
        
        if (aliasSplited.length) {
            const packageName = aliasSplited.shift();
            
            if (packageName) return packageName;
        }
        
        return 'lib';
    }
    
    function getFormattedData(fieldToChangeData, data) {
        return data
                .filter(function(reportElement) {
                    return !Array.isArray(reportElement);
                })
                .map(function(reportElement) {
                    if (Array.isArray(reportElement)) return;
                    
                    const newElement = {};
                    
                    newElement[fieldToChangeData.valueField] = reportElement.value;
                    newElement[fieldToChangeData.descriptionField] = reportElement.description;
                    newElement[fieldToChangeData.strategyField] = reportElement.strategy;
                    
                    return newElement;
                });
    }
    
    function handleErrors(data) {
        const errorIndex = data.findIndex(function(reportElement) {
            return Array.isArray(reportElement);
        });
        
        if (errorIndex === -1) return;
            
        if (data.length === 1) {
            const message = 'Ocorreu um erro ao buscar a lista de relatórios da tela.';
            
            ScreenService.errorNotification(message);
            
            throw message + 'Consulte a requisição correspondente para mais detalhes.';
        } else {
            const isQr2Parametrized = data[errorIndex].findIndex(function(error) {
                return error === 'Servidor de relatórios QR2 não está parametrizado.';
            }) === -1;
            
            const isBirtParametrized = data[errorIndex].findIndex(function(error) {
                return error === 'Servidor de relatórios BIRT não está parametrizado.';
            }) === -1;
            
            if (isQr2Parametrized || isBirtParametrized) {
                ScreenService.errorNotification('Alguns relatórios podem não estar disponíveis.');
            }
            
        	ZhLogger.warning('Alguns relatórios podem não estar disponíveis. Consulte a requisição correspondente para mais detalhes.');
        }
    }
    
    function setDataSource(fieldToChangeData, data) {
        if (!fieldToChangeData.dataSource.data || fieldToChangeData.dataSource.data.length == 0) {
            fieldToChangeData.dataSource.data = data;
        } else {
            const valueField = fieldToChangeData.valueField;
            const descriptionField = fieldToChangeData.descriptionField;
            const strategyField = fieldToChangeData.strategyField;
            
            const removingDuplicatedReports = function(report, index, reports) {
                return index === reports.findIndex(function(r) {
                    return r[valueField] === report[valueField] &&
                           r[descriptionField] === report[descriptionField] &&
                           r[strategyField] === report[strategyField];
                });
            };
            
            const newData = fieldToChangeData.dataSource.data.concat(data);
            
            fieldToChangeData.dataSource.data = newData.filter(removingDuplicatedReports);
        }
    }
    
    function applyDefaultValue(fieldToChangeData) {
        if (!fieldToChangeData.defaultValue || fieldToChangeData.defaultValue === 'firstRow') {
            const firstRow = fieldToChangeData.dataSource.getFirstRow();
            
            fieldToChangeData.defaultValue = firstRow[fieldToChangeData.valueField];
        }
        
        fieldToChangeData.applyDefaultValue();
    }
    
    function clickOnSelectedOption(fieldName) {
        const baseSelector = "#" + fieldName + " + div";
        const baseElement = document.querySelector(baseSelector);
        
        if (!baseElement) {
            return setTimeout(function recursiveCallClickOnSelectedOption() {
                clickOnSelectedOption(fieldName);
            }, 500);
        }
        
        document.querySelector(baseSelector).click();
        
        const selectedElementSelector = baseSelector + ' li[class*="selected"]';
        
        document.querySelector(selectedElementSelector).click();
    }
    
    function selectWidget(container, widgetName) {
        var widget = container.getWidget(widgetName);
        
        if (widget) {
            return widget;
        } else {
            throw "A widget informada não existe dentro desse container.";
        }
    }
    
    function selectField(widget, fieldName) {
        var field = widget.getField(fieldName);
        
        if (field) {
            if (field.dataSource) {
                return field;
            } else {
                throw "O field informado não contém a propriedade dataSource.";    
            }
        } else {
            throw "O field informado não existe na widget informada.";
        }
    }
    
    function validContainer(container) {
        if (!container) {
            throw "Container não existe.";
        }
    }
    
    function validCustomReportsConfig(config) {
        if (!config.widgetName) {
            throw 'A propriedade widgetName não existe dentro de customReportsConfig.';
        }
        
        if (!config.fieldName) {
            throw 'A propriedade fieldName não existe dentro de customReportsConfig.';
        }
    }
    
    function eventExistsByCode(events, searchedCode) {
        return events.filter(function(event){
            return event.code.indexOf(searchedCode) != -1;
        }).length > 0;
    }
    
    function eventExistsByName(events, searchedName) {
        return events.findIndex(function(event){
            return event.name === searchedName;
        }) !== -1;
    }
    
    function addCodeOnEvent(events, eventName, eventCode, placeOn) {
        if (!placeOn) placeOn = 'right';
        
        const eventIndex = events.findIndex(function(event) {
            return event.name === eventName;
        });
        const event = events[eventIndex];
        const endSemicolonIndex = event.code.lastIndexOf(';');
        const codeHasEndSemicolon = endSemicolonIndex !== -1 && endSemicolonIndex === event.code.length - 1;
        
        if (!codeHasEndSemicolon) {
            event.code += ';';
        }
        
        if (placeOn === 'right') {
            events[eventIndex].code += eventCode;
        } else {
            const newCode = eventCode + ';' + events[eventIndex].code;
            
            events[eventIndex].code = newCode;
        }
    }
    
    function addEvent(params) {
        const entity = params.entity;
        const event = params.event;
        
        const name = event.name;
        const code = event.code;
        const events = entity.events;
        
        if (eventExistsByName(events, name)) {
            if (!eventExistsByCode(events, code)) {
                addCodeOnEvent(events, name, code, event.placeOn);
            }
        } else {
            events.push({
                name: name,
                code: code,
            });
            
            if (event.bind) {
                eventEngine.bindEvent(entity, name, code);
            }
        }
    }
    
    function getSelectedOption(fieldName) {
        const selector = '#' + fieldName + ' + div li[class*="selected"]';
        
        return document.querySelector(selector);
    }
    
    function getCurrent(fieldName) {
        const selector = '#' + fieldName + ' + div > span';
        
        return document.querySelector(selector).textContent;
    }
    
    this.onEnterReportWidget = function(fieldName) {
        const field = templateManager.container.getActiveWidget().getField(fieldName);
        
        if (!field.value()) {
            applyDefaultValue(field);
        }
        
        if (!getSelectedOption(fieldName)) {
            setTimeout(function runClickOnSelectedOption() {
                const selectedOption = getSelectedOption(fieldName);
                
                if (!selectedOption) {
                    return setTimeout(runClickOnSelectedOption, 500);
                }
                
                const current = getCurrent(fieldName);
                
                if (!current) {
                    return clickOnSelectedOption(fieldName);
                }
            }, 500);
        }
    };
    
    function beforeOpenReport(params) {
        const reportParams = params.reportParams;
        const reportNameSplitted = reportParams.__report_name.split('.');
        const reportName = reportNameSplitted[0];
        const strategy = reportNameSplitted[1];
        
        reportParams.__report_name = reportName;
        reportParams.STRATEGY = strategy;
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('ReportDecorator', ReportDecorator);
});

// FILE: js/Util/ShortcutDecorator.js
function ShortcutDecorator(ApplicationContext, templateManager) {

    /**
     *  Default keys for CRUD icons
     */
    const defaultCRUDShortcutKeys = {
        "add": "mod+a",
        "edit": "mod+e",
        "delete": "mod+d"
    };

    /**
     * 
     * Get the 'mod' key name according to OS (Ctrl or Command)
     * @returns String(Ctrl, Command) / Boolean(false)
     */
    function getTooltipModKey() {
        var platform = window.navigator.platform,
            commandPlatforms = ['Macintosh', 'MacIntel', 'MacPPC', 'Mac68K'],
            ctrlPlatforms = ['Win32', 'Win64', 'Windows', 'WinCE'],
            mod = false;

        if (commandPlatforms.indexOf(platform) !== -1) {
            mod = 'Command + ';
        } else if (ctrlPlatforms.indexOf(platform) !== -1 || /Linux/.test(platform)) {
            mod = 'Ctrl + ';
        }
        return mod;
    }

    /**
     * 
     * Get shortcut key from shortcutKeys object of widget, if not defined, 
     * from project property and if not defined, 
     * from defaultCRUDShortcutKeys const in this function.
     * @param {any} widget 
     * @param {any} shortcut 
     * @returns String(Shortcut key) / Undefined
     */
    function getShortcutKey(widget, shortcut) {
        return (
            Util.getNestedProperty(widget, ['shortcutKeys', shortcut]) ||
            Util.getNestedProperty(templateManager, ['project', 'shortcutKeys', shortcut]) ||
            defaultCRUDShortcutKeys[shortcut]
        );
    }
/**
 * 
 * Iterate recursively in widgets of container applying shortcut key in actions. 
 * @param {any} widgets 
 */
function applyShortcutOnAction(widgets) {
        var modTooltip = getTooltipModKey();
        widgets.forEach(function (widget) {
            if (widget.actions) {
                widget.actions.forEach(function (action) {
                    if (action.icon === 'plus' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'add');
                    } else if (action.icon === 'pencil' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'edit');
                    } else if (action.icon === 'trash' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'delete');
                    } else if (!action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, action.icon);
                    }
                    if (modTooltip && action.shortcutKey && !action.tooltip && templateManager.project.showShortcutKeyAsTooltip !== false) {
                        action.tooltip = action.shortcutKey.toUpperCase().replace('MOD+', modTooltip);
                    }
                });
            }
            if (widget.widgets && widget.widgets.length) {
                applyShortcutOnAction(widget.widgets);
            }
        });
    }

    /**
     * 
     * If project have the property enableCRUDShortcutKeys == true, applies the shortcut keys in actions
     * @param {any} containerName 
     * @param {any} container 
     * @returns Container(container)
     */
    this.apply = function (containerName, container) {
        if (templateManager.project.enableCRUDShortcutKeys == true) {
            applyShortcutOnAction(container.widgets);
        }
        return container;
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ShortcutDecorator', ShortcutDecorator);
    ContainerDecoratorProvider.registerProvider(new ShortcutDecorator(ApplicationContext, templateManager));
});


// FILE: js/Util/Utilities.js
function LibUtilities(ScreenService, eventAggregator, templateManager, requestDataSourceEngine, restEngine, MessageService) {
    var self = this;
    this.authenticationContainer = 'login#authentication';
    this.expiredSessionMethod = null;

    this.showErrorMessage = function(response) {
        return MessageService.getMessageByCode('lib#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode('lib#' + messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    if(!message.includes('ALE_SESSION_WILL_BE_ENDED')) {
                        return ScreenService.showMessage(message, 'ERROR');
                    }
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.redirectToLogin = function() {
        if(window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var validateExpiredSessionRoute = 'lib_validateExpiredSession';
        var logoutRoute = 'lib_logout';
        var loginRoute  = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if(route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if(window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if(eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if(route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;

        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'USER',
                value: localStorage.getItem('LOGIN_USER'),
                operator: '='
            }, {
                name: 'IS_LOGGED',
                value: localStorage.getItem('IS_LOGGED') || false,
                operator: '='
            }, {
                name: 'KEEP_CONNECTED',
                value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
                operator: '='
            }, {
                name: 'LOGIN_LANGUAGE',
                value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
                operator: '='
            }  ];

            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter).then(function() {
                ScreenService.hideLoader();
                ScreenService.openLink(args.owner.url);
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        return requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    eventAggregator.subscribe("AboutOnClick", function () {
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: getCurrentModule()
            } ];
            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource('lib#/lib_getVersionInfo', filter).then(function (response) {
                ScreenService.hideLoader();
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.current = templateManager.versionData.dependencies.frontend;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                };
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    });

    eventAggregator.subscribe("EventOnRequestError", function (eventData) {
        if(eventData.e && eventData.e.e && eventData.e.e.data && eventData.e.e.data.data) {
            var error = eventData.e.e.data.data;
            var config = eventData.e.e.data.config;
            //Expired or Invalid Token
            if(error.errorCode == 101 || error.errorCode == 100)  {
                eventAggregator.publish('LibInvalidToken', error.errorCode);
            } else if(error.errorCode == 102){
                eventAggregator.publish('LibInvalidTokenSupportOperator');
            } else if(error.errorCode == 20){
                eventAggregator.publish('LibConcurrentAccess');
            }
        }
    });

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    /**
     * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
     * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
     * @param $productId      => Código do Produto (opcional).
     * @param $operatorCode   => Código do Operador (opcional).
     * @param $operatorName   => Nome do Operador (opcional).
     * @param $organizationId => Código do organização (opcional).
     */
    this.saveCollectedData = function(collectionIds, operationId, productId, operatorCode, operatorName, organizationId) {
        var row = {
            COLLECTION_IDS : collectionIds,
            OPERATION_ID   : operationId,
            PRODUCT_ID     : productId		|| null,
            OPERATOR_CODE  : operatorCode   || null,
            OPERATOR_NAME  : operatorName   || null,
            ORGANIZATION_ID: organizationId || null,
            changed        : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveCollectedData', [row]);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});


// FILE: js/Util/ViewFloatMask.js
var ViewFloatMask = function(maskEngine) {
    var keypressHandler = function (e, element, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var thousandsCode = opt.thousands.charCodeAt();
        var isThousands = (thousandsCode === e.which);
        var is_number = (e.which >= 48 && e.which <= 57);
        var is_dot = (e.which == 46);
        var is_comma = (e.which == 44);
        var is_minus = (e.which == 45);
        var hasSeparator = (element.val().indexOf(opt.decimal) != -1 || element.val().indexOf(opt.thousands) != -1);
        var hasNegative = (element.val().length > 0 || !opt.allowNegative);

        if ((hasSeparator && (is_dot || is_comma)) || (hasNegative && is_minus) || isThousands)
            e.preventDefault();

        if (is_dot || is_comma || is_number || is_minus)
            element.data('maskOptions', opt);
        else
            e.preventDefault();
    };

    var keyupHandler = function (e, element, params) {

        if(params.precision){
            var value = element[0].value;
            var decimals = value.substring(value.indexOf(params.decimal || ",")+1, value.length);
            if(value != decimals){
                if(decimals.length > params.precision){
                    element[0].value = value.substring(0, value.indexOf(params.decimal || ",")+(params.precision + 1));
                }

            }

        }

    };

    var viewFloat = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        params.showRound = !('showRound' in params) ? true : params.showRound;
        params.showSymbol = false;

        element.on("keypress", function(event) {
            keypressHandler(event, element, params);
        });

        element.on("keyup", function(event) {
            keyupHandler(event, element, params);
        });

        element.on("blur", function() {
            element.text(unMaskViewFloat(element.val(), params));
            element.val(viewFloatFormat(element.val(), params));
            params.callback(element.text());
        });

        if (!!params.showRound) {
            element.on("focus", function() {
                var value = element.text();
                var viewValue = viewFloatFormat(element.val(), params);
                var currentValue = viewFloatFormat(element.text(), params);

                if (currentValue !== viewValue)
                    value = element.val();

                element.val(viewFloatFormat(value, params));
                element.select();
            });
        }
    };

    var viewFloatFormat = function(value, params) {
        var defaultOptions = Util.clone($.currency.i18n[$.currency.lang]);

        var valueFormated = "";
        try {
            var options = Util.extend(defaultOptions, (params || {}), true);
            valueFormated = $.currency.string(value, options);
        } catch (e) {
            valueFormated = value;
        }
        return valueFormated;
    };

    var unMaskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var fltValue = opt.allowZero && !opt.allowEmpty ? 0 : "";
        var valueFormated = value;

        if (!Util.isEmptyOrBlank(valueFormated)) {
            var decimal = opt.decimal;
            var thousands = opt.thousands;

            if (typeof valueFormated === "string") {
                if(opt.showSymbol)
                    valueFormated = valueFormated.replace(opt.symbol, '');

                if(thousands)
                    valueFormated = valueFormated.replace(new RegExp('\\' + thousands, 'g'), '');

                valueFormated = valueFormated.replace(new RegExp('\\' + decimal, 'g'), '.');
                valueFormated = removeInvalidChar(valueFormated);
            }
            fltValue = parseFloat(valueFormated);

            if ((!opt.allowZero && fltValue === 0) || $.currency.isInvalidValue(fltValue, opt))
                valueFormated = "";
        }
        return valueFormated;
    };

    var removeInvalidChar = function(value) {
        var index = value.indexOf('.');
        return index + 1 === value.length ? value.slice(0, index) : value;
    };

    var maskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);

        if (value)
            value = value.replace(new RegExp('\\' + '.', 'g'), opt.decimal);

        return value;
    };

    this.register = maskEngine.registerMask('viewFloat', viewFloat, viewFloatFormat);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewFloatMask', ViewFloatMask);
});

// FILE: js/Util/WidgetsMetaData.js
function WidgetsMetaData() {
    this.getExpiredSession = function() {
        return {
            "name": "EXPIRED_SESSION",
            "label": "Sessão Expirada",
            "isVisible": true,
            "showDescriptionOrder": false,
            "showAsModal": true,
            "template": "widget/form.html",
            "tekLibExpiredSession": true,
            "actions": [
                {
                    "name": "CONFIRM",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventConfirm(args)"
                        }
                    ]
                },
                {
                    "name": "CANCEL",
                    "label": "Sair",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventCancel()"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": [],
            "fields": [
                {
                    "name": "OPERATOR",
                    "label": "Usuário",
                    "template": "field/text-edit.html",
                    "class": 12,
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "PASSWORD",
                    "label": "Senha",
                    "template": "field/password.html",
                    "class": 12,
                    "focused": true,
                    "validations":{
                        required: true
                    },
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true,
                    "events":[
                        {
                            "name": "FieldOnKeyup",
                            "code": "LibLoginController.expiredSessionFieldOnKeyupPassword(args)"
                        }
                    ]
                }
            ]
        };
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('WidgetsMetaData', WidgetsMetaData);
});


// FILE: js/Service/Login.js
function LibLoginService(requestDataSourceEngine){
    this.startSession = function(user) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'IS_LOGGED',
            value: localStorage.getItem('IS_LOGGED') || false,
            operator: '='
        }, {
            name: 'LOGIN_TOKEN',
            value: localStorage.getItem('LOGIN_TOKEN'),
            operator: '='
        }, {
            name: 'LOGIN_LANGUAGE',
            value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_startSession', filter, 1, undefined, undefined, undefined, 120000)
            .then(function(data) {
                if(typeof data == 'string') {
                    return {
                        isError : true,
                        message : data
                    };
                }
                return data && data.dataset ? data.dataset.system : false;
            });
    };

    this.validateUser = function(cdOperator, password, isSupportOperator) {
        var filter = [ {
            name: 'USER',
            value: cdOperator,
            operator: '='
        }, {
            name: 'PASSWORD',
            value: password,
            operator: '='
        }, {
            name: 'IS_SUPPORT_OPERATOR',
            value: isSupportOperator,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('login#/validateUser', filter);
    };

    this.logout = function() {
        var filter = [ {
            name: 'PROJECT',
            value: window.tekLoginProjectId,
            operator: '='
        }, {
            name: 'ERASE_HASH',
            value: localStorage.getItem('LOGIN_ERASE_HASH'),
            operator: '='
        }, {
            name: 'HASH',
            value: localStorage.getItem('LOGIN_HASH'),
            operator: '='
        } ];
        return requestDataSourceEngine.getDataSource('lib#/lib_logout', filter, 1, undefined, undefined, undefined, 120000);
    };

    this.getModulesVersion = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_getModulesVersion', []);
    };

    this.generateModulesVersionFile = function() {
        return requestDataSourceEngine.setDataSource('lib#/lib_generateModulesVersionFile', [ { changed: true } ]);
    };

    this.downloadModulesVersionFile = function(fileName) {
        var serviceUrl = window.serviceUrl;
        window.open(serviceUrl + "/lib_downloadModulesVersionFile?requestType=DataSet&dataset[0][FILE_NAME]=" + fileName);
    };

    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'ACCESS_PROFILE',
            value: accessProfile,
            operator: '='
        }, {
            name: 'OTHER_SESSION_VALUES',
            value: otherSessionValues,
            operator: '='
        }, {
            name: 'LOGIN_USER',
            value: localStorage.getItem('LOGIN_USER'),
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_updateSession', filter);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginService', LibLoginService);
});



// FILE: js/Controller/Login.js
function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var updateLanguage = function () {
        var language = localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var redirectToRedirectionProduct = function (redirectUrl, defer) {
        if (window.ZhNativeInterface) {
            window.ZhNativeInterface.redirect(redirectUrl);
        } else {
            window.location = redirectUrl;
        }
        defer.resolve({});
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var redirectUrl = localStorage.getItem('LOGIN_REDIRECT_URL');
            var authToken = updateToken();

            if(redirectUrl && !redirectUrl.includes(window.location.hostname)) {
                redirectToRedirectionProduct(redirectUrl, defer);
            } else if(!user && !authToken) {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }

                defer.resolve(response);
            }
        }
    };

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    // Requisição getModulesVersion chamada para atualizar o tempo de sessão local (produto ou módulo)
                    LibLoginService.getModulesVersion();
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});

// FILE: js/Listener/ZdNext.js
function LibZdNextListener(ScreenService, LibLoginService, LibUtilities) {
    window.addEventListener('message', handleMessageEvent);

    function handleMessageEvent(event) {
        if(event.data.type === 'logoutByZdNext') {
            logoutByZdNext()
        }
    }

    var logoutByZdNext = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibZdNextListener', LibZdNextListener);
});


// FILE: bower_components/zh-image-uploader/dist/zh-image-uploader.js

// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-image-uploader';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/controllers/ImageUploaderController.js
function ImageUploaderController($scope, templateManager) {

	var self = this;

	var widgetAccessorDescriptor = {
		get: function() {
			return $scope.widget;
		}
	};

	var fieldAcessosDescriptor = {
		get: function() {
			return $scope.field;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);
	Object.defineProperty(self, "field", fieldAcessosDescriptor);

	/***************************************************
	 * Functions used by template
	 ***************************************************/
	this.getChooseFileText = function() {
		return self.field.chooseFileText || "Choose a file...";
	};

	this.getNoFileSelectedText = function() {
		return self.field.noFileSelectedText || "No file selected.";
	};

	this.getCurrentThumbnail = function() {
		var currentThumbnail = "//:0";
		if (typeof self.widget.currentRow[self.field.name] === "string") {
			var root = templateManager.project.fileServiceBaseUrl || "";
			currentThumbnail = root + self.widget.currentRow[self.field.name];
		} else if (Array.isArray(self.widget.currentRow[self.field.name]) && self.widget.currentRow[self.field.name].length > 0) {
			currentThumbnail = self.widget.currentRow[self.field.name][0].b64File;
		}

		return currentThumbnail;
	};

	this.isVisibleInCurrentFormTemplate = function() {};

	this.isEditMode = function() {
		return self.widget.mode === 'edit';
	};

	this.deselectFile = function(file) {
		self.widget.currentRow[self.field.name].splice(self.widget.currentRow[self.field.name].indexOf(file), 1);
		self.field.fileList = self.widget.currentRow[self.field.name];

		_callChangeEvent();
		_callAfterDeselectFileEvent();
	};

	function _callChangeEvent() {
		if (self.field.change) {
			self.field.change();
		}
	}

	function _callAfterDeselectFileEvent() {
		if (self.field.afterDeselectFile) {
			self.field.afterDeselectFile();
		}
	}
}

// FILE: src/directives/ImageUploaderDirective.js
(function() {
	var ZeedhiDirectives = angular.module('ZeedhiDirectives');
	ZeedhiDirectives.directive('zhImageUploader', zhImageUploader);

	function zhImageUploader(templateManager, ZHPromise) {

		var linkFunction = function(scope, element, attrs) {
			var container = element.parent();
			var field = scope.field;
			var widget = field.widget;
			var filesPromises = [];

			field.fileList = [];
			element.shouldNotPreventDefaultOnClick = true;
			element.parent().find("ul").nonbounce();
			element.on('change', onFileSelected);

			function onFileSelected() {
				var files = this.files;

				if (files.length > 0) {
					_ensureCurrentRowPropertyExistence();
					_setCurrentRowAsChanged();
					_putFilesIntoPromises(files);
					_executeAllPromises();
				}

				this.value = "";
			}

			function _ensureCurrentRowPropertyExistence() {
				widget.currentRow = widget.currentRow || {};
				widget.currentRow[field.name] = widget.currentRow[field.name] || [];
			}

			function _setCurrentRowAsChanged() {
				widget.currentRow.changed = true;
			}

			function _putFilesIntoPromises(files) {
				for (var i in files) {
					if (files.hasOwnProperty(i) && files[i] instanceof File) {
						filesPromises.push(_buildFileData(files[i]));
					}
				}
			}

			function _buildFileData(file) {
				var deferred = ZHPromise.defer();
				var fileReader = new FileReader();

				fileReader.onload = function(event) {
					widget.currentRow[field.name] = [_addExtraDataToFile(file)];
					field.fileList = widget.currentRow[field.name];
					deferred.resolve();
				};

				fileReader.readAsDataURL(file);
				return deferred;
			}

			function _addExtraDataToFile(file) {
				return Util.extend(file, {
					b64File: event.target.result
				}, true);
			}

			function _executeAllPromises() {
				ZHPromise.all(filesPromises)
					.then(function() {
						_callAfterSelectFileEvent();
						field.validate(field.value());
						templateManager.updateTemplate();
					});
			}

			function _callAfterSelectFileEvent() {
				if (field.afterSelectFile) {
					field.afterSelectFile();
				}
			}
		};

		return {
			restrict: 'A',
			link: linkFunction
		};
	}
})();

// FILE: src/registers/ImageUploaderRegister.js
function ImageUploaderRegister(DecodeValueFieldService){
    DecodeValueFieldService.registerType("zh-image-uploader#field/image-uploader.html", "image");
}

Configuration(function(ContextRegister) {
  ContextRegister.register('ImageUploaderRegister', ImageUploaderRegister);
});
